#ifndef _TUNNEL_CAMERA_H_
#define _TUNNEL_CAMERA_H_

#include "others/includes.h"

#define RAPPORT_VITESSE 3.

#define DUMPVARIABLE(name,v) { \
 cout << (name) << " ::= " << (v) << endl; \
}
#define DUMPVECTOR(name,v) { \
cout << (name) << " :: [" << (v)[0] << " , " << (v)[1] << " , " << (v)[2] << "]   (" << NORMVECTOR(v) << ")" << endl; \
}
#define DUMPMATRIX(name,m) { \
DUMPVECTOR(name,(m)[0]); \
DUMPVECTOR(name,(m)[1]); \
DUMPVECTOR(name,(m)[2]); \
}
#define SETVECTOR(v,x,y,z) { \
(v)[0] = (x); \
(v)[1] = (y); \
(v)[2] = (z); \
}
#define COPYVECTOR(a,b) { \
(a)[0] = (b)[0]; \
(a)[1] = (b)[1]; \
(a)[2] = (b)[2]; \
}
#define ADDVECTOR(a,v,b) { \
(a)[0] += (v)*(b)[0]; \
(a)[1] += (v)*(b)[1]; \
(a)[2] += (v)*(b)[2]; \
}
#define SUBVECTOR(a,v,b) { \
(a)[0] -= (v)*(b)[0]; \
(a)[1] -= (v)*(b)[1]; \
(a)[2] -= (v)*(b)[2]; \
}
#define NORMVECTOR(v) (sqrt((v)[0]*(v)[0]+(v)[1]*(v)[1]+(v)[2]*(v)[2]))
#define NORMALIZEVECTOR(v) { \
double nv = NORMVECTOR(v); \
(v)[0] = (v)[0]/nv; \
(v)[1] = (v)[1]/nv; \
(v)[2] = (v)[2]/nv; \
}
#define DOTPRODUCT(a,b) ((a)[0]*(b)[0]+(a)[1]*(b)[1]+(a)[2]*(b)[2])
#define CROSSPRODUCT(a,b,v) { \
v[0] = (a)[1]*(b)[2]-(a)[2]*(b)[1]; \
v[1] = (a)[2]*(b)[0]-(a)[0]*(b)[2]; \
v[2] = (a)[0]*(b)[1]-(a)[1]*(b)[0]; \
}

class Camera
{
 private:
  double _position[3];
  double _direction[3];
  double _right[3];
  double _up[3];
  double _focal;
  double _fov;
  double _aspect;
 protected:
 public:
  Camera();
  ~Camera();

  inline void setPosition(double pos[3]) { _position[0] = pos[0]; _position[1] = pos[1]; _position[2] = pos[2]; };
  inline void setDirection(double dir[3]) { _direction[0] = dir[0]; _direction[1] = dir[1]; _direction[2] = dir[2]; };
  inline void setUp(double up[3]) { _up[0] = up[0]; _up[1] = up[1]; _up[2] = up[2]; };
  inline void setRight(double up[3]) { _right[0] = up[0]; _right[1] = up[1]; _right[2] = up[2]; };
  inline void setFocal(double f) { _focal = f; };
  inline void setFov(double f) { _fov = f; };
  inline void setAspect(double a) { _aspect = a; };

  inline double* getPosition() { return _position; };
  inline double* getDirection() { return _direction; };
  inline double* getUp() { return _up; };
  inline double* getRight() { return _right; };
  inline double getFocal() { return _focal; };
  inline double getFov() { return _fov; };
  inline double getAspect() { return _aspect; };

  void goTo(double,double);
  void lookAt(double,double);
  void straffTo(double,double);
};

#endif
