#ifndef _TUNNEL_SCENENFF_H_
#define _TUNNEL_SCENENFF_H_

#include "others/includes.h"

#include "data/LightSource.h"
#include "data/MaterialReflectanceSpectrum.h"

class PointNFF
{
 private:
  double _x,_y,_z;
 protected:
 public:
  PointNFF();
  PointNFF(const double&,const double&,const double&);
  PointNFF(const PointNFF&);
  ~PointNFF();

  void set(const double&,const double&,const double&);
  inline double operator[](int idx) const { return (idx==0 ? _x : (idx==1 ? _y : _z)); }
};

class PolygonNFF
{
 private:
  cList< PointNFF* > _points;
  sMaterialReflectanceSpectrum* _material;
  sLightSource* _light;
  enum eTunnelSurface _type;
 protected:
 public:
  PolygonNFF();
  PolygonNFF(const PolygonNFF&);
  ~PolygonNFF();

  inline void set(enum eTunnelSurface t) { _type = t; };
  inline void set(sLightSource* s) { if (s) _light = new sLightSource(*s); };
  inline void set(sMaterialReflectanceSpectrum* m) { if (m) _material = new sMaterialReflectanceSpectrum(*m); };
  inline enum eTunnelSurface getType() { return _type; }

  void add(const PointNFF&);
  inline int card() const { return _points.card(); }
  inline PointNFF& operator[](int idx) const { return (*(_points[idx])); }
};

class SceneNFF
{
 private:
  cList< PolygonNFF* > _listPolygons;
 protected:
 public:
  SceneNFF();
  SceneNFF(const SceneNFF&);
  ~SceneNFF();

  void add(const PolygonNFF&);
  inline int card() const { return _listPolygons.card(); }
  inline PolygonNFF& operator[](int idx) const { return (*(_listPolygons[idx])); }
};

ostream& operator<<(ostream&,const SceneNFF&);
ostream& operator<<(ostream&,const PolygonNFF&);
ostream& operator<<(ostream&,const PointNFF&);

#endif
