#ifndef _TUNNEL_GUI_FLMAINWINDOW_H_
#define _TUNNEL_GUI_FLMAINWINDOW_H_

#include "others/includes.h"
#include "gui/Flincludes.h"
#include "gui/FlMenu.h"
#include "gui/FlGlVisuScene.h"
#include "gui/FlGlVisuSpectre.h"
#include "gui/FlGlVisuSource.h"
#include "gui/FlGlVisuCourbe.h"
#include "gui/FlSelectSpectre.h"
#include "gui/FlSelectSource.h"
#include "gui/FlSelectCourbe.h"

class FlMainWindow : public Fl_Window
{
private:
  static FlMainWindow* _windowPointer;
  int quit;
 public:
  static inline FlMainWindow* getPointer() { return _windowPointer; };
protected:
  Fl_Menu_Bar* _menuBar;
  FlGlVisuScene* _visuScene;
  FlSelectCourbe* _selectCourbe;
  FlGlVisuCourbe* _visuCourbe;
  FlSelectSpectre* _selectSpectre;
  FlGlVisuSpectre* _visuSpectre;
  FlSelectSource* _selectSource;
  FlGlVisuSource* _visuSource;
public:
  FlMainWindow();
  ~FlMainWindow();
  
  void draw();
  void resize(const int& x,const int& y,const int& w,const int& h);
  int handle(int event);

  /* menu callback functions */
  inline void cb_import_i(Fl_Widget* o, void* v);
  static void cb_import(Fl_Widget* o, void* v);
  inline void cb_export_i(Fl_Widget* o, void* v);
  static void cb_export(Fl_Widget* o, void* v);
  inline void cb_export3d_i(Fl_Widget* o, void* v);
  static void cb_export3d(Fl_Widget* o, void* v);
  inline void cb_quit_i(Fl_Widget* o, void* v);
  static void cb_quit(Fl_Widget* o, void* v);

  inline void cb_simulationparameters_i(Fl_Widget* o, void* v);
  inline void cb_launchsimulation_i(Fl_Widget* o, void* v);
  inline void cb_visualize_i(Fl_Widget* o, void* v);
  inline void cb_shootlaunch_i(Fl_Widget* o, void* v);
  inline void cb_shootidle_i(void* v);
  static void cb_simulationparameters(Fl_Widget* o, void* v);
  static void cb_launchsimulation(Fl_Widget* o, void* v);
  static void cb_visualize(Fl_Widget* o, void* v);
  static void cb_shootlaunch(Fl_Widget* o, void* v);
  static void cb_shootidle(void* v);

  inline void cb_newtunnel_i(Fl_Widget* o, void* v);
  inline void cb_loadtunnel_i(Fl_Widget* o, void* v);
  inline void cb_savetunnel_i(Fl_Widget* o, void* v);
  static void cb_newtunnel(Fl_Widget* o, void* v);
  static void cb_loadtunnel(Fl_Widget* o, void* v);
  static void cb_savetunnel(Fl_Widget* o, void* v);

  inline void cb_loadlights_i(Fl_Widget* o, void* v);
  inline void cb_savelights_i(Fl_Widget* o, void* v);
  inline void cb_listlights_i(Fl_Widget* o, void* v);
  static void cb_loadlights(Fl_Widget* o, void* v);
  static void cb_savelights(Fl_Widget* o, void* v);
  static void cb_listlights(Fl_Widget* o, void* v);
  
  inline void cb_loadfloors_i(Fl_Widget* o, void* v);
  inline void cb_savefloors_i(Fl_Widget* o, void* v);
  inline void cb_listfloors_i(Fl_Widget* o, void* v);
  static void cb_loadfloors(Fl_Widget* o, void* v);
  static void cb_savefloors(Fl_Widget* o, void* v);
  static void cb_listfloors(Fl_Widget* o, void* v);
  
  inline void cb_loadwalls_i(Fl_Widget* o, void* v);
  inline void cb_savewalls_i(Fl_Widget* o, void* v);
  inline void cb_listwalls_i(Fl_Widget* o, void* v);
  static void cb_loadwalls(Fl_Widget* o, void* v);
  static void cb_savewalls(Fl_Widget* o, void* v);
  static void cb_listwalls(Fl_Widget* o, void* v);
  
  inline void cb_loadroofs_i(Fl_Widget* o, void* v);
  inline void cb_saveroofs_i(Fl_Widget* o, void* v);
  inline void cb_listroofs_i(Fl_Widget* o, void* v);
  static void cb_loadroofs(Fl_Widget* o, void* v);
  static void cb_saveroofs(Fl_Widget* o, void* v);
  static void cb_listroofs(Fl_Widget* o, void* v);

  /* other public functions */
  void setDefaultsParameters();
  int quitter();
  inline Fl_Menu_Bar* getMenu() { return _menuBar; };
};

#endif
