#ifndef _TUNNEL_GLOBALDATA_H_
#define _TUNNEL_GLOBALDATA_H_

#include "others/includes.h"

#include "data/LightSource.h"
#include "data/MaterialReflectanceSpectrum.h"
#include "data/SceneNFF.h"

#include "radiosite/Structures.h"
#include "radiosite/Surface.h"

class GlobalData
{
 public:
  typedef enum
    {
      is_findsource,
      is_linkshoot,
      is_getseen,
      is_shoot,
      is_initdB
    } eInteractiveShootStep;
  static Surface* is_crtS;
  static Surface* is_Ssou;
  static int is_i;
  static int is_step;
  static Surface** is_seen;
  static int is_nbseen;
  static int is_crtseen;
  static eInteractiveShootStep is_etape;
 private:
  static sRequiredRadianceCurve _requiredRadianceCurve;
  static sMaterialReflectanceSpectrum _materialReflectanceSpectrum;
  static sLightSource _lightSource;
  static sTunnelParameters _tunnelParameters;
  static sSimulationParameters _simulParameters;
  static cList< sLightSource* > _lightList;
  static cList< sMaterialReflectanceSpectrum* > _floorList;
  static cList< sMaterialReflectanceSpectrum* > _wallList;
  static cList< sMaterialReflectanceSpectrum* > _roofList;
  static enum eTunnelModel _viewedScene;
  static int _solutionInitialized;
 private:
  static SceneNFF* scenenff;
  static Env* maillage;
 public:
  static void initialize();
  static void initializeDefaultData();

  static int loadRequiredRadianceCurve(istream&);
  static int loadRequiredRadianceCurve(char*);
  static sRequiredRadianceCurve& getRequiredRadianceCurve();

  static int loadMaterialReflectanceSpectrum(char*);
  static sMaterialReflectanceSpectrum& getMaterialReflectanceSpectrum();

  static int loadLightSource(char*);
  static sLightSource& getLightSource();

  static int loadTunnelParameters(istream&);
  static int loadTunnelParameters(char*);
  static int saveTunnelParameters(ostream&);
  static int saveTunnelParameters(char*);
  static void dumpTunnelParameters(const sTunnelParameters&);
  static sTunnelParameters& getTunnelParameters();
  static void setTunnelParameters(sTunnelParameters&);
  static sTunnelParameters getTunnelDefaultParameters();

  static int loadSimulationParameters(istream&,sSimulationParameters&);
  static int loadSimulationParameters(istream&);
  static int loadSimulationParameters(char*,sSimulationParameters&);
  static int loadSimulationParameters(char*);
  static int saveSimulationParameters(ostream&,sSimulationParameters&);
  static int saveSimulationParameters(char*,sSimulationParameters&);
  static sSimulationParameters& getSimulationParameters();
  static void setSimulationParameters(sSimulationParameters&);
  static sSimulationParameters getSimulationDefaultParameters();

  static int loadLightList(istream&);
  static int loadLightList(char*);
  static int saveLightList(ostream&);
  static int saveLightList(char*);
  static int getLightIndexFromName(char*);
  static cList< sLightSource* >& getLightList();
  static void setLightList(const cList< sLightSource* >&);

  static int loadFloorList(istream&);
  static int loadFloorList(char*);
  static int saveFloorList(ostream&);
  static int saveFloorList(char*);
  static int getFloorIndexFromName(char*);
  static cList< sMaterialReflectanceSpectrum* >& getFloorList();
  static void setFloorList(const cList< sMaterialReflectanceSpectrum* >&);

  static int loadWallList(istream&);
  static int loadWallList(char*);
  static int saveWallList(ostream&);
  static int saveWallList(char*);
  static int getWallIndexFromName(char*);
  static cList< sMaterialReflectanceSpectrum* >& getWallList();
  static void setWallList(const cList< sMaterialReflectanceSpectrum* >&);

  static int loadRoofList(istream&);
  static int loadRoofList(char*);
  static int saveRoofList(ostream&);
  static int saveRoofList(char*);
  static int getRoofIndexFromName(char*);
  static cList< sMaterialReflectanceSpectrum* >& getRoofList();
  static void setRoofList(const cList< sMaterialReflectanceSpectrum* >&);

  static enum eErrorCode loadResolutionSearchResult(istream&);
  static enum eErrorCode loadResolutionSearchResult(char*);
  static enum eErrorCode saveResolutionSearchResult(ostream&);
  static enum eErrorCode saveResolutionSearchResult(char*);

  static enum eTunnelModel getVisualizedModel();
  static void setVisualizedModel(enum eTunnelModel);

  static enum eErrorCode loadLightSpectrum(char*,sLightSourceSpectrum*);
  static enum eErrorCode loadLight(char*,sLightSource*);
  static enum eErrorCode loadMaterial(char*,sMaterialReflectanceSpectrum*);

  static void initializeSceneNFF();
  static void initializeSceneMaillage();
  static SceneNFF* getInitialTunnelNFF();
  static Env* getMaillage3d();
  static void saveMaillage3d(ostream&);
  static int saveMaillage3d(char*);
  static Env* getMaillageSolution();
  static void saveMaillageSolution(ostream&);
  static int saveMaillageSolution(char*);
  static int solutionInitialized();

  static void initializeInteractiveShoot();
  static int oneStepInteractiveShoot();

  static void lancerRecherche();

  static void dumpGlobalData();
};

#endif
