#ifndef _TEMPLATE_ELEMENT_H_
#define _TEMPLATE_ELEMENT_H_

template <class T> class cList;
template <class T> class cListIterator;

/**
 * template container element class.
 *
 * \param class T, the type of contained elements.
 */
template <class T> class cElement {
private:
protected:
  /** next element in the container. */
  cElement* _next;
  /** previous element in the container. */
  cElement* _prev;
  /** the data contained. */
  T _data;
  /**
   * the constructor.
   * initialize the element (no next, no previous, no data).
   */
  cElement() :
    _next(NULL) ,
    _prev(NULL)
    {
    }
  /**
   * the constructor.
   * initialize the element (no next, no previous).
   * \param T& v, the data contained.
   */
  cElement(T& v) :
    _next(NULL) ,
    _prev(NULL)
    {
      _data = v;
    }
  /**
   * the constructor.
   * initialize the element (no data).
   * \param cElement* p, the previous element in the container.
   * \param cElement* n, the next element in the container.
   */
  cElement(cElement * p,cElement * n) :
    _next(n) ,
    _prev(p)
    {
      if (p != NULL) p->_next = this;
      if (n != NULL) n->_prev = this;
    }
  /**
   * the constructor.
   * initialize the element.
   * \param cElement* p, the previous element in the container.
   * \param cElement* n, the next element in the container.
   * \param T& v, the data contained.
   */
  cElement(cElement * p,cElement * n,T& v) :
    _next(n) ,
    _prev(p)
    {
      _data = v;
      if (p != NULL) p->_next = this;
      if (n != NULL) n->_prev = this;
    }
  /**
   * the destructor.
   */
  ~cElement()
    {
      if (_next != NULL) _next->_prev = _prev;
      if (_prev != NULL) _prev->_next = _next;
    }

  friend class cList<T>;
  friend class cListIterator<T>;
  friend ostream& operator<<< T >(ostream& out,cList<T>& v);
};

#endif
