#ifndef _CAMERA_CAMERAFILE_H_
#define _CAMERA_CAMERAFILE_H_

#include "Camera/includes.h"
#include "Camera/CameraGenerique.h"
#include "Camera/CameraOrthographique.h"
#include "Camera/CameraPerspective.h"

#ifdef USENAMESPACE
namespace Camera
{
#endif /* #ifdef USENAMESPACE */

  class CameraFile
    {
    public:
      /** flags specifying reading mode. */
      static byte in;
      /** flags specifying writting mode. */
      static byte out;
      /** flags specifying gzip encoding. */
      static byte gzipper;
      /** flags specifying binary encoding. */
      static byte binary;
    private:
    protected:
      /** compression is used ? */
      int _gzipped;
      /** binary representation used ? */
      int _binary;
      /** the name of the file. */
      string _fname;
      /** flags describing file opening mode. */
      byte _flags;
    protected:
      /*!
	\brief constructor.
	\param fname const char*, the name of the file.
	\param flags byte, flags specifying opening mode.
      */
      CameraFile(const char* fname,byte flags = 0);
      /*!
	\brief destructor.
      */
      ~CameraFile();
    public:
    }; /* class CameraFile */

  class CameraFileInput :
    public CameraFile
    {
    private:
    protected:
      /** stream associated with the file. */
      istream* _file;
    public:
      /*!
	\brief constructor.
	\param fname const char*, the name of the file.
	\param flags byte, flags specifying opening mode.
      */
      CameraFileInput(const char* fname,byte flags = 0);
      /*!
	\brief destructor.
      */
      ~CameraFileInput();
      friend CameraFileInput& operator >> (CameraFileInput& in,CameraGenerique& cam);
      friend CameraFileInput& operator >> (CameraFileInput& in,CameraOrthographique& cam);
      friend CameraFileInput& operator >> (CameraFileInput& in,CameraPerspective& cam);
    }; /* class CameraFileInput */

  extern CameraFileInput& operator >> (CameraFileInput& in,CameraGenerique& cam);
  extern CameraFileInput& operator >> (CameraFileInput& in,CameraOrthographique& cam);
  extern CameraFileInput& operator >> (CameraFileInput& in,CameraPerspective& cam);

  class CameraFileOutput :
    public CameraFile
    {
    private:
    protected:
      /** stream associated with the file. */
      ostream* _file;
    public:
      /*!
	\brief constructor.
	\param fname const char*, the name of the file.
	\param flags byte, flags specifying opening mode.
      */
      CameraFileOutput(const char* fname,byte flags = 0);
      /*!
	\brief destructor.
      */
      ~CameraFileOutput();
      friend CameraFileOutput& operator << (CameraFileOutput& out,CameraGenerique& cam);
      friend CameraFileOutput& operator << (CameraFileOutput& out,CameraOrthographique& cam);
      friend CameraFileOutput& operator << (CameraFileOutput& out,CameraPerspective& cam);
    }; /* class CameraFileOutput */

  extern CameraFileOutput& operator << (CameraFileOutput& out,CameraGenerique& cam);
  extern CameraFileOutput& operator << (CameraFileOutput& out,CameraOrthographique& cam);
  extern CameraFileOutput& operator << (CameraFileOutput& out,CameraPerspective& cam);

#ifdef USENAMESPACE
} /* namespace Camera */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _CAMERA_CAMERAFILE_H_ */
