#ifndef _CAMERA_CAMERAGENERIQUE_H_
#define _CAMERA_CAMERAGENERIQUE_H_

#include "Camera/includes.h"

#ifdef USENAMESPACE
namespace Camera
{
#endif /* #ifdef USENAMESPACE */

/*! \class CameraGenerique
 * \brief base class for cameras objects
 * \author E. Guillou
 *
 * This object is the base class for camera hierarchy.
 * It encapsulate external parameters of the camera (ie : position and orientation in 3d space) and the type of the camera local coordinate system (direct or indirect).
 */

class CameraGenerique {
private:
  /**
   * the position of the camera in respect to the global coordinate system.
   */
  sPoint3d _position;
  /**
   * the looking direction of the camera.
   */
  sVector3d _direction;
  /**
   * the up vector.
   */
  sVector3d _up;
  /**
   * the right vector.
   */
  sVector3d _right;
protected:
  /**
   * the type of the camera.
   */
  Camera_Types _type;
  /**
   * the type of the camera local coordinate system (direct or indirect).
   */
  Coordinates_Definition _ccs;
public:
  /**
   * Generic camera default constructor.
   */
  CameraGenerique();
  /**
   * Generic camera initialization constructor.
   */
  CameraGenerique(Coordinates_Definition);
  /**
   * Generic camera copy constructor.
   */
  CameraGenerique(CameraGenerique&);
  /**
   * Generic camera destructor.
   */
  ~CameraGenerique();
  /**
   * Generic camera copy operator.
   */
  CameraGenerique& operator=(CameraGenerique&);
  /**
   * Generic camera private members access method.
   * Access method to the type of camera.
   */
  Camera_Types& Type();
  /**
   * Generic camera private members access method.
   * Access method to the type of coordinate system.
   */
  Coordinates_Definition& Coordinates();
  /**
   * Generic camera private members access method.
   * Access method to the position of the camera.
   */
  sPoint3d& Position();
  /**
   * Generic camera private members access method.
   * Access method to the direction of the camera.
   */
  sVector3d& Direction();
  /**
   * Generic camera private members access method.
   * Access method to the up vector of the camera.
   */
  sVector3d& Up();
  /**
   * Generic camera private members access method.
   * Access method to the right vector of the camera.
   */
  sVector3d& Right();
  /**
   * Generic camera private members initialization method.
   * initialization method of the type of camera.
   */
  void Type(Camera_Types);
  /**
   * Generic camera private members initialization method.
   * initialization method of the type of coordinate system.
   */
  void Coordinates(Coordinates_Definition);
  /**
   * Generic camera private members initialization method.
   * initialization method of the position of camera.
   */
  void Position(sPoint3d&);
  /**
   * Generic camera private members initialization method.
   * initialization method of the direction of camera.
   */
  void Direction(sVector3d&);
  /**
   * Generic camera private members initialization method.
   * initialization method of the up vector of camera.
   */
  void Up(sVector3d&);
  /**
   * Generic camera private members initialization method.
   * initialization method of the right vector of camera.
   */
  void Right(sVector3d&);
  /**
   * access method to computed attributes.
   * access method to the rotation matrix describing the orientation of the camer in respect to the global coordinate system.
   */
  double** Rotation();
  /**
   * access method to computed attributes.
   * access method to the translation vector describing the position of the camer in respect to the global coordinate system.
   */
  double* Translation();
}; /* class CameraGenerique */

#ifdef USENAMESPACE
} /* namespace Camera */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _CAMERA_CAMERAGENERIQUE_H_ */
