#ifndef _CAMERA_CAMERAORTHOGRAPHIQUE_H_
#define _CAMERA_CAMERAORTHOGRAPHIQUE_H_

#include "Camera/includes.h"
#include "Camera/CameraGenerique.h"

#ifdef USENAMESPACE
namespace Camera
{
#endif /* #ifdef USENAMESPACE */

/*! \class CameraOrthographique
 * \brief class for perspective cameras objects
 * \author E. Guillou
 *
 * This object encapsulates internal parameters of an orthographic camera.
 * It inherits external parameters from CameraGenerique object.
 */

class CameraOrthographique : public CameraGenerique {
private:
  /**
   * the horizontal opening size of the camera.
   */
  double _ouvertureu;
  /**
   * the vertical opening size of the camera.
   */
  double _ouverturev;
  /**
   * the pixel size of the camera.
   * ie : width of a pixel in meters.
   */
  double _pixelsize;
  /**
   * the aspect ratio of the camera.
   * ie : width/height of a pixel.
   */
  double _aspectratio;
protected:
public:
  /**
   * Orthographic camera default constructor.
   */
  CameraOrthographique();
  /**
   * Orthographic camera initialization constructor.
   */
  CameraOrthographique(Coordinates_Definition);
  /**
   * Orthographic camera copy constructor.
   */
  CameraOrthographique(CameraOrthographique&);
  /**
   * Orthographic camera destructor.
   */
  ~CameraOrthographique();
  /**
   * Orthographic camera copy operator.
   */
  CameraOrthographique& operator=(CameraOrthographique&);
  /**
   * Orthographic camera private members access method.
   * Access method to the horizontal opening size of the camera.
   */
  double& OuvertureU();
  /**
   * Orthographic camera private members access method.
   * Access method to the vertical opening size of the camera.
   */
  double& OuvertureV();
  /**
   * Orthographic camera private members access method.
   * Access method to the pixel size of the camera.
   */
  double& PixelSize();
  /**
   * Orthographic camera private members access method.
   * Access method to the aspect ratio of the camera.
   */
  double& AspectRatio();
  /**
   * Orthographic camera private members initialization method.
   * initialization method of the horizontal opening size of the camera.
   */
  void OuvertureU(double);
  /**
   * Orthographic camera private members initialization method.
   * initialization method of the vertical opening size of the camera.
   */
  void OuvertureV(double);
  /**
   * Orthographic camera private members initialization method.
   * initialization method of the pixel size of the camera.
   */
  void PixelSize(double);
  /**
   * Orthographic camera private members initialization method.
   * initialization method of the aspect ratio of the camera.
   */
  void AspectRatio(double);
}; /* class CameraOrthographique */

#ifdef USENAMESPACE
} /* namespace Camera */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _CAMERA_CAMERAORTHOGRAPHIQUE_H_ */
