#ifndef _GEOMETRIC_INCLUDES_H_
#define _GEOMETRIC_INCLUDES_H_

#include <iostream>
#include <iomanip>
#include <cmath>

using namespace std;

/* pour la bibliotheque utilitaires */

#include <Utility/Includes.h>
#include <Utility/BinaryCoding.h>
#include <Utility/Mode.h>
#include <Utility/Debugger.h>
#include <Utility/Memory.h>
#include <Utility/Statistic.h>
#include <Utility/CounterStatistic.h>
#include <Utility/Utility.h>

#ifdef USENAMESPACE
using namespace DebugManager;
using namespace MemoryManager;
using namespace CallingStatisticManager;
using namespace CounterStatisticManager;
using namespace BinaryCoder;
using namespace Serializer;
using namespace ErrorCode;
#endif /* #ifdef USENAMESPACE */

/* pour la bibliotheque math */

#include <Math/includes.h>
#include <Math/imath.h>
#include <Math/dmath.h>
#include <Math/cmath.h>
#include <Math/NR.h>
#include <Math/functions.h>

#ifdef USENAMESPACE
using namespace NumericalRecipes;
#endif /* #ifdef USENAMESPACE */

#ifdef USENAMESPACE
/*! \namespace Geometric
 * Namespace containing basic declarations for 2d and 3d simple objects.
 */
namespace Geometric
{
#endif /* #ifdef USENAMESPACE */

  /*! \typedef  double sGeometricData
   * type used to defined 2d/3d basic datas (ie : coordinates).
   */
typedef double sGeometricData;
  /*! \def maximumNumberOfPointInPolygon
   * defines the maximum number of points used to describe a 2d/3d polygon.
   */
#define maximumNumberOfPointInPolygon 50
/*! \enum Coordinates_Definition
 * enumeration for coordinate system definition.
 */
typedef enum {
  GEO_DIRECT = 0 ,
  GEO_INDIRECT = 1
} Coordinates_Definition;

#ifdef USENAMESPACE
} /* namespace Geometric */
#endif /* #ifdef USENAMESPACE */

#ifdef USENAMESPACE
/*! \namespace ErrorCode
 * Namespace containing Error codes.
 */
namespace ErrorCode
{
#endif /* #ifdef USENAMESPACE */

  /*! \def GEOMETRICERROR
   * Geometric manipulation error code.
   */
#define GEOMETRICERROR 2000

#ifdef USENAMESPACE
} /* namespace ErrorCode */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _GEOMETRIC_INCLUDES_H_ */
