#ifndef _MATH_NRCHAPTER02SECTION01_H_
#define _MATH_NRCHAPTER02SECTION01_H_

/*!
 * \file Math/NRChapter02Section01.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 1.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section01 2.1 Gauss-Jordan Elimination
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

 /*!
    \fn void gaussj(double **a,int n,double **b,int m)
    \brief gauss-jordan elimination.
    \ingroup chapitre02section01
    \param a is the input matrix.a is replaced by its matrix inverse.
    \param b is the input containing the m right-hand side vectors. b is replaced by the corresponding set of solution vectors.
    \param n defines the size of a.
    \param m defines the number of right-hand side vectors

    As described in "Numerical Recipes in c" pp. 39-40

    Linear equation solution by Gauss-Jordan elimination, equation 2.1.1 (see NR). a[1..n][1..n] is the input matrix, b[1..n][1..m] is input containing the m right-hand side vectors. On output a is replaced by its matrix inverse, and b is replaced by the corresponding set of solution vectors.
  */
  extern void gaussj(double **a,int n,double **b,int m);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION01_H_ */
