#ifndef _MATH_NRCHAPTER02SECTION05_H_
#define _MATH_NRCHAPTER02SECTION05_H_

/*!
 * \file Math/NRChapter02Section05.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 5.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section05 2.5 Iterative Improvement of a Solution to Linear Equations
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void mprove(double **a, double **alud, int n, int indx[], double b[], double x[])
    \brief improves the solution of a system of equations.
    \ingroup chapitre02section05
    \param a the coefficients of the equations.
    \param alud the LU decomposition of matrix a.
    \param n number of equations/unknowns.
    \param indx the row permutation vector.
    \param b the right-hand side vector.
    \param x the solution vector.

    As described in "Numerical Recipes in c" pp. 56

    Improves a solution vector x[1..n] of the linear set of equations A .  X = B. The matrix a[1..n][1..n], and the vectors b[1..n] and x[1..n] are input, as is the dimension n. Also input is alud[1..n][1..n], the LU decomposition of a as returned by ludcmp, and the vector indx[1..n] also returned by that routine. On output, only x[1..n] is modified, to an improved set of values.
  */
  extern void mprove(double **a, double **alud, int n, int indx[], double b[], double x[]);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION05_H_ */
