#ifndef _MATH_NRCHAPTER02SECTION08_H_
#define _MATH_NRCHAPTER02SECTION08_H_

/*!
 * \file Math/NRChapter02Section08.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 8.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section08 2.8 Vandermonde Matrices and Toeplitz Matrices
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void vander(double x[], double w[], double q[], int n)
    \brief solves Vandermonde systems.
    \ingroup chapitre02section08
    \param x vector containing equation coefficients.
    \param w outputs the result.
    \param q right-hand side coefficients.
    \param n size of the vectors.

    As described in "Numerical Recipes in c" pp. 92

    Solves the Vandermonde linear system PNi=1 xk1 i wi = qk (k = 1; : : : ;N). Input consists of the vectors x[1..n] and q[1..n]; the vector w[1..n] is output.
  */
  extern void vander(double x[], double w[], double q[], int n);
  /*!
    \fn void toeplz(double r[], double x[], double y[], int n)
    \brief solves Toeplitz systems.
    \ingroup chapitre02section08
    \param r left-hand side coefficients.
    \param x outputs the result.
    \param y right-hand side coefficients.
    \param n size of the vectors.

    As described in "Numerical Recipes in c" pp. 95

    Solves the Toeplitz system PNj=1 R(N+ij)xj = yi (i = 1; : : :;N). The Toeplitz matrix need not be symmetric. y[1..n] and r[1..2*n-1] are input arrays; x[1..n] is the output array.
  */
  extern void toeplz(double r[], double x[], double y[], int n);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION08_H_ */
