#ifndef _MATH_NRCHAPTER02SECTION09_H_
#define _MATH_NRCHAPTER02SECTION09_H_

/*!
 * \file Math/NRChapter02Section09.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 9.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section09 2.9 Cholesky Decomposition
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void choldc(double **a, int n, double p[])
    \brief Cholesky decomposition.
    \ingroup chapitre02section09
    \param a matrix containing left-hand side coefficients. returns the Cholesky factor L.
    \param n size of the matrix.
    \param p returns the diagonal elements of the decomposition.

    As described in "Numerical Recipes in c" pp. 97

    Given a positive-definite symmetric matrix a[1..n][1..n], this routine constructs its Cholesky decomposition, A = L  LT . On input, only the upper triangle of a need be given; it is not modified. The Cholesky factor L is returned in the lower triangle of a, except for its diagonal elements which are returned in p[1..n].
  */
  extern void choldc(double **a, int n, double p[]);
  /*!
    \fn void cholsl(double **a, int n, double p[], double b[], double x[])
    \brief Cholesky backsubstitution.
    \ingroup chapitre02section09
    \param a the Cholesky factor L.
    \param n size of the matrix.
    \param p the diagonal elements of the decomposition.
    \param b the right-hand side coefficients.
    \param x outputs the solution vector.

    As described in "Numerical Recipes in c" pp. 97-98

    Solves the set of n linear equations A  x = b, where a is a positive-denite symmetric matrix. a[1..n][1..n] and p[1..n] are input as the output of the routine choldc. Only the lower triangle of a is accessed. b[1..n] is input as the right-hand side vector. The solution vector is returned in x[1..n]. a, n, and p are not modied and can be left in place for successive calls with different right-hand sides b. b is not modified unless you identify b and x in the calling sequence, which is allowed.
  */
  extern void cholsl(double **a, int n, double p[], double b[], double x[]);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION09_H_ */
