#ifndef _MATH_NRCHAPTER08SECTION04_H_
#define _MATH_NRCHAPTER08SECTION04_H_

/*!
 * \file Math/NRChapter08Section04.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 8, Section 4.
 */

#include "Math/includes.h"

/** \defgroup chapitre08section04 8.4 Indexing and Ranking
 * \ingroup chapitre08
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void indexx(unsigned long n, double arr[], unsigned long indx[])
    \brief constrct an index for an array
    \ingroup chapitre08section04
    \param n size of the array.
    \param arr input array.
    \param indx output indexes on array arr.

    As described in "Numerical Recipes in c" pp. 338-340

    Indexes an array arr[1..n], i.e., outputs the array indx[1..n] such that arr[indx[j]] is in ascending order for j = 1;2; : : :;N. The input quantities n and arr are not changed.
  */
  void indexx(unsigned long n, double arr[], unsigned long indx[]);
  /*!
    \fn void sort3(unsigned long n, double ra[], double rb[], double rc[])
    \brief sort, use an index to sort 3 or more arrays
    \ingroup chapitre08section04
    \param n size of the arrays.
    \param ra the array to sort.
    \param rb the first array to apply rearrangement on.
    \param rc the second array to apply rearrangement on.

    As described in "Numerical Recipes in c" pp. 340

    Sorts an array ra[1..n] into ascending numerical order while making the corresponding rearrangements of the arrays rb[1..n] and rc[1..n]. An index table is constructed via the routine indexx.
   */
  void sort3(unsigned long n, double ra[], double rb[], double rc[]);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER08SECTION04_H_ */
