#ifndef _MATH_NRCHAPTER10SECTION01_H_
#define _MATH_NRCHAPTER10SECTION01_H_

/*!
 * \file Math/NRChapter10Section01.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipies Chapter 10, Section 1.
 */

#include "Math/includes.h"

/** \defgroup chapitre10Section01 10.1 Golden Section Search in One Dimension
 * \ingroup chapitre10
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void mnbrak(double *ax,double *bx,double *cx,double *fa,double *fb,double *fc,double (*func)(double))
    \brief bracket the minimum of a function
    \ingroup chapitre10Section01
    \param ax is the first initial point.ax is replaced by the first evaluated point.
    \param bx is the second initial point.bx is replaced by the second evaluated point.
    \param func is the function for which we want to bracket the minimum.
    \param cx returns the third evaluated point.
    \param fa returns the function value at first evaluated point.
    \param fb returns the function value at second evaluated point.
    \param fc returns the function value at third evaluated point.

    As described in "Numerical Recipes in c" pp. 400-401.

    Given a function func, and given distinct initial points ax and bx, this routine searches in the downhill direction (defined by the function as evaluated at the inial points) and returns new points ax, bx, cx that bracket a minimum of the function. Also returned are the function values at the three points, fa, fb, and fc.
  */
  extern void mnbrak(double *ax,double *bx,double *cx,double *fa,double *fb,double *fc,double (*func)(double));

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER10SECTION01_H_ */
