#ifndef _MATH_NRCHAPTER10SECTION02_H_
#define _MATH_NRCHAPTER10SECTION02_H_

/*!
 * \file Math/NRChapter10Section02.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipies Chapter 10, Section 2.
 */

#include "Math/includes.h"

/** \defgroup chapitre10Section02 10. 2 Parabolic Interpolation and Brent's Method in One Dimension
 * \ingroup chapitre10
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn double brent(double ax,double bx,double cx,double (*f)(double),double tol,double *xmin)
    \brief find minimum of a function by Brent's method
    \ingroup chapitre10Section02
    \param ax is the first bracketting point.
    \param bx is the second bracketting point.
    \param cx is the third bracketting point.
    \param f is the function for which we want to find the minimum.
    \param tol is the fractional precision for Brent's method.
    \param xmin returns the abcissa of the function minimum.
    \return the minimum function value (ie : f(xmin))

    As described in "Numerical Recipes in c" pp. 404-405.

    Given a function f, and given bracketting triplet of abscissas ax, bx, cx (such that bx is between ax and cx, and f(bx) is less than both f(ax) and f(cx)), this routine isolates the minimum to a fractional precision of about tol using Brent's method. The abscissa of the minimum is returned as xmin, and the minimum function value is returned as brent, the returned function value.
  */
  extern double brent(double ax,double bx,double cx,double (*f)(double),double tol,double *xmin);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER10SECTION02_H_ */
