#ifndef _MATH_NRCHAPTER10SECTION03_H_
#define _MATH_NRCHAPTER10SECTION03_H_

/*!
 * \file Math/NRChapter10Section03.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipies Chapter 10, Section 3.
 */

#include "Math/includes.h"

/** \defgroup chapitre10Section03 10. 3 One-Dimensional Search with  First Derivatives
 * \ingroup chapitre10
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn double dbrent(double ax,double bx,double cx,double (*f)(double),double (*df)(double),double tol,double *xmin)
    \brief find minimum of a function using derivative information
    \ingroup chapitre10Section03
    \param ax is the first bracketting point.
    \param bx is the second bracketting point.
    \param cx is the third bracketting point.
    \param f is the function for which we want to find the minimum.
    \param df is the derivate function of f,for which we want to find the minimum.
    \param tol is the fractional precision for modified Brent's method.
    \param xmin returns the abcissa of the function minimum.
    \return the minimum function value (ie : f(xmin))

    As described in "Numerical Recipes in c" pp. 406-408.

    Given a function f, and its derivate function df, and given a bracketting triplet of abscissas  ax, bx, cx (such that bx is between ax and cx, and f(bx) is less than both f(ax) and f(cx)), this routine isolates the minimum to a fractional precision of about tol using a modification of Brent's method that uses derivatives. The abscissa of the minimum is returned as xmin, and the minimum function value is returned as dbrent, the returned function value.
  */
  extern double dbrent(double ax,double bx,double cx,double (*f)(double),double (*df)(double),double tol,double *xmin);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER10SECTION03_H_ */
