#ifndef _MATH_NRCHAPTER10SECTION05_H_
#define _MATH_NRCHAPTER10SECTION05_H_

/*!
 * \file Math/NRChapter10Section05.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipies Chapter 10, Section 5.
 */

#include "Math/includes.h"

/** \defgroup chapitre10Section05 10. 5 Direction Set (Powell's) Methods in Multidimensions
 * \ingroup chapitre10
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  extern int ncom;
  extern double* pcom;
  extern double* xicom;
  extern double (*nrfunc)(double []);
  /*!
    \fn void linmin(double p[],double xi[],int n,double *fret,double (*func)(double []))
    \brief minimum of a function along a ray in N-dimensions
    \ingroup chapitre10Section05
    \param p is the actual point, replaced by the new point.
    \param xi is the direction of displacement, replaced by the vector describing the movement of p.
    \param n is the dimension of p and xi.
    \param func is the function for which we want to find the minimum.
    \param fret returns the value of func at the new location of p.

    As described in "Numerical Recipes in c" p. 419.

    Given a n-dimensionnal point p[1..n] and an n-dimensional direction xi[1..n], moves and resets p to where the function func(p) takes on a minimum along direction xi from p, and replaces x1 by the actual vector displacement that p was moved. Also returns as fret the value of func at the returned location p. This is actually all accomplished by calling the routine mnbrak and brent.
  */
  extern void linmin(double p[],double xi[],int n,double *fret,double (*func)(double []));
  /*!
    \fn double f1dim(double x)
    \brief function used by linmin
    \ingroup chapitre10Section05
    \param x is the point where to evaluate the function
    \return the value of the function to minimize at point x

    As described in "Numerical Recipes in c" p. 419.

    Must accompany linmin. This is the function linmin tries to minimize.
  */
  extern double f1dim(double x);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER10SECTION05_H_ */
