#ifndef _MATH_NRCHAPTER10SECTION06_H_
#define _MATH_NRCHAPTER10SECTION06_H_

/*!
 * \file Math/NRChapter10Section06.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipies Chapter 10, Section 6.
 */

#include "Math/includes.h"

/** \defgroup chapitre10Section06 10. 6 Conjugate Gradient Methods in Multidimensions
 * \ingroup chapitre10
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void frpmn(double p[],int n,double ftol,int *iter,double *fret,double (*func)(double[]),void (*dfunc)(double [],double []))
    \brief minimize in N-dimensions by conjugate gradient
    \ingroup chapitre10Section06
    \param p is the actual point, replaced by the new point where minimum of func is reached.
    \param n is the dimension of p.
    \param ftol
    \param func is the function for which we want to find the minimum.
    \param dfunc is a function returning the gradient of func.
    \param fret returns the value of func at the new location of p.
    \param iter returns the number of iterations performed to reach the minimum of func.

    As described in "Numerical Recipes in c" pp. 423-424.

    Given a n-dimensionnal point p[1..n], Fletcher-Reeves-Polak-Ribiere minimization is performed on a function func, using its gradient as calculated by a routine dfunc. The convergence tolerance on the function value is input as ftol. Returned quantities are p (the location of the minimum), iter (the number of iterations that were performed), and fret (the minimum value of the function). The routine linmin is called to perform line minimizations.
  */
  void frpmn(double p[],int n,double ftol,int *iter,double *fret,double (*func)(double[]),void (*dfunc)(double [],double []));

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER10SECTION06_H_ */
