#ifndef _MATH_CMATH_H_
#define _MATH_CMATH_H_

/*!
  \file Math/cmath.h
  \brief mathematical functions on complex values

  Complex utility routines used by all, or some, of the mathematical algorithms.
 */

#include "Math/includes.h"

/** \defgroup appendixC Appendix C: Complex Arithmetic
 * \ingroup NR
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*! \struct FCOMPLEX
    \ingroup appendixC
   */
  typedef struct FCOMPLEX {
    /** real part of the complex */
    float r;
    /** imaginary part of the complex */
    float i;
  } fcomplex;
  /*!
    \fn fcomplex Cadd(fcomplex a, fcomplex b)
    \ingroup appendixC
  */
  fcomplex Cadd(fcomplex a, fcomplex b);
  /*!
    \fn fcomplex Csub(fcomplex a, fcomplex b)
    \ingroup appendixC
  */
  fcomplex Csub(fcomplex a, fcomplex b);
  /*!
    \fn fcomplex Cmul(fcomplex a, fcomplex b)
    \ingroup appendixC
  */
  fcomplex Cmul(fcomplex a, fcomplex b);
  /*!
    \fn fcomplex Complex(float re, float im)
    \ingroup appendixC
  */
  fcomplex Complex(float re, float im);
  /*!
    \fn fcomplex Conjg(fcomplex z)
    \ingroup appendixC
  */
  fcomplex Conjg(fcomplex z);
  /*!
    \fn fcomplex Cdiv(fcomplex a, fcomplex b)
    \ingroup appendixC
  */
  fcomplex Cdiv(fcomplex a, fcomplex b);
  /*!
    \fn float Cabs(fcomplex z)
    \ingroup appendixC
  */
  float Cabs(fcomplex z);
  /*!
    \fn fcomplex Csqrt(fcomplex z)
    \ingroup appendixC
  */
  fcomplex Csqrt(fcomplex z);
  /*!
    \fn fcomplex RCmul(float x, fcomplex a)
    \ingroup appendixC
  */
  fcomplex RCmul(float x, fcomplex a);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* _MATH_CMATH_H_ */
