#ifndef _MATH_INCLUDES_H_
#define _MATH_INCLUDES_H_

/*!
  \file Math/includes.h
  \brief include file for mathematical library

  defines common variables and functions for mathematical library
 */

#include <math.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>

using namespace std;

/* pour la bibliotheque utilitaires */

#include <Utility/Includes.h>
#include <Utility/BinaryCoding.h>
#include <Utility/Mode.h>
#include <Utility/Debugger.h>
#include <Utility/Memory.h>
#include <Utility/Statistic.h>
#include <Utility/CounterStatistic.h>
#include <Utility/Utility.h>

#ifdef USENAMESPACE
using namespace DebugManager;
using namespace MemoryManager;
using namespace CallingStatisticManager;
using namespace CounterStatisticManager;
using namespace BinaryCoder;
using namespace Serializer;
using namespace ErrorCode;
#endif /* #ifdef USENAMESPACE */

#ifndef NULL
/*!
  \def NULL
  \brief redefines NULL if necessary.
 */
#define NULL 0
#endif

#ifndef INFINITY
/*!
  \def INFINITY
  \brief redefines INFINITY if necessary.
 */
#define INFINITY 1e10
#endif

/*!
  \def NR_END
  \brief array translation index for numerical recipies routines.
 */
#define NR_END 1
/*!
  \def FREE_ARG
  \brief type of element to free for numerical recipies routines.
 */
#define FREE_ARG char*
/*!
  \def ITMAX
  \brief maximum iteration number for numerical recipies routines.
 */
#define ITMAX 30
/*!
  \def GOLD
  \brief the gold number for numerical recipies routines.
 */
#define GOLD 1.618034
#define CGOLD 0.3819660
/*!
  \def SWAP(a,b)
  \brief swap values of two variables.
 */
#define SWAP(a,b) { \
  swap = (a); \
  (a) = (b); \
  (b) = swap; \
}
/*!
  \def SHFT(a,b,c,d)
  \brief swap values of four variables.
 */
#define SHFT(a,b,c,d) { \
  (a) = (b); \
  (b) = (c); \
  (c) = (d); \
}
/*!
  \def MOV3(a,b,c,d,e,f)
  \brief exchange values between two triplets of variables.
 */
#define MOV3(a,b,c,d,e,f) { \
  (a) = (d); \
  (b) = (e); \
  (c) = (f); \
}
  /*!
    \def SIGN(a,b)
    \brief computes the ??? of two integers
    \param a the first integer value
    \param b the second integer value
    \return |a| if b>= 0., -|a| otherwise
  */
#define SIGN(a,b) ( \
  (b) >= 0 ? ABS(a) : -ABS(a) \
)

#ifdef USENAMESPACE
/*! \namespace ErrorCode
 * Namespace containing Error codes.
 */
namespace ErrorCode
{
#endif /* #ifdef USENAMESPACE */

  /*! \def MATHEMATICALERROR
   * an error occured in a mathematical algorithm.
   */
#define MATHEMATICALERROR 1000
  /*! \def CONVERGENCEERROR
   * no convergence of an algorithm after a maximum number of iterations.
   */
#define CONVERGENCEERROR 1001
  /*! \def SINGULARMATRIXERROR
   * a mathematical algorithm is done on a singular matrix.
   */
#define SINGULARMATRIXERROR 1002
  /*! \def INVALIDMATHEMATICALARGUMENTERROR
   * argument is invalid for this algorithm.
   */
#define INVALIDMATHEMATICALARGUMENTERROR 1003

#ifdef USENAMESPACE
} /* namespace ErrorCode */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifdef _MATH_INCLUDES_H_ */
