#ifndef _RADIOSITE_OPTIONS_H_
#define _RADIOSITE_OPTIONS_H_

#include "Radiosite/Includes.h"
#include "Radiosite/Enumeration.h"
#include "Radiosite/Globals.h"

typedef struct
{
  int _maximumRecursiveRaytraceLevel;
  int _surfaceVisibilityRaySampling;
  int _pixelAntiAliasing;
  enum eOnOff _pixelJittering;
  enum eMonteCarloMethod _montecarloMethod;
  double _montecarloSampling;
} sRaytracerOption;

typedef struct
{
  enum eOnOff _photometricSolidSubdivisionMode;
  enum eOnOff _waveletMode;
  enum eOnOff _tvertexEliminationMode;
  enum eOnOff _interPatchVisibilityTest;
  enum eOnOff _hierarchicalLinkRefinementMode;;

  double _areaThreshold;
  double _areaLightThreshold;
  double _globalEpsilon;
  double _energyEpsilon;
  double _visibilityEpsilon;
  double _formfactorEpsilon;
  double _kernelEpsilon;
  double _photometricSolidSubdivisionEpsilon;
  int _maximumIteration;

  enum eWaveletBasis _waveletBasis;
  enum eFormFactorCalculation _formfactorMethod;
  enum eSimulationMethod _simulationMethod;
} sRadiosityOption;

typedef struct
{
  enum eOnOff _binaryFileMode;
  enum eOnOff _surfaceSplittingMode;
  enum eOnOff _interactiveMode;
  enum eRenderingMode _renderingMode;
  enum eInteractiveMode _interactiveType;
  int _interactiveWhen;
  int _interactiveWhat;
  char _interactiveFile[255];
  double _surfaceMaximumLength;
  double _surfaceMaximumRatio;
  double _spectrumSamplingThreshold;
  enum eOnOff _skylightingMode;
  enum eSkyModel _skyModel;
  double _skyHorizonLine;
  double _sunTheta;
  double _sunPhi;
} sSceneOption;

typedef struct
{
  USESTRUCT sRaytracerOption raytrace;
  USESTRUCT sRadiosityOption radiosity;
  USESTRUCT sSceneOption scene;
} sOption;

extern sOption globalOptions;

extern void initializeDefaultOptions();
extern void dumpOptions();
extern void loadOptions(char* fname);

#endif
