#ifndef _RADIOSITE_PATCH_H_
#define _RADIOSITE_PATCH_H_

#include "Radiosite/Includes.h"
#include "Radiosite/Enumeration.h"
#include "Radiosite/Globals.h"

#include "Radiosite/Utility.h"
#include "Radiosite/Options.h"

#include "Radiosite/Material.h"

typedef struct __patchid
{
  tSurfaceID _surfaceID;
  tPatchID _patchID;
  __patchid() { _surfaceID = 0; _patchID = 0; };
  __patchid(tSurfaceID sid,tPatchID pid) { _surfaceID = sid; _patchID = pid; };
  int operator < (const __patchid e2) const { return ((_surfaceID < e2._surfaceID) || ((_surfaceID == e2._surfaceID) && (_patchID < e2._patchID))); }
  int operator == (const __patchid e2) const { return ((_surfaceID == e2._surfaceID) && (_patchID == e2._patchID)); }
} sPatchID;

extern ostream& operator << (ostream& out,const __patchid& p);

typedef struct __patch
{
  __patch();
  struct __patch* parent() const;
  struct __patch* fils(int i) const;
  struct __surface* surface() const;
  struct __material* material() const;
  struct __lightsource* lightsource() const;
  struct __link* fromLink() const;
  struct __link* toLink() const;
  sPatchID patchID;
  /* donnees geometriques */
  USESTRUCT sPoint3d vertices[maximumNumberOfPointPerPatch];
  int size;
  USESTRUCT sPoint2d mapping[maximumNumberOfPointPerPatch];
  USESTRUCT sVector3d uVector;
  USESTRUCT sVector3d vVector;
  USESTRUCT sPoint3d center;
  int level;
  float area;
  float areaCoefficient[4];
  float width;
  float height;
  /* donnees geometriques */ /* subdivision du patch */
  int subdivided;
  /* donnees photometriques */ /* agregation forte sur les luminances et autres donnees pour la radiosite*/
  sSpectrumPhotometric* coeff;
  sSpectrumPhotometric* scoeff;
  sSpectrumPhotometric* dcoeff;
  sSpectrumPhotometric* decoeff;
  sSpectrumPhotometric** w_coeff;
  sSpectrumPhotometric** w_dcoeff;
  sSpectrumPhotometric** w_decoeff;
  /* donnees photometriques */ /* les liens du patch */
  sPatchID fromLinkID;
  sPatchID toLinkID;
} sPatch;

extern __patch* createPatch();
extern void deletePatch(__patch*& s);

extern void allocateRadiosityData(__patch* res);

extern void initializePatchData(__patch& pol);
extern void initializePatchMappingData(__patch& pol,sPoint2d* uppermap,int idson);

extern __patch* generatePatch(sPoint3d& p0,sPoint3d& p1,sPoint3d& p2,sPoint3d& p3);
extern void generateChildPatch(__patch* a);

extern __patch* whichOneToSubdivide(__patch* a,__patch* b);

extern void getLightAngleFromDirection(const __patch& frompatch,sVector3d& dir,double& thetaa,double& psii);
extern void getMaxMinContributionToPatch(const __patch& topatch,const __patch& frompatch,double& maxcontrib,double& mincontrib);

extern double getRadiosity(int k,const __patch& b,sPoint3d& x,double Area);
extern double evaluateRadiosity(int k,const __patch& a,const __patch& b,double s,double t,double Area);

extern void computeUVspectrum(__patch* p,int factor, double u,double v,double* s);

extern void patchStats();

#endif
