#ifndef _RADIOSITE_SURFACE_H_
#define _RADIOSITE_SURFACE_H_

#include "Radiosite/Includes.h"
#include "Radiosite/Enumeration.h"
#include "Radiosite/Globals.h"

#include "Radiosite/Utility.h"
#include "Radiosite/Options.h"

#include "Radiosite/Material.h"

extern tSurfaceID surfaceFirstID;
extern tSurfaceID surfaceCurrentID;

typedef struct __surface
{
  /* methode membres de la structure */
  struct __surface* top() const;
  struct __surface* up() const;
  struct __surface* child(int i) const;
  struct __surface* next() const;
  struct __surface* prev() const;
  struct __patch* patch() const;
  struct __material* material() const;
  struct __lightsource* lightsource() const;
  float width() const;
  float height() const;
  float metricArea() const;
  /* donnees de la structure */
  tSurfaceID surfaceID;
  /* donnees photometriques : agregation faible sur les proprietes des materiaux */
  tMaterialID materialID;
  int alreadyDone;
  /* donnees geometriques */
  USESTRUCT sPoint3d vertices[maximumNumberOfPointPerSurface];
  int size;
  /* donnees geometriques : donnees du mapping */
  USESTRUCT sPoint2d middleuv;
  /* donnees geometriques : donnees acceleratrice */
  USESTRUCT sVector3d uVector;
  USESTRUCT sVector3d vVector;
  /* donnees geometriques : plan soutenant le patch */
  USESTRUCT sPlane3d plane;
  /* donnees geometriques : pre-calculs pour les intersections (methode 2, en 2d) */
  int coord;
  USESTRUCT sPoint2d aibi[maximumNumberOfPointPerSurface];
  sGeometricData ci[maximumNumberOfPointPerSurface];
  /* donnees geometriques : geometrie sous-jacente : subdivision de la surface */
  int level;
  enum eSurfaceSplittingMode subdivided;
  int numberOfChildSurfaces;
} sSurface;

extern sSurface* createSurface();
extern void deleteSurface(sSurface*& s);

extern void initializeSceneRadiosityData();

extern void initializeSurfacePlaneData(sSurface& pol);
extern void initializeSurfaceMappingData(sSurface& pol,sPoint2d* uppermid,sPoint2d& duv,eSurfaceSplittingMode uppersub,int idson);

extern sSurface* generateSurface(sPoint3d& p0,sPoint3d& p1,sPoint3d& p2,sPoint3d& p3,sSurface* top,sSurface* up,int idson = 0);

extern void setPhotometricProperty(sSurface& p,tMaterialID material);

extern void surfaceStats();

#endif
