#ifndef _RADIOSITE_UTILITY_H_
#define _RADIOSITE_UTILITY_H_

#include "Radiosite/Includes.h"
#include "Radiosite/Enumeration.h"
#include "Radiosite/Globals.h"

#define maximumNumberOfPointPerSurface 4
#define maximumNumberOfPointPerPatch 4

#define maxNumberOfLightSubdivisionPlane 40
#define maxNumberOfLightSubdivisionDirection 40

#define tObjectID long long

typedef tObjectID tSurfaceID;
typedef tObjectID tPatchID;
typedef tObjectID tMaterialID;
typedef tObjectID tLightSourceID;
typedef tObjectID tLightSourceSpectrumID;

#define raddeg 57.2957795
#define ln_10 2.302585093
#define e_1 2.718281828

#define CLAMPRGB(v) ((v) > 1.0 ? 1. : ((v) < 0. ? 0. : (v)))

#define GAMMACORRECTRGB(v) pow(v,gammaCorrectionFactor)

#define positive(n) (n == 1)

#define negative(n) (n == -1)

#define f_ln(a) (ln_10*log(a))

#define f_e(a) (pow(e_1,a))

#define LERP(a,l,h)     ((l)+(((h)-(l))*(a)))

#define V3Lerp(lo,hi,alpha,result) initVector3d(result,LERP(alpha,x(lo),x(hi)),LERP(alpha,y(lo),y(hi)),LERP(alpha,z(lo),z(hi)))

#define angleVector3d(a,b) ((ABS(normVector3d(a)*normVector3d(b)) < epsilonValue) ? 0. : \
			    ((scalarProductVector3d(a,b)/(normVector3d(a)*normVector3d(b))) > 1. ? 0. : \
			     ((scalarProductVector3d(a,b)/(normVector3d(a)*normVector3d(b))) < -1. ? PPI : \
			      acos(scalarProductVector3d(a,b)/(normVector3d(a)*normVector3d(b))))))

#define maximumComponentForObjectID(maxid) ((int)(ceil(f_e((f_ln(maxid))/3.))))

#define decomposeObjectID(id,comp,maxcomp) ((double)( \
  comp == 2 ? id % maxcomp :  \
  comp == 1 ? (id / maxcomp) % maxcomp : \
  id / (maxcomp * maxcomp) \
  )) / ((double)(maxcomp - 1.))

#define computeReflectedRayDirection(from,normal,newdir) { \
  sGeometricData _ps = ((scalarProductVector3d((from),(normal)) < 0.) ? -2. : 2.) * scalarProductVector3d(normal,from); \
  copyVector3d(normal,newdir); \
  mulVectorScalar3d(newdir,_ps); \
  addVectorPoint3d(newdir,newdir,from); \
}

#define computeTransmittedRayDirection(from,normal,newdir,nin,nout) { \
  sGeometricData _ps = ((scalarProductVector3d((from),(normal)) < 0.) ? -2. : 2.) * scalarProductVector3d(normal,from); \
  copyVector3d(normal,newdir); \
  mulVectorScalar3d(newdir,_ps); \
  addVectorPoint3d(newdir,newdir,from); \
}

#define areaRelatedCoefficient(A,B,D,C,a) { \
  sVector3d B_A; sousVectorPoint3d(B_A,A,B); \
  sVector3d C_A; sousVectorPoint3d(C_A,A,C); \
  sVector3d D_C; sousVectorPoint3d(D_C,C,D); \
  sVector3d D_B; sousVectorPoint3d(D_B,B,D); \
  (a)[0] = normVector3d(B_A)*normVector3d(C_A)*sin(angleVector3d(B_A,C_A)); \
  (a)[1] = normVector3d(B_A)*normVector3d(D_B)*sin(angleVector3d(B_A,D_B)); \
  (a)[2] = normVector3d(D_C)*normVector3d(C_A)*sin(angleVector3d(D_C,C_A)); \
  (a)[3] = normVector3d(D_C)*normVector3d(D_B)*sin(angleVector3d(D_C,D_B)); \
}

#define Jacobien(j,ppatch,s,t) { \
  sVector3d _PP2; sousVectorPoint3d(_PP2,(ppatch).vertices[0],(ppatch).vertices[1]); \
  sVector3d _PP3; sousVectorPoint3d(_PP3,(ppatch).vertices[0],(ppatch).vertices[2]); \
  sVector3d _PP4; sousVectorPoint3d(_PP4,(ppatch).vertices[0],(ppatch).vertices[3]); \
  sVector3d _u; copyVector3d(_PP2,_u); \
  normalizeVector3d(_u); \
  sVector3d _v; crossProductVector3d(_v,(ppatch).surface()->plane.v,_u); \
  point2d(_P1,0.,0.); \
  point2d(_P2,scalarProductVector3d(_PP2,_u),scalarProductVector3d(_PP2,_v)); \
  point2d(_P3,scalarProductVector3d(_PP3,_u),scalarProductVector3d(_PP3,_v)); \
  point2d(_P4,scalarProductVector3d(_PP4,_u),scalarProductVector3d(_PP4,_v)); \
  (j) = (s*(-_P3.x*_P2.y-_P2.x*_P4.y+_P2.x*_P3.y+_P4.x*_P2.y)+t*(-_P2.x*_P4.y+_P3.x*_P4.y+_P4.x*_P2.y-_P4.x*_P3.y)+_P2.x*_P4.y-_P4.x*_P2.y); \
  (j) = ABS(j); \
}

#define getUpperMappingValues(uin,vin,surface,uout,vout) { \
  (uout) = x((surface).mapping[0]) + (x((surface).mapping[2]) - x((surface).mapping[0]))*(uin); \
  (vout) = y((surface).mapping[0]) + (y((surface).mapping[2]) - y((surface).mapping[0]))*(vin); \
}

#define getPointMappingOnPolygon(point,surface,uv) { \
 point2d(_uv0,getu(point,(surface).coord)-getu((surface).vertices[0],(surface).coord),getv(point,(surface).coord)-getv((surface).vertices[0],(surface).coord)); \
 point2d(_uv1,getu((surface).uVector,(surface).coord),getv((surface).uVector,(surface).coord)); \
 point2d(_uv2,getu((surface).vVector,(surface).coord),getv((surface).vVector,(surface).coord)); \
 initPoint2d(uv,det2(_uv0,_uv2)/det2(_uv1,_uv2),det2(_uv1,_uv0)/det2(_uv1,_uv2)); \
}

#define getPointPolygonOnMapping(point,surface,uv) { \
 USESTRUCT sVector3d __utemp; copyVector3d((surface).uVector,__utemp); mulVectorScalar3d(__utemp,x(uv)); \
 USESTRUCT sVector3d __vtemp; copyVector3d((surface).vVector,__vtemp); mulVectorScalar3d(__vtemp,y(uv)); \
 copyPoint3d((surface).vertices[0],(point)); \
 addVectorPoint3d((point),(point),__utemp); \
 addVectorPoint3d((point),(point),__vtemp); \
}

#define quadrilateralPoint(verts,p2d,p3d) initPoint3d(p3d, \
(1-y(p2d))*((1-x(p2d))*x((verts)[0]) + x(p2d)*x((verts)[1]))+ y(p2d)*((1-x(p2d))*x((verts)[3]) + x(p2d)*x((verts)[2])), \
(1-y(p2d))*((1-x(p2d))*y((verts)[0]) + x(p2d)*y((verts)[1]))+ y(p2d)*((1-x(p2d))*y((verts)[3]) + x(p2d)*y((verts)[2])), \
(1-y(p2d))*((1-x(p2d))*z((verts)[0]) + x(p2d)*z((verts)[1]))+ y(p2d)*((1-x(p2d))*z((verts)[3]) + x(p2d)*z((verts)[2])))

#define intersectPrimaryRayWithPolygon(ray,surface) { \
 BEGINSTAT(intersectPrimaryRayWithPolygon); \
 if (scalarProductVector3d((ray).rayLine.v,(surface).plane.v) < 0.) { \
   sGeometricData _t = parameterPointOnLineIntersectingPlane3d((surface).plane,(ray).rayLine); \
   if ((_t >= 0.) && (_t < (ray).currentItersectionRayParameter)) { \
     sPoint3d _pp; \
     copyPoint3d((ray).rayLine.v,_pp); \
     mulVectorScalar3d(_pp,_t); \
     addVectorPoint3d(_pp,_pp,(ray).rayLine.p); \
     sGeometricData _u = getu(_pp,(surface).coord); \
     sGeometricData _v = getv(_pp,(surface).coord); \
      int _i = 0; \
      while (_i < (surface).size) { \
	if (((surface).aibi[_i].x*_u+(surface).aibi[_i].y*_v+(surface).ci[_i]) >= 0.) break; \
	_i++; \
      } \
      if (_i == (surface).size) { \
              copyPoint3d(_pp,(ray).currentIntersectionPoint3d); \
	(ray).currentItersectionRayParameter = _t; \
	(ray).currentIntersectionHighestSurface = &(surface); \
      } \
   } \
 } \
 ENDSTAT(intersectPrimaryRayWithPolygon); \
}

#define intersectSecondaryRayWithPolygon(ray,surface) { \
 BEGINSTAT(intersectSecondaryRayWithPolygon); \
 sGeometricData _t = parameterPointOnLineIntersectingPlane3d((surface).plane,(ray).rayLine); \
   if ((_t >= 0.) && (_t < (ray).currentItersectionRayParameter)) { \
     sPoint3d _pp; \
     copyPoint3d((ray).rayLine.v,_pp); \
     mulVectorScalar3d(_pp,_t); \
     addVectorPoint3d(_pp,_pp,(ray).rayLine.p); \
     sGeometricData _u = getu(_pp,(surface).coord); \
     sGeometricData _v = getv(_pp,(surface).coord); \
      int _i = 0; \
      while (_i < (surface).size) { \
	if (((surface).aibi[_i].x*_u+(surface).aibi[_i].y*_v+(surface).ci[_i]) > 0.) break; \
	_i++; \
      } \
      if (_i == (surface).size) { \
        if (ABS(_t) < 1e-3) { \
          if (scalarProductVector3d((ray).rayLine.v,(surface).plane.v) < 0.) { \
             copyPoint3d(_pp,(ray).currentIntersectionPoint3d); \
             (ray).currentItersectionRayParameter = _t; \
             (ray).currentIntersectionHighestSurface = &(surface); \
          } \
        } else { \
              copyPoint3d(_pp,(ray).currentIntersectionPoint3d); \
	(ray).currentItersectionRayParameter = _t; \
	(ray).currentIntersectionHighestSurface = &(surface); \
        } \
      } \
 } \
 ENDSTAT(intersectSecondaryRayWithPolygon); \
}

#endif
