#ifndef _RLE_EXCEPTION_H_
#define _RLE_EXCEPTION_H_

#include "Rle/includes.h"
#include "Rle/Enumeration.h"

#ifdef USENAMESPACE
/*!
  \namespace Rle
  \brief namespace contaning utility functions and datas for rle compression/decompression.
*/
namespace Rle
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \class RleException
    \brief encapsulates exceptions occuring when compressing/decompressing rle buffers.
  */
  class RleException {
  private:
    /** the error code associated to the exception. */
    eRleErrorCode _errorCode;
    /** name of the file where exception occured. */
    string _filename;
    /** name of the function where exception occured. */
    string _funcname;
    /** line number where exception occured. */
    int _linenumber;
    /** error message associated to the exception. */
    string _errormessage;
  public:
    /** constructor */
    RleException(eRleErrorCode ec,const string& Fn,const string& fn,const int ln,const string& em);
    /** destructor */
    ~RleException();
    /** stream dumping function */
    friend ostream& operator<<(ostream& err,const RleException& e);
  };
  /*!
    \fn ostream& operator<<(ostream& err,const RleException& e)
    \brief stream dumping function.
    \param err ostream&, a reference on the error stream.
    \param e const RleException&, a reference on the exception.
    \return the reference on the error stream.
  */
  extern ostream& operator<<(ostream& err,const RleException& e);

#ifdef USENAMESPACE
} /* namespace Rle */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _RLE_EXCEPTION_H_ */
