#ifndef _SOURCE_IESFILE_H_
#define _SOURCE_IESFILE_H_

#include "Source/Includes.h"
#include "Source/LightSource.h"

typedef struct IE_Data /* IESNA Standard File data */
{
  struct /* File information */
  {
    string name; /* Name */
    enum { /* Format */
      IESNA_86, /* LM-63-1986 */
      IESNA_91, /* LM-63-1991 */
      IESNA_95 /* LM-63-1995 */
    } format;
  } file;
  map < string , string > ies_labels;
/*   struct IE_Label { /* Label line linked list element */ */
/*     string line; /* Label line pointer */ */
/*     struct IE_Label *pnext; /* Next list element pointer */ */
/*   } *plline; /* Label line linked list pointer */ */
  struct { /* Lamp data */
    int num_lamps; /* Number of lamps */
    float lumens_lamp; /* Lumens per lamp */
    float multiplier; /* Candela multiplying factor */
    string tilt_fname; /* TILT file name pointer (optional) */
    struct { /* TILT data structure */
      enum { /* Lamp-to-luminaire geometry */
        LampVert = 1, /* Lamp vertical base up or down */
        LampHorz = 2, /* Lamp horizontal */
        LampTilt = 3 /* Lamp tilted */
      } orientation;
      int num_pairs; /* # of angle-multiplying factor pairs */
      float *angles; /* Angles array pointer */
      float *mult_factors; /* Multiplying factors array pointer */
    } tilt;
  } lamp;
  enum { /* Measurement units */
    Feet = 1, /* Imperial */
    Meters = 2 /* Standard Internationale */
  } units;
  struct { /* Luminous cavity dimensions */
    float width; /* Opening width */
    float length; /* Opening length */
    float height; /* Cavity height */
  } dim;
  struct { /* Electrical data */
    float ball_factor; /* Ballast factor */
    float blp_factor; /* Ballast-lamp photometric factor */
    float input_watts; /* Input watts */
  } elec;
  struct { /* Photometric data */
    enum { /* Photometric goniometer type */
      Type_A = 3, /* Type A */
      Type_B = 2, /* Type B */
      Type_C = 1 /* Type C */
    } gonio_type;
    int num_vert_angles; /* Number of vertical angles */
    int num_horz_angles; /* Number of horizontal angles */
    float *vert_angles; /* Vertical angles array */
    float *horz_angles; /* Horizontal angles array */
    float **pcandela; /* Candela values array pointers array */
  } photo;
} IE_DATA;

#endif
