#ifndef _SPECTRUM_CIE_H_
#define _SPECTRUM_CIE_H_

#include "Spectrum/Includes.h"
#include "Spectrum/Spectrum.h"

/*
  CIE standard Spectral Luminous Efficiency Function
  1988 Spectral Luminous Efficiency Function for photopic vision
*/
extern sDetailledSpectrum photopicLuminousEfficiency;

/*
  CIE standard Spectral Luminous Efficiency Function
  Spectral Luminous Efficiency Function for scotopic vision 
*/
extern sDetailledSpectrum scotopicLuminousEfficiency;

/*
  CIE standard observers
  2 Observer (CIE 1931 ) - for small objects. 
*/
extern sDetailledSpectrum observerX2;
extern sDetailledSpectrum observerY2;
extern sDetailledSpectrum observerZ2;

/*
  CIE standard observers
  10 Observer (CIE 1964) - for large objects. 
*/
extern sDetailledSpectrum observerX10;
extern sDetailledSpectrum observerY10;
extern sDetailledSpectrum observerZ10;

/* White Points Defined in CIE 1931 Standard
  Name          x              y        Color Temperature      Comments
  A            0.4476      0.4075            2854 K            Incandescent Light
  B            0.3840      0.3516            4874 K            Direct Sunlight
  C            0.3101      0.3162            6774 K            Indirect Sunlight
  D5000     0.3457      0.3586            5000 K            Bright Incandescent Light
  D6500     0.3127      0.3297            6504 K            "Natural" Daylight
  E            0.3333      0.3333            5500 K            Normalized Reference
*/

/*
  CIE standard illuminants A
  Comment: 
  NBS Index: 72 dark orange yellow
  Chromaticity: (0.447521, 0.407406, 0.145073) luminance 0.289505
  Coefficient of Extinction: 0.000000
  Index of Refraction: 0.000000
  Average Energy: 0.365691
*/
extern sDetailledSpectrum illuminantA;
extern sDetailledSpectrum illuminantB;
extern sDetailledSpectrum illuminantC;
extern sDetailledSpectrum illuminantD55;
extern sDetailledSpectrum illuminantD65;
extern sDetailledSpectrum illuminantD75;

extern void initializeCIEStandardData();
extern void registerCIEStandardData();

#endif
