#ifndef _TEMPLATE_BASICMATRIXSPECIALIZATION_H_
#define _TEMPLATE_BASICMATRIXSPECIALIZATION_H_

#include "Template/include.h"
#include "Template/BasicMatrix.h"

/**
 * template matrix container class.
 * specialize cBasicMatrix for 1x1 matrix.
 *
 * \param class telt, the type of contained elements.
 */
template <class telt> class cBasicMatrix<1,1,telt>
{
 private:
  #define _typeBasicMatrix cBasicMatrix<1,1,telt>
 protected:
  /** pointer to matrix elements. */
  telt* _data;
 public:
  /**
   * the default constructor.
   * initialize a vector.
   */
  cBasicMatrix()
    {
      _data = new telt[1];
    }
  /**
   * the initialization constructor.
   * initialize a matrix with an  element.
   * \param const telt& v, the data.
   */
  cBasicMatrix(const telt& v)
    {
      _data[0] = v;
    }
  /**
   * the initialization constructor.
   * initialize a matrix with an array of elements.
   * \param const telt* data, the array of datas.
   */
  cBasicMatrix(const telt* data)
    {
      _data = new telt[1];
      _data[0] = data[0];
    }
  /**
   * the copy constructor.
   * initialize a matrix with another matrix.
   * \param const cBasicMatrix<1,1,telt>& m, the matrix to be copied
   */
  cBasicMatrix(const cBasicMatrix<1,1,telt>& m)
    {
      _data = new telt[1];
      _data[0] = m._data[0];
    }
  /**
   * the destructor.
   */
  virtual ~cBasicMatrix()
    {
      if (_data != NULL)
	{
	  delete [] _data;
	}
    }
  /**
   * access method to the height of a matrix.
   * \return a the height of a matrix.
   */
  int numberOfRows() const
    {
      return 1;
    }
  /**
   * access method to the width of a matrix.
   * \return a the width of a matrix.
   */
  int numberOfColumns() const
    {
      return 1;
    }
  /**
   * access method to a row of the matrix.
   * \param int idx, the index of the row.
   * \return a cRowVector object describing this row in the matrix.
   */
  virtual cRowVector<1,telt> getRow(const int& idx)
    {
      if ((idx < 0) || (idx >= 1)) {}
      else {
	cRowVector<1,telt> res(_data,idx,1); return res;
      }
    }
  /**
   * access method to a column of the matrix.
   * \param int idx, the index of the column.
   * \return a cColumnVector object describing this column in the matrix.
   */
  virtual cColumnVector<1,telt> getColumn(const int& idx)
    {
      if ((idx < 0) || (idx >= 1)) {}
      else {
	cColumnVector<1,telt> res(_data,idx,1); return res;
      }
    }
  /**
   * conversion to telt (ie: access  to the element of the matrix.
   * \return the element in the matrix.
   */
  operator telt()
    {
      return _data[0];
    }
  /**
   * access method to an element of the matrix (as left member of an expression).
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a reference on the element in the matrix.
   */
  virtual telt& operator()(const int& idxr,const int& idxc)
    {
      if ((idxr < 0) || (idxr >= 1)) {}
      else if ((idxc < 0) || (idxc >= 1)) {}
      else return _data[idxr*1+idxc];
    }
  /**
   * access method to an element of the matrix (as right member of an expression).
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a copy of the element in the matrix.
   */
  virtual telt operator()(const int& idxr,const int& idxc) const
    {
      if ((idxr < 0) || (idxr >= 1)) {}
      else if ((idxc < 0) || (idxc >= 1)) {}
      else return _data[idxr*1+idxc];
    }
  /**
   * copy operator on matrix.
   * \param const cBasicMatrix<1,1,telt>& m, the matrix to be copied.
   * \return a reference on the copy of matrix lst.
   */
  cBasicMatrix<1,1,telt>& operator=(const cBasicMatrix<1,1,telt>& m)
    {
      if (&m != this)
	{
	  _data[0] = m._data[0];
	}
      return *this;
    }
  /**
   * utility function.
   * computes the inverse of a matrix.
   */
  cBasicMatrix<1,1,telt> inv()
  {
    cBasicMatrix<1,1,telt> res;
    res(0,0) = telt(1) / operator()(0,0);
    return res;
  }
  /**
   * utility function.
   * computes the pseudo-inverse of a matrix.
   */
  cBasicMatrix<1,1,telt> pinv()
  {
    cBasicMatrix<1,1,telt> res;
    res(0,0) = telt(1) / operator()(0,0);
    return res;
  }
  template <int nr,int nc> cBasicMatrix<nr,nc,telt> operator*(const cBasicMatrix<nr,nc,telt>& m)
    {
      cBasicMatrix<nr,nc,telt> res;
      for (int i=0;i<nr;i++)
	{
	  for (int j=0;j<nc;j++)
	    {
	      res(i,j) = operator()(0,0) * m(i,j);
	    }
	}
      return res;
    }

  friend class cBasicMatrix<1,1,telt>;
  friend class cMatrix<1,1,telt>;
  friend class cRowVector<1,telt>;
  friend class cColumnVector<1,telt>;
};

#endif
