#ifndef _TEMPLATE_EXCEPTION_H_
#define _TEMPLATE_EXCEPTION_H_

#include "Template/include.h"
//#include <std.h>
//#include <exception>
//#include <stdexcept>

/** \def ExceptionErrorCode, the basic code of an exception. */
#define ExceptionErrorCode 0
/** \def ExceptionErrorCode_Message, the code of a message exception. */
#define ExceptionErrorCode_Message ExceptionErrorCode+1
/** \def ExceptionErrorCode_Warning, the code of a warning exception. */
#define ExceptionErrorCode_Warning ExceptionErrorCode+2
/** \def ExceptionErrorCode_Fatal, the code of a fatal exception. */
#define ExceptionErrorCode_Fatal ExceptionErrorCode+3

/**
 * template Exception class.
 *
 * \param int ErrorCode, the code describing the error.
 * \param class Sender, who throwed the exception.
 */
template < int ErrorCode, class Sender > class cException {
 private:
  /** the string describing the exception. */
  char _exceptionString[255];
  /** the file in which the exception occured. */
  char _fileName[255];
  /** the function in which the exception occured. */
  char _functionName[255];
  /** the line number on which the exception occured. */
  int _lineNumber;
 protected:
 public:
  /**
   * the constructor.
   * initialize the exception.
   * \param char* es, the string describing the exception.
   * \param char* Fn, the file where exception occured.
   * \param char* fn, the function in which occured the exception.
   * \param int ln, the line number in the source code.
   */
  cException(char* es = NULL,char* Fn = NULL,char* fn = NULL,int ln = 0) :
    _lineNumber(ln)
    {
      if (es != NULL) sprintf(_exceptionString,es);
      else _exceptionString[0] = '\0';
      if (Fn != NULL) sprintf(_fileName,Fn);
      else _fileName[0] = '\0';
      if (fn != NULL) sprintf(_functionName,fn);
      else _functionName[0] = '\0';
      cerr << "Error<" << setw(4) << ErrorCode << "> [" << typeid(Sender).name() << "::" << fn << ",(" << Fn << "," << ln << ")] : " << es << endl;
    }
  /**
   * the destructor.
   */
  ~cException()
    {
    }
  /**
   * private attribute access method.
   * access to the describing string of the exception.
   * \return the string describing the exception.
   */
  char* getExceptionString() const
    {
      return _exceptionString;
    }
  /**
   * private attribute access method.
   * access to the function name of the exception.
   * \return the function name of the exception.
   */
  char* getFunctionName() const
    {
      return _functionName;
    }
  /**
   * private attribute access method.
   * access to the file name of the exception.
   * \return the file name of the exception.
   */
  char* getFileName() const
    {
      return _fileName;
    }
  /**
   * private attribute access method.
   * access to the line number of the exception.
   * \return the line number of the exception.
   */
  int getLineNumber() const
    {
      return _lineNumber;
    }
};

/**
 * stream dumping of an exception.
 * dumps an exception on a stream.
 * \param ostream& out, the dumping stream.
 * \param const cException< ErrorCode, Sender >& e, the exception to dump.
 * \return a reference on the dumping stream.
 */
template < int ErrorCode, class Sender > ostream& operator<<(ostream& out,const cException< ErrorCode, Sender >& e)
{
  out << "Error<" << setw(4) << ErrorCode << "> [" << typeid(Sender).name() << "::" << e.getFunctionName() << ",(" << e.getFileName() << "," << e.getLineNumber() << ")] : " << e.getExceptionString();
  return out;
};

/** \def THROWERROR(cod,es,c)
 * \param cod, the error code.
 * \param es, the message string.
 * \param c, the class that throws the exception.
 * throw an error to the system
 */
#define THROWERROR(cod,es,c) throw cException<cod,c>(es,__FILE__,__FUNCTION__,__LINE__)

/** \def MESSAGEERROR(es,c)
 * \param es, the message string.
 * \param c, the class that throws the exception.
 * throw a non-destructive error to the system (with ExceptionErrorCode_Message error code).
 */
#define MESSAGEERROR(es,c) THROWERROR(ExceptionErrorCode_Message,es,c)
/** \def WARNINGERROR(es,c)
 * \param es, the message string.
 * \param c, the class that throws the exception.
 * throw a non-destructive warning to the system (with ExceptionErrorCode_Warning error code).
 */
#define WARNINGERROR(es,c) THROWERROR(ExceptionErrorCode_Warning,es,c)
/** \def FATALERROR(es,c)
 * \param es, the message string.
 * \param c, the class that throws the exception.
 * throw a destructive error to the system (with ExceptionErrorCode_Fatal error code).
 */
#define FATALERROR(es,c) THROWERROR(ExceptionErrorCode_Fatal,es,c)

#endif
