#ifndef _TEMPLATE_FILE_H_
#define _TEMPLATE_FILE_H_

#include "Template/include.h"
#include "Template/Element.h"

/** \def FileErrorCode, the basic code of a file exception. */
#define FileErrorCode 50
/** \def FileErrorCode_EmptyFile, the code of an exception due to empty file access. */
#define FileErrorCode_EmptyFile FileErrorCode+1
/** \def FileErrorCode_MemoryNew, the code of an exception due to memory allocation in cFile class. */
#define FileErrorCode_MemoryNew FileErrorCode+2
/** \def FileErrorCode_MemoryFree, the code of an exception due to memory desallocation in cFile class. */
#define FileErrorCode_MemoryFree FileErrorCode+3

/**
 * template file container class.
 *
 * \param class T, the type of contained elements.
 */
template <class T> class cFile {
private:
  /** the last added element */
  cElement<T>* _head;
  /** the first added element */
  cElement<T>* _tail;
  /** the number of elements in the file */
  int _card;
protected:
  /**
   * internal method to push an element in the file.
   * \param T& v, the element to add on top of the file.
   */
  void _push(T& v)
    {
      cElement<T>* tmp;
      tmp = new cElement<T>(_head,NULL,v);
      _head = tmp;
      if (_tail == NULL) _tail = _head;
      _card++;
    }
  /**
   * internal method to pop an element from the file.
   * \return the element that previously was on bottom of the file.
   */
  T& _pop()
    {
      if (_tail == NULL) {}
      else {
	cElement<T>* tmp = _tail;
	_tail = tmp->_next;
	T elt = tmp->_data;
	_card--;
	if (_tail != NULL) _tail->_prev = NULL;
	if (_head == tmp) _head = NULL;
	delete tmp;
	return elt;
      }
    }
public:
  /**
   * the constructor.
   * initialize a file to empty file.
   */
  cFile() :
    _head(NULL) ,
    _tail(NULL) ,
    _card(0)
    {
    }
  /**
   * the destructor.
   * removes a file from memory.
   */
  ~cFile()
    {
      while (_head)
	{
	  cElement<T>* tmp = _head;
	  _head = tmp->_prev;
	  delete tmp;
	}
    }
  /**
   * access to the bottom of the file.
   * use: T v = (T)<cFile object>;
   */
  operator T()
    {
      if (_tail != NULL) return _tail->_data;
    }
  /**
   * access method to the effective size of the file.
   * \return the number of elements contained in the file.
   */
  operator int()
    {
      return _card;
    }
  /**
   * operator to push an element in the file.
   * use: <cFile object>  <<  v;
   * \param T& v, the element to add on top of the file.
   * \return a reference on the current file.
   */
  cFile<T>& operator <<(T& v)
    {
      _push(v);
      return (*this);
    }
  /**
   * operator to pop an element from the file.
   * use: <cFile object>  >>  v;
   * \param T& v, the element that was on bottom of the file.
   * \return a reference on the current file.
   */
  cFile<T>& operator >>(T& v)
    {
      v = _pop();
      return (*this);
    }
};

#endif
