#ifndef _TEMPLATE_LISTITERATOR_H_
#define _TEMPLATE_LISTITERATOR_H_

#include "Template/include.h"
#include "Template/Element.h"
#include "Template/List.h"

/** \def ListIteratorErrorCode, the basic code of a list iterator exception. */
#define ListIteratorErrorCode 30
/** \def ListIteratorErrorCode_NotInitialized, the code of an exception due to uninitialized list iterator. */
#define ListIteratorErrorCode_NotInitialized ListIteratorErrorCode+0
/** \def ListIteratorErrorCode_RunningIteration, the code of an exception due to running iterator on the reference list. */
#define ListIteratorErrorCode_RunningIteration ListIteratorErrorCode+1
/** \def ListIteratorErrorCode_EndAchieve, the code of an exception due to access after the end of the list in an iterator. */
#define ListIteratorErrorCode_EndAchieve ListIteratorErrorCode+2

/**
 * template list iterator class.
 *
 * \param class T, the type of elements contained in the list to iterate.
 */
template < class T > class cListIterator {
private:
  /** the reference list to iterate on. */
  cList< T >& _list;
  /** the current element of the iteration. */
  cElement< T >* _crtelt;
  /** the current element index of the iteration. */
  int _index;
  /** is the iterator initialized. */
  int _initialized;
protected:
public:
  /**
   * the default constructor.
   * initialize the iterator to nothing.
   */
  cListIterator() :
    _crtelt(NULL) ,
    _index(0) ,
    _initialized(0)
    {
    }
  /**
   * the initialization constructor.
   * initialize the iterator with the given list.
   * \param cList< T >& lst, the list to iterate on.
   */
  cListIterator(cList< T >& lst) :
    _list(lst) ,
    _crtelt(lst._head->_next) ,
    _index(1) ,
    _initialized(1)
    {
      if (lst.iterate()) {}
    }
  /**
   * the destructor.
   */
  ~cListIterator()
    {
    }
  /**
   * is the iteration finished.
   * \return a boolean (true if already after the end of the list, false otherwise).
   */
  int finished()
    {
      if (!_initialized) {}
      return (_crtelt == _list._tail);
    }
  /**
   * gets the current element in the list.
   * \return a reference on the element in the list.
   */
  T& getCurrent()
    {
      if (!_initialized) {}
      else if (_crtelt == NULL) {}
      else return _crtelt->_data;
    }
  /**
   * goes on the next element.
   * \return a reference on the current iterator.
   */
  cListIterator< T >& operator++()
    {
      if (!_initialized) {}
      else if (_crtelt == NULL) {}
      else {
	_index++;
	_crtelt = _crtelt->_next;
      }
      return *this;
    }
  /**
   * initialize an iterator with a list.
   * \param cList< T >& lst, the list to iterate on.
   * \return a reference on the current iterator.
   */
  cListIterator< T >& operator=(cList< T >& lst)
    {
      if (lst.iterate()) {}
      else {
	_list = lst;
	_index = 1;
	_initialized = 1;
	_crtelt = lst._head->_next;
      }
    }
};

#endif
