#ifndef _TEMPLATE_STACK_H_
#define _TEMPLATE_STACK_H_

#include "Template/include.h"
#include "Template/Element.h"

/** \def StackErrorCode, the basic code of a stack exception. */
#define StackErrorCode 40
/** \def StackErrorCode_EmptyStack, the code of an exception due to empty stack access. */
#define StackErrorCode_EmptyStack StackErrorCode+1
/** \def StackErrorCode_MemoryNew, the code of an exception due to memory allocation in cStack class. */
#define StackErrorCode_MemoryNew StackErrorCode+2
/** \def StackErrorCode_MemoryFree, the code of an exception due to memory desallocation in cStack class. */
#define StackErrorCode_MemoryFree StackErrorCode+3

/**
 * template stack container class.
 *
 * \param class T, the type of contained elements.
 */
template <class T> class cStack {
private:
  /** the last added element */
  cElement<T>* _head;
  /** the number of elements in the stack */
  int _card;
protected:
  /**
   * internal method to push an element in the stack.
   * \param T& v, the element to add on top of the stack.
   */
  void _push(T& v)
    {
      cElement<T>* tmp;
      tmp = new cElement<T>(_head,NULL,v);
      _head = tmp;
      _card++;
    }
  /**
   * internal method to pop an element from the stack.
   * \return the element that previously was on top of the stack.
   */
  T& _pop()
    {
      if (_head == NULL) {}
      else {
	cElement<T>* tmp = _head;
	_head = tmp->_prev;
	T elt = tmp->_data;
	_card--;
	if (_head != NULL) _head->_next = NULL;
	delete tmp;
	return elt;
      }
    }
public:
  /**
   * the constructor.
   * initialize a stack to empty stack.
   */
  cStack() :
    _head(NULL) ,
    _card(0)
    {
    }
  /**
   * the destructor.
   * removes a stack from memory.
   */
  ~cStack()
    {
      while (_head)
	{
	  cElement<T>* tmp = _head;
	  _head = tmp->_prev;
	  delete tmp;
	}
    }
  /**
   * access to the top of the stack.
   * use: T v = (T)<cStack object>;
   */
  operator T()
    {
      if (_head != NULL) return _head->_data;
    }
  /**
   * access method to the effective size of the stack.
   * \return the number of elements contained in the stack.
   */
  operator int()
    {
      return _card;
    }
  int size()
    {
      return _card;
    }
  /**
   * operator to push an element in the stack.
   * use: <cStack object>  <<  v;
   * \param T& v, the element to add on top of the stack.
   * \return a reference on the current stack.
   */
  cStack<T>& operator <<(T& v)
    {
      _push(v);
      return (*this);
    }
  /**
   * operator to pop an element from the stack.
   * use: <cStack object>  >>  v;
   * \param T& v, the element that was on top of the stack.
   * \return a reference on the current stack.
   */
  cStack<T>& operator >>(T& v)
    {
      v = _pop();
      return (*this);
    }
};

#endif
