#ifndef _TEMPLATE_VECTOR_H_
#define _TEMPLATE_VECTOR_H_

#include "Template/include.h"
#include "Template/BasicMatrix.h"

/** \def VectorErrorCode, the basic code of a vector exception. */
#define VectorErrorCode 60
/** \def VectorErrorCode_MemoryNew, the code of an exception due to memory allocation in cVector class. */
#define VectorErrorCode_MemoryNew VectorErrorCode+0
/** \def VectorErrorCode_MemoryFree, the code of an exception due to memory desallocation in cVector class. */
#define VectorErrorCode_MemoryFree VectorErrorCode+1
/** \def VectorErrorCode_InvalidIndex, the code of an exception due to errored index in the vector. */
#define VectorErrorCode_InvalidIndex VectorErrorCode+2

/**
 * template vector container class.
 *
 * \param int vectorLength, the length of the vector.
 * \param class vectorElement, the type of contained elements.
 */
template < int vectorLength, class vectorElement > class cVector : public cBasicMatrix<1,vectorLength,vectorElement>
{
 private:
  #define _vectorType cVector<vectorLength,vectorElement>
  /** pointer to vector elements. */
/*   vectorElement* _data; */
 protected:
 public:
  /**
   * the default constructor.
   * initialize a vector.
   */
  cVector()
    {
      _data = new vectorElement[vectorLength];
    }
  /**
   * the initialization constructor.
   * initialize a vector with an array of elements.
   * \param vectorElement* pElt, the array of datas.
   */
  cVector(vectorElement* pElt)
    {
      _data = new vectorElement[vectorLength];
      for (int i=0;i<vectorLength;i++) _data[i] = pElt[i];
    }
  /**
   * the copy constructor.
   * initialize a vector with a copy of a row aligned vector.
   * \param cRowVector<vectorLength,vectorElement>& rv, the vector to copy
   */
  cVector(cRowVector<vectorLength,vectorElement>& rv)
    {
      _data = new vectorElement[vectorLength];
      for (int i=0;i<vectorLength;i++) _data[i] = rv[i];
    }
  /**
   * the copy constructor.
   * initialize a vector with a copy of a column aligned vector.
   * \param cColumnVector<vectorLength,vectorElement>& rv, the vector to copy
   */
  cVector(cColumnVector<vectorLength,vectorElement>& rv)
    {
      _data = new vectorElement[vectorLength];
      for (int i=0;i<vectorLength;i++) _data[i] = rv[i];
    }
  /**
   * the destructor.
   */
  virtual ~cVector()
    {
      delete [] _data;
    }
  /**
   * access method to the length of a vector.
   * \return a the length of a vector.
   */
  int length()
    {
      return vectorLength;
    }
  /**
   * access method to an element in the vector.
   * \param int index, the index of the element.
   * \return a reference on the element in the vector.
   */
  vectorElement& operator[](const int& index) const
    {
      if ((index < 0) || (index >= vectorLength)) {}
      else return _data[index];
    }
  /**
   * copy operator on vectors.
   * \param cVector<vectorLength,vectorElement>& lst, the vector to be copied.
   * \return the copy of vector lst.
   */
  cVector<vectorLength,vectorElement>& operator =(const cVector<vectorLength,vectorElement>& lst)
    {
      if (this != &lst)
	{
	  for (int i=0;i<vectorLength;i++)
	    {
	      _data[i] = lst._data[i];
	    }
	}
      return (*this);
    }

  /**
   * unary ~ operator.
   * \return the normalized vector.
   */
  cVector<vectorLength,vectorElement> operator~()
    {
      vectorElement n = *(*this);
      return ((*this) / n);
    }
};

#endif
