#ifndef _TGAFILE_ENUMERATION_H_
#define _TGAFILE_ENUMERATION_H_

#ifdef USENAMESPACE
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \def TF_ORIGINALFORMAT
    \brief tga file is in the original format.
  */
#define TF_ORIGINALFORMAT 0
  /*!
    \def TF_NEWFORMAT
    \brief tga file is in the new format.
  */
#define TF_NEWFORMAT 1
  /*!
    \def CMT_NOCOLORMAP
    \brief no colormap in the file.
  */
#define CMT_NOCOLORMAP 0
  /*!
    \def CMT_COLORMAP
    \brief data are colormaped in the file.
  */
#define CMT_COLORMAP 1
  /*!
    \def IT_NOIMAGEDATA
    \brief no image data in the file.
  */
#define IT_NOIMAGEDATA 0x00
  /*!
    \def IT_UNCOMPRESSEDCOLORMAPPED
    \brief format of the image data.
  */
#define IT_UNCOMPRESSEDCOLORMAPPED 0x01
  /*!
    \def IT_UNCOMPRESSEDTRUECOLOR
    \brief format of the image data.
  */
#define IT_UNCOMPRESSEDTRUECOLOR 0x02
  /*!
    \def IT_UNCOMPRESSEDBW
    \brief format of the image data.
  */
#define IT_UNCOMPRESSEDBW 0x03
  /*!
    \def IT_RLECOMPRESSEDCOLORMAPPED
    \brief format of the image data.
  */
#define IT_RLECOMPRESSEDCOLORMAPPED 0x09
  /*!
    \def IT_RLECOMPRESSEDTRUECOLOR
    \brief format of the image data.
  */
#define IT_RLECOMPRESSEDTRUECOLOR 0x0A
  /*!
    \def IT_RLECOMPRESSEDBW
    \brief format of the image data.
  */
#define IT_RLECOMPRESSEDBW 0x0B
  /*!
    \def IDM_ALL
    \brief mask to access image data attributes.
  */
#define IDM_ALL 0x3F
  /*!
    \def IDM_ORIGIN
    \brief mask to access image data origin specification.
  */
#define IDM_ORIGIN 0x30
  /*!
    \def IDM_ATTRIBUTESIZE
    \brief mask to access image data attribute size.
  */
#define IDM_ATTRIBUTESIZE 0x0F
  /*!
    \def ID_ORIGINBOTTOMLEFT
    \brief flag specifying image data origin position.
  */
#define ID_ORIGINBOTTOMLEFT 0x00
  /*!
    \def ID_ORIGINBOTTOMRIGHT
    \brief flag specifying image data origin position.
  */
#define ID_ORIGINBOTTOMRIGHT 0x01
  /*!
    \def ID_ORIGINTOPLEFT
    \brief flag specifying image data origin position.
  */
#define ID_ORIGINTOPLEFT 0x02
  /*!
    \def ID_ORIGINTOPRIGHT
    \brief flag specifying image data origin position.
  */
#define ID_ORIGINTOPRIGHT 0x03
  /*!
    \enum eTgaErrorCode
    \brief enumeration containing tga file loading/saving error codes.
   */
typedef enum
  {
    TEC_NONE = 0 , /**< no error. */
    TEC_FILENOTEXIST = 1 , /**< input file doesn't exists. */
    TEC_FILEINVALIDFLAGS = 2 , /**< invalid flag in the tga file. */
    TEC_FILEINVALIDFORMAT = 3 , /**< invalid file format. */
    TEC_DATAINVALIDFORMAT = 4 , /**< invalid data format in the file. */
    TEC_MEMORYERROR = 5 /**< memory allocation/desallocation error. */
  } eTgaErrorCode;

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGAFILE_ENUMERATION_H_ */
