#ifndef _TGAFILE_EXCEPTION_H_
#define _TGAFILE_EXCEPTION_H_

#include "Tga/includes.h"
#include "Tga/Enumeration.h"

#ifdef USENAMESPACE
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \class TgaException
    \brief encapsulates exceptions occuring when loading/saving tga files.
  */
  class TgaException {
  private:
    /** the error code associated to the exception. */
    eTgaErrorCode _errorCode;
    /** name of the file where exception occured. */
    char _filename[255];
    /** name of the function where exception occured. */
    char _funcname[255];
    /** line number where exception occured. */
    int _linenumber;
    /** error message associated to the exception. */
    char _errormessage[255];
  public:
    /** constructor */
    TgaException(eTgaErrorCode ec,const char* Fn,const char* fn,const int ln,const char* em);
    /** destructor */
    ~TgaException();
    /** stream dumping function */
    friend ostream& operator<<(ostream& err,const TgaException& e);
  };
  /*!
    \fn ostream& operator<<(ostream& err,const TgaException& e)
    \brief stream dumping function.
    \param err ostream&, a reference on the error stream.
    \param e const RleException&, a reference on the exception.
    \return the reference on the error stream.
  */
  extern ostream& operator<<(ostream& err,const TgaException& e);

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGAFILE_EXCEPTION_H_ */
