#ifndef _TGAFILE_STRUCTURES_H_
#define _TGAFILE_STRUCTURES_H_

#ifdef USENAMESPACE
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \struct sColorMapSpecification
    \brief description of the color map (if any) used for the image.
   */
  typedef struct {
    /** index of the first entry in the colormap. */
    /* field 4.1 */ byte _cmap_firstentryindex[2];
    /** number of enties in the colormap. */
    /* field 4.2 */ byte _cmap_colormaplength[2];
    /** number of bits per entry. */
    /* field 4.3 */ byte _cmap_colormapentrysize;
  } sColorMapSpecification;
  /*!
    \struct sImageSpecification
    \brief image screen location and size.
   */
  typedef struct {
    /** absolute horizontal coordinate for the lower left corner of the image. */
    /* field 5.1 */ byte _image_xorigin[2];
    /** absolute vertical coordinate for the lower left corner of the image. */
    /* field 5.2 */ byte _image_yorigin[2];
    /** width of the image in pixels. */
    /* field 5.3 */ byte _image_width[2];
    /** height of the image in pixels. */
    /* field 5.4 */ byte _image_height[2];
    /** number of bits per pixel, including attribute datas. */
    /* field 5.5 */ byte _image_depth;
    /** image internal descriptor. */
    /* field 5.6 */ byte _image_descriptor;
  } sImageSpecification;
  /*!
    \struct sTgaFileHeader
    \brief header of a tga file.
   */
  typedef struct {
    /** indicates the number of bytes contained in the image ID. */
    /* field 1 */ byte _header_IDLength;
    /** indicates if image datas are (aren't) color mapped. */
    /* field 2 */ byte _header_ColorMapType;
    /** indicates how image datas are stored. */
    /* field 3 */ byte _header_ImageType;
    /** description of the color map (if any) used for the image. */
    /* field 4 */ sColorMapSpecification _header_ColorMapSpecification;
    /** image screen location and size. */
    /* field 5 */ sImageSpecification _header_ImageSpecification;
  } sTgaFileHeader;
  /*!
    \struct sTgaFileFooter
    \brief footer of a tga file.
   */
  typedef struct {
    /** offset of the extension area (if any) in the file. */
    /* field 1 */ byte _footer_extensionAreaOffset[4];
    /** offset of the developer area (if any) in the file. */
    /* field 2 */ byte _footer_developerAreaOffset[4];
    /** signature of the file : "TRUEVISION-XFILE". */
    /* field 3 */ char _footer_signature[16];
    /** ending char : ".". */
    /* field 4 */ byte _footer_endchar;
    /** string terminator : 0x00. */
    /* field 5 */ byte _footer_terminator;
  } sTgaFileFooter;
  /*!
    \struct sExtensionAreaHeader
    \brief header of the extension area (if any).
   */
  typedef struct {
    /** size of the extension area. */
    short _extensionSize;
    /** 40 characters for the name. 41th char must be null char. */
    byte _authorName[41];
    /** four lines of 80 characters. each line ended with null char. */
    byte _authorComments[324];
    /** 6 shorts for date and time of image saving. in order : month, day, year, hour, minutes, seconds. */
    short _dateTimeStamp[6];
    /** 40 characters for the job name (or ID). terminated by a null char. */
    byte _jobNameID[41];
    /** time elapsed for the job (hour, minutes, seconds). */
    short _jobTime[3];
    /** 40 characters for the software ID (last char is null). */
    byte _softwareID[41];
    /** version number of the software (first two bytes : version number * 100, last byte : version letter). */
    byte _softwareVersion[3];
    /** key color in effect when image was saved (in A.R.G.B format). */
    long _keyColor;
    /** the pixel size ratio : width , height. */
    short _pixelAspectRatio[2];
    /** fractionnal gamma value : numerator , denominator. */
    short _gammaValue[2];
    /** offset (from the beginning of the file) to the start of the color correction table. */
    long _colorCorrectionOffset;
    /** offset (from the beginning of the file) to the start of the postage stamp image. */
    long _postageStampOffset;
    /** offset (from the beginning of the file) to the start of the scan line table. */
    long _scanLineOffset;
    /** this field contains a values that specifies the type of alpha channel data contained in the file. */
    byte _attributeType;
  } sExtensionAreaHeader;

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGAFILE_STRUCTURES_H_ */
