#ifndef _TGA_TGAFILE_H_
#define _TGA_TGAFILE_H_

#include "Tga/includes.h"
#include "Tga/Enumeration.h"
#include "Tga/Structures.h"

#ifdef USENAMESPACE
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \class TgaFile
    \brief base class for input/output tga files.
   */
  class TgaFile {
  public:
    /** flags specifying reading mode. */
    static byte in;
    /** flags specifying writting mode. */
    static byte out;
    /** flags specifying rle encoding. */
    static byte coder;
    /** flags specifying color mapped datas. */
    static byte mapper;
  protected:
    /** name of the file. */
    char _file_Name[255];
    /** flags used for file opening mode. */
    byte _file_flags;
    /** size of the file. */
    long _file_Size;
    /** buffer where file datas are loaded/encoded. */
    byte* _file_Data;
    //@{
    /** pointers referencing various areas in the buffer. */
    sTgaFileHeader* _tga_Header;
    byte* _tga_Data;
    byte* _tga_ImageID;
    byte* _tga_ColorMapData;
    byte* _tga_ImageData;
    byte* _tga_DeveloperArea;
    byte* _tga_DeveloperFields;
    byte* _tga_DeveloperDirectory;
    byte* _tga_ExtensionArea;
    sExtensionAreaHeader* _tga_ExtensionAreaHeader;
    byte* _tga_ScanLineTable;
    byte* _tga_PostageStampImage;
    byte* _tga_ColorCorrectionTable;
    sTgaFileFooter* _tga_Footer;
    //@}
    /** width in pixels. */
    int _image_Width;
    /** height in pixels. */
    int _image_Height;
    /** number of bytes per pixels. */
    int _image_Depth;
    /** decoded color map datas. */
    byte* _image_decodedColormap;
    /** uncompressed image datas. */
    byte* _image_uncompressedImageData;
    /** unmapped image datas. */
    byte* _image_unmappedImageData;
    /** decoded image datas. */
    byte* _image_uncodedImageData;
    /** color mapping is used ? */
    int _image_colormapped;
    /** is rle compression used ? */
    int _image_rlecompressed;
    /** in what tga format (original or new) is the file. */
    int _tga_FileFormat;
    /** size of the image ID. */
    long _tga_imageIDSize;
    /** size of the color map. */
    long _tga_colormapSize;
    /** size of the image datas. */
    long _tga_imageDataSize;
  protected:
    /*!
      \brief constructor.
      \param fname const char*, the name of the file.
      \param flags byte, flags specifying opening mode.
    */
    TgaFile(const char* fname,byte flags = 0);
    /*!
      \brief destructor.
    */
    ~TgaFile();
    /*!
      \fn void initializeFile(const char* fname,byte flags = 0)
      \brief initialization function.
      \param fname const char*, the name of the file.
      \param flags byte, flags specifying opening mode.
    */
    void initializeFile(const char* fname,byte flags = 0);
    /*!
      \fn void resetFile()
      \brief reset function.
     */
    void resetFile();
  }; /* class TgaFile */

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGA_TGAFILE_H_ */
