#ifndef _TGA_TGAFILEINPUT_H_
#define _TGA_TGAFILEINPUT_H_

#include "Tga/includes.h"
#include "Tga/Enumeration.h"
#include "Tga/Structures.h"
#include "Tga/TgaFile.h"

#ifdef USENAMESPACE
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \class TgaFileInput
    \brief class for tga files reading.
   */
  class TgaFileInput :
    public TgaFile
    {
    private:
      /** decodes color map data. such as there is one byte per color component. */
      void decodeColorMap();
      /** uncompress image data. */
      void uncompressImage();
      /** unmap image datas (from indexed to true-color). */
      void unmapImage();
      /** decodes image data. such as there is one byte per color component. */
      void decodeImage();
      /** decodes tga file header. */
      void decodeHeader();
      /** decodes tga file datas. */
      void decodeData();
    public:
      /*!
	\brief constructor.
	\param fname const char*, the name of the file.
	\param flags byte, flags specifying opening mode.
      */
      TgaFileInput(const char* fname,byte flags = 0);
      /*!
	\brief destructor.
      */
      ~TgaFileInput();
      /*!
	\fn void loadFile()
	\brief loads the file datas.
      */
      void loadFile();
      /*!
	\fn void decodeFile()
	\brief decodes the file datas.
      */
      void decodeFile();
    public:
      //@{
      /** access methods to the file datas. */
      inline byte* getColormapData() { return _image_decodedColormap; };
      inline byte* getColormappedData() { return _image_uncompressedImageData; };
      inline byte* getCompressedData() { return _tga_ImageData; };
      inline byte* getImageData() { return _image_uncodedImageData; };
      //@}
      //@{
      /** access methods to the file attributes. */
      inline int getWidth() { return _image_Width; };
      inline int getHeight() { return _image_Height; };
      inline int getDepth() { return _image_Depth; };
      inline int isColormapped() { return _image_colormapped; };
      inline int isCompressed() { return _image_rlecompressed; };
      //@}
    }; /* class TgaFileInput */

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGA_TGAFILEINPUT_H_ */
