#ifndef _TGA_TGAFILEOUTPUT_H_
#define _TGA_TGAFILEOUTPUT_H_

#include "Tga/includes.h"
#include "Tga/Enumeration.h"
#include "Tga/Structures.h"
#include "Tga/TgaFile.h"

#ifdef USENAMESPACE
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \class TgaFileOutput
    \brief class for tga files saving.
   */
  class TgaFileOutput :
    public TgaFile
    {
    private:
       /** encodes color map data. */
      void encodeColorMap();
      /** compress image data. */
      void compressImage();
      /** map image datas (from true-color to indexed). */
      void mapImage();
      /** encodes image data. */
      void encodeImage();
      /** encodes tga file header. */
      void encodeHeader();
    public:
      /*!
	\brief constructor.
	\param fname const char*, the name of the file.
	\param flags byte, flags specifying opening mode.
      */
      TgaFileOutput(const char* fname,byte flags = 0);
      /*!
	\brief destructor.
      */
      ~TgaFileOutput();
      /*!
	\fn void saveFile()
	\brief saves the file datas.
      */
      void saveFile();
      /*!
	\fn void encodeFile()
	\brief encodes the file datas.
      */
      void encodeFile();
    public:
      //@{
      /** initialization methods for the file datas. */
      void setColormapData(const byte*,int);
      void setColormappedData(const byte*);
      void setImageData(const byte*);
      //@}
      //@{
      /** initialization methods for the file attributes. */
      inline void setWidth(const int ww) { _image_Width = ww; };
      inline void setHeight(const int hh) { _image_Height = hh; };
      inline void setDepth(const int dd) { _image_Depth = dd; };
      inline void setColormapped() { _image_colormapped = 0; /* pas gere pour le moment */ };
      inline void unsetColormapped() { _image_colormapped = 0; };
      inline void setCompressed() { _image_rlecompressed = 0; /* pas gere pour le moment */ };
      inline void unsetCompressed() { _image_rlecompressed = 0; };
      //@}
    }; /* class TgaFileOutput */

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGA_TGAFILEOUTPUT_H_ */
