#ifndef _TGAFILE_INCLUDES_H_
#define _TGAFILE_INCLUDES_H_

#include <string>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>

/* pour la bibliotheque utilitaires */

#include <Utility/Includes.h>
#include <Utility/BinaryCoding.h>
#include <Utility/Mode.h>
#include <Utility/Debugger.h>
#include <Utility/Memory.h>
#include <Utility/Statistic.h>
#include <Utility/CounterStatistic.h>
#include <Utility/Utility.h>

#ifdef USENAMESPACE
using namespace DebugManager;
using namespace MemoryManager;
using namespace CallingStatisticManager;
using namespace CounterStatisticManager;
using namespace BinaryCoder;
using namespace Serializer;
using namespace ErrorCode;
#endif /* #ifdef USENAMESPACE */

/* pour la bibliotheque rle */

#include <Rle/includes.h>
#include <Rle/Enumeration.h>
#include <Rle/Exception.h>
#include <Rle/RLE.h>

#ifdef USENAMESPACE
using namespace Rle;
#endif /* #ifdef USENAMESPACE */

#ifdef USENAMESPACE
/*!
  \namespace Tga
  \brief namespace contaning utility functions and datas for tga file manipulation.
*/
namespace Tga
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \def DECODESHORT(v)
    \brief returns the value of a short (encoded in an array).
  */
#define DECODESHORT(v) ( \
 ((v)[1] << 8) + (v)[0] \
)
  /*!
    \def DECODELONG(v)
    \brief returns the value of a long (encoded in an array).
  */
#define DECODELONG(v) ( \
 ((((((v)[3] << 8) + (v)[2]) << 8) + (v)[1]) << 8) + (v)[0] \
)
  /*!
    \def ENCODESHORT(f,v)
    \brief encodes the value of a short in an array.
  */
#define ENCODESHORT(f,v) { \
 (f)[1] = ((v) >> 8); (f)[0] = ((v) - 256 * (f)[1]); \
}
  /*!
    \brief defines the type 'byte'.
  */
  typedef unsigned char byte;

#ifdef USENAMESPACE
} /* namespace Tga */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _TGAFILE_INCLUDES_H_ */
