#ifndef _UTILITY_INCLUDES_H_
#define _UTILITY_INCLUDES_H_

#include <iostream>
#include <iomanip>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <cmath>
#include <procbuf.h>

#include <time.h>
#include <sys/timeb.h>

#include <map>
#include <stack>

using namespace std;

#ifndef NULL
/*! \def NULL
 * defines the undefined value.
 */
#define NULL 0
#endif

#ifndef INFINITY
/*! \def INFINITY
 * defines an infinite value
 */
#define INFINITY 1e20
#endif

/*! \enum eOnOff
 * enumeration for On/Off switches.
 */
typedef enum
  {
    OFF = 0 , /**< Off value. */
    ON = 1 /**< On value. */
  } eOnOff;

/*! \def isON(v)
 * macro to test if a switch is On.
 */
#define isON(v) ((v) == ON)
/*! \def isOFF(v)
 * macro to test if a switch is Off.
 */
#define isOFF(v) ((v) == OFF)

/*! \def verySmallEpsilonValue
 * defines a very very small epsilon value
 */
#define verySmallEpsilonValue 1.e-30
/*! \def smallEpsilonValue
 * defines a very small epsilon value
 */
#define smallEpsilonValue 0.00000001
/*! \def epsilonValue
 * defines a small epsilon value
 */
#define epsilonValue 0.00001
/*! \def largeValue
 * defines a large value
 */
#define largeValue 1e20
/*! \def hugeValue
 * defines a huge value
 */
#define hugeValue 1e30

/*! \def ABS(a)
 * macro returning the absolute value of a number.
 */
#define ABS(a)          (((a)<0) ? -(a) : (a))

/*! \def MIN(a,b)
 * macro returning the minimum of two values.
 */
#define MIN(a,b) ((a) < (b) ? (a) : (b))

#define MAX(a,b) ( \
  (a) < (b) ? (b) : (a) \
)
#define SQR(v) ( \
  (v) * (v) \
)

/*! \def CLAMP(a,b1,b2)
 * macro that clamps a value within an interval.
 */
#define CLAMP(a,b1,b2) (a) =  ((a) < (b1) ? (b1) : ((a) > (b2) ? (b2) : (a)))

/*! \def sqr(v)
 * macro returning the square of a number.
 */
#define sqr(v) ((v)*(v))

#ifdef USENAMESPACE
/*! \namespace ErrorCode
 * Namespace containing Error codes.
 */
namespace ErrorCode
{
#endif /* #ifdef USENAMESPACE */
  /*! \def NOERROR
   * No error code.
   */
#define NOERROR 0
  /*! \def MEMORYERROR
   * Memory management error code.
   */
#define MEMORYERROR 100
  /*! \def ARRAYTOOSMALL
   * Out of range error code.
   */
#define ARRAYTOOSMALL 101
  /*! \def FILEERROR
   * File Management error code.
   */
#define FILEERROR 200
  /*! \def FILEDOESNOTEXIST
   * Inexistant file error code.
   */
#define FILEDOESNOTEXIST 201
#ifdef USENAMESPACE
} /* namespace ErrorCode */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _UTILITY_INCLUDES_H_ */
