#ifndef _UTILITY_STATISTIC_H_
#define _UTILITY_STATISTIC_H_

#include "Utility/Includes.h"
#include "Utility/Mode.h"

#ifdef USENAMESPACE
/*! \namespace CallingStatisticManager
 * Namespace containing time and calling statistics.
 */
namespace CallingStatisticManager
{
#endif /* #ifdef USENAMESPACE */

  /*! \def MaxCallingStack
   * maximum number of recursive calls.
   */
#define MaxCallingStack 100
  /*! \class CallingStat
   * \brief class to register calling statistics.
   */
  class CallingStat
  {
  private:
    /*!
     * stack of computed time to manage recursive functions.
     */
    timeb _stack[MaxCallingStack];
    /*!
     * index of filling in the stack.
     */
    int _stackfilling;
    /*!
     * name of the statistic information.
     */
    string _name;
    /*!
     * name of the file.
     */
    string _filename;
    /*!
     * name of the function.
     */
    string _functionname;
    /*!
     * line number in the file.
     */
    int _linenumber;
    /*!
     * number of call to this function.
     */
    unsigned long _numberofcall;
    /*!
     * maximum time of a call.
     */
    double _maximumtime;
    /*!
     * minimum time of a call.
     */
    double _minimumtime;
    /*!
     * total time of all calls.
     */
    double _totaltime;
  protected:
  public:
    /*!
     * CallingStat initialization constructor.
     * \param stat string, the name of the variable.
     * \param Fname string, name of the file.
     * \param fname string, name of the function.
     * \param line int, line number.
     */
    CallingStat(string stat,string Fname,string fname,int line);
    /*!
     * CallingStat destructor.
     */
    ~CallingStat();
    /*!
     * CallingStat timer start function.
     */
    void start();
    /*!
     * CallingStat timer stop function.
     */
    void stop();
    /*!
     * CallingStat dumping function.
     */
    static void callingStats();
  };
  /*! \var map< string , CallingStat* > __callingstatistics
   * map containing all the registered statistic informations.
   */
  extern map< string , CallingStat* > __callingstatistics;
  /*! \typedef  map < string , CallingStat* >::const_iterator __callingstatisticsIterator
   * iterator on the map containing all the registered statistic informations.
   */
  typedef  map < string , CallingStat* >::const_iterator __callingstatisticsIterator;
  /*! \def BEGINSTAT
   * Macro to initialize the start of a time/call statistic.
   */
#define BEGINSTAT(f) \
static CallingStat* __f = NULL; \
if ((isON(getTimeStatisticMode())) || (isON(getCallStatisticMode()))) { \
if (__f == NULL) __f = new CallingStat(#f,__FILE__,__FUNCTION__,__LINE__); \
__f->start(); \
}
  /*! \def ENDSTAT
   * Macro to stop a time/call statistic.
   */
#define ENDSTAT(f) \
if ((isON(getTimeStatisticMode())) || (isON(getCallStatisticMode()))) { \
__f->stop(); \
}

#ifdef USENAMESPACE
} /* namespace CallingStatisticManager */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _UTILITY_STATISTIC_H_ */
