#ifndef _WAVELET_GAUSS_H_
#define _WAVELET_GAUSS_H_

#include "Wavelet/Includes.h"

extern sWaveletData gauss_quadrature_point[4][4];
extern sWaveletData gauss_quadrature_weight[4][4];

/*
  Gauss-Legendre Quadrature to get Projection coefficients of
  functions on Multiwavelet basis functions of order m
  and vanishing moment M and resolution j.
  Lowest resolution is at root with j=0.
*/

/*
  f[] is the projected function evaluated
  at M Gauss points.
  M is the Vanishing moment.
  m is the order of the basis function.
  J is the wavelet resolution. J=0 for lowest resolution.
*/
extern sWaveletData projection_coeff_1v(int M,int m, int J,sWaveletData *f);

/*
  f[] is the projected function evaluated
  at M^2 Gauss points.
  M is the Vanishing moment.
  m,n are the order of the basis functions along
  variable u and v respectively.
  J is the wavelet resolution. J=0 for lowest resolution.
*/
extern sWaveletData projection_coeff_2v(int M,int m,int n, int J,sWaveletData *f);

/*
  f[] is the projected function evaluated
  at M^4 Gauss points.
  M is the Vanishing moment.
  m,n,o,p are the order of the basis functions along
  u,v,s and t respectively.
  J_a & J_b is the wavelet resolution at patch a
  and b respectively. 0 being lowest resolution.
*/
extern sWaveletData projection_coeff_4v(int M,int m,int n,int o, int p, int J_a,int J_b,sWaveletData *f);

#endif
