#include "Spectrum/Converter.h"

#include "Spectrum/Lambda.h"
#include "Spectrum/CIE.h"

/* this code was taken from the MGF documentation */

#ifdef  NTSC
#define  CIE_x_r                0.670           /* standard NTSC primaries */
#define  CIE_y_r                0.330
#define  CIE_x_g                0.210
#define  CIE_y_g                0.710
#define  CIE_x_b                0.140
#define  CIE_y_b                0.080
#define  CIE_x_w                0.3101          /* monitor white point */
#define  CIE_y_w                0.3162
#else
#ifdef EBU
#define  CIE_x_r                0.640           /* nominal EBU primaries */
#define  CIE_y_r                0.330
#define  CIE_x_g                0.290
#define  CIE_y_g                0.600
#define  CIE_x_b                0.150
#define  CIE_y_b                0.060
#define  CIE_x_w                0.3127          /* monitor white point */
#define  CIE_y_w                0.3291
#else
#ifdef SMPTE
#define  CIE_x_r                0.630           /* nominal SMPTE primaries */
#define  CIE_y_r                0.340
#define  CIE_x_g                0.310
#define  CIE_y_g                0.595
#define  CIE_x_b                0.155
#define  CIE_y_b                0.070
#define  CIE_x_w                0.3127          /* monitor white point */
#define  CIE_y_w                0.3291
#else
#define  CIE_x_r                0.640           /* nominal CRT primaries */
#define  CIE_y_r                0.330
#define  CIE_x_g                0.290
#define  CIE_y_g                0.600
#define  CIE_x_b                0.150
#define  CIE_y_b                0.060
#define  CIE_x_w                0.3333333333          /* monitor white point */
#define  CIE_y_w                0.3333333333
#endif
#endif
#endif

#define CIE_D           (       CIE_x_r*(CIE_y_g - CIE_y_b) + \
                                CIE_x_g*(CIE_y_b - CIE_y_r) + \
                                CIE_x_b*(CIE_y_r - CIE_y_g)     )
#define CIE_C_rD        ( (1./CIE_y_w) * \
                                ( CIE_x_w*(CIE_y_g - CIE_y_b) - \
                                  CIE_y_w*(CIE_x_g - CIE_x_b) + \
                                  CIE_x_g*CIE_y_b - CIE_x_b*CIE_y_g     ) )
#define CIE_C_gD        ( (1./CIE_y_w) * \
                                ( CIE_x_w*(CIE_y_b - CIE_y_r) - \
                                  CIE_y_w*(CIE_x_b - CIE_x_r) - \
                                  CIE_x_r*CIE_y_b + CIE_x_b*CIE_y_r     ) )
#define CIE_C_bD        ( (1./CIE_y_w) * \
                                ( CIE_x_w*(CIE_y_r - CIE_y_g) - \
                                  CIE_y_w*(CIE_x_r - CIE_x_g) + \
                                  CIE_x_r*CIE_y_g - CIE_x_g*CIE_y_r     ) )

#define CIE_rf          (CIE_y_r*CIE_C_rD/CIE_D)
#define CIE_gf          (CIE_y_g*CIE_C_gD/CIE_D)
#define CIE_bf          (CIE_y_b*CIE_C_bD/CIE_D)

sSpectrumPhotometric  XYZRVB[3][3] = {     /* XYZ to RGB */
        {(CIE_y_g - CIE_y_b - CIE_x_b*CIE_y_g + CIE_y_b*CIE_x_g)/CIE_C_rD,
         (CIE_x_b - CIE_x_g - CIE_x_b*CIE_y_g + CIE_x_g*CIE_y_b)/CIE_C_rD,
         (CIE_x_g*CIE_y_b - CIE_x_b*CIE_y_g)/CIE_C_rD},
        {(CIE_y_b - CIE_y_r - CIE_y_b*CIE_x_r + CIE_y_r*CIE_x_b)/CIE_C_gD,
         (CIE_x_r - CIE_x_b - CIE_x_r*CIE_y_b + CIE_x_b*CIE_y_r)/CIE_C_gD,
         (CIE_x_b*CIE_y_r - CIE_x_r*CIE_y_b)/CIE_C_gD},
        {(CIE_y_r - CIE_y_g - CIE_y_r*CIE_x_g + CIE_y_g*CIE_x_r)/CIE_C_bD,
         (CIE_x_g - CIE_x_r - CIE_x_g*CIE_y_r + CIE_x_r*CIE_y_g)/CIE_C_bD,
         (CIE_x_r*CIE_y_g - CIE_x_g*CIE_y_r)/CIE_C_bD}
};

sSpectrumPhotometric  RVBXYZ[3][3] = {     /* RGB to XYZ */
        {CIE_x_r*CIE_C_rD/CIE_D,CIE_x_g*CIE_C_gD/CIE_D,CIE_x_b*CIE_C_bD/CIE_D},
        {CIE_y_r*CIE_C_rD/CIE_D,CIE_y_g*CIE_C_gD/CIE_D,CIE_y_b*CIE_C_bD/CIE_D},
        {(1.-CIE_x_r-CIE_y_r)*CIE_C_rD/CIE_D,
         (1.-CIE_x_g-CIE_y_g)*CIE_C_gD/CIE_D,
         (1.-CIE_x_b-CIE_y_b)*CIE_C_bD/CIE_D}
};

sSpectrumPhotometric  RVBYIQ[3][3] = {
  {  0.299 , 0.587 , 0.114 } ,
  {  0.596 , -0.275 , -0.321 } ,
  {  0.212 , -0.523 , 0.311 }
};

sSpectrumPhotometric  YIQRVB[3][3] = {
  {  1.0 , 0.956 , 0.621 } ,
  {  1.0 , -0.272 , -0.647 } ,
  {  1.0 , -1.105 , 1.702 }
};

/*
  fonctions de manipulation des spectres reduits
*/

sSpectrumPhotometric gammaCorrectionFactor = 2.3;

void clampComponent(sSpectrumPhotometric& r, sSpectrumPhotometric& g, sSpectrumPhotometric& b)
{
  if(r > 1.0) r = 1; else if (r < 0.) r = 0.;
  if(g > 1.0) g = 1; else if (g < 0.) g = 0.;
  if(b > 1.0) b = 1; else if (b < 0.) b = 0.;
}

void gammaCorrection(sSpectrumPhotometric& R, sSpectrumPhotometric& V, sSpectrumPhotometric& B)
{
  R = pow(R,1. / gammaCorrectionFactor);
  V = pow(V,1. / gammaCorrectionFactor);
  B = pow(B,1. / gammaCorrectionFactor);
}

/*
  colorspace conversion
*/

void XYZtoRGB(sSpectrumPhotometric X, sSpectrumPhotometric Y, sSpectrumPhotometric Z, sSpectrumPhotometric& R, sSpectrumPhotometric& V, sSpectrumPhotometric& B)
{
  R = (X * XYZRVB[0][0]) + (Y * XYZRVB[0][1]) + (Z * XYZRVB[0][2]);
  V = (X * XYZRVB[1][0]) + (Y * XYZRVB[1][1]) + (Z * XYZRVB[1][2]);
  B = (X * XYZRVB[2][0]) + (Y * XYZRVB[2][1]) + (Z * XYZRVB[2][2]);
}

void RGBtoXYZ(sSpectrumPhotometric R, sSpectrumPhotometric V, sSpectrumPhotometric B, sSpectrumPhotometric& X, sSpectrumPhotometric& Y, sSpectrumPhotometric& Z)
{
  X = (R * RVBXYZ[0][0]) + (V * RVBXYZ[0][1]) + (B * RVBXYZ[0][2]);
  Y = (R * RVBXYZ[1][0]) + (V * RVBXYZ[1][1]) + (B * RVBXYZ[1][2]);
  Z = (R * RVBXYZ[2][0]) + (V * RVBXYZ[2][1]) + (B * RVBXYZ[2][2]);
}

void XYZtoLAB(sSpectrumPhotometric X, sSpectrumPhotometric Y, sSpectrumPhotometric Z, sSpectrumPhotometric& L, sSpectrumPhotometric& a, sSpectrumPhotometric& b)
{
}

void LABtoXYZ(sSpectrumPhotometric L, sSpectrumPhotometric a, sSpectrumPhotometric b, sSpectrumPhotometric& X, sSpectrumPhotometric& Y, sSpectrumPhotometric& Z)
{
}

void YIQtoRGB(sSpectrumPhotometric Y, sSpectrumPhotometric I, sSpectrumPhotometric Q, sSpectrumPhotometric& R, sSpectrumPhotometric& V, sSpectrumPhotometric& B)
{
  R = (Y * YIQRVB[0][0]) + (I * YIQRVB[0][1]) + (Q * YIQRVB[0][2]);
  V = (Y * YIQRVB[1][0]) + (I * YIQRVB[1][1]) + (Q * YIQRVB[1][2]);
  B = (Y * YIQRVB[2][0]) + (I * YIQRVB[2][1]) + (Q * YIQRVB[2][2]);
}

void RGBtoYIQ(sSpectrumPhotometric R, sSpectrumPhotometric V, sSpectrumPhotometric B, sSpectrumPhotometric& Y, sSpectrumPhotometric& I, sSpectrumPhotometric& Q)
{
  Y = (R * RVBYIQ[0][0]) + (V * RVBYIQ[0][1]) + (B * RVBYIQ[0][2]);
  I = (R * RVBYIQ[1][0]) + (V * RVBYIQ[1][1]) + (B * RVBYIQ[1][2]);
  Q = (R * RVBYIQ[2][0]) + (V * RVBYIQ[2][1]) + (B * RVBYIQ[2][2]);
}

void HSVtoRGB(sSpectrumPhotometric H,sSpectrumPhotometric S,sSpectrumPhotometric V,sSpectrumPhotometric& R,sSpectrumPhotometric& G,sSpectrumPhotometric& B)
{
  if (S == 0.0) {
    R = V;        // gray pixel
    G = V;
    B = V;
    return;
  }
  if (H == 1.0) H = 0.0;
  H *= 6;
  int index = int(H);
  sSpectrumPhotometric frac = H - index;
  sSpectrumPhotometric p = V * (1.0 - S);
  sSpectrumPhotometric q = V * (1.0 - (S*frac));
  sSpectrumPhotometric t = V * (1.0 - (S * (1.0-frac)));
  // break into cases based on the sextant of the color wheel
  switch (index) {
  case 0:
    R = V;
    G = t;
    B = p;
    break;
  case 1:
    R = q;
    G = V;
    B = p;
    break;
  case 2:
    R = p;
    G = V;
    B = t;
    break;
  case 3:
    R = p;
    G = q;
    B = V;
    break;
  case 4:
    R = t;
    G = p;
    B = V;
    break;
  case 5:
    R = V;
    G = p;
    B = q;
    break;
  }
}

void RGBtoHSV(sSpectrumPhotometric R, sSpectrumPhotometric G, sSpectrumPhotometric B, sSpectrumPhotometric& H, sSpectrumPhotometric& S, sSpectrumPhotometric& V)
{
  sSpectrumPhotometric min = MIN(R,MIN(G,B));
  sSpectrumPhotometric max = MAX(R,MAX(G,B));
  V = max;
  sSpectrumPhotometric delta = max - min;
  if (max != 0.) S = delta / max;
  else {
    S = 0.;
    H = -1.;
    return;
  }
  if (R == max) H = (G - B) / delta;
  else if (G == max) H = 2. + (B - R) / delta;
  else H = 4 + (R - G) / delta;
  H *= 60.;
  if (H < 0.) H += 360.;
}

void HLStoRGB(sSpectrumPhotometric H,sSpectrumPhotometric L,sSpectrumPhotometric S,sSpectrumPhotometric& R,sSpectrumPhotometric& G,sSpectrumPhotometric& B)
{
}

void RGBtoHLS(sSpectrumPhotometric R, sSpectrumPhotometric G, sSpectrumPhotometric B, sSpectrumPhotometric& H, sSpectrumPhotometric& L, sSpectrumPhotometric& S)
{
  double max = MAX(R,MAX(G,B));
  double min = MIN(R,MIN(G,B));
  L = (max+min)/2;
  if (max == min) {
    S = 0;
    H = -1.;
  } else {
    if (L <= 0.5) S = (max-min)/(max+min);
    else S = (max-min)/(2-max-min);
    double delta = max-min;
    if (R == max) H = (G-B)/delta;
    else if (G == max) H = 2 + (B-R)/delta;
    else if (B == max) H = 4 + (R-G)/delta;
    H = H*60;
    if (H < 0.0) H = H + 360;
  }
}

void XYZtoXY(sSpectrumPhotometric X, sSpectrumPhotometric Y, sSpectrumPhotometric Z, sSpectrumPhotometric& x, sSpectrumPhotometric& y)
{
  sSpectrumPhotometric S = X + Y + Z;
  x = X / S;
  y = Y / S;
}

void CYMKtoRGB(sSpectrumPhotometric C,sSpectrumPhotometric M,sSpectrumPhotometric Y,sSpectrumPhotometric K,sSpectrumPhotometric& R,sSpectrumPhotometric& V,sSpectrumPhotometric& B)
{
  float brightness = 1.0 - K;
  R = (brightness * (1.0 - C)) + 0.5;
  V = (brightness * (1.0 - M)) + 0.5;
  B = (brightness * (1.0 - Y)) + 0.5;
}

void CYMtoRGB(sSpectrumPhotometric C,sSpectrumPhotometric M,sSpectrumPhotometric Y,sSpectrumPhotometric& R,sSpectrumPhotometric& V,sSpectrumPhotometric& B)
{
  R = 1.0 - C;
  V = 1.0 - M;
  B = 1.0 - Y;
}

void RGBtoCorrectedRGB(sSpectrumPhotometric& R, sSpectrumPhotometric& V, sSpectrumPhotometric& B)
{
  clampComponent(R,V,B);
  gammaCorrection(R,V,B);
}

typedef struct {
  sSpectrumPhotometric mirek;  /* temp (in microreciprocal kelvin) */
  sSpectrumPhotometric ut;     /* u coord of intersection w/ blackbody locus */
  sSpectrumPhotometric vt;     /* v coord of intersection w/ blackbody locus */
  sSpectrumPhotometric tt;     /* slope of isotemp. line */
} isotemp;
isotemp isotempdata[] = {
  /*  {Mirek, Ut,       Vt,      Tt      } */
  {0,     0.18006,  0.26352,  -0.24341},
  {10,    0.18066,  0.26589,  -0.25479},
  {20,    0.18133,  0.26846,  -0.26876},
  {30,    0.18208,  0.27119,  -0.28539},
  {40,    0.18293,  0.27407,  -0.30470},
  {50,    0.18388,  0.27709,  -0.32675},
  {60,    0.18494,  0.28021,  -0.35156},
  {70,    0.18611,  0.28342,  -0.37915},
  {80,    0.18740,  0.28668,  -0.40955},
  {90,    0.18880,  0.28997,  -0.44278},
  {100,   0.19032,  0.29326,  -0.47888},
  {125,   0.19462,  0.30141,  -0.58204},
  {150,   0.19962,  0.30921,  -0.70471},
  {175,   0.20525,  0.31647,  -0.84901},
  {200,   0.21142,  0.32312,  -1.0182 },
  {225,   0.21807,  0.32909,  -1.2168 },
  {250,   0.22511,  0.33439,  -1.4512 },
  {275,   0.23247,  0.33904,  -1.7298 },
  {300,   0.24010,  0.34308,  -2.0637 },
  {325,   0.24702,  0.34655,  -2.4681 },
  {350,   0.25591,  0.34951,  -2.9641 },
  {375,   0.26400,  0.35200,  -3.5814 },
  {400,   0.27218,  0.35407,  -4.3633 },
  {425,   0.28039,  0.35577,  -5.3762 },
  {450,   0.28863,  0.35714,  -6.7262 },
  {475,   0.29685,  0.35823,  -8.5955 },
  {500,   0.30505,  0.35907,  -11.324 },
  {525,   0.31320,  0.35968,  -15.628 },
  {550,   0.32129,  0.36011,  -23.325 },
  {575,   0.32931,  0.36038,  -40.770 },
  {600,   0.33724,  0.36051, -116.45  }
};
int niso = sizeof(isotempdata)/sizeof(isotemp);

void XYtoCCT(sSpectrumPhotometric X, sSpectrumPhotometric Y, sSpectrumPhotometric& CCT)
{
  int j;
  sSpectrumPhotometric us,vs;
  sSpectrumPhotometric uj,vj,tj,di,dj,mi,mj;
  sSpectrumPhotometric Tc;
  /* convert (x,y) to CIE 1960 (u,v) */
  us = (2*X) / (-X + 6*Y + 1.5);
  vs = (3*Y) / (-X + 6*Y + 1.5);
  /* search for closest isotemp lines */
  for (j=0; j<niso; j++) {
    uj = isotempdata[j].ut;
    vj = isotempdata[j].vt;
    tj = isotempdata[j].tt;
    mj = isotempdata[j].mirek;
    /* dj = distance from (us,vs) to this isotemp line */
    dj = ((vs - vj) - tj * (us - uj)) / sqrt(1 + tj*tj);
    /* we stop when (di/dj) < 0.0, i.e. when distance changes
       sign, because this means we have found isotemp lines
       * that "straddle" our point. */
    if ((j!=0) && (di/dj < 0.0)) {
      Tc = 1000000.0 / (mi + (di / (di - dj)) * (mj - mi));
      break;
    }
    di = dj;
    mi = mj;
  }
   if (j == niso) CCT = 0;
   else CCT = Tc;
}

/*
  spectra conversion
*/

sSpectrumPhotometric Km = 683.;

void computeKmFactor(sSpectrumPhotometric Kmax,int night)
{
  Km = Kmax / SPECTRUMintegrate(night ? scotopicLuminousEfficiency : photopicLuminousEfficiency);
}

void SPECTRUMtoXYZ(sDetailledSpectrum& SpectreIn, sSpectrumPhotometric& X, sSpectrumPhotometric& Y, sSpectrumPhotometric& Z,int small)
{
  sDetailledSpectrum* tmp;
  X = 0.;
  tmp = (small ? &observerX2 : &observerX10);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    X += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
  Y = 0.;
  tmp = (small ? &observerY2 : &observerY10);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    Y += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
  Z = 0.;
  tmp = (small ? &observerZ2 : &observerZ10);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    Z += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
}

void SPECTRUMtoXYZ(sSampledSpectrum& SpectreIn, sSpectrumPhotometric& X, sSpectrumPhotometric& Y, sSpectrumPhotometric& Z,int small)
{
  sDetailledSpectrum* tmp;
  X = 0.;
  tmp = (small ? &observerX2 : &observerX10);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    X += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
  Y = 0.;
  tmp = (small ? &observerY2 : &observerY10);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    Y += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
  Z = 0.;
  tmp = (small ? &observerZ2 : &observerZ10);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    Z += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
}

void SPECTRUMtoRADIANCE(sSampledSpectrum& SpectreIn, sSpectrumPhotometric& L1)
{
  L1 = 0.0;
  for (int i=1;i < photopicLuminousEfficiency._filledSize;i++) {
    sSpectrumPhotometric l0 = photopicLuminousEfficiency._lambda[i-1]; sSpectrumPhotometric v0 = photopicLuminousEfficiency._data[i-1];
    sSpectrumPhotometric l1 = photopicLuminousEfficiency._lambda[i];     sSpectrumPhotometric v1 = photopicLuminousEfficiency._data[i];
    L1 += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
}

sSpectrumPhotometric SPECTRUMintegrate(sDetailledSpectrum& SpectreIn)
{
  sSpectrumPhotometric I=0.0;
  for (int i=1;i<SpectreIn._filledSize;i++) {
    sSpectrumPhotometric l0 = SpectreIn._lambda[i-1];
    sSpectrumPhotometric l1 = SpectreIn._lambda[i];
    sSpectrumPhotometric v0 = SpectreIn._data[i-1];
    sSpectrumPhotometric v1 = SpectreIn._data[i];
    I += (l1 - l0) * (v0 + v1) / 2.;
  }
  return I;
}

sSpectrumPhotometric SPECTRUMintegrate(sSampledSpectrum& SpectreIn)
{
  sSpectrumPhotometric I=0.0;
  for (int i=1;i<sampledSpectrumSize;i++) {
    sSpectrumPhotometric l0 = sampledSpectrum[i-1];
    sSpectrumPhotometric l1 = sampledSpectrum[i];
    sSpectrumPhotometric v0 = SpectreIn._data[i-1];
    sSpectrumPhotometric v1 = SpectreIn._data[i];
    I += (l1 - l0) * (v0 + v1) / 2.;
  }
  return I;
}

sSpectrumPhotometric SPECTRUMradiance(sDetailledSpectrum& SpectreIn,int night)
{
  sDetailledSpectrum* tmp;
  sSpectrumPhotometric Y=0.0;
  tmp = (night ? &scotopicLuminousEfficiency : &photopicLuminousEfficiency);
  for (int i=1;i < (*tmp)._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    Y += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
  return Y;
}

sSpectrumPhotometric SPECTRUMradiance(sSampledSpectrum& SpectreIn,int night)
{
  sDetailledSpectrum* tmp;
  sSpectrumPhotometric Y=0.0;
  tmp = (night ? &scotopicLuminousEfficiency : &photopicLuminousEfficiency);
  for (int i=1;i < observerY10._filledSize;i++) {
    sSpectrumPhotometric l0 = (*tmp)._lambda[i-1]; sSpectrumPhotometric v0 = (*tmp)._data[i-1];
    sSpectrumPhotometric l1 = (*tmp)._lambda[i];     sSpectrumPhotometric v1 = (*tmp)._data[i];
    Y += Km * (l1 - l0) * (v0 * SpectreIn(l0) + v1 * SpectreIn(l1)) / 2.;
  }
  return Y;
}
