#include "SelectiveFilter.h"

#include "Spectral.h"


/**************************************************************/
/* Ce filtre est un filtre qui prserve les contours en       */
/* s'adaptant aux drives des points autour du point en      */
/* traitement. pdest et psrc doivent avoir la mme taille.    */
void SelectiveFilter::selective_filter(const ImageRGB &psrc, ImageRGB &pdest,bool color,unsigned int param){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = .72 * sqr(param);// on ajuste sigma en fonction de la taille voulue 
				// on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
			        // gauss(x) = exp(- (x*x / sigma)) 
				// gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

//   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y,w,h;
  float res_r, res_g, res_b;
//   pix vpix,cpix;
  ImageRGBPixel vpix,cpix;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

//   float filter_red  [3][3];
//   float filter_green[3][3];
//   float filter_blue [3][3];

  float *filter_red	= new float[sqr(taille)];
  float *filter_green	= new float[sqr(taille)];
  float *filter_blue	= new float[sqr(taille)];

  float *gauss	= new float[sqr(taille)];

  float filter_red_sum, filter_green_sum, filter_blue_sum;
  
  /* si les prconditions ne sont pas remplies, on quitte */
//   if((pdest == NULL) || (psrc == NULL) ||
//      (psrc->width != pdest->width) || (psrc->height != pdest->height))
//    return;
  

  h = psrc.Width();	// c'est normal, tout est inverse
  w = psrc.Height();

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
        gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	  exp(-((sqr(vx) + sqr(vy)) / sigma));
    }
  }

  if(color){
    //cout << "filtre couleur" << endl;
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	  filter_red_sum = filter_green_sum = filter_blue_sum = 0.0;
	  /* recupere le point que l'on traite */
	  cpix = psrc[vx][vy];
	  /* fabrique la matrice de convolution en fonction des variations */
	  /* de l'intensit (fabrique le filtre local)                     */
	  for(j = 0;j < taille;j++){
	    for(i = 0;i < taille;i++){
	      x = vx - param + i;
	      y = vy - param + j;
	      if((x >= 0) && (x < w) && (y >= 0) && (y < h)){
		if((j == param) && (i == param)){
		    filter_red  [centre] = 0.5;
		    filter_green[centre] = 0.5;
		    filter_blue [centre] = 0.5;
		}
		else {
		    vpix = psrc[x][y];
		    
		    res_r = cpix[0] - vpix[0];
		    if(res_r < 0.0) res_r = -res_r;
		    if(res_r != 0)
		      filter_red  [j * taille + i] = 1.0/res_r;
		    else
		      filter_red  [j * taille + i] = 1.0;
		    
		    res_g = cpix[1] - vpix[1];
		    if(res_g < 0.0) res_g = -res_g;
		    if(res_g != 0)
		      filter_green[j * taille + i] = 1.0/res_g;
		    else
		      filter_green[j * taille + i] = 1.0;
		    
		    res_b = cpix[2] - vpix[2];
		    if(res_b < 0.0) res_b = -res_b;
		    if(res_b != 0)
		      filter_blue [j * taille + i] = 1.0/res_b;
		    else
		      filter_blue [j * taille + i] = 1.0;
		}
		filter_red_sum   += filter_red  [j * taille + i];
		filter_green_sum += filter_green[j * taille + i];
		filter_blue_sum  += filter_blue [j * taille + i];
	      }
	    }
	  }
	  for(j=0;j<taille;j++){
	    for(i=0;i<taille;i++){
	       if(filter_red_sum != 0.0)
		 filter_red  [j * taille + i] /= filter_red_sum;
	       if(filter_green_sum != 0.0)
		 filter_green[j * taille + i] /= filter_green_sum;
	       if(filter_blue_sum != 0.0)
		 filter_blue [j * taille + i] /= filter_blue_sum;
	    }
	  }
	 /* on applique le filtre calcul */
	  res_r = 0.0;
	  res_g = 0.0;
	  res_b = 0.0;
	  for(j=0;j<taille;j++){
	    for(i=0;i<taille;i++){
		vpix = psrc[vx - param + i][vy - param + j];
		res_r += vpix[0] * filter_red  [j * taille + i];
		res_g += vpix[1] * filter_green[j * taille + i];
		res_b += vpix[2] * filter_blue [j * taille + i];
	    }
	  }

	  /* on sature le rsultat */
	  unsigned char tab[3];
	  tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	  tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
	  /* on place le rsultat dans l'image destination */      
// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	  pdest[vx][vy] = ImageRGBPixel(tab,3);
      }
    }
  }
  
  else{
    //cout << "filtre gris" << endl;
  for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	  filter_red_sum = 0.0;
	  /* recupere le point que l'on traite */
	  cpix =psrc[vx][vy];
	  /* fabrique la matrice de convolution en fonction des variations */
	  /* de l'intensit (fabrique le filtre local)                     */
	  for(j = 0;j < taille;j++){
	    for(i = 0;i < taille;i++){
	      x = vx - param + i;
	      y = vy - param + j;
	      if((x >= 0) && (x < w) && (y >= 0) && (y < h)){
		if((j == param) && (i == param)){
		    filter_red  [centre] = 0.5;
		} 
		else {
		    vpix = psrc[x][y];
		    
		    res_r = cpix[0] - vpix[0];
		    if(res_r < 0.0) res_r = -res_r;
		    if(res_r != 0)
		      filter_red  [j * taille + i] = 1.0/res_r;
		    else
		      filter_red  [j * taille + i] = 1.0;
		    
		  }
		filter_red_sum   += filter_red  [j * taille + i];
	      }
	    }
	  }
	  for(j = 0;j<taille;j++){
	    for(i = 0;i<taille;i++){
	       if(filter_red_sum != 0.0)
		 filter_red  [j * taille + i] /= filter_red_sum;
	    }
	  }
	 /* on applique le filtre calcul */
	  res_r = 0.0;
	  for(j = 0;j<taille;j++){
	    for(i = 0;i<taille;i++){
		vpix = psrc[vx - param + i][vy - param + j];
		res_r += vpix[0] * filter_red  [j * taille + i];
	    }
	  }
	  
	  /* on sature le rsultat */
	  unsigned char tab[3];
	  tab[0] = tab[1] = tab[2] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  
	  /* on place le rsultat dans l'image destination */      
// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	  pdest[vx][vy] = ImageRGBPixel(tab,3);
	}
    }
  }
  delete filter_red;
  delete filter_green;
  delete filter_blue;
  delete gauss;

}


void SelectiveFilter::gradient(const ImageRGB &psrc, ImageRGB &pdest,bool color,float param){

//   int32 vx,vy;
  int vx,vy;
  int w,h;
  float res_r, res_g, res_b;

//   pix vpix,cpix;
  ImageRGBPixel cpixg,cpixd,cpixh,cpixb;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

//   float filter_red  [3][3];
//   float filter_green[3][3];
//   float filter_blue [3][3];

  w = psrc.Width();
  h = psrc.Height();

  if(color){
    //cout << "gradient couleur" << endl;
    for(vy = 1;vy < h - 1; vy++){
      for(vx = 1;vx < w - 1; vx++){
	  /* recupere le point que l'on traite */
	  cpixg = psrc[vy][vx - 1];
	  cpixd = psrc[vy][vx + 1];
	  cpixh = psrc[vy + 1][vx];
	  cpixb = psrc[vy - 1][vx];

	  res_r = param * sqrt(sqr(cpixg[0] - cpixd[0]) + sqr(cpixh[0] - cpixb[0]));
	  res_g = param * sqrt(sqr(cpixg[1] - cpixd[1]) + sqr(cpixh[1] - cpixb[1]));
	  res_b = param * sqrt(sqr(cpixg[2] - cpixd[2]) + sqr(cpixh[2] - cpixb[2]));

	  /* on sature le rsultat */
	  unsigned char tab[3];
	  tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	  tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
	  /* on place le rsultat dans l'image destination */      
// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	  pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
  
  else{
    //cout << "filtre gris" << endl;
    for(vy = 1;vy < h - 1; vy++){
      for(vx = 1;vx < w - 1; vx++){
	  /* recupere le point que l'on traite */
	  cpixg = psrc[vy][vx - 1];
	  cpixd = psrc[vy][vx + 1];
	  cpixh = psrc[vy + 1][vx];
	  cpixb = psrc[vy - 1][vx];

	  res_r = param * sqrt(sqr(cpixg[0] - cpixd[0]) + sqr(cpixh[0] - cpixb[0]));

	  /* on sature le rsultat */
	  unsigned char tab[3];
	  tab[0] = tab[1] = tab[2] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  
	  /* on place le rsultat dans l'image destination */      
// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	  pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
}

void SelectiveFilter::light_filter(const ImageRGB &psrc, ImageRGB &pdest,float param){

//   int32 vx,vy;
  int vx,vy;
  int w,h,taille;
  float res_r, res_g, res_b;

  w = psrc.Width();
  h = psrc.Height();
  taille = w * h;

//   pix vpix,cpix;
  ImageRGBPixel cpix1,cpix2,cpix3,cpix4;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *filter_h	= new float[sqr(taille - 1)];
  float *filter_v	= new float[sqr(taille - 1)];
  float *lum_obs       	= new float[sqr(taille)];
  float *lum_calc      	= new float[sqr(taille)];



  for(vy = 0;vy < h - 1; vy++){
    for(vx = 0;vx < w - 1; vx++){
      /* recupere le point que l'on traite */
      cpix1 = psrc[vy]	  [vx];
      cpix2 = psrc[vy]	  [vx + 1];
      cpix3 = psrc[vy + 1][vx];
      cpix4 = psrc[vy + 1][vx + 1];
      
      res_r = param * sqrt(sqr(cpix1[0] - cpix2[0]) + sqr(cpix3[0] - cpix4[0]));
      res_g = param * sqrt(sqr(cpix1[1] - cpix2[1]) + sqr(cpix3[1] - cpix4[1]));
      res_b = param * sqrt(sqr(cpix1[2] - cpix2[2]) + sqr(cpix3[2] - cpix4[2]));
      
      /* on sature le rsultat */
      unsigned char tab[3];
      tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
      tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
      tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
      /* on place le rsultat dans l'image destination */      
      // 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
      pdest[vy][vx] = ImageRGBPixel(tab,3);
    }
  }
  delete filter_h;
  delete filter_v;
  delete lum_obs;
  delete lum_calc;
}

void SelectiveFilter::aniso_diff(const ImageRGB &im,ImageRGB &d,float kappa,float lambda,int option){

  unsigned int rows,cols,x,y;
  unsigned long r_c;
  rows = im.Height();
  cols = im.Width();
  r_c  = rows * cols;

//   float cN,cS,cE,cW,deltaN,deltaS,deltaE,deltaW;
  
  for(x = 1;x < cols - 1;x++){
    for(y = 1;y < rows - 1;y++){

      ImageRGBPixel po,pn,ps,pe,pw;
      po = im[y]	[x];
      pn = im[y - 1]	[x];
      ps = im[y + 1]	[x];
      pw = im[y]	[x - 1];
      pe = im[y]	[x + 1];

      int por,pog,pob,pnr,png,pnb,psr,psg,psb,pwr,pwg,pwb,per,peg,peb;
      int param2 = 2;

      por = po[0] + param2;
      pog = po[1] + param2;
      pob = po[2] + param2;

      pnr = pn[0] + param2;
      png = pn[1] + param2;
      pnb = pn[2] + param2;

      psr = ps[0] + param2;
      psg = ps[1] + param2;
      psb = ps[2] + param2;

      pwr = pw[0] + param2;
      pwg = pw[1] + param2;
      pwb = pw[2] + param2;

      per = pe[0] + param2;
      peg = pe[1] + param2;
      peb = pe[2] + param2;

      float dny,dsy,dwy,dey,poy;
      
      poy = Spectral::get_y(po);
//       //cout << "avant : r = " << (int)po[0] << " , g = " << (int)po[1] << " , b = " << (int)po[2] << " , y = " << poy << endl;

      int n = 1;
      int param = 0;

      if((((param * por) / pnr) == (param * pog) / png) && (((param * por) / pnr) == (param * pob) / pnb)){
	dny = Spectral::get_y(pn) - poy;
	n++;
      }
      else
	dny = 0;
      if((((param * por) / psr) == (param * pog) / psg) && (((param * por) / psr) == (param * pob) / psb)){
	dsy = Spectral::get_y(ps) - poy;
	n++;
      }
      else
	dny = 0;
      if((((param * por) / pwr) == (param * pog) / pwg) && (((param * por) / pwr) == (param * pob) / pwb)){
	dwy = Spectral::get_y(pw) - poy;
	n++;
      }
      else
	dny = 0;
      if((((param * por) / per) == (param * pog) / peg) && (((param * por) / per) == (param * pob) / peb)){
	dey = Spectral::get_y(pe) - poy;
	n++;
      }
      else
	dny = 0;

      if(option == 1){
	d[y][x] = Spectral::modif_y( im[y][x] , poy + lambda * ( exp(-sqr(dny/kappa)) * dny + exp(-sqr(dsy/kappa)) * dsy + exp(-sqr(dwy/kappa)) * dwy + exp(-sqr(dey/kappa)) * dey ));
      }
      else if(option == 2){
	d[y][x] = Spectral::modif_y( im[y][x] , poy + lambda * (( dny / 1. + sqr(dny/kappa)) +  ( dsy / 1. + sqr(dsy/kappa)) +  ( dwy / 1. + sqr(dwy/kappa)) +  ( dey / 1. + sqr(dey/kappa))));
      }
      else{
	d[y][x] = Spectral::modif_y( im[y][x] , (poy + dny + dsy + dwy + dey) / n );
      }

//       po = d[y][x];
//       //cout << "apres : r = " << (int)po[0] << " , g = " << (int)po[1] << " , b = " << (int)po[2] << " , y = " << (int)Spectral::get_y(po) << endl << endl;
    }
  }
}

#define TAILLE_SIGMA 1

void SelectiveFilter::gauss_filter_sub(const ImageRGB &psrc, ImageRGB &pdest,bool color,unsigned int param,bool *t){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = TAILLE_SIGMA * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y,w,h;
  float res_r, res_g, res_b;
  //   pix vpix,cpix;
  ImageRGBPixel vpix;//,cpix;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *gauss	= new float[sqr(taille)];
  float sum;
  
  /* si les prconditions ne sont pas remplies, on quitte */
  //   if((pdest == NULL) || (psrc == NULL) ||
  //      (psrc->width != pdest->width) || (psrc->height != pdest->height))
  //    return;
  

  w = psrc.Width();
  h = psrc.Height();
  unsigned char tab[3];

//   for(vy = 0;vy < h; vy++){
//     for(vx = 0;vx < w; vx++){
//       if((*(t++)) == inside){
// 	tab[0] = 255;
// 	tab[1] = tab[2] = 0;
//       }	  
//       else{
// 	tab[2] = 255;
// 	tab[1] = tab[0] = 0;
//       }	  
//       pdest[vy][vx] = ImageRGBPixel(tab,3);
//     }
//   }
//   return;

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	exp(-((sqr(vx) + sqr(vy)) / sigma));
    }
  }

  if(color){
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
// 	    cout << "pixel : " << vx << " , " << vy << endl;
	sum = 0.0;
	// 	cpix = psrc[vy][vx];
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    // 		cout << "test sur : " << x << " , " << y << endl;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      // 		cout << "test 1 ok " << endl;
	      if(t[vy * w + vx] == t[y * w + x]){
		// 		cout << "test 2 ok " << endl;
		vpix = psrc[y][x];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		res_g += vpix[1] * g;
		res_b += vpix[2] * g;
		sum += g;
		// 		cout << "sum = " << sum << endl;
	      }
	    }
	  }
	}

	// 		cout << "sum = " << sum << endl;

	// 	if(sum != 0){

	res_r /= sum;
	res_g /= sum;
	res_b /= sum;

	// 	}
	// 	else{
	// 	  res_r = res_g = res_b = 0;
	// 	}

//   else{
// 	  for(j=0;j<taille;j++){
// 	    for(i=0;i<taille;i++){
// 	      x = vx - param + i;
// 	      y = vy - param + j;
// 	      // 		cout << "test sur : " << x << " , " << y << endl;
// 	      if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
// 		if(t[vy * w + vx] == t[y * w + x]){
// 		  vpix = psrc[y][x];
// 		  float g = gauss[j * taille + i];
// 		  res_r += vpix[0] * g;
// 		  res_g += vpix[1] * g;
// 		  res_b += vpix[2] * g;
// 		  sum += g;
// 		}
// 	      }
// 	    }
// 	  }
// 	  res_r /= sum;
// 	  res_g /= sum;
// 	  res_b /= sum;
// 	}

	  // 		cout << "res_r = " << res_r << endl;

	/* on sature le rsultat */
	tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
	/* on place le rsultat dans l'image destination */      
	// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	pdest[vy][vx] = ImageRGBPixel(tab,3);
// 		cout << "pdest[vy][vx][0] = " << (int)pdest[vy][vx][0] << endl;
// 	getchar();
      }
    }
  }

  else{
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	sum = 0.0;
	res_r = 0.0;
// 	if(t[vy * w + vx]){
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      if(t[y * w + x] == t[vy * w + vx]){
		vpix = psrc[y][x];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		sum += g;
	      }
	    }
	  }
	}
	// 	if(sum != 0){
	res_r /= sum;
	// 	}
	// 	else{
	// 	  res_r = 0;
//       }
      // 	else{
      // 	  for(j=0;j<taille;j++){
      // 	    for(i=0;i<taille;i++){
      // 	      x = vx - param + i;
      // 	      y = vy - param + j;
      // 	      if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
      // 		vpix = psrc[y][x];
      // 		float g = gauss[j * taille + i];
      // 		res_r += vpix[0] * g;
      // 		sum += g;
      // 	      }
      // 	    }
      // 	  }
      // 	  res_r /= sum;
      //       }


      /* on sature le rsultat */
	tab[0] = tab[1] = tab[2] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  
	/* on place le rsultat dans l'image destination */      
	// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
  delete gauss;
}

void SelectiveFilter::gauss_filter_sub(const ImageRGB &psrc, ImageRGB &pdest,bool color,unsigned int param,bool *t,bool *t2){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = TAILLE_SIGMA * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y,w,h;
  float res_r, res_g, res_b;
  //   pix vpix,cpix;
  ImageRGBPixel vpix;//,cpix;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *gauss	= new float[sqr(taille)];
  float sum;
  
  w = psrc.Width();
  h = psrc.Height();
  unsigned char tab[3];

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	exp(-((sqr(vx) + sqr(vy)) / sigma));
    }
  }

  if(color){
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	sum = 0.0;
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      if( ((!t[vy * w + vx])||(t[y * w + x])) && ((!t2[vy * w + vx])||(t2[y * w + x])) ){
		vpix = psrc[y][x];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		res_g += vpix[1] * g;
		res_b += vpix[2] * g;
		sum += g;
	      }
	    }
	  }
	}
	res_r /= sum;
	res_g /= sum;
	res_b /= sum;

	/* on sature le rsultat */
	tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
	/* on place le rsultat dans l'image destination */      
	pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
  else{
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	sum = 0.0;
	res_r = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      if( ((!t[vy * w + vx])||(t[y * w + x])) && ((!t2[vy * w + vx])||(t2[y * w + x])) ){
		vpix = psrc[y][x];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		sum += g;
	      }
	    }
	  }
	}
	res_r /= sum;

      /* on sature le rsultat */
	tab[0] = tab[1] = tab[2] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  
	/* on place le rsultat dans l'image destination */      
	pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
  delete gauss;
}

void SelectiveFilter::gauss_filter_sub(const ImageRGB &psrc, ImageRGB &pdest,bool color,unsigned int param){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = TAILLE_SIGMA * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y,w,h;
  float res_r, res_g, res_b;
  //   pix vpix,cpix;
  ImageRGBPixel vpix;//,cpix;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *gauss	= new float[sqr(taille)];
  float sum;
  
  /* si les prconditions ne sont pas remplies, on quitte */
  //   if((pdest == NULL) || (psrc == NULL) ||
  //      (psrc->width != pdest->width) || (psrc->height != pdest->height))
  //    return;
  

  w = psrc.Width();
  h = psrc.Height();

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
      gauss[(param + vx) * taille + (param - vy)] = 
      gauss[(param - vx) * taille + (param + vy)] =
      gauss[(param - vx) * taille + (param - vy)] = 
      exp(-((sqr(vx) + sqr(vy)) / sigma));
    }
  }

  if(color){
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
// 	    cout << "pixel : " << vx << " , " << vy << endl;
	sum = 0.0;
// 	cpix = psrc[vy][vx];
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
// 	    cout << "test : " << x << " , " << y << endl;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      vpix = psrc[y][x];
	      float g = gauss[j * taille + i];
	      res_r += vpix[0] * g;
	      res_g += vpix[1] * g;
	      res_b += vpix[2] * g;
	      sum += g;
// 	    cout << "sum =  " << sum << endl;
	    }
	  }
	}

// 	getchar();

// 	if(sum != 0){
	  res_r /= sum;
	  res_g /= sum;
	  res_b /= sum;
// 	}
// 	else{
// 	  res_r = res_g = res_b = 0;
// 	}

	/* on sature le rsultat */
	unsigned char tab[3];
	tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
	/* on place le rsultat dans l'image destination */      
	// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
  
  else{
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	sum = 0.0;
	res_r = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      vpix = psrc[y][x];
	      float g = gauss[j * taille + i];
	      res_r += vpix[0] * g;
	      sum += g;
	    }
	  }
	}

// 	if(sum != 0){
	  res_r /= sum;
// 	}
// 	else{
// 	  res_r = 0;
// 	}

	/* on sature le rsultat */
	unsigned char tab[3];
	tab[0] = tab[1] = tab[2] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  
	/* on place le rsultat dans l'image destination */      
	// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	pdest[vy][vx] = ImageRGBPixel(tab,3);
      }
    }
  }
  delete gauss;
}




//       int dnr,dng,dnb,dsr,dsg,dsb,dwr,dwg,dwb,der,deg,deb,por,pog,pob;
      
//       por = po[0];
//       pog = po[1];
//       pob = po[2];

//       dnr = pn[0] - por;
//       dng = pn[1] - pog;
//       dnb = pn[2] - pob;

//       dsr = ps[0] - por;
//       dsg = ps[1] - pog;
//       dsb = ps[2] - pob;

//       dwr = pw[0] - por;
//       dwg = pw[1] - pog;
//       dwb = pw[2] - pob;

//       der = pe[0] - por;
//       deg = pe[1] - pog;
//       deb = pe[2] - pob;

//  Uncomment the following to see a progression of images
//  subplot(ceil(sqrt(niterations)),ceil(sqrt(niterations)), i)
//  imagesc(diff), colormap(gray), axis image

//       end
// 	fprintf('\n');








#if(0)

function diff = anisodiff(im, niter, kappa, lambda, option)

if ndims(im)==3
  error('Anisodiff only operates on 2D grey-scale images');
end

im = double(im);
[rows,cols] = size(im);
diff = im;
  
for i = 1:niter
  fprintf('\rIteration %d',i);

  % Construct diffl which is the same as diff but
  % has an extra padding of zeros around it.
  diffl = zeros(rows+2, cols+2);
  diffl(2:rows+1, 2:cols+1) = diff;

  % North, South, East and West differences
  deltaN = diffl(1:rows,2:cols+1)   - diff;
  deltaS = diffl(3:rows+2,2:cols+1) - diff;
  deltaE = diffl(2:rows+1,3:cols+2) - diff;
  deltaW = diffl(2:rows+1,1:cols)   - diff;

  % Conduction

  if option == 1
    cN = exp(-(deltaN/kappa).^2);
    cS = exp(-(deltaS/kappa).^2);
    cE = exp(-(deltaE/kappa).^2);
    cW = exp(-(deltaW/kappa).^2);
  elseif option == 2
    cN = 1./(1 + (deltaN/kappa).^2);
    cS = 1./(1 + (deltaS/kappa).^2);
    cE = 1./(1 + (deltaE/kappa).^2);
    cW = 1./(1 + (deltaW/kappa).^2);
  end

  diff = diff + lambda*(cN.*deltaN + cS.*deltaS + cE.*deltaE + cW.*deltaW);

%  Uncomment the following to see a progression of images
%  subplot(ceil(sqrt(niterations)),ceil(sqrt(niterations)), i)
%  imagesc(diff), colormap(gray), axis image

end
fprintf('\n');




void SelectiveFilter::gfilter(const ImageRGB &psrc, ImageRGB &pdest,bool color,unsigned int param){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = .72 * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y,w,h;
  float res_r, res_g, res_b;
  //   pix vpix,cpix;
  ImageRGBPixel vpix,cpix;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  //   float filter_red  [3][3];
  //   float filter_green[3][3];
  //   float filter_blue [3][3];

  float *filter_red	= new float[sqr(taille)];
  float *filter_green	= new float[sqr(taille)];
  float *filter_blue	= new float[sqr(taille)];

  float *gauss	= new float[sqr(taille)];

  float filter_red_sum, filter_green_sum, filter_blue_sum;
  
  /* si les prconditions ne sont pas remplies, on quitte */
  //   if((pdest == NULL) || (psrc == NULL) ||
  //      (psrc->width != pdest->width) || (psrc->height != pdest->height))
  //    return;
  

  h = psrc.Width();	// c'est normal, tout est inverse
  w = psrc.Height();

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	exp(-((sqr(vx) + sqr(vy)) / sigma));
    }
  }

  for(j = 0;j < taille;j++){
    for(i = 0;i < taille;i++){
      vpix = psrc[x][y];
		    
      res_r = cpix[0] - vpix[0];
      if(res_r < 0.0) res_r = -res_r;
      if(res_r != 0)
	filter_red  [j * taille + i] = 1.0/res_r;
      else
	filter_red  [j * taille + i] = 1.0;
		    
      res_g = cpix[1] - vpix[1];
      if(res_g < 0.0) res_g = -res_g;
      if(res_g != 0)
	filter_green[j * taille + i] = 1.0/res_g;
      else
	filter_green[j * taille + i] = 1.0;
		    
      res_b = cpix[2] - vpix[2];
      if(res_b < 0.0) res_b = -res_b;
      if(res_b != 0)
	filter_blue [j * taille + i] = 1.0/res_b;
      else
	filter_blue [j * taille + i] = 1.0;

      filter_red_sum   += filter_red  [j * taille + i];
      filter_green_sum += filter_green[j * taille + i];
      filter_blue_sum  += filter_blue [j * taille + i];
    }
  }

  if(color){
    //cout << "filtre couleur" << endl;
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	filter_red_sum = filter_green_sum = filter_blue_sum = 0.0;
	/* recupere le point que l'on traite */
	cpix = psrc[vx][vy];
	/* fabrique la matrice de convolution en fonction des variations */
	/* de l'intensit (fabrique le filtre local)                     */
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    if(filter_red_sum != 0.0)
	      filter_red  [j * taille + i] /= filter_red_sum;
	    if(filter_green_sum != 0.0)
	      filter_green[j * taille + i] /= filter_green_sum;
	    if(filter_blue_sum != 0.0)
	      filter_blue [j * taille + i] /= filter_blue_sum;
	  }
	}
	/* on applique le filtre calcul */
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    vpix = psrc[vx - param + i][vy - param + j];
	    res_r += vpix[0] * filter_red  [j * taille + i];
	    res_g += vpix[1] * filter_green[j * taille + i];
	    res_b += vpix[2] * filter_blue [j * taille + i];
	  }
	}

	/* on sature le rsultat */
	unsigned char tab[3];
	tab[0] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	tab[1] = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	tab[2] = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);
	  
	/* on place le rsultat dans l'image destination */      
	// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	pdest[vx][vy] = ImageRGBPixel(tab,3);
      }
    }
  }
  
  else{
    //cout << "filtre gris" << endl;
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++){
	filter_red_sum = 0.0;
	/* recupere le point que l'on traite */
	cpix =psrc[vx][vy];
	/* fabrique la matrice de convolution en fonction des variations */
	/* de l'intensit (fabrique le filtre local)                     */
	for(j = 0;j < taille;j++){
	  for(i = 0;i < taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    if((x >= 0) && (x < w) && (y >= 0) && (y < h)){
	      if((j == param) && (i == param)){
		filter_red  [centre] = 0.5;
	      } 
	      else {
		vpix = psrc[x][y];
		    
		res_r = cpix[0] - vpix[0];
		if(res_r < 0.0) res_r = -res_r;
		if(res_r != 0)
		  filter_red  [j * taille + i] = 1.0/res_r;
		else
		  filter_red  [j * taille + i] = 1.0;
		    
	      }
	      filter_red_sum   += filter_red  [j * taille + i];
	    }
	  }
	}
	for(j = 0;j<taille;j++){
	  for(i = 0;i<taille;i++){
	    if(filter_red_sum != 0.0)
	      filter_red  [j * taille + i] /= filter_red_sum;
	  }
	}
	/* on applique le filtre calcul */
	res_r = 0.0;
	for(j = 0;j<taille;j++){
	  for(i = 0;i<taille;i++){
	    vpix = psrc[vx - param + i][vy - param + j];
	    res_r += vpix[0] * filter_red  [j * taille + i];
	  }
	}
	  
	/* on sature le rsultat */
	unsigned char tab[3];
	tab[0] = tab[1] = tab[2] = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	  
	/* on place le rsultat dans l'image destination */      
	// 	  put_pix_alpha_replace(pdest, vx, vy, COL((uint8)res_r,(uint8)res_g,(uint8)res_b));
	pdest[vx][vy] = ImageRGBPixel(tab,3);
      }
    }
  }
  delete filter_red;
  delete filter_green;
  delete filter_blue;
  delete gauss;
}



#endif


void SelectiveFilter::gauss_filter_sub(float *psrc,float *pdest,int w,int h,bool color,unsigned int param,bool *t,bool *t2){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = TAILLE_SIGMA * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y;
//   ww = w * 3;
  float res_r, res_g, res_b;
//   ImageRGBPixel vpix;//,cpix;
  float *vpix;//,cpix;
  unsigned long courant,autre;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *gauss	= new float[sqr(taille)];
  float sum;
  
//   unsigned char tab[3];
//   float *tab;

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	exp(-((sqr(vx) + sqr(vy)) / sigma));
//       cout << gauss[(param + vx) * taille + (param + vy)] << endl;
    }
  }
//   getchar();

  courant = 0;

  if(color){
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++,courant++){
	sum = 0.0;
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    autre = y * w + x;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      if((!(t[courant] ^ t2[courant])) || ((t[courant]) ? ((t[autre])&&(!t2[autre])) : ((t2[autre])&&(!t[autre])))){
		vpix = &psrc[autre * 3];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		res_g += vpix[1] * g;
		res_b += vpix[2] * g;
		sum += g;
	      }
	    }
	  }
	}
	res_r /= sum;
	res_g /= sum;
	res_b /= sum;

	*(pdest++) = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	*(pdest++) = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	*(pdest++) = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);	  
      }
    }
  }
  else{
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++,courant++){
	sum = 0.0;
	res_r = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    autre = y * w + x;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      if((!(t[courant] ^ t2[courant])) || ((t[courant]) ? ((t[autre])&&(!t2[autre])) : ((t2[autre])&&(!t[autre])))){
		vpix = &psrc[(y * w + x) * 3];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		sum += g;
	      }
	    }
	  }
	}
	res_r /= sum;

	res_r = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	*(pdest++) = res_r;
	*(pdest++) = res_r;
	*(pdest++) = res_r;
      }
    }
  }
  delete gauss;
}

void SelectiveFilter::gauss_filter_sub(float *psrc,float *pdest,int w,int h,bool color,unsigned int param,bool *t){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = TAILLE_SIGMA * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y;
//   ww = w * 3;
  float res_r, res_g, res_b;
//   ImageRGBPixel vpix;//,cpix;
  float *vpix;//,cpix;
  unsigned long courant,autre;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *gauss	= new float[sqr(taille)];
  float sum;
  
//   unsigned char tab[3];
//   float *tab;

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	exp(-((sqr(vx) + sqr(vy)) / sigma));
//       cout << gauss[(param + vx) * taille + (param + vy)] << endl;
    }
  }
//   getchar();

  courant = 0;

  if(color){
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++,courant++){
	sum = 0.0;
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    autre = y * w + x;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
// 	      if((!(t[courant] ^ t2[courant])) || ((t[courant]) ? ((t[autre])&&(!t2[autre])) : ((t2[autre])&&(!t[autre])))){
	      if(!(t[courant] ^ t[autre])){
		vpix = &psrc[autre * 3];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		res_g += vpix[1] * g;
		res_b += vpix[2] * g;
		sum += g;
	      }
	    }
	  }
	}
	res_r /= sum;
	res_g /= sum;
	res_b /= sum;

	*(pdest++) = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	*(pdest++) = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	*(pdest++) = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);	  
      }
    }
  }
  else{
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++,courant++){
	sum = 0.0;
	res_r = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    autre = y * w + x;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      if(!(t[courant] ^ t[autre])){
		vpix = &psrc[(y * w + x) * 3];
		float g = gauss[j * taille + i];
		res_r += vpix[0] * g;
		sum += g;
	      }
	    }
	  }
	}
	res_r /= sum;

	res_r = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	*(pdest++) = res_r;
	*(pdest++) = res_r;
	*(pdest++) = res_r;
      }
    }
  }
  delete gauss;
}

void SelectiveFilter::gauss_filter_sub(float *psrc,float *pdest,int w,int h,bool color,unsigned int param){

  int taille = 2 * param + 1;	// largeur du carre dans lequel on calcule la gaussienne
  float sigma = TAILLE_SIGMA * sqr(param);// on ajuste sigma en fonction de la taille voulue 
  // on choisit sigma tel que la gaussienne vale 1/4 au bord de la fenetre 
  // gauss(x) = exp(- (x*x / sigma)) 
  // gauss(param) = 1/4  quand  sigma = .72
  int centre = (taille + 1) * param; // indice du pixel central sur un tableau monodimensionnel

  //   int32 vx,vy;
  int vx,vy;
  int i,j;
  int x,y;
//   ww = w * 3;
  float res_r, res_g, res_b;
//   ImageRGBPixel vpix;//,cpix;
  float *vpix;//,cpix;
  unsigned long courant,autre;
  /* definition di filtre utilise (en forme de cloche => gaussien) */

  float *gauss	= new float[sqr(taille)];
  float sum;
  
//   unsigned char tab[3];
//   float *tab;

  for(vx = 0;vx <= param;vx++){
    for(vy = 0;vy <= param;vy++){
      gauss[(param + vx) * taille + (param + vy)] =
	gauss[(param + vx) * taille + (param - vy)] = 
	gauss[(param - vx) * taille + (param + vy)] =
	gauss[(param - vx) * taille + (param - vy)] = 
	exp(-((sqr(vx) + sqr(vy)) / sigma));
//       cout << gauss[(param + vx) * taille + (param + vy)] << endl;
    }
  }
//   getchar();

  courant = 0;

  if(color){
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++,courant++){
	sum = 0.0;
	res_r = 0.0;
	res_g = 0.0;
	res_b = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    autre = y * w + x;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      vpix = &psrc[autre * 3];
	      float g = gauss[j * taille + i];
	      res_r += vpix[0] * g;
	      res_g += vpix[1] * g;
	      res_b += vpix[2] * g;
	      sum += g;
	    }
	  }
	}
	res_r /= sum;
	res_g /= sum;
	res_b /= sum;

	*(pdest++) = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	*(pdest++) = (res_g > 255.0)? 255.0 : ((res_g < 0.0)? 0.0:res_g);
	*(pdest++) = (res_b > 255.0)? 255.0 : ((res_b < 0.0)? 0.0:res_b);	  
      }
    }
  }
  else{
    for(vy = 0;vy < h; vy++){
      for(vx = 0;vx < w; vx++,courant++){
	sum = 0.0;
	res_r = 0.0;
	for(j=0;j<taille;j++){
	  for(i=0;i<taille;i++){
	    x = vx - param + i;
	    y = vy - param + j;
	    autre = y * w + x;
	    if((x >= 0) && (y >= 0) && (x < w) && (y < h)){
	      vpix = &psrc[(y * w + x) * 3];
	      float g = gauss[j * taille + i];
	      res_r += vpix[0] * g;
	      sum += g;
	    }
	  }
	}
	res_r /= sum;

	res_r = (res_r > 255.0)? 255.0 : ((res_r < 0.0)? 0.0:res_r);
	*(pdest++) = res_r;
	*(pdest++) = res_r;
	*(pdest++) = res_r;
      }
    }
  }
  delete gauss;
}

void SelectiveFilter::im2float(ImageRGB &s,float *d,int w,int h){
  int x,y;
  ImageRGBPixel p;
  for(y = 0;y < h;y++){
    for(x = 0;x < w;x++){
//       cout << x << "  " << y << endl;
      p = s[y][x];
      *(d++) = p[0];
      *(d++) = p[1];
      *(d++) = p[2];
    }
  }
}

void SelectiveFilter::float2im(float *d,ImageRGB &s,int w,int h){
  int x,y;
  unsigned char p[3];
  float f;
  for(y = 0;y < h;y++){
    for(x = 0;x < w;x++){
      f = *(d++);
      p[0] = (f <= 0) ? 0 : (f >= 255) ? 255 : floor(f);
      f = *(d++);
      p[1] = (f <= 0) ? 0 : (f >= 255) ? 255 : floor(f);
      f = *(d++);
      p[2] = (f <= 0) ? 0 : (f >= 255) ? 255 : floor(f);
      s[y][x] = ImageRGBPixel(p,3);
    }
  }
}



