
#include "Spectral.h"

#ifdef  NTSC
// standard NTSC primaries R
const float Spectral::CIE_x_r = 0.670;
const float Spectral::CIE_y_r = 0.330;
// standard NTSC primaries G
const float Spectral::CIE_x_g = 0.210;
const float Spectral::CIE_y_g = 0.710;
// standard NTSC primaries B
const float Spectral::CIE_x_b = 0.140;
const float Spectral::CIE_y_b = 0.080;
// standard NTSC primaries White point
const float Spectral::CIE_x_w = 0.313;
const float Spectral::CIE_y_w = 0.329;
#endif
#ifdef NOMINALCRT
// nominal CRT primaries R
const float Spectral::CIE_x_r = 0.640;
const float Spectral::CIE_y_r = 0.330;
// nominal CRT primaries G
const float Spectral::CIE_x_g = 0.290;
const float Spectral::CIE_y_g = 0.600;
// nominal CRT primaries B
const float Spectral::CIE_x_b = 0.150;
const float Spectral::CIE_y_b = 0.060;
// nominal CRT primaries White point
const float Spectral::CIE_x_w = 0.313;
const float Spectral::CIE_y_w = 0.329;
#endif
#ifdef REC709
// Rec.709 primaries R
const float Spectral::CIE_x_r = 0.640;
const float Spectral::CIE_y_r = 0.330;
// Rec.709 primaries G
const float Spectral::CIE_x_g = 0.300;
const float Spectral::CIE_y_g = 0.600;
// Rec.709 primaries B
const float Spectral::CIE_x_b = 0.150;
const float Spectral::CIE_y_b = 0.060;
// Rec.709 primaries White point
const float Spectral::CIE_x_w = 0.3127;
const float Spectral::CIE_y_w = 0.3290;
#endif

const float Spectral::CIE_D    = (CIE_x_r*(CIE_y_g - CIE_y_b) + CIE_x_g*(CIE_y_b - CIE_y_r) + CIE_x_b*(CIE_y_r - CIE_y_g));
const float Spectral::CIE_C_rD = ((1./CIE_y_w) * ( CIE_x_w*(CIE_y_g - CIE_y_b) - CIE_y_w*(CIE_x_g - CIE_x_b) + CIE_x_g*CIE_y_b - CIE_x_b*CIE_y_g));
const float Spectral::CIE_C_gD = ((1./CIE_y_w) * ( CIE_x_w*(CIE_y_b - CIE_y_r) - CIE_y_w*(CIE_x_b - CIE_x_r) - CIE_x_r*CIE_y_b + CIE_x_b*CIE_y_r));
const float Spectral::CIE_C_bD = ((1./CIE_y_w) * ( CIE_x_w*(CIE_y_r - CIE_y_g) - CIE_y_w*(CIE_x_r - CIE_x_g) + CIE_x_r*CIE_y_g - CIE_x_g*CIE_y_r));

const float Spectral::CIE_rf   = (CIE_y_r*CIE_C_rD/CIE_D);
const float Spectral::CIE_gf   = (CIE_y_g*CIE_C_gD/CIE_D);
const float Spectral::CIE_bf   = (CIE_y_b*CIE_C_bD/CIE_D);

const float Spectral::xyz2rgb_m[3][3] = {     /* XYZ to RGB */
  {(CIE_y_g - CIE_y_b - CIE_x_b*CIE_y_g + CIE_y_b*CIE_x_g)/CIE_C_rD,
							(CIE_x_b - CIE_x_g - CIE_x_b*CIE_y_g + CIE_x_g*CIE_y_b)/CIE_C_rD,
														(CIE_x_g*CIE_y_b - CIE_x_b*CIE_y_g)/CIE_C_rD},
  {(CIE_y_b - CIE_y_r - CIE_y_b*CIE_x_r + CIE_y_r*CIE_x_b)/CIE_C_gD,
							(CIE_x_r - CIE_x_b - CIE_x_r*CIE_y_b + CIE_x_b*CIE_y_r)/CIE_C_gD,
														(CIE_x_b*CIE_y_r - CIE_x_r*CIE_y_b)/CIE_C_gD},
  {(CIE_y_r - CIE_y_g - CIE_y_r*CIE_x_g + CIE_y_g*CIE_x_r)/CIE_C_bD,
   (CIE_x_g - CIE_x_r - CIE_x_g*CIE_y_r + CIE_x_r*CIE_y_g)/CIE_C_bD,
   (CIE_x_r*CIE_y_g - CIE_x_g*CIE_y_r)/CIE_C_bD}
};

const float Spectral::rgb2xyz_m[3][3] = {     /* RGB to XYZ */
//   {CIE_x_r*CIE_C_rD/CIE_D,CIE_x_g*CIE_C_gD/CIE_D,CIE_x_b*CIE_C_bD/CIE_D},
//   {CIE_y_r*CIE_C_rD/CIE_D,CIE_y_g*CIE_C_gD/CIE_D,CIE_y_b*CIE_C_bD/CIE_D},
//   {(1.-CIE_x_r-CIE_y_r)*CIE_C_rD/CIE_D,
//    (1.-CIE_x_g-CIE_y_g)*CIE_C_gD/CIE_D,
//    (1.-CIE_x_b-CIE_y_b)*CIE_C_bD/CIE_D}
  {1. / 3. , 1. / 3. , 1. / 3.},
  {1. / 3. , 1. / 3. , 1. / 3.},
  {1. / 3. , 1. / 3. , 1. / 3.}
};

float* Spectral::rgb2xyz(const ImageRGBPixel &p){
  static float res[3];
  float t_p[3];
  t_p[0] = p[0] / 255.;
  t_p[1] = p[1] / 255.;
  t_p[2] = p[2] / 255.;
  res[0] = (t_p[0] * Spectral::rgb2xyz_m[0][0] + t_p[1] * Spectral::rgb2xyz_m[0][1] + t_p[2] * Spectral::rgb2xyz_m[0][2]);
  res[1] = (t_p[1] * Spectral::rgb2xyz_m[1][0] + t_p[1] * Spectral::rgb2xyz_m[1][1] + t_p[2] * Spectral::rgb2xyz_m[1][2]);
  res[2] = (t_p[2] * Spectral::rgb2xyz_m[2][0] + t_p[1] * Spectral::rgb2xyz_m[2][1] + t_p[2] * Spectral::rgb2xyz_m[2][2]);
  return res;
}

unsigned char* Spectral::xyz2rgb(const float p[3]){
  static unsigned char res[3];
  float t_p[3];
  t_p[0] = p[0];
  t_p[1] = p[1];
  t_p[2] = p[2];
  res[0] = floor((t_p[0] * Spectral::xyz2rgb_m[0][0] + t_p[1] * Spectral::xyz2rgb_m[0][1] + t_p[2] * Spectral::xyz2rgb_m[0][2]) * 255.);
  res[1] = floor((t_p[1] * Spectral::xyz2rgb_m[1][0] + t_p[1] * Spectral::xyz2rgb_m[1][1] + t_p[2] * Spectral::xyz2rgb_m[1][2]) * 255.);
  res[2] = floor((t_p[2] * Spectral::xyz2rgb_m[2][0] + t_p[1] * Spectral::xyz2rgb_m[2][1] + t_p[2] * Spectral::xyz2rgb_m[2][2]) * 255.);
  return res;
}

float Spectral::get_y(const ImageRGBPixel& p){
//   //   return (0.2125 * (float)(p[0]) + 0.7124 * (float)(p[1]) + 0.0721 * (float)(p[2])) / 255.;
//   return ((rgb2xyz_m[1][0] * (float)(p[0])) + (rgb2xyz_m[1][1] * (float)(p[1])) + (rgb2xyz_m[1][2] * (float)(p[2]))) / 255.;

//   cout << "get_y : " << (int)p[0] << " " << (int)p[1] << " " << (int)p[2] << " " << (((float)(p[0] + p[1] + p[2])) / 3.) << endl;
  return ((float)(p[0] + p[1] + p[2])) / (255. * 3.);
}

// ImageRGBPixel Spectral::modif_y(const ImageRGBPixel& p,float y){
// //   unsigned char res[3];

//   //   float dy =  y - get_y(p);
//   //   pg[0] = p[0] + (255 * (rgb2xyz[0][1] * dy));
//   //   pg[1] = p[1] + (255 * (rgb2xyz[1][1] * dy));
//   //   pg[2] = p[2] + (255 * (rgb2xyz[2][1] * dy));
//   float *t = rgb2xyz(p);
//   t[1] = y;
//   //res = xyz2rgb(t);
  
//   return ImageRGBPixel(xyz2rgb(t),3);
// }

ImageRGBPixel Spectral::modif_y(const ImageRGBPixel& p,float y){
  static unsigned char res[3];

//   cout << "modif_y : avant : r = " << (int)p[0] << " , g = " << (int)p[1] << " , b = " << (int)p[2] << " , y = " << get_y(p) << endl; 
  float f;
  int r,g,b;
//   cout << "modif_y : y = " << y << " , y/y(p) = " << f << endl; 
  if(get_y(p) == 0.){
    res[0] = res[1] = res[2] = (255. / 3.) * f ;
  }
  else{
    f = y / get_y(p);
    r = p[0] * f;
    g = p[1] * f;
    b = p[2] * f;
    res[0] = (r < 0) ? 0 : (r > 255) ? 255 : r;
    res[1] = (g < 0) ? 0 : (g > 255) ? 255 : g;
    res[2] = (b < 0) ? 0 : (b > 255) ? 255 : b;
  }

  ImageRGBPixel pres(res,3);
//   cout << "modif_y : apres : r = " << (int)res[0] << " , g = " << (int)res[1] << " , b = " << (int)res[2] << " , y = " << get_y(pres) << endl; 

  return ImageRGBPixel(res,3);
}

ImageRGBPixel Spectral::normalize(const ImageRGBPixel p){
  double tmp = p[0] + p[1] + p[2];
  unsigned char pn[3];
  if(tmp > 0){
    pn[0] = (unsigned int)floor(255. * (double)(p[0]) / tmp);
    pn[1] = (unsigned int)floor(255. * (double)(p[1]) / tmp);
    pn[2] = (unsigned int)floor(255. * (double)(p[2]) / tmp);
  }
  else{
    pn[0] = pn[1] = pn[2] = 0;
  }
  return ImageRGBPixel(pn,3); 
}
