#include "data/Camera.h"

void initializeRotationMatrix(double M[3][3],double axe[3],double angle)
{
  double x = axe[0];
  double y = axe[1];
  double z = axe[2];
  double s = sin(angle);
  double c = cos(angle);
  double t = 1-c;
  double x2 = x*x; double xy = x*y; double xz = x*z;
  double y2 = y*y; double yz = y*z;
  double z2 = z*z;
  M[0][0] = t*x2+c; M[1][0] = t*xy+s*z; M[2][0] = t*xz-s*y;
  M[0][1] = t*xy-s*z; M[1][1] = t*y2+c; M[2][1] = t*yz+s*x;
  M[0][2] = t*xz+s*y; M[1][2] = t*yz-s*x; M[2][2] = t*z2+c;
}

void rotateVector(double v[3],double M[3][3])
{
  double vtemp[3]; vtemp[0] = v[0]; vtemp[1] = v[1]; vtemp[2] = v[2];
  for (int i=0;i<3;i++)
    {
      v[i] = 0.;
      for (int j=0;j<3;j++) v[i] += M[i][j]*vtemp[j];
    }
}

Camera::Camera()
{
  _position[0] = 0.; _position[1] = -10.; _position[2] = 1.6;
  _direction[0] = 0.; _direction[1] = 1.; _direction[2] = 0.;
  _right[0] = 1.; _right[1] = 0.; _right[2] = 0.;
  _up[0] = 0.; _up[1] = 0.; _up[2] = 1.;
  _focal = 0.05;
  _fov = 26.99146656;
  _aspect = 1.;
}

Camera::~Camera()
{
}

void Camera::goTo(double dx,double dy)
{
  ADDVECTOR(_position,(dy * RAPPORT_VITESSE),_direction);
  lookAt(dx,0.);
}

void Camera::lookAt(double dx,double dy)
{
  double alphah = M_PI * (dy * _fov / 2.) / 180.;
  double alphaw = -M_PI * (dx * _aspect * _fov / 2.) / 180.;
  double M[3][3];
  initializeRotationMatrix(M,_up,alphaw / RAPPORT_VITESSE);
  rotateVector(_direction,M); NORMALIZEVECTOR(_direction);
  rotateVector(_right,M); NORMALIZEVECTOR(_right);
  rotateVector(_up,M); NORMALIZEVECTOR(_up);
  initializeRotationMatrix(M,_right,alphah / RAPPORT_VITESSE);
  rotateVector(_direction,M); NORMALIZEVECTOR(_direction);
  rotateVector(_right,M); NORMALIZEVECTOR(_right);
  rotateVector(_up,M); NORMALIZEVECTOR(_up);
}

void Camera::straffTo(double dx,double dy)
{
  ADDVECTOR(_position,(dy * RAPPORT_VITESSE),_up);
  ADDVECTOR(_position,(dx * RAPPORT_VITESSE),_right);
}
