#include "data/SceneNFF.h"

PointNFF::PointNFF() :
  _x(0.) , _y(0.) , _z(0.)
{
}

PointNFF::PointNFF(const double& xx,const double& yy,const double& zz) :
  _x(xx) , _y(yy) , _z(zz)
{
}

PointNFF::PointNFF(const PointNFF& p) :
  _x(p._x) , _y(p._y) , _z(p._z)
{
}

PointNFF::~PointNFF()
{
}

void PointNFF::set(const double& xx,const double& yy,const double& zz)
{
  _x = xx;
  _y = yy;
  _z = zz;
}

PolygonNFF::PolygonNFF() :
  _points() ,
  _material(NULL) ,
  _light(NULL),
  _type(TUNNELSURFACE_FLOOR)
{
}

PolygonNFF::PolygonNFF(const PolygonNFF& p) :
  _points() ,
  _material(NULL) ,
  _light(NULL) ,
  _type(p._type)
{
  _points = p._points;
  if (p._material != NULL) _material = new sMaterialReflectanceSpectrum(*(p._material));
  if (p._light != NULL) _light = new sLightSource(*(p._light));
}

PolygonNFF::~PolygonNFF()
{
  if (_material != NULL) delete _material;
  if (_light != NULL) delete _light;
}

void PolygonNFF::add(const PointNFF& p)
{
  PointNFF* tmp = new PointNFF(p);
  _points.add(tmp);
}

SceneNFF::SceneNFF() :
  _listPolygons()
{
}

SceneNFF::SceneNFF(const SceneNFF& s) :
  _listPolygons()
{
  _listPolygons = s._listPolygons;
}

SceneNFF::~SceneNFF()
{
}

void SceneNFF::add(const PolygonNFF& p)
{
  PolygonNFF* tmp = new PolygonNFF(p);
  _listPolygons.add(tmp);
}

ostream& operator<<(ostream& out,const SceneNFF& scn)
{
  for (int i=1;i<=scn.card();i++)
    out << scn[i] << endl;
  return out;
}

ostream& operator<<(ostream& out,const PolygonNFF& poly)
{
  out << "p " << poly.card() << endl;
  for (int i=1;i<=poly.card();i++)
    out << poly[i] << endl;
  return out;
}

ostream& operator<<(ostream& out,const PointNFF& pt)
{
  out << pt[0] << "  " << pt[1] << "  " << pt[2];
  return out;
}
