#include "fonct.h"

fonct::fonct(){}

void fonct::clear(){
  _f.clear();
}

bool fonct::existe_cle(const double x)const{
  return _f.find(x) != _f.end();
}

bool fonct::empty()const{
  return _f.empty();
}

const map<double,double>& fonct::get_map()const{
  return _f;
}

double fonct::operator*()const{
  double res = 0;
  map<double,double>::const_iterator it;
  for(it = _f.begin();it != _f.end();it++)
    res += it->second;
  return res;
}

double fonct::evaluer(const double x)const{
  map<double,double>::const_iterator it;
  double x1,x2,y1,y2;
  if(_f.empty()){
    //cout << __FUNCTION__ << "(" << x << ")  = 0 : cas 0" << endl;
    return 0;
  }
  it = _f.find(x);
  if(it != _f.end()){
    //cout << __FUNCTION__ << "(" << x << ") = " << it->second << " : cas 1" << endl;
    return it->second;
  }
  else{
    it = _f.upper_bound(x);
    if(it == _f.end()){
      //cout << __FUNCTION__ << "(" << x << ") = " << ((--it)++)->second << " : cas 2" << endl;
      return (--it)->second;
    }
    if(it == _f.begin()){
      //cout << __FUNCTION__ << "(" << x << ") = " << it->second << " : cas 3" << endl;
      return it->second;
    }
    x2 = it->first;
    y2 = it->second;
    it--;
    x1 = it->first;
    y1 = it->second;
    //cout << __FUNCTION__ << "(" << x << ") = " << y1 + (((x - x1) * (y2 - y1)) / (x2 - x1)) << " : cas 4" << endl;
    return y1 + (((x - x1) * (y2 - y1)) / (x2 - x1));
  }
}

double fonct::evaluer_lower(const double x)const{
  map<double,double>::const_iterator it;
  if(_f.empty()){
    return 0;
  }
  it = _f.upper_bound(x);
  if(it == _f.begin()){
    return it->second;
  } 
  else{
    return (--it)->second;
  }
}

// double fonct::evaluer_lower(const double x)const{
//   map<double,double>::const_iterator it;
//   if(_f.empty()){
//     return 0;
//   }
//   it = _f.upper_bound(x);
//   if((it == _f.begin()){
//     if((it->second == 0)){
//       return 0;
//     }
//     else{
//       return it->second + .5;
//     } 
//   }
//   else{
//     if(it-- == _f.end()){
//       return (it)->second;
//     }
//     else{
//       return ((it)->second) + .5;
//     }
//   }
//}

double& fonct::operator[](const double x){
  /*map<double,double>::iterator it = _f.find(x);
  if(it != _f.end())
    return it->second;
    else*/
    return _f[x];
}

void fonct::operator+=(const double x){
  map<double,double>::iterator it = _f.find(x);
  if(it != _f.end()){
    //cout << __FUNCTION__ << "incrementation de " << it->second;
    (it->second)++;
    //cout << " = " << it->second;
  }
  else
    _f[x] = 1;
}

void fonct::operator-=(const double x){
  map<double,double>::iterator it = _f.find(x);
  if(it != _f.end())
    (it->second)--;
  else
    _f[x] = -1;
}

fonct fonct::operator*(const double d)const{
  fonct res;
  map<double,double>::const_iterator it;
  for(it = _f.begin();it != _f.end();it++)
    res[it->first] = it->second * d;
  return res;
}

fonct fonct::operator/(const double d)const{
  fonct res;
  map<double,double>::const_iterator it;
  for(it = _f.begin();it != _f.end();it++)
    res[it->first] = it->second / d;
  return res;
}

fonct fonct::operator!()const{
  fonct res;
  map<double,double>::const_iterator it;
  for(it = _f.begin();it != _f.end();it++)
    res[it->second] = it->first;
  return res;  
}

fonct fonct::integr()const{	// integrale (sommets)
  double cumul;
  fonct res;
  res[0] = 0;
  map<double,double>::const_iterator it = _f.begin();
  for(it = _f.begin(),cumul = 0;it != _f.end();it++){
    cumul += it->second;
    res[it->first] = cumul;
    //cout << __FUNCTION__ << " : " << it->first << " -> " << cumul << endl;
  }
#if(0)
  if(it == _f.begin())
    res[1] = 1;
  else
#endif
    res[1] = cumul;
  return res;  
}

fonct fonct::operator++()const{	// integrale (milieux)
  double cumul,inter;
  fonct res;
  res[0] = 0;
  map<double,double>::const_iterator it = _f.begin();
  for(it = _f.begin(),cumul = 0;it != _f.end();it++){
    if(it->second != 0){
      inter = cumul + (it->second / 2);
      cumul += it->second;
      res[it->first] = inter;
    }
  }
  if(it == _f.begin())
    res[1] = 1;
  else if((--it)->first != 1)
    res[1] = cumul;
  return res;  
}

fonct fonct::operator^(const fonct &f)const{
  fonct res;
  map<double,double>::const_iterator it;
  fonct recip = !(*this);
  for(it = f._f.begin();it != f._f.end();it++){
    //cout << "operator^  : " << recip.evaluer(it->first) << " -> " << it->first << " -> " << it->second << endl;
    res[recip.evaluer(it->first)] = it->second;
  }
  for(it = _f.begin();it != _f.end();it++){
    //cout << "operator^  : " << it->first << " -> " << it->second << " -> " << evaluer(it->second) << endl;
    res[it->first] = f.evaluer(it->second);
  }
  return res;
}

fonct fonct::operator>>(const fonct &f)const{
  map<double,double>::const_iterator it;
  double s,sf;
  s = *(*this);		// *fonct = surface sous la courbe
  sf = *f;
  if(s == 0)
    s = 1;
  if(sf == 0)
    sf = 1;

  return ((++(*this)) * sf) ^ (!((++f) * s));

#if(0)
  for(it = _f.begin();it != _f.end();it++)
    cout << "f1( " << it->first << " )\t=\t" << it->second << endl;
  cout << endl;

  fonct f_test = ++(*this);
  for(it = f_test._f.begin();it != f_test._f.end();it++)
    cout << "int_f1( " << it->first << " )\t=\t" << it->second << endl;
  cout << endl;

  f_test = ++(f);
  for(it = f_test._f.begin();it != f_test._f.end();it++)
    cout << "int_f2( " << it->first << " )\t=\t" << it->second << endl;
  cout << endl;

  f_test = (++(*this)) * sf;
  for(it = f_test._f.begin();it != f_test._f.end();it++)
    cout << "(++(*this)) * sf)( " << it->first << " )\t=\t" << it->second << endl;
  cout << endl;

  f_test = (++(f)) * s;
  for(it = f_test._f.begin();it != f_test._f.end();it++)
    cout << "((++(f)) * s)( " << it->first << " )\t=\t" << it->second << endl;
  cout << endl;

  f_test = ((++(*this)) * sf) ^ (!((++f) * s));
  for(it = f_test._f.begin();it != f_test._f.end();it++)
    cout << "res( " << it->first << " )\t=\t" << it->second << endl;
  cout << endl;
#endif()

}







