#include "gui/FlGenerateTunnel.h"
#include "gui/FlEvents.h"
#include "gui/Labels.h"

#include "image/tunnel_planar.xpm"
#include "image/tunnel_triangular.xpm"
#include "image/tunnel_spherical.xpm"

#include "image/image_tunnel_planar.xpm"
#include "image/image_tunnel_triangular.xpm"
#include "image/image_tunnel_spherical.xpm"

#include "image/tunnel_source1_1.xpm"
#include "image/tunnel_source1_2.xpm"
#include "image/tunnel_source1_3.xpm"

#include "image/tunnel_source2_1.xpm"
#include "image/tunnel_source2_2.xpm"
#include "image/tunnel_source2_3.xpm"

#include "image/tunnel_source3_1.xpm"
#include "image/tunnel_source3_2.xpm"
#include "image/tunnel_source3_3.xpm"

#include "others/GlobalData.h"
#include "data/LightSource.h"
#include "data/MaterialReflectanceSpectrum.h"

#define LabelStartIndex 0

FlGenerateTunnel::FlGenerateTunnel() :
  Fl_Window(500,300,labels[LabelStartIndex+0])
{
  char tmp[255];

  box(FL_UP_BOX);
  labelsize(12);
  user_data((void*)(this));
  resizable(NULL);

  /* get tunnel and lists data from GlobalData object */
  temporaryParameters = GlobalData::getTunnelParameters();
  const cList< sLightSource* >& lightlist = GlobalData::getLightList();
  const cList< sMaterialReflectanceSpectrum* >& floorlist = GlobalData::getFloorList();
  const cList< sMaterialReflectanceSpectrum* >& walllist = GlobalData::getWallList();
  const cList< sMaterialReflectanceSpectrum* >& rooflist = GlobalData::getRoofList();

  {
    Fl_Tabs* o = _tabulator = new Fl_Tabs(0,0,500,270);
    {
      Fl_Group* o = _tunnelShape = new Fl_Group(5,20,490,245,labels[LabelStartIndex+1]);
      {
	Fl_Group* o = _tunnelShapeButton = new Fl_Group(5,20,85,245);
	{
	  Fl_Button* o = _tunnelShapeTriangular = new Fl_Button(10,30,70,70,labels[LabelStartIndex+2]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.shape._roofShape == TUNNELSHAPE_TRIANGULAR);
	  o->callback((Fl_Callback*)(FlGenerateTunnel::cb_tunnelshape));
	}
	{
	  Fl_Button* o = _tunnelShapeSpherical = new Fl_Button(10,105,70,70,labels[LabelStartIndex+3]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.shape._roofShape == TUNNELSHAPE_SPHERICAL);
	  o->callback((Fl_Callback*)(FlGenerateTunnel::cb_tunnelshape));
	}
	{
	  Fl_Button* o = _tunnelShapePlanar = new Fl_Button(10,180,70,70,labels[LabelStartIndex+4]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.shape._roofShape == TUNNELSHAPE_PLANAR);
	  o->callback((Fl_Callback*)(FlGenerateTunnel::cb_tunnelshape));
	}
	o->end();
      }
      {
	Fl_Group* o = _tunnelShapeLane = new Fl_Group(90,20,110,70);
	{
	  Fl_Light_Button* o = _tunnelShapeOneLane = new Fl_Light_Button(95,30,100,20,labels[LabelStartIndex+5]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.shape._floorLanes == TUNNELSHAPE_ONELANE);
	}
	{
	  Fl_Light_Button* o = _tunnelShapeTwoLanes = new Fl_Light_Button(95,60,100,20,labels[LabelStartIndex+6]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.shape._floorLanes == TUNNELSHAPE_TWOLANES);
	}
	o->end();
      }
      {
	Fl_Group* o = _tunnelShapeDimension = new Fl_Group(90,90,140,190);
	{
	  Fl_Float_Input* o = _tunnelShapeLength = new Fl_Float_Input(115,100,110,20,labels[LabelStartIndex+7]);
	  sprintf(tmp,"%f",temporaryParameters.shape._length);
	  o->value(tmp);
	}
	{
	  Fl_Float_Input* o = _tunnelShapeMaxHeight = new Fl_Float_Input(115,135,110,20,labels[LabelStartIndex+8]);
	  sprintf(tmp,"%f",temporaryParameters.shape._maxHeight);
	  o->value(tmp);
	}
	{
	  Fl_Float_Input* o = _tunnelShapeWidth = new Fl_Float_Input(115,170,110,20,labels[LabelStartIndex+9]);
	  sprintf(tmp,"%f",temporaryParameters.shape._width);
	  o->value(tmp);
	}
	{
	  Fl_Float_Input* o = _tunnelShapeMinHeight = new Fl_Float_Input(115,205,110,20,labels[LabelStartIndex+10]);
	  sprintf(tmp,"%f",temporaryParameters.shape._minHeight);
	  o->value(tmp);
	  if (temporaryParameters.shape._roofShape == TUNNELSHAPE_PLANAR) o->deactivate();
	}
	{
	  Fl_Float_Input* o = _tunnelShapeAngle = new Fl_Float_Input(115,240,110,20,labels[LabelStartIndex+11]);
	  sprintf(tmp,"%f",temporaryParameters.shape._angle);
	  o->value(tmp);
	  if (temporaryParameters.shape._roofShape != TUNNELSHAPE_TRIANGULAR) o->deactivate();
	}
	o->end();
      }
      {
	Fl_Group* o = _tunnelShapeImage = new Fl_Group(242,22,246,246);
	o->box(FL_DOWN_BOX);
	o->end();
      }
      o->end();
    }
    {
      Fl_Group* o = _tunnelLight = new Fl_Group(5,20,490,245,labels[LabelStartIndex+12]);
      {
	Fl_Multi_Browser* o = _tunnelLightSelector = new Fl_Multi_Browser(5,25,150,220,labels[LabelStartIndex+13]);
	o->add(labels[LabelStartIndex+14]);
	for (cListIterator< sLightSource* > iter = lightlist;!iter.finished();++iter)
	  {
	    sLightSource* crt = iter.getCurrent();
	    o->add(crt->_name);
	  }
	for (cListIterator< int > iter = temporaryParameters.light._lightSelected;!iter.finished();++iter)
	  {
	    int crt = iter.getCurrent();
	    o->select(crt,1);
	  }
      }
      {
	Fl_Group* o = _tunnelLightButton = new Fl_Group(170,25,320,110);
	{
	  Fl_Button* o = _tunnelLightCenter = new Fl_Button(170,30,100,100,labels[LabelStartIndex+15]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.light._lightPosition == TUNNELLIGHT_CENTER);
	  o->callback((Fl_Callback*)(FlGenerateTunnel::cb_tunnellight));
	}
	{
	  Fl_Button* o = _tunnelLightMiddle = new Fl_Button(280,30,100,100,labels[LabelStartIndex+16]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.light._lightPosition == TUNNELLIGHT_MIDDLE);
	  o->callback((Fl_Callback*)(FlGenerateTunnel::cb_tunnellight));
	}
	{
	  Fl_Button* o = _tunnelLightSide = new Fl_Button(390,30,100,100,labels[LabelStartIndex+17]);
	  o->type(FL_RADIO_BUTTON);
	  o->value(temporaryParameters.light._lightPosition == TUNNELLIGHT_SIDE);
	  o->callback((Fl_Callback*)(FlGenerateTunnel::cb_tunnellight));
	}
	o->end();
      }
      {
	Fl_Group* o = _tunnelLightParameters = new Fl_Group(160,145,325,80);
	{
	  Fl_Light_Button* o = _tunnelLightSymmetrical = new Fl_Light_Button(285,150,200,20,labels[LabelStartIndex+18]);
	  o->type(FL_TOGGLE_BUTTON);
	  o->value(temporaryParameters.light._lightSymmetrical == ON);
	}
	{
	  Fl_Float_Input* o = _tunnelLightSpacing = new Fl_Float_Input(365,175,120,20,labels[LabelStartIndex+19]);
	  sprintf(tmp,"%f",temporaryParameters.light._lightSpacing);
	  o->value(tmp);
	}
	{
	  Fl_Float_Input* o = _tunnelLightSlope = new Fl_Float_Input(365,200,120,20,labels[LabelStartIndex+20]);
	  sprintf(tmp,"%f",temporaryParameters.light._lightSlope);
	  o->value(tmp);
	}
	o->end();
      }
      o->end();
    }
    {
      Fl_Group* o = _tunnelMaterial = new Fl_Group(5,20,490,245,labels[LabelStartIndex+21]);
      {
	Fl_Multi_Browser* o = _tunnelFloorSelector = new Fl_Multi_Browser(15,25,150,220,labels[LabelStartIndex+22]);
	o->add(labels[LabelStartIndex+23]);
	for (cListIterator< sMaterialReflectanceSpectrum* > iter = floorlist;!iter.finished();++iter)
	  {
	    sMaterialReflectanceSpectrum* crt = iter.getCurrent();
	    o->add(crt->_spectrumName);
	  }
	for (cListIterator< int > iter = temporaryParameters.material._floorSelected;!iter.finished();++iter)
	  {
	    int crt = iter.getCurrent();
	    o->select(crt,1);
	  }
      }
      {
	Fl_Multi_Browser* o = _tunnelWallSelector = new Fl_Multi_Browser(175,25,150,220,labels[LabelStartIndex+24]);
	o->add(labels[LabelStartIndex+25]);
	for (cListIterator< sMaterialReflectanceSpectrum* > iter = walllist;!iter.finished();++iter)
	  {
	    sMaterialReflectanceSpectrum* crt = iter.getCurrent();
	    o->add(crt->_spectrumName);
	  }
	for (cListIterator< int > iter = temporaryParameters.material._wallSelected;!iter.finished();++iter)
	  {
	    int crt = iter.getCurrent();
	    o->select(crt,1);
	  }
      }
      {
	Fl_Multi_Browser* o = _tunnelRoofSelector = new Fl_Multi_Browser(335,25,150,220,labels[LabelStartIndex+26]);
	o->add(labels[LabelStartIndex+27]);
	for (cListIterator< sMaterialReflectanceSpectrum* > iter = rooflist;!iter.finished();++iter)
	  {
	    sMaterialReflectanceSpectrum* crt = iter.getCurrent();
	    o->add(crt->_spectrumName);
	  }
	for (cListIterator< int > iter = temporaryParameters.material._roofSelected;!iter.finished();++iter)
	  {
	    int crt = iter.getCurrent();
	    o->select(crt,1);
	  }
      }
      o->end();
    }
    o->value(_tunnelShape);
    o->end();
  }

  _validateButton = new Fl_Button(100,275,100,20,labels[LabelStartIndex+28]); _validateButton->callback((Fl_Callback*)(FlGenerateTunnel::cb_validate));
  _cancelButton = new Fl_Button(300,275,100,20,labels[LabelStartIndex+29]); _cancelButton->callback((Fl_Callback*)(FlGenerateTunnel::cb_cancel));

  end();

  icn_planar = new Fl_Pixmap(tunnel_planar_xpm); icn_planar->label(_tunnelShapePlanar);
  icn_triangular = new Fl_Pixmap(tunnel_triangular_xpm); icn_triangular->label(_tunnelShapeTriangular);
  icn_spherical = new Fl_Pixmap(tunnel_spherical_xpm); icn_spherical->label(_tunnelShapeSpherical);

  image_planar = new Fl_Pixmap(image_tunnel_planar_xpm);
  image_triangular = new Fl_Pixmap(image_tunnel_triangular_xpm);
  image_spherical = new Fl_Pixmap(image_tunnel_spherical_xpm);

  image_sources[2][0] = new Fl_Pixmap(tunnel_source1_1_xpm);
  image_sources[2][1] = new Fl_Pixmap(tunnel_source1_2_xpm);
  image_sources[2][2] = new Fl_Pixmap(tunnel_source1_3_xpm);
  image_sources[0][0] = new Fl_Pixmap(tunnel_source2_1_xpm);
  image_sources[0][1] = new Fl_Pixmap(tunnel_source2_2_xpm);
  image_sources[0][2] = new Fl_Pixmap(tunnel_source2_3_xpm);
  image_sources[1][0] = new Fl_Pixmap(tunnel_source3_1_xpm);
  image_sources[1][1] = new Fl_Pixmap(tunnel_source3_2_xpm);
  image_sources[1][2] = new Fl_Pixmap(tunnel_source3_3_xpm);

  set_modal();
  _buttonClicked = 0;
}

FlGenerateTunnel::~FlGenerateTunnel()
{
}

void FlGenerateTunnel::draw()
{
  image_sources[(int)(temporaryParameters.shape._roofShape)][0]->label(_tunnelLightCenter);
  image_sources[(int)(temporaryParameters.shape._roofShape)][1]->label(_tunnelLightMiddle);
  image_sources[(int)(temporaryParameters.shape._roofShape)][2]->label(_tunnelLightSide);
  Fl_Window::draw();
  if (_tabulator->value() == _tunnelShape)
    {
      switch (temporaryParameters.shape._roofShape)
	{
	case TUNNELSHAPE_PLANAR:
	  image_planar->draw(245,25,240,240);
	  break;
	case TUNNELSHAPE_TRIANGULAR:
	  image_triangular->draw(245,25,240,240);
	  break;
	case TUNNELSHAPE_SPHERICAL:
	  image_spherical->draw(245,25,240,240);
	  break;
	}
    }
}

void FlGenerateTunnel::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Window::resize(xx,yy,ww,hh);
}

int FlGenerateTunnel::handle(int event)
{
  if ((event == FL_KEYBOARD) || (event == FL_SHORTCUT)) return 1;
  return Fl_Window::handle(event);
}

int FlGenerateTunnel::clicked()
{
  return _buttonClicked;
}

void FlGenerateTunnel::cb_tunnellight_i(Fl_Widget* o, void* v)
{
  if (o == _tunnelLightCenter) temporaryParameters.light._lightPosition = TUNNELLIGHT_CENTER;
  else if (o == _tunnelLightMiddle) temporaryParameters.light._lightPosition = TUNNELLIGHT_MIDDLE;
  else temporaryParameters.light._lightPosition = TUNNELLIGHT_SIDE;
  redraw();
}
void FlGenerateTunnel::cb_tunnellight(Fl_Widget* o, void* v)
{
  ((FlGenerateTunnel*)(o->parent()->parent()->parent()->parent()))->cb_tunnellight_i(o,v);
}

void FlGenerateTunnel::cb_tunnelshape_i(Fl_Widget* o, void* v)
{
  if (o == _tunnelShapeTriangular)
    {
      temporaryParameters.shape._roofShape = TUNNELSHAPE_TRIANGULAR;
      _tunnelShapeMinHeight->activate();
      _tunnelShapeAngle->activate();
    }
  else if (o == _tunnelShapeSpherical)
    {
      temporaryParameters.shape._roofShape = TUNNELSHAPE_SPHERICAL;
      _tunnelShapeMinHeight->activate();
      _tunnelShapeAngle->deactivate();
    }
  else
    {
      temporaryParameters.shape._roofShape = TUNNELSHAPE_PLANAR;
      _tunnelShapeMinHeight->deactivate();
      _tunnelShapeAngle->deactivate();
    }
  redraw();
}
void FlGenerateTunnel::cb_tunnelshape(Fl_Widget* o, void* v)
{
  ((FlGenerateTunnel*)(o->parent()->parent()->parent()->parent()))->cb_tunnelshape_i(o,v);
}

void FlGenerateTunnel::cb_validate_i(Fl_Widget* o, void* v)
{
  /* retreiving shape parameters */
  temporaryParameters.shape._floorLanes = (_tunnelShapeOneLane->value() ? TUNNELSHAPE_ONELANE : TUNNELSHAPE_TWOLANES);
  sscanf(_tunnelShapeLength->value(),"%lf",&(temporaryParameters.shape._length));
  sscanf(_tunnelShapeMaxHeight->value(),"%lf",&(temporaryParameters.shape._maxHeight));
  sscanf(_tunnelShapeWidth->value(),"%lf",&(temporaryParameters.shape._width));
  sscanf(_tunnelShapeMinHeight->value(),"%lf",&(temporaryParameters.shape._minHeight));
  sscanf(_tunnelShapeAngle->value(),"%lf",&(temporaryParameters.shape._angle));
  /* retreiving light parameters */
  temporaryParameters.light._lightSelected.freeMemory();
  for (int i = 1;i<=_tunnelLightSelector->size();i++)
    {
      //cout << "Light " << i << " :: " << _tunnelLightSelector->select(i) << endl;
      if (!(_tunnelLightSelector->select(i))) temporaryParameters.light._lightSelected.add(i);
    }
  temporaryParameters.light._lightSymmetrical = (_tunnelLightSymmetrical->value() ? ON : OFF);
  sscanf(_tunnelLightSpacing->value(),"%lf",&(temporaryParameters.light._lightSpacing));
  sscanf(_tunnelLightSlope->value(),"%lf",&(temporaryParameters.light._lightSlope));
  /* retreiving material parameters */
  temporaryParameters.material._floorSelected.freeMemory();
  for (int i = 1;i<=_tunnelFloorSelector->size();i++)
    if (!(_tunnelFloorSelector->select(i))) temporaryParameters.material._floorSelected.add(i);
  temporaryParameters.material._wallSelected.freeMemory();
  for (int i = 1;i<=_tunnelWallSelector->size();i++)
    if (!(_tunnelWallSelector->select(i))) temporaryParameters.material._wallSelected.add(i);
  temporaryParameters.material._roofSelected.freeMemory();
  for (int i = 1;i<=_tunnelRoofSelector->size();i++)
    if (!(_tunnelRoofSelector->select(i))) temporaryParameters.material._roofSelected.add(i);
  /* update tunnel and simulation data in GlobalData object */
  GlobalData::setTunnelParameters(temporaryParameters);
  _buttonClicked = 1;
}
void FlGenerateTunnel::cb_validate(Fl_Widget* o, void* v)
{
  ((FlGenerateTunnel*)(o->parent()))->cb_validate_i(o,v);
}

void FlGenerateTunnel::cb_cancel_i(Fl_Widget* o, void* v)
{
  _buttonClicked = 1;
}
void FlGenerateTunnel::cb_cancel(Fl_Widget* o, void* v)
{
  ((FlGenerateTunnel*)(o->parent()))->cb_cancel_i(o,v);
}
