#include "gui/FlGlVisuCourbe.h"
#include "gui/FlEvents.h"
#include "gui/FlSelectCourbe.h"

#include "others/GlobalData.h"

FlGlVisuCourbe::FlGlVisuCourbe(const int& xx,const int& yy,const int& w,const int& h,const char* title = NULL) :
  Fl_Gl_Window(xx,yy,w,h,title) ,
  _firstTimeDrawing(1) ,
  _axesDrawingList(0) ,
  _curveDrawingList(0) ,
  _clickedAbcisse(0) ,
  _curveSelector(NULL)
{
}

FlGlVisuCourbe::~FlGlVisuCourbe()
{
  if (glIsList(_axesDrawingList)) glDeleteLists(_axesDrawingList,1);
  if (glIsList(_curveDrawingList)) glDeleteLists(_curveDrawingList,1);
}

void FlGlVisuCourbe::draw()
{
  if (_firstTimeDrawing) { initializeAxesDrawingList(); initializeCurveDrawingList(); _firstTimeDrawing = 0; }
  /* initialisation de GL */
  glViewport(0,0,w(),h());
  glClearColor(1.,1.,1.,1.);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1.,1.,-1.,1.,-1.,1.);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  if (glIsList(_axesDrawingList)) { glCallList(_axesDrawingList); }
  if (glIsList(_curveDrawingList)) { glCallList(_curveDrawingList); }
}

void FlGlVisuCourbe::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Gl_Window::resize(xx,yy,ww,hh);
}

int FlGlVisuCourbe::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case FL_PUSH:
    case FL_DRAG:
      if (Fl::event_button() == FL_LEFT_MOUSE)
	{
	  _clickedAbcisse = Fl::event_x();
	  _curveSelector->handle(TUNNEL_VIEWERCLICKED);
	}
      res = 1;
      break;
    case TUNNEL_DATACHANGED:
      initializeCurveDrawingList();
      redraw();
      res = 1;
      break;
    case TUNNEL_NEEDTOREDRAW:
      redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Gl_Window::handle(event);
  return res;
}

void FlGlVisuCourbe::link(FlSelectCourbe* selector)
{
  _curveSelector = selector;
}

int FlGlVisuCourbe::getAbcisse()
{
  return _clickedAbcisse;
}

/*
 * private methods
 */

void FlGlVisuCourbe::initializeAxesDrawingList()
{
  if (glIsList(_axesDrawingList)) glDeleteLists(_axesDrawingList,1);
  _axesDrawingList = glGenLists(1);
  glNewList(_axesDrawingList,GL_COMPILE);
  glColor3f(0.5,0.5,0.5);
  glBegin(GL_LINES);
  for (int i = 0; i < 18; i++)
    {
      glVertex2f(-0.8 + (0.1 * i), -1.0); glVertex2f(-0.8 + (0.1 * i), 1.0);
      glVertex2f(-1.0, -0.8 + (0.1 * i)); glVertex2f(1.0, -0.8 + (0.1 * i));
    }
  glEnd();
  glColor3f(0.,0.,1.);
  glBegin(GL_LINES);
  glVertex2f(-1.0, -0.9); glVertex2f(1.0, -0.9);
  glVertex2f(-0.9, -1.0); glVertex2f(-0.9, 1.0);
  glEnd();
  glEndList();
}

void FlGlVisuCourbe::initializeCurveDrawingList()
{
  if (glIsList(_curveDrawingList)) glDeleteLists(_curveDrawingList,1);
  const sRequiredRadianceCurve& rrc = GlobalData::getRequiredRadianceCurve();
  if (rrc._curveDataSize > 0)
    {
      _curveDrawingList = glGenLists(1);
      glNewList(_curveDrawingList,GL_COMPILE);
      glColor3f(1.,0.,0.);
      glBegin(GL_LINE_STRIP);
      for (int i=0;i<rrc._curveDataSize;i++)
	{
	  glVertex2f((rrc._curveAbcisse[i] - 80.) / 100., (rrc._curveValue[i] - 180.) / 200.);
	}
      glEnd();
      glEndList();
    }
}
