#include "gui/FlGlVisuScene.h"
#include "gui/FlEvents.h"

#include "others/GlobalData.h"

#include "radiosite/Spectre.h"

#define DEGREETORADIAN(d) (M_PI * (d) / 180.)

#define WINTONORMALIZED2DCOORDINATES(xf,yf,xr,yr,wf,hf,xn,yn) { \
 (xn) = ((xf) - (xr)) / (wf); \
 (yn) = ((yr) - (yf)) / (hf); \
 }
#define NORMALIZED2DTOCAMERACOORDINATES(x2,y2,focal,fov,aspect,pc) { \
 (pc)[0] = (x2) * (aspect) * 2. * (focal) * tan(DEGREETORADIAN(fov) / 2.); \
 (pc)[1] = (y2) * 2. * (focal) * tan(DEGREETORADIAN(fov) / 2.); \
 (pc)[2] = (focal); \
 }
#define CAMERATOGLOBAL(pc,pos,dir,up,right,pg) { \
COPYVECTOR(pg,pos);\
ADDVECTOR(pg,(pc)[0],right);\
ADDVECTOR(pg,(pc)[1],up);\
ADDVECTOR(pg,(pc)[2],dir);\
}
#define GLDRAWVECTOR(v) { \
glVertex3f((v)[0],(v)[1],(v)[2]); \
}

FlGlVisuScene::FlGlVisuScene(const int& xx,const int& yy,const int& w,const int& h,const char* title = NULL) :
  Fl_Gl_Window(xx,yy,w,h,title) ,
  _resolution(NULL) ,
  _scene(NULL) ,
  _scenelist(0) ,
  _camera() ,
  _moving(0) ,
  _rotating(0) ,
  _straffing(0)
{
  _camera.setAspect(w/h);
  switch (GlobalData::getVisualizedModel())
    {
    case TUNNELMODEL_NFF:
      _scene = GlobalData::getInitialTunnelNFF();
      break;
    case TUNNELMODEL_INITIAL:
      _resolution = GlobalData::getMaillage3d();
      break;
    case TUNNELMODEL_SOLUTION:
      _scene = GlobalData::getInitialTunnelNFF();
      break;
    }
}

FlGlVisuScene::~FlGlVisuScene()
{
  if (glIsList(_scenelist)) glDeleteLists(_scenelist,1);
  GlobalData::initializeSceneNFF();
  GlobalData::initializeSceneMaillage();
}

void FlGlVisuScene::draw()
{
  if (_moving) { moveTo(_mouseX,_mouseY); }
  if (_rotating) { rotateTo(_mouseX,_mouseY); }
  if (_straffing) { straffTo(_mouseX,_mouseY); }
  /* initialisation de GL */
  glClearColor(0.,0.,0.,1.);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_CULL_FACE);
  glDrawCamera();
  glColor3f(1.,1.,1.);
  if (_moving || _rotating || _straffing) glDrawScreenLine(_referenceMouseX,_referenceMouseY,_mouseX,_mouseY);
  if (_scene != NULL)
    {
      glPolygonMode(GL_BACK,GL_LINE);
      glPolygonMode(GL_FRONT,GL_FILL);
      if (glIsList(_scenelist)) glCallList(_scenelist);
      else {
	_scenelist = glGenLists(1);
	glNewList(_scenelist,GL_COMPILE_AND_EXECUTE);
	for (int i=1;i<=_scene->card();i++) { glDrawPolygon((*_scene)[i]); if (i%1000==0) cout << "drawing poly :: " << i << endl; }
	glEndList();
      }
    }
  if (_resolution != NULL)
    {
      if (GlobalData::getVisualizedModel() == TUNNELMODEL_INITIAL)
	{
	  glPolygonMode(GL_BACK,GL_LINE);
	  glPolygonMode(GL_FRONT,GL_LINE);
	}
      else
	{
	  glPolygonMode(GL_BACK,GL_LINE);
	  glPolygonMode(GL_FRONT,GL_FILL);
	}
      if (glIsList(_scenelist)) glCallList(_scenelist);
      else {
	_scenelist = glGenLists(1);
	glNewList(_scenelist,GL_COMPILE_AND_EXECUTE);
	for (Surface* S=_resolution->slist;S != NULL;S=S->nexts) { glDrawSurface(*S); }
	glEndList();
      }
    }
}

void FlGlVisuScene::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Gl_Window::resize(xx,yy,ww,hh);
}

int FlGlVisuScene::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case TUNNEL_DATACHANGED:
      if (glIsList(_scenelist)) { glDeleteLists(_scenelist,1); _scenelist = 0; }
      switch (GlobalData::getVisualizedModel())
	{
	  case TUNNELMODEL_NFF:
	    GlobalData::initializeSceneNFF();
	    GlobalData::initializeSceneMaillage();
	    _scene = NULL; _resolution = NULL;
	    _scene = GlobalData::getInitialTunnelNFF();
	  break;
	  case TUNNELMODEL_INITIAL:
	    GlobalData::initializeSceneNFF();
	    GlobalData::initializeSceneMaillage();
	    _scene = NULL; _resolution = NULL;
	    _resolution = GlobalData::getMaillage3d();
	  break;
	case TUNNELMODEL_SOLUTION:
	    GlobalData::initializeSceneNFF();
	    GlobalData::initializeSceneMaillage();
	    _scene = NULL; _resolution = NULL;
	    _resolution = GlobalData::getMaillageSolution();
	  break;
	}
      redraw();
      res = 1;
      break;
    case TUNNEL_NEEDTOREDRAW:
      if (glIsList(_scenelist)) { glDeleteLists(_scenelist,1); _scenelist = 0; }
      redraw();
      res = 1;
      break;
    case FL_PUSH:
      switch (Fl::event_button())
	{
	case FL_LEFT_MOUSE:
	  _moving = 1;
	  _rotating = 0;
	  _straffing = 0;
	  _referenceMouseX = Fl::event_x();
	  _referenceMouseY = Fl::event_y();
	  _mouseX = Fl::event_x();
	  _mouseY = Fl::event_y();
	  Fl::add_idle(FlGlVisuScene::movingIdle,this);
	  break;
	case FL_MIDDLE_MOUSE:
	  _moving = 0;
	  _rotating = 1;
	  _straffing = 0;
	  _referenceMouseX = Fl::event_x();
	  _referenceMouseY = Fl::event_y();
	  _mouseX = Fl::event_x();
	  _mouseY = Fl::event_y();
	  Fl::add_idle(FlGlVisuScene::rotatorIdle,this);
	  break;
	case FL_RIGHT_MOUSE:
	  _moving = 0;
	  _rotating = 0;
	  _straffing = 1;
	  _referenceMouseX = Fl::event_x();
	  _referenceMouseY = Fl::event_y();
	  _mouseX = Fl::event_x();
	  _mouseY = Fl::event_y();
	  Fl::add_idle(FlGlVisuScene::straffIdle,this);
	  break;
	}
      redraw();
      res = 1;
      break;
    case FL_DRAG:
      _mouseX = Fl::event_x();
      _mouseY = Fl::event_y();
      res = 1;
      break;
    case FL_RELEASE:
      switch (Fl::event_button())
	{
	case FL_LEFT_MOUSE:
	  _moving = 0;
	  Fl::remove_idle(FlGlVisuScene::movingIdle,this);
	  break;
	case FL_MIDDLE_MOUSE:
	  _rotating = 0;
	  Fl::remove_idle(FlGlVisuScene::rotatorIdle,this);
	  break;
	case FL_RIGHT_MOUSE:
	  _straffing = 0;
	  Fl::remove_idle(FlGlVisuScene::straffIdle,this);
	  break;
	}
      redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Gl_Window::handle(event);
  return res;
}

/*
 * Protected Methods
 */

void FlGlVisuScene::glDrawCamera()
{
  double* position = _camera.getPosition();
  double* direction = _camera.getDirection();
  double* up = _camera.getUp();
  double focal = _camera.getFocal();
  double fov = _camera.getFov();
  double aspect = _camera.getAspect();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0,0,w(),h());
  gluPerspective(fov,aspect,focal,1.e6);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt
    (
     position[0],position[1],position[2],
     position[0]+focal*direction[0],position[1]+focal*direction[1],position[2]+focal*direction[2],
     up[0],up[1],up[2]
     );
}

void FlGlVisuScene::glDrawScreenLine(double xx0,double yy0,double xx1,double yy1)
{
  double* position = _camera.getPosition();
  double* direction = _camera.getDirection();
  double* up = _camera.getUp();
  double* right = _camera.getRight();
  double focal = _camera.getFocal();
  double fov = _camera.getFov();
  double aspect = _camera.getAspect();

  double dx0,dy0,dx1,dy1;
  double pc0[3];
  double pc1[3];
  double origine[3];
  double extremite[3];

  WINTONORMALIZED2DCOORDINATES(xx0,yy0,w()/2.,h()/2.,w(),h(),dx0,dy0);
  WINTONORMALIZED2DCOORDINATES(xx1,yy1,w()/2.,h()/2.,w(),h(),dx1,dy1);
  NORMALIZED2DTOCAMERACOORDINATES(dx0,dy0,focal,fov,aspect,pc0);
  NORMALIZED2DTOCAMERACOORDINATES(dx1,dy1,focal,fov,aspect,pc1);
  CAMERATOGLOBAL(pc0,position,direction,up,right,origine);
  CAMERATOGLOBAL(pc1,position,direction,up,right,extremite);
  glBegin(GL_LINE_STRIP);
  GLDRAWVECTOR(origine);
  GLDRAWVECTOR(extremite);
  glEnd();
}

void FlGlVisuScene::glDrawPatch(int lvl,Patch& p)
{
  double c[4][3];
  for (int i=0;i<4;i++) for (int j=0;j<3;j++) c[i][j] = 255.;
  if (p.nextp != NULL)
    {
      for (int i=0;i<4;i++)
	glDrawPatch(2*lvl,p.nextp[i]);
    }
  else
    {
      if (GlobalData::getVisualizedModel() == TUNNELMODEL_INITIAL) {
	glBegin(GL_POLYGON);
	glVertex3f(p.P1.x,p.P1.y,p.P1.z);
	glVertex3f(p.P2.x,p.P2.y,p.P2.z);
	glVertex3f(p.P3.x,p.P3.y,p.P3.z);
	glVertex3f(p.P4.x,p.P4.y,p.P4.z);
	glEnd();
      } else {
	if (p.s->type_surface != TUNNELSURFACE_LIGHT)
	  {
	    int M = (int)(GlobalData::getSimulationParameters()._finalstepParameters._waveletBasis);
	    double Area = (int)(GlobalData::getSimulationParameters()._finalstepParameters._patchAreaThreshold);
// 	    double r,g,b;
// 	    p.getColor(M,lvl,(p.min_u+p.max_u)*0.5,(p.min_v+p.max_v)*0.5,&r,&g,&b);
// 	    if ((fabs(r) < epsilonValue) && (fabs(g) < epsilonValue) && (fabs(b) < epsilonValue))
// 	      {
// 		SPECTRUMtoRGB(p.s->rho,&r,&g,&b);
// 		r /= 255.; g /= 255.; b /= 255.;
// 		glPolygonMode(GL_BACK,GL_LINE);
// 		glPolygonMode(GL_FRONT,GL_LINE);
// 		glColor3f(r,g,b);
// 		glBegin(GL_POLYGON);
// 		glVertex3f(p.P1.x,p.P1.y,p.P1.z);
// 		glVertex3f(p.P2.x,p.P2.y,p.P2.z);
// 		glVertex3f(p.P3.x,p.P3.y,p.P3.z);
// 		glVertex3f(p.P4.x,p.P4.y,p.P4.z);
// 		glEnd();
// 	      }
// 	    else
// 	      {
		double S1[maxNumberOfPointsInSpectrum];
		double S2[maxNumberOfPointsInSpectrum];
		double S3[maxNumberOfPointsInSpectrum];
		double S4[maxNumberOfPointsInSpectrum];
		computeUVspectrum(M,&p,lvl,0,0,S1,p.area);  /* Pour le point P1 */
		computeUVspectrum(M,&p,lvl,1,0,S2,p.area);  /* Pour le point P2 */
		computeUVspectrum(M,&p,lvl,1,1,S3,p.area);  /* Pour le point P3 */
		computeUVspectrum(M,&p,lvl,0,1,S4,p.area);  /* Pour le point P4 */
		SPECTRUMtoCorrectedRGB(S1,&(c[0][0]),&(c[0][1]),&(c[0][2]));
		SPECTRUMtoCorrectedRGB(S2,&(c[1][0]),&(c[1][1]),&(c[1][2]));
		SPECTRUMtoCorrectedRGB(S3,&(c[2][0]),&(c[2][1]),&(c[2][2]));
		SPECTRUMtoCorrectedRGB(S4,&(c[3][0]),&(c[3][1]),&(c[3][2]));
		glPolygonMode(GL_BACK,GL_LINE);
		glPolygonMode(GL_FRONT,GL_FILL);
		glBegin(GL_POLYGON);
		glColor3f(c[0][0]/255.,c[0][1]/255.,c[0][2]/255.);
		glVertex3f(p.P1.x,p.P1.y,p.P1.z);
		glColor3f(c[1][0]/255.,c[1][1]/255.,c[1][2]/255.);
		glVertex3f(p.P2.x,p.P2.y,p.P2.z);
		glColor3f(c[2][0]/255.,c[2][1]/255.,c[2][2]/255.);
		glVertex3f(p.P3.x,p.P3.y,p.P3.z);
		glColor3f(c[3][0]/255.,c[3][1]/255.,c[3][2]/255.);
		glVertex3f(p.P4.x,p.P4.y,p.P4.z);
		glEnd();
// 	      }
	  }
	else
	  {
	    glPolygonMode(GL_BACK,GL_LINE);
	    glPolygonMode(GL_FRONT,GL_FILL);
	    glColor3f(1.,1.,1.);
	    glBegin(GL_POLYGON);
	    glVertex3f(p.P1.x,p.P1.y,p.P1.z);
	    glVertex3f(p.P2.x,p.P2.y,p.P2.z);
	    glVertex3f(p.P3.x,p.P3.y,p.P3.z);
	    glVertex3f(p.P4.x,p.P4.y,p.P4.z);
	    glEnd();
	  }
      }
    }
}

void FlGlVisuScene::glDrawSurface(Surface& S)
{
  if (S._splitting) {
    for (int i=0;i<S._splitting;i++)
      glDrawSurface(*(S._splitted[i]));
  } else {
  if (GlobalData::getVisualizedModel() == TUNNELMODEL_INITIAL)
    {
      glPolygonMode(GL_BACK,GL_LINE);
      glPolygonMode(GL_FRONT,GL_FILL);
      double r,g,b;
      switch ((enum eTunnelSurface)(S.type_surface))
	{
	case TUNNELSURFACE_FLOOR:
	case TUNNELSURFACE_WALL:
	case TUNNELSURFACE_ROOF:
	  SPECTRUMtoRGB(S.rho,&r,&g,&b);
	  r /= 255.; g /= 255.; b /= 255.;
	  break;
	case TUNNELSURFACE_LIGHT:
	  glColor3f(1.,1.,1.);
	  break;
	}
    }
  else if (GlobalData::getVisualizedModel() == TUNNELMODEL_SOLUTION)
    {
      if (GlobalData::getSimulationParameters()._finalstepParameters._simulationMode == SIMULATIONMODE_GATHERING)
	{
	    if (S.type_surface == TUNNELSURFACE_LIGHT) glColor3f(1.,1.,1.);
	    glDrawPatch(1,S.p);
	}
      else
	{
	  if ((&S == GlobalData::is_Ssou) ||
	      ((&S == GlobalData::is_seen[GlobalData::is_crtseen]) && (GlobalData::is_etape == GlobalData::is_shoot)) ||
	      ((&S == GlobalData::is_crtS) && (GlobalData::is_etape == GlobalData::is_linkshoot))) {
	    glColor3f(((&S == GlobalData::is_crtS) && (GlobalData::is_etape == GlobalData::is_linkshoot)),
		      (&S == GlobalData::is_Ssou),
		      ((&S == GlobalData::is_seen[GlobalData::is_crtseen]) && (GlobalData::is_etape == GlobalData::is_shoot)));
	    glPolygonMode(GL_BACK,GL_LINE);
	    glPolygonMode(GL_FRONT,GL_FILL);
	    glBegin(GL_POLYGON);
	    glVertex3f(S.P1.x,S.P1.y,S.P1.z);
	    glVertex3f(S.P2.x,S.P2.y,S.P2.z);
	    glVertex3f(S.P3.x,S.P3.y,S.P3.z);
	    glVertex3f(S.P4.x,S.P4.y,S.P4.z);
	    glEnd();
	  } else {
	    if (S.type_surface == TUNNELSURFACE_LIGHT) glColor3f(1.,1.,1.);
	    glDrawPatch(1,S.p);
	  }
	}
    }
  }
}

void FlGlVisuScene::glDrawPolygon(PolygonNFF& pol)
{
  switch (pol.getType())
    {
    case TUNNELSURFACE_FLOOR:
      glColor3f(1.,0.,0.);
      break;
    case TUNNELSURFACE_WALL:
      glColor3f(0.,1.,0.);
      break;
    case TUNNELSURFACE_ROOF:
      glColor3f(0.,0.,1.);
      break;
    case TUNNELSURFACE_LIGHT:
      glColor3f(1.,1.,1.);
      break;
    }
  glBegin(GL_POLYGON);
  for (int j=pol.card();j>=1;j--)
    {
      PointNFF& pt = pol[j];
      glVertex3f(pt[0],pt[1],pt[2]);
    }
  glEnd();
}

/*
 * Private Methods
 */

void FlGlVisuScene::moveTo(double xx,double yy)
{
  double xr,yr;
  WINTONORMALIZED2DCOORDINATES(_referenceMouseX,_referenceMouseY,0.,0.,w(),h(),xr,yr);
  double xm,ym;
  WINTONORMALIZED2DCOORDINATES(xx,yy,0.,0.,w(),h(),xm,ym);
  double dx,dy;
  dx = xm - xr; dy = ym - yr;
  _camera.goTo(dx,dy);
}

void FlGlVisuScene::rotateTo(double xx,double yy)
{
  double xr,yr;
  WINTONORMALIZED2DCOORDINATES(_referenceMouseX,_referenceMouseY,0.,0.,w(),h(),xr,yr);
  double xm,ym;
  WINTONORMALIZED2DCOORDINATES(xx,yy,0.,0.,w(),h(),xm,ym);
  double dx,dy;
  dx = xm - xr; dy = ym - yr;
  _camera.lookAt(dx,dy);
}

void FlGlVisuScene::straffTo(double xx,double yy)
{
  double xr,yr;
  WINTONORMALIZED2DCOORDINATES(_referenceMouseX,_referenceMouseY,0.,0.,w(),h(),xr,yr);
  double xm,ym;
  WINTONORMALIZED2DCOORDINATES(xx,yy,0.,0.,w(),h(),xm,ym);
  double dx,dy;
  dx = xm - xr; dy = ym - yr;
  _camera.straffTo(dx,dy);
}

void FlGlVisuScene::movingIdle_i()
{
  redraw();
}
void FlGlVisuScene::rotatorIdle_i()
{
  redraw();
}
void FlGlVisuScene::straffIdle_i()
{
  redraw();
}

void FlGlVisuScene::movingIdle(void* data)
{
  (static_cast< FlGlVisuScene* >(data))->movingIdle_i();
}
void FlGlVisuScene::rotatorIdle(void* data)
{
  (static_cast< FlGlVisuScene* >(data))->rotatorIdle_i();
}
void FlGlVisuScene::straffIdle(void* data)
{
  (static_cast< FlGlVisuScene* >(data))->straffIdle_i();
}
