#include "gui/FlGlVisuSource.h"
#include "gui/FlEvents.h"
#include "gui/FlSelectSource.h"

#include "others/GlobalData.h"
#include "data/LightSource.h"

GLdouble sourceColor[40][3]={{0.5, 0.0, 0.5}, {0.3, 0.0, 0.55}, {0.1, 0.0, 0.6}, {0.0, 0.0, 0.65}, 
			{0.0, 0.0, 0.7}, {0.0, 0.0, 0.75}, {0.0, 0.0, 0.8}, {0.0, 0.0, 0.85}, 
			{0.0, 0.0, 1.0}, {0.0, 0.14, 0.86}, {0.0, 0.28, 0.72}, {0.0, 0.42, 0.58}, 
			{0.0, 0.56, 0.44}, {0.0, 0.7, 0.3}, {0.0, 0.84, 0.16}, {0.0, 0.84, 0.0}, 
			{0.0, 0.84, 0.0}, {0.17, 0.84, 0.0}, {0.34, 0.84, 0.0}, {0.51, 0.84, 0.0}, 
			{0.68, 0.84, 0.0}, {0.85, 0.84, 0.0}, {0.85, 0.84, 0.0}, {0.86, 0.7, 0.0}, 
			{0.87, 0.56, 0.0}, {0.88, 0.42, 0.0}, {0.89, 0.28, 0.0}, {0.90, 0.14, 0.0}, 
			{0.91, 0.0, 0.0}, {0.92, 0.0, 0.0}, {0.93, 0.0, 0.0}, {0.94, 0.0, 0.0}, 
			{0.95, 0.0, 0.0}, {0.96, 0.0, 0.0}, {0.97, 0.0, 0.0}, {0.98, 0.0, 0.0}, 
			{0.99, 0.0, 0.0}, {1.0, 0.0, 0.0}, {1.0, 0.0, 0.0}, {1.0, 0.0, 0.0}};

FlGlVisuSource::FlGlVisuSource(const int& xx,const int& yy,const int& w,const int& h,const char* title = NULL) :
  Fl_Gl_Window(xx,yy,w,h,title) ,
  _firstTimeDrawing(1) ,
  _visuMode(VISUSOURCE_GEOMETRY) ,
  _axesDrawingList(0) ,
  _sourceDrawingList(0) ,
  _clickedAbcisse(0) ,
  _sourceSelector(NULL) ,
  _thetacam(45.) ,
  _rhocam(45.) ,
  _fovcam(27.)
{
}

FlGlVisuSource::~FlGlVisuSource()
{
  if (glIsList(_axesDrawingList)) glDeleteLists(_axesDrawingList,1);
  if (glIsList(_sourceDrawingList)) glDeleteLists(_sourceDrawingList,1);
}

void FlGlVisuSource::draw()
{
  if (_firstTimeDrawing) { initializeAxesDrawingList(); initializeSourceDrawingList(); _firstTimeDrawing = 0; }
  /* initialisation de GL */
  glClearColor(1.,1.,1.,1.);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0,0,w(),h());
  switch (_visuMode)
    {
    case VISUSOURCE_PHOTOMETRY:
      glOrtho(-1.,1.,-1.,1.,-1.,1.);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      break;
    case VISUSOURCE_GEOMETRY:
      gluPerspective(_fovcam,((double)w())/h(),0.05,1.e6);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      gluLookAt(500.*cos(M_PI*_thetacam/180.),500.*sin(M_PI*_thetacam/180.),128.+500.*sin(M_PI*_rhocam/180.),0.,0.,128.,0.,0.,1.);
      break;
    }
  if (glIsList(_axesDrawingList)) { glCallList(_axesDrawingList); }
  if (glIsList(_sourceDrawingList)) { glCallList(_sourceDrawingList); }
}

void FlGlVisuSource::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Gl_Window::resize(xx,yy,ww,hh);
}

double px,py;

int FlGlVisuSource::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case FL_PUSH:
      if (Fl::event_button() == FL_LEFT_MOUSE)
	{
	  switch (_visuMode)
	    {
	    case VISUSOURCE_PHOTOMETRY:
	      _clickedAbcisse = Fl::event_x();
	      _sourceSelector->handle(TUNNEL_VIEWERCLICKED);
	      break;
	    case VISUSOURCE_GEOMETRY:
	      px = Fl::event_x();
	      py = Fl::event_y();
	      break;
	    }
	}
      else if (Fl::event_button() == FL_MIDDLE_MOUSE)
	{
	  switch (_visuMode)
	    {
	    case VISUSOURCE_PHOTOMETRY:
	      break;
	    case VISUSOURCE_GEOMETRY:
	      px = Fl::event_x();
	      py = Fl::event_y();
	      break;
	    }
	}
      res = 1;
      break;
    case FL_DRAG:
      if (Fl::event_button() == FL_LEFT_MOUSE)
	{
	  switch (_visuMode)
	    {
	    case VISUSOURCE_PHOTOMETRY:
	      _clickedAbcisse = Fl::event_x();
	      _sourceSelector->handle(TUNNEL_VIEWERCLICKED);
	      break;
	    case VISUSOURCE_GEOMETRY:
	      _thetacam = (_thetacam + (px - Fl::event_x()));
	      while (_thetacam > 359.) _thetacam-=359.;
	      while (_thetacam < 0.) _thetacam+=359.;
	      _rhocam = (_rhocam + (Fl::event_y() - py));
	      if (_rhocam > 89.) _rhocam=89.;
	      if (_rhocam < 0.) _rhocam=0.;
	      px = Fl::event_x();
	      py = Fl::event_y();
	      redraw();
	      break;
	    }
	}
      else if (Fl::event_button() == FL_MIDDLE_MOUSE)
	{
	  switch (_visuMode)
	    {
	    case VISUSOURCE_PHOTOMETRY:
	      break;
	    case VISUSOURCE_GEOMETRY:
	      _fovcam = _fovcam + (Fl::event_y() - py);
	      if (_fovcam >159.) _fovcam = 159.;
	      if (_fovcam < 27.) _fovcam = 27.;
	      px = Fl::event_x();
	      py = Fl::event_y();
	      redraw();
	      break;
	    }
	}
      res = 1;
      break;
    case FL_SHORTCUT:
    case FL_KEYBOARD:
      if (Fl::event_key() == FL_Tab)
	{
	  _visuMode = (_visuMode == VISUSOURCE_PHOTOMETRY ? VISUSOURCE_GEOMETRY : VISUSOURCE_PHOTOMETRY);
	  handle(TUNNEL_DATACHANGED);
	}
      res = 1;
      break;
    case TUNNEL_DATACHANGED:
      initializeAxesDrawingList();
      initializeSourceDrawingList();
      redraw();
      res = 1;
      break;
    case TUNNEL_NEEDTOREDRAW:
      redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Gl_Window::handle(event);
  return res;
}

void FlGlVisuSource::link(FlSelectSource* selector)
{
  _sourceSelector = selector;
}

int FlGlVisuSource::getAbcisse()
{
  return _clickedAbcisse;
}

/*
 * private methods
 */

void FlGlVisuSource::initializeAxesDrawingList()
{
  if (glIsList(_axesDrawingList)) glDeleteLists(_axesDrawingList,1);
  _axesDrawingList = glGenLists(1);
  glNewList(_axesDrawingList,GL_COMPILE);
  switch (_visuMode)
    {
    case VISUSOURCE_PHOTOMETRY:
      glColor3f(0.5,0.5,0.5);
      glBegin(GL_LINES);
      for (int i = 0; i < 18; i++)
	{
	  glVertex2f(-0.8 + (0.1 * i), -1.0); glVertex2f(-0.8 + (0.1 * i), 1.0);
	  glVertex2f(-1.0, -0.8 + (0.1 * i)); glVertex2f(1.0, -0.8 + (0.1 * i));
	}
      glEnd();
      glColor3f(0.,0.,1.);
      glBegin(GL_LINES);
      glVertex2f(-1.0, -0.9); glVertex2f(1.0, -0.9);
      glVertex2f(-0.9, -1.0); glVertex2f(-0.9, 1.0);
      glEnd();
      break;
    case VISUSOURCE_GEOMETRY:
      glColor3f(0.,0.,1.);
      glBegin(GL_LINES);
      glVertex3f(0.,0.,0.); glVertex3f(255.,0.,0.);
      glVertex3f(0.,0.,0.); glVertex3f(0.,255.,0.);
      glVertex3f(0.,0.,0.); glVertex3f(0.,0.,255.);
      glEnd();
      break;
    }
  glEndList();
}

void FlGlVisuSource::initializeSourceDrawingList()
{
  if (glIsList(_sourceDrawingList)) glDeleteLists(_sourceDrawingList,1);
  const sLightSource& ls = GlobalData::getLightSource();
  double mx = 0.;
  for (int i=0;i<ls._spectrum._nbvalues;i++)
    if (ls._spectrum._values[1][i] > mx) mx = ls._spectrum._values[1][i];
  _sourceDrawingList = glGenLists(1);
  glNewList(_sourceDrawingList,GL_COMPILE);
  switch (_visuMode)
    {
    case VISUSOURCE_PHOTOMETRY:
      for (int i=0;i<ls._spectrum._nbvalues;i++)
	{
	  double xx = (ls._spectrum._values[0][i]-370.)/10.;
	  double yy0 = (i == 0 ? -0.9 : (ls._spectrum._values[1][i-1]*1.9/mx) - 0.9);
	  double yyn = (ls._spectrum._values[1][i]*1.9/mx) - 0.9;
	  glBegin(GL_POLYGON);
	  glColor3f(sourceColor[(int)xx][0],sourceColor[(int)xx][1],sourceColor[(int)xx][2]);
	  glVertex2f(-0.9 + ((xx - 1) * 0.05), -0.9);
	  glVertex2f(-0.9 + ((xx - 1) * 0.05), yy0);
	  glColor3f(sourceColor[((int)xx) + 1][0],sourceColor[((int)xx) + 1][1],sourceColor[((int)xx) + 1][2]);
	  glVertex2f(-0.9 + (xx * 0.05), yyn);
	  glVertex2f(-0.9 + (xx * 0.05), -0.9);
	  glEnd();
	}
      break;
    case VISUSOURCE_GEOMETRY:
      for (int idplan0 = 0;idplan0 < ls._nbPlans;idplan0++)
	{
	  double rho0 = ls._firstPlan + (idplan0 * ls._stepPlans);
	  double crho0 = cos(M_PI * rho0 / 180.);
	  double srho0 = sin(M_PI * rho0 / 180.);
	  glBegin(GL_LINE_STRIP);
	  for (int iddir0 = 0;iddir0 < (ls._nbDir-1);iddir0++)
	    {
	      double theta0 = 90. - iddir0 * ls._stepDir;
	      double ctheta0 = cos(M_PI * theta0 / 180.);
	      double stheta0 = sin(M_PI * theta0 / 180.);
	      double v00 = ls._values[idplan0][iddir0];
	      glColor3f(fabs(ctheta0),fabs(stheta0),v00/255.);
	      glVertex3f(crho0 * ctheta0 * v00 , srho0 * ctheta0 * v00 , stheta0 * v00);
	    }
	  for (int iddir0 = 0;iddir0 < (ls._nbDir-1);iddir0++)
	    {
	      double theta0 = 90. - iddir0 * ls._stepDir;
	      double ctheta0 = cos(M_PI * theta0 / 180.);
	      double stheta0 = sin(M_PI * theta0 / 180.);
	      double v00 = ls._values[idplan0][iddir0];
	      glColor3f(fabs(ctheta0),fabs(stheta0),v00/255.);
	      glVertex3f((0. - crho0) * ctheta0 * v00 , srho0 * ctheta0 * v00 , stheta0 * v00);
	    }
	  glEnd();
	}
      break;
    }
  glEndList();
}
