#include "gui/FlGlVisuSpectre.h"
#include "gui/FlEvents.h"
#include "gui/FlSelectSpectre.h"

#include "others/GlobalData.h"
#include "data/MaterialReflectanceSpectrum.h"

GLdouble spectrumColor[40][3]={{0.5, 0.0, 0.5}, {0.3, 0.0, 0.55}, {0.1, 0.0, 0.6}, {0.0, 0.0, 0.65}, 
			{0.0, 0.0, 0.7}, {0.0, 0.0, 0.75}, {0.0, 0.0, 0.8}, {0.0, 0.0, 0.85}, 
			{0.0, 0.0, 1.0}, {0.0, 0.14, 0.86}, {0.0, 0.28, 0.72}, {0.0, 0.42, 0.58}, 
			{0.0, 0.56, 0.44}, {0.0, 0.7, 0.3}, {0.0, 0.84, 0.16}, {0.0, 0.84, 0.0}, 
			{0.0, 0.84, 0.0}, {0.17, 0.84, 0.0}, {0.34, 0.84, 0.0}, {0.51, 0.84, 0.0}, 
			{0.68, 0.84, 0.0}, {0.85, 0.84, 0.0}, {0.85, 0.84, 0.0}, {0.86, 0.7, 0.0}, 
			{0.87, 0.56, 0.0}, {0.88, 0.42, 0.0}, {0.89, 0.28, 0.0}, {0.90, 0.14, 0.0}, 
			{0.91, 0.0, 0.0}, {0.92, 0.0, 0.0}, {0.93, 0.0, 0.0}, {0.94, 0.0, 0.0}, 
			{0.95, 0.0, 0.0}, {0.96, 0.0, 0.0}, {0.97, 0.0, 0.0}, {0.98, 0.0, 0.0}, 
			{0.99, 0.0, 0.0}, {1.0, 0.0, 0.0}, {1.0, 0.0, 0.0}, {1.0, 0.0, 0.0}};

FlGlVisuSpectre::FlGlVisuSpectre(const int& xx,const int& yy,const int& w,const int& h,const char* title = NULL) :
  Fl_Gl_Window(xx,yy,w,h,title) ,
  _firstTimeDrawing(1) ,
  _axesDrawingList(0) ,
  _spectreDrawingList(0) ,
  _clickedAbcisse(0) ,
  _spectreSelector(NULL)
{
}

FlGlVisuSpectre::~FlGlVisuSpectre()
{
  if (glIsList(_axesDrawingList)) glDeleteLists(_axesDrawingList,1);
  if (glIsList(_spectreDrawingList)) glDeleteLists(_spectreDrawingList,1);
}

void FlGlVisuSpectre::draw()
{
  if (_firstTimeDrawing) { initializeAxesDrawingList(); initializeSpectrumDrawingList(); _firstTimeDrawing = 0; }
  /* initialisation de GL */
  glClearColor(1.,1.,1.,1.);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1.,1.,-1.,1.,-1.,1.);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  if (glIsList(_axesDrawingList)) { glCallList(_axesDrawingList); }
  if (glIsList(_spectreDrawingList)) { glCallList(_spectreDrawingList); }
}

void FlGlVisuSpectre::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Gl_Window::resize(xx,yy,ww,hh);
}

int FlGlVisuSpectre::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case FL_PUSH:
    case FL_DRAG:
      if (Fl::event_button() == FL_LEFT_MOUSE)
	{
	  _clickedAbcisse = Fl::event_x();
	  _spectreSelector->handle(TUNNEL_VIEWERCLICKED);
	}
      res = 1;
      break;
    case TUNNEL_DATACHANGED:
      initializeSpectrumDrawingList();
      redraw();
      res = 1;
      break;
    case TUNNEL_NEEDTOREDRAW:
      redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Gl_Window::handle(event);
  return res;
}

void FlGlVisuSpectre::link(FlSelectSpectre* selector)
{
  _spectreSelector = selector;
}

int FlGlVisuSpectre::getAbcisse()
{
  return _clickedAbcisse;
}

/*
 * private methods
 */

void FlGlVisuSpectre::initializeAxesDrawingList()
{
  if (glIsList(_axesDrawingList)) glDeleteLists(_axesDrawingList,1);
  _axesDrawingList = glGenLists(1);
  glNewList(_axesDrawingList,GL_COMPILE);
  glColor3f(0.5,0.5,0.5);
  glBegin(GL_LINES);
  for (int i = 0; i < 18; i++)
    {
      glVertex2f(-0.8 + (0.1 * i), -1.0); glVertex2f(-0.8 + (0.1 * i), 1.0);
      glVertex2f(-1.0, -0.8 + (0.1 * i)); glVertex2f(1.0, -0.8 + (0.1 * i));
    }
  glEnd();
  glColor3f(0.,0.,1.);
  glBegin(GL_LINES);
  glVertex2f(-1.0, -0.9); glVertex2f(1.0, -0.9);
  glVertex2f(-0.9, -1.0); glVertex2f(-0.9, 1.0);
  glEnd();
  glEndList();
}

void FlGlVisuSpectre::initializeSpectrumDrawingList()
{
  if (glIsList(_spectreDrawingList)) glDeleteLists(_spectreDrawingList,1);
  const sMaterialReflectanceSpectrum& mrs = GlobalData::getMaterialReflectanceSpectrum();
  if (mrs._spectrumDataSize > 0)
    {
      _spectreDrawingList = glGenLists(1);
      glNewList(_spectreDrawingList,GL_COMPILE);
      for (int i=0;i<mrs._spectrumDataSize;i++)
	{
	  double xx = (mrs._spectrumWavelength[i]-370.)/10.;
	  double yy0 = (i == 0 ? -0.9 : (mrs._spectrumValue[i-1]*1.9) - 0.9);
	  double yyn = (mrs._spectrumValue[i]*1.9) - 0.9;
	  glBegin(GL_POLYGON);
	  glColor3f(spectrumColor[(int)xx][0],spectrumColor[(int)xx][1],spectrumColor[(int)xx][2]);
	  glVertex2f(-0.9 + ((xx - 1) * 0.05), -0.9);
	  glVertex2f(-0.9 + ((xx - 1) * 0.05), yy0);
	  glColor3f(spectrumColor[((int)xx) + 1][0],spectrumColor[((int)xx) + 1][1],spectrumColor[((int)xx) + 1][2]);
	  glVertex2f(-0.9 + (xx * 0.05), yyn);
	  glVertex2f(-0.9 + (xx * 0.05), -0.9);
	  glEnd();
	}
      glEndList();
    }
}
