#include "gui/FlMainWindow.h"
#include "gui/FlEvents.h"
#include "gui/FlSimulationParameters.h"
#include "gui/FlGenerateTunnel.h"
#include "gui/FlManageLightList.h"
#include "gui/FlManageMaterialList.h"
#include "gui/FlSelectResolution.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"

#include "recherche/Recherche.h"

#define LabelsStartIndex 30

FlMainWindow* FlMainWindow::_windowPointer = (FlMainWindow*)NULL;

#define SelectionWindowHeight 80
#define SelectionWindowWidth  200
#define VisualizationWindowHeight 120
#define VisualizationWindowWidth  200
#define SceneWindowHeight 3*(SelectionWindowHeight+VisualizationWindowHeight)
#define SceneWindowWidth  600
#define MainWindowHeight SceneWindowHeight+20
#define MainWindowWidth SceneWindowWidth+VisualizationWindowWidth

FlMainWindow::FlMainWindow() :
  Fl_Window(MainWindowWidth,MainWindowHeight,Parameters::_programName)
{
  box(FL_UP_BOX);
  labelsize(12);
  user_data((void*)(this));
  resizable(NULL);

  _menuBar = new Fl_Menu_Bar(0,0,MainWindowWidth,20);
  _visuScene = new FlGlVisuScene(0,20,SceneWindowWidth,SceneWindowHeight);
  _selectCourbe = new FlSelectCourbe(SceneWindowWidth+1,20,SelectionWindowWidth,SelectionWindowHeight);
  _visuCourbe = new FlGlVisuCourbe(SceneWindowWidth+1,20+SelectionWindowHeight,VisualizationWindowWidth,VisualizationWindowHeight);
  _selectSpectre = new FlSelectSpectre(SceneWindowWidth+1,20+SelectionWindowHeight+VisualizationWindowHeight,SelectionWindowWidth,SelectionWindowHeight);
  _visuSpectre = new FlGlVisuSpectre(SceneWindowWidth+1,20+2*SelectionWindowHeight+VisualizationWindowHeight,VisualizationWindowWidth,VisualizationWindowHeight);
  _selectSource = new FlSelectSource(SceneWindowWidth+1,20+2*(SelectionWindowHeight+VisualizationWindowHeight),SelectionWindowWidth,SelectionWindowHeight);
  _visuSource = new FlGlVisuSource(SceneWindowWidth+1,20+3*SelectionWindowHeight+2*VisualizationWindowHeight,VisualizationWindowWidth,VisualizationWindowHeight);

  _selectCourbe->link(_visuCourbe);
  _selectSpectre->link(_visuSpectre);
  _selectSource->link(_visuSource);

  _visuCourbe->link(_selectCourbe);
  _visuSpectre->link(_selectSpectre);
  _visuSource->link(_selectSource);

  end();

  initializeMenu(_menuBar);

  _windowPointer = this;
  quit = 0;
}

FlMainWindow::~FlMainWindow()
{
}

void FlMainWindow::draw()
{
  Fl_Window::draw();
}

void FlMainWindow::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Window::resize(xx,yy,ww,hh);
}

int FlMainWindow::handle(int event)
{
  if (((event == FL_KEYBOARD) || (event == FL_SHORTCUT)) && (Fl::event_key() == FL_Escape)) return 1;
  return Fl_Window::handle(event);
}

/*
 *
 * menu File
 *
 */

void FlMainWindow::cb_import_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_resolutionSearchDirectory,getFilter(Parameters::_resolutionSearchFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::loadResolutionSearchResult(tmp);
      initializeMenu(_menuBar);
      _visuCourbe->handle(TUNNEL_DATACHANGED);
      _visuScene->handle(TUNNEL_DATACHANGED);
      redraw();
    }
  delete fc;
}
void FlMainWindow::cb_export_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_resolutionSearchDirectory,getFilter(Parameters::_resolutionSearchFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveResolutionSearchResult(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_export3d_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_geometricDataDirectory,getFilter(Parameters::_geometricDataFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      switch (GlobalData::getVisualizedModel())
	{
	case TUNNELMODEL_NFF:
	  break;
	case TUNNELMODEL_INITIAL:
	  GlobalData::saveMaillage3d(tmp);
	  break;
	case TUNNELMODEL_SOLUTION:
	  GlobalData::saveMaillageSolution(tmp);
	  break;
	}
    }
  delete fc;
}
void FlMainWindow::cb_quit_i(Fl_Widget* o, void* v)
{
  quit = 1;
  hide();
}
void FlMainWindow::cb_import(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_import_i(o,v);
}
void FlMainWindow::cb_export(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_export_i(o,v);
}
void FlMainWindow::cb_export3d(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_export3d_i(o,v);
}
void FlMainWindow::cb_quit(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_quit_i(o,v);
}

/*
 *
 * menu Simulation
 *
 */

void FlMainWindow::cb_simulationparameters_i(Fl_Widget* o, void* v)
{
  FlSimulationParameters* win = new FlSimulationParameters();
  win->show();
  while (!(win->clicked())) Fl::wait();
  delete win;
  initializeMenu(_menuBar);
}
void FlMainWindow::cb_launchsimulation_i(Fl_Widget* o, void* v)
{
  GlobalData::lancerRecherche();
  initializeMenu(_menuBar);
  redraw();
}
void FlMainWindow::cb_visualize_i(Fl_Widget* o, void* v)
{
  if (((int)v) == TUNNELMODEL_SOLUTION)
    {
      FlSelectResolution* win = new FlSelectResolution();
      win->show();
      while (!(win->clicked())) Fl::wait();
      int res = (win->clicked() >= 1);
      int nb = (win->clicked() - 1);
      delete win;
      if (res)
	{
	  GlobalData::setVisualizedModel(TUNNELMODEL_SOLUTION);
	  Parameters::_solutionNumber = nb;
	  _visuScene->handle(TUNNEL_DATACHANGED);
	  initializeMenu(_menuBar);
	}
    }
  else
    {
      GlobalData::setVisualizedModel((enum eTunnelModel)((int)v));
      _visuScene->handle(TUNNEL_DATACHANGED);
    }
}
static int is_shooting = 0;
void FlMainWindow::cb_shootlaunch_i(Fl_Widget* o, void* v)
{
  if (!is_shooting) { Fl::add_idle(cb_shootidle,NULL); is_shooting = 1; }
  else { Fl::remove_idle(cb_shootidle,NULL); is_shooting = 0; }
}
void FlMainWindow::cb_shootidle_i(void* v)
{
  if (GlobalData::oneStepInteractiveShoot()) { Fl::remove_idle(cb_shootidle,NULL); is_shooting = 0; }
  Fl::handle(TUNNEL_NEEDTOREDRAW,_visuScene);
}
void FlMainWindow::cb_simulationparameters(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_simulationparameters_i(o,v);
}
void FlMainWindow::cb_launchsimulation(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_launchsimulation_i(o,v);
}
void FlMainWindow::cb_visualize(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_visualize_i(o,v);
}
void FlMainWindow::cb_shootlaunch(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_shootlaunch_i(o,v);
}
void FlMainWindow::cb_shootidle(void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_shootidle_i(v);
}

/*
 *
 * menu Tunnel
 *
 */

void FlMainWindow::cb_newtunnel_i(Fl_Widget* o, void* v)
{
  FlGenerateTunnel* win = new FlGenerateTunnel();
  win->show();
  while (!(win->clicked())) Fl::wait();
  if (win->clicked() == 1) _visuScene->handle(TUNNEL_DATACHANGED);
  delete win;
}
void FlMainWindow::cb_loadtunnel_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_tunnelParametersDirectory,getFilter(Parameters::_tunnelParametersFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+0]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::loadTunnelParameters(tmp);
      _visuScene->handle(TUNNEL_DATACHANGED);
    }
  delete fc;
}
void FlMainWindow::cb_savetunnel_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_tunnelParametersDirectory,getFilter(Parameters::_tunnelParametersFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+1]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveTunnelParameters(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_newtunnel(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_newtunnel_i(o,v);
}
void FlMainWindow::cb_loadtunnel(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_loadtunnel_i(o,v);
}
void FlMainWindow::cb_savetunnel(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_savetunnel_i(o,v);
}

/*
 *
 * menu Source
 * Lights description list manager
 *
 */
void FlMainWindow::cb_loadlights_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_lightSourceListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+2]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::loadLightList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_savelights_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_lightSourceListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+3]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveLightList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_listlights_i(Fl_Widget* o, void* v)
{
  FlManageLightList* win = new FlManageLightList();
  win->show();
  while (!(win->clicked())) Fl::wait();
  delete win;
}
void FlMainWindow::cb_loadlights(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_loadlights_i(o,v);
}
void FlMainWindow::cb_savelights(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_savelights_i(o,v);
}
void FlMainWindow::cb_listlights(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_listlights_i(o,v);
}

/*
 *
 * Floors material list manager
 *
 */
void FlMainWindow::cb_loadfloors_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+4]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::loadFloorList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_savefloors_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveFloorList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_listfloors_i(Fl_Widget* o, void* v)
{
  FlManageMaterialList* win = new FlManageMaterialList(TUNNELSURFACE_FLOOR);
  win->show();
  while (!(win->clicked())) Fl::wait();
  delete win;
}
void FlMainWindow::cb_loadfloors(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_loadfloors_i(o,v);
}
void FlMainWindow::cb_savefloors(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_savefloors_i(o,v);
}
void FlMainWindow::cb_listfloors(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_listfloors_i(o,v);
}

/*
 *
 * Walls material list manager
 *
 */
void FlMainWindow::cb_loadwalls_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+6]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::loadWallList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_savewalls_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+7]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveWallList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_listwalls_i(Fl_Widget* o, void* v)
{
  FlManageMaterialList* win = new FlManageMaterialList(TUNNELSURFACE_WALL);
  win->show();
  while (!(win->clicked())) Fl::wait();
  delete win;
}
void FlMainWindow::cb_loadwalls(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_loadwalls_i(o,v);
}
void FlMainWindow::cb_savewalls(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_savewalls_i(o,v);
}
void FlMainWindow::cb_listwalls(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_listwalls_i(o,v);
}

/*
 *
 * Roofs material list manager
 *
 */
void FlMainWindow::cb_loadroofs_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+8]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::loadRoofList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_saveroofs_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_listFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+9]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveRoofList(tmp);
    }
  delete fc;
}
void FlMainWindow::cb_listroofs_i(Fl_Widget* o, void* v)
{
  FlManageMaterialList* win = new FlManageMaterialList(TUNNELSURFACE_ROOF);
  win->show();
  while (!(win->clicked())) Fl::wait();
  delete win;
}
void FlMainWindow::cb_loadroofs(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_loadroofs_i(o,v);
}
void FlMainWindow::cb_saveroofs(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_saveroofs_i(o,v);
}
void FlMainWindow::cb_listroofs(Fl_Widget* o, void* v)
{
  if (_windowPointer != NULL) _windowPointer->cb_listroofs_i(o,v);
}

void FlMainWindow::setDefaultsParameters()
{
}

int FlMainWindow::quitter()
{
  return quit;
}
