#include "gui/FlManageLightList.h"
#include "gui/FlEvents.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"

#define LabelStartIndex 40

FlManageLightList::FlManageLightList() :
  Fl_Window(300,300,labels[LabelStartIndex+0]) ,
  lightlist(GlobalData::getLightList())
{
  box(FL_UP_BOX);
  labelsize(12);
  user_data((void*)(this));
  resizable(NULL);

  {
    Fl_Select_Browser* o = _lightList = new Fl_Select_Browser(5,5,150,245,labels[LabelStartIndex+1]);
    for (cListIterator< sLightSource* > iter = lightlist;!iter.finished();++iter)
      {
	sLightSource* crt = iter.getCurrent();
	o->add(crt->_name,crt);
      }
    o->callback((Fl_Callback*)(FlManageLightList::cb_select));
  }
  {
    Fl_Button* o = _addButton = new Fl_Button(160,10,130,20,labels[LabelStartIndex+2]);
    o->callback((Fl_Callback*)(FlManageLightList::cb_add));
  }
  {
    Fl_Button* o = _delButton = new Fl_Button(160,40,130,20,labels[LabelStartIndex+3]);
    o->callback((Fl_Callback*)(FlManageLightList::cb_delete));
  }
  {
    Fl_Button* o = _newButton = new Fl_Button(160,70,130,20,labels[LabelStartIndex+4]);
    o->callback((Fl_Callback*)(FlManageLightList::cb_new));
  }

  _validateButton = new Fl_Button(10,275,80,20,labels[LabelStartIndex+5]); _validateButton->callback((Fl_Callback*)(FlManageLightList::cb_validate));
  _cancelButton = new Fl_Button(110,275,80,20,labels[LabelStartIndex+6]); _cancelButton->callback((Fl_Callback*)(FlManageLightList::cb_cancel));

  end();
  set_modal();
  _buttonClicked = 0;
}

FlManageLightList::~FlManageLightList()
{
}

void FlManageLightList::draw()
{
  if (_lightList->size() != 0) _newButton->activate();
  else _newButton->deactivate();
  if (_lightList->value() != 0) _delButton->activate();
  else _delButton->deactivate();
  Fl_Window::draw();
}

int FlManageLightList::handle(int event)
{
  if ((event == FL_KEYBOARD) || (event == FL_SHORTCUT)) return 1;
  return Fl_Window::handle(event);
}

int FlManageLightList::clicked()
{
  return _buttonClicked;
}

void FlManageLightList::cb_select_i(Fl_Widget* o, void* v)
{
  redraw();
}
void FlManageLightList::cb_select(Fl_Widget* o, void* v)
{
  ((FlManageLightList*)(o->parent()))->cb_select_i(o,v);
}

void FlManageLightList::cb_add_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_lightSourceListDirectory,getFilter(Parameters::_lightSourceExtension), FileChooser::SINGLE, labels[LabelStartIndex+7]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      sLightSource* tmp = new sLightSource();
      char ttmp[255]; strcpy(ttmp,fc->value());
      enum eErrorCode error = GlobalData::loadLight(ttmp,tmp);
      if (error == ERROR_NONE)
	{
	  lightlist.add(tmp);
	  _lightList->add(tmp->_name,tmp);
	}
    }
  delete fc;
  redraw();
}
void FlManageLightList::cb_add(Fl_Widget* o, void* v)
{
  ((FlManageLightList*)(o->parent()))->cb_add_i(o,v);
}

void FlManageLightList::cb_delete_i(Fl_Widget* o, void* v)
{
  int id = _lightList->value();
  sLightSource* crt = static_cast< sLightSource* >(_lightList->data(id));
  _lightList->remove(id);
  lightlist.del(crt);
  delete crt;
  redraw();
}
void FlManageLightList::cb_delete(Fl_Widget* o, void* v)
{
  ((FlManageLightList*)(o->parent()))->cb_delete_i(o,v);
}

void FlManageLightList::cb_new_i(Fl_Widget* o, void* v)
{
  lightlist.freeMemory();
  for (int i=_lightList->size();i>0;--i)
    _lightList->remove(i);
  redraw();
}
void FlManageLightList::cb_new(Fl_Widget* o, void* v)
{
  ((FlManageLightList*)(o->parent()))->cb_new_i(o,v);
}

void FlManageLightList::cb_validate_i(Fl_Widget* o, void* v)
{
  /* retreiving list parameters */
  GlobalData::setLightList(lightlist);
  _buttonClicked = 1;
}
void FlManageLightList::cb_validate(Fl_Widget* o, void* v)
{
  ((FlManageLightList*)(o->parent()))->cb_validate_i(o,v);
}

void FlManageLightList::cb_cancel_i(Fl_Widget* o, void* v)
{
  _buttonClicked = 1;
}
void FlManageLightList::cb_cancel(Fl_Widget* o, void* v)
{
  ((FlManageLightList*)(o->parent()))->cb_cancel_i(o,v);
}
