#include "gui/FlManageMaterialList.h"
#include "gui/FlEvents.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"

#define LabelStartIndex 48

FlManageMaterialList::FlManageMaterialList(enum eTunnelSurface t) :
  Fl_Window(300,300,labels[LabelStartIndex+0]) ,
  materialtype(t)
{
  materiallist = (materialtype == TUNNELSURFACE_FLOOR ?
		  GlobalData::getFloorList() :
		  (materialtype == TUNNELSURFACE_WALL ?
		   GlobalData::getWallList() : 
		   GlobalData::getRoofList()));
  box(FL_UP_BOX);
  labelsize(12);
  user_data((void*)(this));
  resizable(NULL);

  {
    Fl_Select_Browser* o = _materialList = new Fl_Select_Browser(5,5,150,245,(materialtype == TUNNELSURFACE_FLOOR ?
									      labels[LabelStartIndex+1] :
									      (materialtype == TUNNELSURFACE_WALL ?
									       labels[LabelStartIndex+2] : labels[LabelStartIndex+3])));
    for (cListIterator< sMaterialReflectanceSpectrum* > iter = materiallist;!iter.finished();++iter)
      {
	sMaterialReflectanceSpectrum* crt = iter.getCurrent();
	o->add(crt->_spectrumName,crt);
      }
    o->callback((Fl_Callback*)(FlManageMaterialList::cb_select));
  }
  {
    Fl_Button* o = _addButton = new Fl_Button(160,10,130,20,labels[LabelStartIndex+4]);
    o->callback((Fl_Callback*)(FlManageMaterialList::cb_add));
  }
  {
    Fl_Button* o = _delButton = new Fl_Button(160,40,130,20,labels[LabelStartIndex+5]);
    o->callback((Fl_Callback*)(FlManageMaterialList::cb_delete));
  }
  {
    Fl_Button* o = _newButton = new Fl_Button(160,70,130,20,labels[LabelStartIndex+6]);
    o->callback((Fl_Callback*)(FlManageMaterialList::cb_new));
  }

  _validateButton = new Fl_Button(10,275,80,20,labels[LabelStartIndex+7]); _validateButton->callback((Fl_Callback*)(FlManageMaterialList::cb_validate));
  _cancelButton = new Fl_Button(110,275,80,20,labels[LabelStartIndex+8]); _cancelButton->callback((Fl_Callback*)(FlManageMaterialList::cb_cancel));

  end();
  set_modal();
  _buttonClicked = 0;
}

FlManageMaterialList::~FlManageMaterialList()
{
}

void FlManageMaterialList::draw()
{
  if (_materialList->size() != 0) _newButton->activate();
  else _newButton->deactivate();
  if (_materialList->value() != 0) _delButton->activate();
  else _delButton->deactivate();
  Fl_Window::draw();
}

int FlManageMaterialList::handle(int event)
{
  if ((event == FL_KEYBOARD) || (event == FL_SHORTCUT)) return 1;
  return Fl_Window::handle(event);
}

int FlManageMaterialList::clicked()
{
  return _buttonClicked;
}

void FlManageMaterialList::cb_select_i(Fl_Widget* o, void* v)
{
  redraw();
}
void FlManageMaterialList::cb_select(Fl_Widget* o, void* v)
{
  ((FlManageMaterialList*)(o->parent()))->cb_select_i(o,v);
}

void FlManageMaterialList::cb_add_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_materialListDirectory,getFilter(Parameters::_reflectanceMaterialSpectrumFileExtension), FileChooser::SINGLE, labels[LabelStartIndex+9]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      sMaterialReflectanceSpectrum* tmp = new sMaterialReflectanceSpectrum();
      char ttmp[255]; strcpy(ttmp,fc->value());
      enum eErrorCode error = GlobalData::loadMaterial(ttmp,tmp);
      if (error == ERROR_NONE)
	{
	  materiallist.add(tmp);
	  _materialList->add(tmp->_spectrumName,tmp);
	}
    }
  delete fc;
  redraw();
}
void FlManageMaterialList::cb_add(Fl_Widget* o, void* v)
{
  ((FlManageMaterialList*)(o->parent()))->cb_add_i(o,v);
}

void FlManageMaterialList::cb_delete_i(Fl_Widget* o, void* v)
{
  int id = _materialList->value();
  sMaterialReflectanceSpectrum* crt = static_cast< sMaterialReflectanceSpectrum* >(_materialList->data(id));
  _materialList->remove(id);
  materiallist.del(crt);
  delete crt;
  redraw();
}
void FlManageMaterialList::cb_delete(Fl_Widget* o, void* v)
{
  ((FlManageMaterialList*)(o->parent()))->cb_delete_i(o,v);
}

void FlManageMaterialList::cb_new_i(Fl_Widget* o, void* v)
{
  materiallist.freeMemory();
  for (int i=_materialList->size();i>0;--i)
    _materialList->remove(i);
  redraw();
}
void FlManageMaterialList::cb_new(Fl_Widget* o, void* v)
{
  ((FlManageMaterialList*)(o->parent()))->cb_new_i(o,v);
}

void FlManageMaterialList::cb_validate_i(Fl_Widget* o, void* v)
{
  /* retreiving list parameters */
  switch (materialtype)
    {
    case TUNNELSURFACE_FLOOR:
      GlobalData::setFloorList(materiallist);
      break;
    case TUNNELSURFACE_WALL:
      GlobalData::setWallList(materiallist);
      break;
    case TUNNELSURFACE_ROOF:
      GlobalData::setRoofList(materiallist);
      break;
    }
  _buttonClicked = 1;
}
void FlManageMaterialList::cb_validate(Fl_Widget* o, void* v)
{
  ((FlManageMaterialList*)(o->parent()))->cb_validate_i(o,v);
}

void FlManageMaterialList::cb_cancel_i(Fl_Widget* o, void* v)
{
  _buttonClicked = 1;
}
void FlManageMaterialList::cb_cancel(Fl_Widget* o, void* v)
{
  ((FlManageMaterialList*)(o->parent()))->cb_cancel_i(o,v);
}
