#include "gui/FlSelectCourbe.h"
#include "gui/FlEvents.h"
#include "gui/FlGlVisuCourbe.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"
#include "data/LightSource.h"

#define LabelStartIndex 58

#define SelectCourbeWindowHeight 80
#define SelectCourbeWindowWidth 100

FlSelectCourbe::FlSelectCourbe(const int& xx,const int& yy,const int& ww,const int& hh,const char* title = NULL) :
  Fl_Window(xx,yy,ww,hh,title) ,
  _curveAbcisse(0.) ,
  _curveOrdonnee(0.) ,
  _curveViewer(NULL)
{
  box(FL_UP_BOX);
  labelsize(12);
  label(labels[LabelStartIndex+0]);
  user_data((void*)(this));
  resizable(NULL);

  _nameLabel = new Fl_Box(5,5,190,20,labels[LabelStartIndex+1]);
  _selectionButton = new Fl_Button(5,30,40,40,labels[LabelStartIndex+2]);
  _abcisseLabel = new Fl_Box(50,30,80,20,labels[LabelStartIndex+3]);
  _abcisseOutput = new Fl_Output(135,30,60,20);
  _ordonneeLabel = new Fl_Box(50,55,80,20,labels[LabelStartIndex+4]);
  _ordonneeOutput = new Fl_Output(135,55,60,20);

  end();

  _selectionButton->callback((Fl_Callback*)(FlSelectCourbe::cb_selection),this);
}

FlSelectCourbe::~FlSelectCourbe()
{
}

void FlSelectCourbe::draw()
{
  char tmp[255];
  sprintf(tmp,"%f",_curveAbcisse); _abcisseOutput->value(tmp);
  sprintf(tmp,"%f",_curveOrdonnee); _ordonneeOutput->value(tmp);
  Fl_Window::draw();
}

void FlSelectCourbe::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Window::resize(xx,yy,ww,hh);
}

int FlSelectCourbe::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case TUNNEL_NEEDTOREDRAW:
      redraw();
      res = 1;
      break;
    case TUNNEL_VIEWERCLICKED:
      if (!initializeCoordinates()) redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Window::handle(event);
  return res;
}

void FlSelectCourbe::link(FlGlVisuCourbe* viewer)
{
  _curveViewer = viewer;
}

int FlSelectCourbe::initializeData( char* filename)
{
  int res = 0;
  if (!GlobalData::loadRequiredRadianceCurve(filename))
    {
      if (_curveViewer != NULL) _curveViewer->handle(TUNNEL_DATACHANGED);
      handle(TUNNEL_NEEDTOREDRAW);
    }
  else
    {
      res = 1;
    }
  return res;
}

/*
 * Protected methods
 */

void FlSelectCourbe::cb_selection_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_requiredRadianceCurveDirectory,getFilter(Parameters::_requiredRadianceCurveFileExtension), FileChooser::SINGLE, labels[LabelStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      initializeData(tmp);
    }
  delete fc;
}

void FlSelectCourbe::cb_selection(Fl_Widget* o, void* v)
{
  if (v != NULL) ((FlSelectCourbe*)v)->cb_selection_i(o,v);
}

/*
 * Private methods
 */

int FlSelectCourbe::initializeCoordinates()
{
  int xf = _curveViewer->getAbcisse();
  const sRequiredRadianceCurve& rrc = GlobalData::getRequiredRadianceCurve();
  double xg = 2.* xf / w() -1.;
  double xr = 100. * xg + 80.;
  int id = -1;
  for (int i=0;(i<rrc._curveDataSize) && (id==-1);i++)
    {
      if (rrc._curveAbcisse[i] > xr) id = i;
    }
  if (id <= 0) return ERROR_UNDEFINEDVALUE;
  double d0 = ABS(rrc._curveAbcisse[id-1] - xr);
  double d1 = ABS(rrc._curveAbcisse[id] - xr);
  double vr = (d1*rrc._curveValue[id-1] + d0*rrc._curveValue[id]) / (d0 + d1);
  _curveAbcisse = xr;
  _curveOrdonnee = vr;
  return ERROR_NONE;
}
