#include "gui/FlSelectSource.h"
#include "gui/FlEvents.h"
#include "gui/FlGlVisuSource.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"
#include "data/MaterialReflectanceSpectrum.h"

#define LabelStartIndex 70

FlSelectSource::FlSelectSource(const int& xx,const int& yy,const int& w,const int& h,const char* title = NULL) :
  Fl_Window(xx,yy,w,h,title) ,
  _sourceAbcisse(0.) ,
  _sourceOrdonnee(0.) ,
  _sourceViewer(NULL)
{
  box(FL_UP_BOX);
  labelsize(12);
  label(labels[LabelStartIndex+0]);
  user_data((void*)(this));
  resizable(NULL);

  _nameLabel = new Fl_Box(5,5,90,20,labels[LabelStartIndex+1]);
  _nameOutput = new Fl_Output(100,5,95,20);
  _selectionButton = new Fl_Button(5,30,40,40,labels[LabelStartIndex+2]);
  _abcisseLabel = new Fl_Box(50,30,80,20,labels[LabelStartIndex+3]);
  _abcisseOutput = new Fl_Output(135,30,60,20);
  _ordonneeLabel = new Fl_Box(50,55,80,20,labels[LabelStartIndex+4]);
  _ordonneeOutput = new Fl_Output(135,55,60,20);

  end();

  _selectionButton->callback((Fl_Callback*)(FlSelectSource::cb_selection),this);
}

FlSelectSource::~FlSelectSource()
{
}

void FlSelectSource::draw()
{
  const sLightSource& ls = GlobalData::getLightSource();
  _nameOutput->value(ls._name);
  char tmp[255];
  sprintf(tmp,"%f",_sourceAbcisse); _abcisseOutput->value(tmp);
  sprintf(tmp,"%f",_sourceOrdonnee); _ordonneeOutput->value(tmp);
  Fl_Window::draw();
}

void FlSelectSource::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Window::resize(xx,yy,ww,hh);
}

int FlSelectSource::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case TUNNEL_NEEDTOREDRAW:
      redraw();
      res = 1;
      break;
    case TUNNEL_VIEWERCLICKED:
      if (!initializeCoordinates()) redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Window::handle(event);
  return res;
}

/*
 * Public data handling methods
 */

int FlSelectSource::initializeData( char* filename)
{
  int res = 0;
  if (!GlobalData::loadLightSource(filename))
    {
      if (_sourceViewer != NULL) _sourceViewer->handle(TUNNEL_DATACHANGED);
      handle(TUNNEL_NEEDTOREDRAW);
    }
  else
    {
      res = 1;
    }
  return res;
}

void FlSelectSource::link(FlGlVisuSource* viewer)
{
  _sourceViewer = viewer;
}

/*
 * Protected methods
 */

void FlSelectSource::cb_selection_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_lightSourceDirectory,getFilter(Parameters::_lightSourceExtension), FileChooser::SINGLE, labels[LabelStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      initializeData(tmp);
    }
  delete fc;
}

void FlSelectSource::cb_selection(Fl_Widget* o, void* v)
{
  if (v != NULL) ((FlSelectSource*)v)->cb_selection_i(o,v);
}

/*
 * Private methods
 */

int FlSelectSource::initializeCoordinates()
{
  int xf = _sourceViewer->getAbcisse();
  const sLightSource& ls = GlobalData::getLightSource();
  double mx = 0.;
  for (int i=0;i<ls._spectrum._nbvalues;i++)
    if (ls._spectrum._values[1][i] > mx) mx = ls._spectrum._values[1][i];
  double xg = 2.* xf / w() -1.;
  double xr = 10. * (xg + 0.9) / 0.05 + 370.;
  int id = -1;
  int i;
  for (i=0;(i<ls._spectrum._nbvalues) && (id==-1);i++)
    {
      if (ls._spectrum._values[0][i] > xr) id = i;
    }
  if (id < 0)
    {
    _sourceOrdonnee = ls._spectrum._values[1][ls._spectrum._nbvalues-1];
    }
  else if (id == 0)
    {
    _sourceOrdonnee = ls._spectrum._values[1][0];
    }
  else
  {
    double d0 = ABS(ls._spectrum._values[0][id-1] - xr);
    double d1 = ABS(ls._spectrum._values[0][id] - xr);
    double vr = (d1*ls._spectrum._values[1][id-1] + d0*ls._spectrum._values[1][id]) / (mx*(d0 + d1));
    _sourceOrdonnee = vr;
  }
  _sourceAbcisse = xr;
  return ERROR_NONE;
}
