#include "gui/FlSelectSpectre.h"
#include "gui/FlEvents.h"
#include "gui/FlGlVisuSpectre.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"
#include "data/MaterialReflectanceSpectrum.h"

#define LabelStartIndex 64

FlSelectSpectre::FlSelectSpectre(const int& xx,const int& yy,const int& w,const int& h,const char* title = NULL) :
  Fl_Window(xx,yy,w,h,title) ,
  _spectreAbcisse(0.) ,
  _spectreOrdonnee(0.) ,
  _spectreViewer(NULL)
{
  box(FL_UP_BOX);
  labelsize(12);
  label(labels[LabelStartIndex+0]);
  user_data((void*)(this));
  resizable(NULL);

  _nameLabel = new Fl_Box(5,5,90,20,labels[LabelStartIndex+1]);
  _nameOutput = new Fl_Output(100,5,95,20);
  _selectionButton = new Fl_Button(5,30,40,40,labels[LabelStartIndex+2]);
  _abcisseLabel = new Fl_Box(50,30,80,20,labels[LabelStartIndex+3]);
  _abcisseOutput = new Fl_Output(135,30,60,20);
  _ordonneeLabel = new Fl_Box(50,55,80,20,labels[LabelStartIndex+4]);
  _ordonneeOutput = new Fl_Output(135,55,60,20);

  end();

  _selectionButton->callback((Fl_Callback*)(FlSelectSpectre::cb_selection),this);
}

FlSelectSpectre::~FlSelectSpectre()
{
}

void FlSelectSpectre::draw()
{
  const sMaterialReflectanceSpectrum& mrs = GlobalData::getMaterialReflectanceSpectrum();
  _nameOutput->value(mrs._spectrumName);
  char tmp[255];
  sprintf(tmp,"%f",_spectreAbcisse); _abcisseOutput->value(tmp);
  sprintf(tmp,"%f",_spectreOrdonnee); _ordonneeOutput->value(tmp);
  Fl_Window::draw();
}

void FlSelectSpectre::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Window::resize(xx,yy,ww,hh);
}

int FlSelectSpectre::handle(int event)
{
  int res = 0;
  switch (event)
    {
    case TUNNEL_NEEDTOREDRAW:
      redraw();
      res = 1;
      break;
    case TUNNEL_VIEWERCLICKED:
      if (!initializeCoordinates()) redraw();
      res = 1;
      break;
    }
  if (!res) res = Fl_Window::handle(event);
  return res;
}

/*
 * Public data handling methods
 */

int FlSelectSpectre::initializeData( char* filename)
{
  int res = 0;
  if (!GlobalData::loadMaterialReflectanceSpectrum(filename))
    {
      if (_spectreViewer != NULL) _spectreViewer->handle(TUNNEL_DATACHANGED);
      handle(TUNNEL_NEEDTOREDRAW);
    }
  else
    {
      res = 1;
    }
  return res;
}

void FlSelectSpectre::link(FlGlVisuSpectre* viewer)
{
  _spectreViewer = viewer;
}

/*
 * Protected methods
 */

void FlSelectSpectre::cb_selection_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_reflectanceMaterialSpectrumDirectory,getFilter(Parameters::_reflectanceMaterialSpectrumFileExtension), FileChooser::SINGLE, labels[LabelStartIndex+5]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      char tmp[255]; strcpy(tmp,fc->value());
      initializeData(tmp);
    }
  delete fc;
}

void FlSelectSpectre::cb_selection(Fl_Widget* o, void* v)
{
  if (v != NULL) ((FlSelectSpectre*)v)->cb_selection_i(o,v);
}

/*
 * Private methods
 */

int FlSelectSpectre::initializeCoordinates()
{
  int xf = _spectreViewer->getAbcisse();
  const sMaterialReflectanceSpectrum& mrs = GlobalData::getMaterialReflectanceSpectrum();
  double xg = 2.* xf / w() -1.;
  double xr = 10. * (xg + 0.9) / 0.05 + 370.;
  int id = -1;
  int i;
  for (i=0;(i<mrs._spectrumDataSize) && (id==-1);i++)
    {
      if (mrs._spectrumWavelength[i] > xr) id = i;
    }
  if (id < 0)
    {
    _spectreOrdonnee = mrs._spectrumValue[mrs._spectrumDataSize-1];
    }
  else if (id == 0)
    {
    _spectreOrdonnee = mrs._spectrumValue[0];
    }
  else
  {
    double d0 = ABS(mrs._spectrumWavelength[id-1] - xr);
    double d1 = ABS(mrs._spectrumWavelength[id] - xr);
    double vr = (d1*mrs._spectrumValue[id-1] + d0*mrs._spectrumValue[id]) / (d0 + d1);
    _spectreOrdonnee = vr;
  }
  _spectreAbcisse = xr;
  return ERROR_NONE;
}
