#include "gui/FlSimulationParameters.h"
#include "gui/FlEvents.h"
#include "gui/Labels.h"

#include "others/GlobalData.h"
#include "others/Parameters.h"
#include "others/SystemFuncs.h"

#define LabelsStartIndex 76

FlSimulationParameters* FlSimulationParameters::_win = NULL;

Fl_Menu_Item menusModepre[] = { { "1", 0, 0, 0, FL_MENU_RADIO }, { "2", 0, 0, 0, FL_MENU_RADIO }, { 0 } };
Fl_Menu_Item menusBasispre[] = { { "1", 0, 0, 0, FL_MENU_RADIO }, { "2", 0, 0, 0, FL_MENU_RADIO }, { 0 } };
Fl_Menu_Item menusModefin[] = { { "1", 0, 0, 0, FL_MENU_RADIO }, { "2", 0, 0, 0, FL_MENU_RADIO }, { 0 } };
Fl_Menu_Item menusBasisfin[] = { { "1", 0, 0, 0, FL_MENU_RADIO }, { "2", 0, 0, 0, FL_MENU_RADIO }, { 0 } };

FlSimulationParameters::FlSimulationParameters() :
  Fl_Window(500,205,labels[LabelsStartIndex+0])
{
  char tmp[255];

  box(FL_UP_BOX);
  labelsize(12);
  user_data((void*)(this));
  resizable(NULL);

  /* get simulation parameters from GlobalData object */
  temporaryParameters = GlobalData::getSimulationParameters();

  /* parameters manager tabs */
  {
    Fl_Tabs* o = _tabulator = new Fl_Tabs(0,0,380,175);
    {
      Fl_Group* o = _preprocessParameters = new Fl_Group(5,20,370,175,labels[LabelsStartIndex+1]);
      {
	Fl_Float_Input* o = _preprocessParametersAreaThreshold = new Fl_Float_Input(175,25,180,20,labels[LabelsStartIndex+2]);
	sprintf(tmp,"%f",temporaryParameters._preprocessParameters._patchAreaThreshold);
	o->value(tmp);
      }
      {
	Fl_Float_Input* o = _preprocessParametersLightAreaThreshold = new Fl_Float_Input(175,50,180,20,labels[LabelsStartIndex+3]);
	sprintf(tmp,"%f",temporaryParameters._preprocessParameters._lightAreaThreshold);
	o->value(tmp);
      }
      {
	Fl_Float_Input* o = _preprocessParametersThreshold = new Fl_Float_Input(175,75,180,20,labels[LabelsStartIndex+4]);
	sprintf(tmp,"%f",temporaryParameters._preprocessParameters._globalThreshold);
	o->value(tmp);
      }
      {
	Fl_Float_Input* o = _preprocessParametersErrorThreshold = new Fl_Float_Input(175,100,180,20,labels[LabelsStartIndex+5]);
	sprintf(tmp,"%f",temporaryParameters._preprocessParameters._errorThreshold);
	o->value(tmp);
      }
      {
	Fl_Value_Slider* o = _preprocessParametersConfigNumber = new Fl_Value_Slider(175,125,180,20,labels[LabelsStartIndex+6]);
	o->type(FL_HOR_NICE_SLIDER);
	o->range(1.,(double)maxNumberOfConfiguration);
	o->step(1.);
	o->value(temporaryParameters._configurationNumber);
	o->align(FL_ALIGN_LEFT);
      }
      {
	Fl_Menu_Button* o = _preprocessParametersMode = new Fl_Menu_Button(20,150,160,20,labels[LabelsStartIndex+7]);
	menusModepre[0].label(labels[LabelsStartIndex+8]);
	menusModepre[1].label(labels[LabelsStartIndex+9]);
	if (temporaryParameters._preprocessParameters._simulationMode == SIMULATIONMODE_SHOOTING)
	  menusModepre[0].setonly();
	else
	  menusModepre[1].setonly();
	o->menu(menusModepre);
      }
      {
	Fl_Menu_Button* o = _preprocessParametersWaveletBasis = new Fl_Menu_Button(200,150,160,20,labels[LabelsStartIndex+10]);
	menusBasispre[0].label(labels[LabelsStartIndex+11]);
	menusBasispre[1].label(labels[LabelsStartIndex+12]);
	switch (temporaryParameters._preprocessParameters._waveletBasis) {
	case WAVELETBASIS_LINEAR:
	  menusBasispre[0].setonly();
	  break;
	case WAVELETBASIS_QUADRATIC:
	  menusBasispre[1].setonly();
	  break;
	}
	o->menu(menusBasispre);
      }
      o->end();
    }
    {
      Fl_Group* o = _finalParameters = new Fl_Group(5,20,370,150,labels[LabelsStartIndex+13]);
      {
	Fl_Float_Input* o = _finalParametersAreaThreshold = new Fl_Float_Input(175,25,180,20,labels[LabelsStartIndex+14]);
	sprintf(tmp,"%f",temporaryParameters._finalstepParameters._patchAreaThreshold);
	o->value(tmp);
      }
      {
	Fl_Float_Input* o = _finalParametersLightAreaThreshold = new Fl_Float_Input(175,50,180,20,labels[LabelsStartIndex+15]);
	sprintf(tmp,"%f",temporaryParameters._finalstepParameters._lightAreaThreshold);
	o->value(tmp);
      }
      {
	Fl_Float_Input* o = _finalParametersThreshold = new Fl_Float_Input(175,75,180,20,labels[LabelsStartIndex+16]);
	sprintf(tmp,"%f",temporaryParameters._finalstepParameters._globalThreshold);
	o->value(tmp);
      }
      {
	Fl_Float_Input* o = _finalParametersErrorThreshold = new Fl_Float_Input(175,100,180,20,labels[LabelsStartIndex+17]);
	sprintf(tmp,"%f",temporaryParameters._finalstepParameters._errorThreshold);
	o->value(tmp);
      }
      {
	Fl_Menu_Button* o = _finalParametersMode = new Fl_Menu_Button(20,125,160,20,labels[LabelsStartIndex+18]);
	menusModefin[0].label(labels[LabelsStartIndex+19]);
	menusModefin[1].label(labels[LabelsStartIndex+20]);
	if (temporaryParameters._finalstepParameters._simulationMode == SIMULATIONMODE_SHOOTING)
	  menusModefin[0].setonly();
	else
	  menusModefin[1].setonly();
	o->menu(menusModefin);
      }
      {
	Fl_Menu_Button* o = _finalParametersWaveletBasis = new Fl_Menu_Button(200,125,160,20,labels[LabelsStartIndex+21]);
	menusBasisfin[0].label(labels[LabelsStartIndex+22]);
	menusBasisfin[1].label(labels[LabelsStartIndex+23]);
	switch (temporaryParameters._finalstepParameters._waveletBasis) {
	case WAVELETBASIS_LINEAR:
	  menusBasisfin[0].setonly();
	  break;
	case WAVELETBASIS_QUADRATIC:
	  menusBasisfin[1].setonly();
	  break;
	}
	o->menu(menusBasisfin);
      }
      o->end();
    }
    o->value(_preprocessParameters);
    o->end();
  }

  /* parameters manager buttons */
  _loadButton = new Fl_Button(390,55,100,20,labels[LabelsStartIndex+24]); _loadButton->callback((Fl_Callback*)(FlSimulationParameters::cb_load));
  _saveButton = new Fl_Button(390,85,100,20,labels[LabelsStartIndex+25]); _saveButton->callback((Fl_Callback*)(FlSimulationParameters::cb_save));
  _defaultButton = new Fl_Button(390,115,100,20,labels[LabelsStartIndex+26]); _defaultButton->callback((Fl_Callback*)(FlSimulationParameters::cb_default));

  /* window manager buttons */
  _validateButton = new Fl_Button(60,180,100,20,labels[LabelsStartIndex+27]); _validateButton->callback((Fl_Callback*)(FlSimulationParameters::cb_validate));
  _cancelButton = new Fl_Button(220,180,100,20,labels[LabelsStartIndex+28]); _cancelButton->callback((Fl_Callback*)(FlSimulationParameters::cb_cancel));

  end();

  set_modal();
  _buttonClicked = 0;
  _win = this;
}

FlSimulationParameters::~FlSimulationParameters()
{
}

void FlSimulationParameters::draw()
{
  Fl_Window::draw();
}

void FlSimulationParameters::resize(const int& xx,const int& yy,const int& ww,const int& hh)
{
  Fl_Window::resize(xx,yy,ww,hh);
}

int FlSimulationParameters::handle(int event)
{
  if ((event == FL_KEYBOARD) || (event == FL_SHORTCUT)) return 1;
  return Fl_Window::handle(event);
}

int FlSimulationParameters::clicked()
{
  return _buttonClicked;
}

void FlSimulationParameters::init()
{
  /* update simulation preprocess data in GlobalData object */
  sscanf(_preprocessParametersAreaThreshold->value(),"%lf",&(temporaryParameters._preprocessParameters._patchAreaThreshold));
  sscanf(_preprocessParametersLightAreaThreshold->value(),"%lf",&(temporaryParameters._preprocessParameters._lightAreaThreshold));
  sscanf(_preprocessParametersThreshold->value(),"%lf",&(temporaryParameters._preprocessParameters._globalThreshold));
  sscanf(_preprocessParametersErrorThreshold->value(),"%lf",&(temporaryParameters._preprocessParameters._errorThreshold));
  temporaryParameters._configurationNumber = _preprocessParametersConfigNumber->value();
  temporaryParameters._preprocessParameters._simulationMode = (menusModepre[0].value() ? SIMULATIONMODE_SHOOTING : SIMULATIONMODE_GATHERING);
  temporaryParameters._preprocessParameters._waveletBasis =
    (menusBasispre[0].value() ?
     WAVELETBASIS_LINEAR :
     WAVELETBASIS_QUADRATIC);
     /* update simulation final step data in GlobalData object */
  sscanf(_finalParametersAreaThreshold->value(),"%lf",&(temporaryParameters._finalstepParameters._patchAreaThreshold));
  sscanf(_finalParametersLightAreaThreshold->value(),"%lf",&(temporaryParameters._finalstepParameters._lightAreaThreshold));
  sscanf(_finalParametersThreshold->value(),"%lf",&(temporaryParameters._finalstepParameters._globalThreshold));
  sscanf(_finalParametersErrorThreshold->value(),"%lf",&(temporaryParameters._finalstepParameters._errorThreshold));
  temporaryParameters._finalstepParameters._simulationMode = (menusModefin[0].value() ? SIMULATIONMODE_SHOOTING : SIMULATIONMODE_GATHERING);
  temporaryParameters._finalstepParameters._waveletBasis =
    (menusBasisfin[0].value() ?
     WAVELETBASIS_LINEAR :
     WAVELETBASIS_QUADRATIC);
}

void FlSimulationParameters::reinit()
{
  char tmp[255];
  sprintf(tmp,"%f",temporaryParameters._preprocessParameters._patchAreaThreshold);
  _preprocessParametersAreaThreshold->value(tmp);
  sprintf(tmp,"%f",temporaryParameters._preprocessParameters._lightAreaThreshold);
  _preprocessParametersLightAreaThreshold->value(tmp);
  sprintf(tmp,"%f",temporaryParameters._preprocessParameters._globalThreshold);
  _preprocessParametersThreshold->value(tmp);
  sprintf(tmp,"%f",temporaryParameters._preprocessParameters._errorThreshold);
  _preprocessParametersErrorThreshold->value(tmp);
  _preprocessParametersConfigNumber->value(temporaryParameters._configurationNumber);
  if (temporaryParameters._preprocessParameters._simulationMode == SIMULATIONMODE_SHOOTING)
    menusModepre[0].setonly();
  else
    menusModepre[1].setonly();
  if (temporaryParameters._preprocessParameters._waveletBasis == WAVELETBASIS_LINEAR)
    menusBasispre[0].setonly();
  else
    menusBasispre[1].setonly();
  sprintf(tmp,"%f",temporaryParameters._finalstepParameters._patchAreaThreshold);
  _finalParametersAreaThreshold->value(tmp);
  sprintf(tmp,"%f",temporaryParameters._finalstepParameters._lightAreaThreshold);
  _finalParametersLightAreaThreshold->value(tmp);
  sprintf(tmp,"%f",temporaryParameters._finalstepParameters._globalThreshold);
  _finalParametersThreshold->value(tmp);
  sprintf(tmp,"%f",temporaryParameters._finalstepParameters._errorThreshold);
  _finalParametersErrorThreshold->value(tmp);
  if (temporaryParameters._finalstepParameters._simulationMode == SIMULATIONMODE_SHOOTING)
    menusModefin[0].setonly();
  else
    menusModefin[1].setonly();
  if (temporaryParameters._finalstepParameters._waveletBasis == WAVELETBASIS_LINEAR)
    menusBasisfin[0].setonly();
  else
    menusBasisfin[1].setonly();
  redraw();
}

void FlSimulationParameters::cb_load_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_simulationParametersDirectory,getFilter(Parameters::_simulationParametersFileExtension), FileChooser::SINGLE, labels[LabelsStartIndex+29]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      sSimulationParameters tmp;
      char ttmp[255]; strcpy(ttmp,fc->value());
      if (GlobalData::loadSimulationParameters(ttmp,tmp) == ERROR_NONE)
	{
	  temporaryParameters = tmp;
	  reinit();
	}
    }
  delete fc;
}
void FlSimulationParameters::cb_load(Fl_Widget* o, void* v)
{
  _win->cb_load_i(o,v);
}

void FlSimulationParameters::cb_save_i(Fl_Widget* o, void* v)
{
  FileChooser * fc = new FileChooser(Parameters::_simulationParametersDirectory,getFilter(Parameters::_simulationParametersFileExtension), FileChooser::CREATE, labels[LabelsStartIndex+30]);
  fc->color((Fl_Color)196);
  fc->show();
  while (fc->visible())
    Fl::wait();
  if (fc->count() == 1)
    {
      init();
      char tmp[255]; strcpy(tmp,fc->value());
      GlobalData::saveSimulationParameters(tmp,temporaryParameters);
    }
  delete fc;
}
void FlSimulationParameters::cb_save(Fl_Widget* o, void* v)
{
  _win->cb_save_i(o,v);
}

void FlSimulationParameters::cb_default_i(Fl_Widget* o, void* v)
{
  temporaryParameters = GlobalData::getSimulationDefaultParameters();
  reinit();
  redraw();
}
void FlSimulationParameters::cb_default(Fl_Widget* o, void* v)
{
  _win->cb_default_i(o,v);
}

void FlSimulationParameters::cb_validate_i(Fl_Widget* o, void* v)
{
  init();
  /* update simulation data in GlobalData object */
  GlobalData::setSimulationParameters(temporaryParameters);
  _buttonClicked = 1;
}
void FlSimulationParameters::cb_validate(Fl_Widget* o, void* v)
{
  _win->cb_validate_i(o,v);
}

void FlSimulationParameters::cb_cancel_i(Fl_Widget* o, void* v)
{
  _buttonClicked = 1;
}
void FlSimulationParameters::cb_cancel(Fl_Widget* o, void* v)
{
  _win->cb_cancel_i(o,v);
}
