#include "others/GlobalData.h"

#include "others/Parameters.h"

#include "radiosite/StaticMemory.h"
#include "radiosite/Link.h"
#include "radiosite/Solver.h"
#include "radiosite/Shoot.h"

#include "recherche/Nff.h"
#include "recherche/Maillage.h"
#include "recherche/Recherche.h"

sRequiredRadianceCurve GlobalData::_requiredRadianceCurve;
sMaterialReflectanceSpectrum GlobalData::_materialReflectanceSpectrum;
sLightSource GlobalData::_lightSource;
sTunnelParameters GlobalData::_tunnelParameters;
sSimulationParameters GlobalData::_simulParameters;

cList< sLightSource* > GlobalData::_lightList;
cList< sMaterialReflectanceSpectrum* > GlobalData::_floorList;
cList< sMaterialReflectanceSpectrum* > GlobalData::_wallList;
cList< sMaterialReflectanceSpectrum* > GlobalData::_roofList;

enum eTunnelModel GlobalData::_viewedScene;

int GlobalData::_solutionInitialized;

SceneNFF* GlobalData::scenenff = NULL;
Env* GlobalData::maillage = NULL;

void GlobalData::initialize()
{
  //_tunnelParameters = getTunnelDefaultParameters();
  _simulParameters = getSimulationDefaultParameters();
  _requiredRadianceCurve._curveStopDistance = 0;
  _requiredRadianceCurve._curveDataSize = 0;
  _materialReflectanceSpectrum._spectrumName[0] = '\0';
  _materialReflectanceSpectrum._spectrumN = 0.;
  _materialReflectanceSpectrum._spectrumK = 0.;
  _materialReflectanceSpectrum._spectrumType[0] = '\0';
  _materialReflectanceSpectrum._spectrumDataSize = 0;
  _materialReflectanceSpectrum._spectrumPrice = 0.;
  _solutionInitialized = 0;
  _lightList.freeMemory();
  _floorList.freeMemory();
  _wallList.freeMemory();
  _roofList.freeMemory();
  _viewedScene = TUNNELMODEL_NFF;
}

void GlobalData::initializeDefaultData()
{
  loadRequiredRadianceCurve(Parameters::_requiredRadianceCurveFilename);
  loadMaterialReflectanceSpectrum(Parameters::_reflectanceMaterialSpectrumFilename);
  loadLightSource(Parameters::_lightSourceFilename);

  loadLightList(Parameters::_lightSourceListFilename);
  loadFloorList(Parameters::_floorMaterialListFilename);
  loadWallList(Parameters::_wallMaterialListFilename);
  loadRoofList(Parameters::_roofMaterialListFilename);

  loadTunnelParameters(Parameters::_tunnelFilename);
  loadSimulationParameters(Parameters::_simulationFilename,_simulParameters);
}

int GlobalData::loadRequiredRadianceCurve(istream& inputfile)
{
  int res = ERROR_NONE;
  double _curveStopDistance;
  int _curveDataSize;
  double _curveAbcisse[maxNumberOfPointsInRequiredRadianceCurve];
  double _curveValue[maxNumberOfPointsInRequiredRadianceCurve];
  char tmpline[255];
  _curveDataSize = 0;
  inputfile.getline(tmpline,255,'\n');
  while ((!inputfile.eof()) && (!res))
    {
      switch (tmpline[0])
	{
	case '#':
	  break;
	case 'd':
	  sscanf(&(tmpline[1]),"%lf",&_curveStopDistance);
	  break;
	default:
	  if (isdigit(tmpline[0]))
	    {
	      double xx,yy;
	      sscanf(tmpline,"%lf%lf",&xx,&yy);
	      _curveAbcisse[_curveDataSize] = xx;
	      _curveValue[_curveDataSize] = yy;
	      _curveDataSize++;
	    }
	  else
	    res = ERROR_FILEFORMAT;
	  break;
	}
      inputfile.getline(tmpline,255,'\n');
    }
  if (res == ERROR_NONE)
    {
      _requiredRadianceCurve._curveStopDistance = _curveStopDistance;
      _requiredRadianceCurve._curveDataSize = _curveDataSize;
      for (int i=0;i<_curveDataSize;i++)
	{
	  _requiredRadianceCurve._curveAbcisse[i] = _curveAbcisse[i];
	  _requiredRadianceCurve._curveValue[i] = _curveValue[i];
	}
    }
  return res;
}

int GlobalData::loadRequiredRadianceCurve( char* fname)
{
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadRequiredRadianceCurve(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_requiredRadianceCurveDirectory,
				       Parameters::_requiredRadianceCurveFilename,
				       Parameters::_requiredRadianceCurveFileExtension);
    }
  return ERROR_NONE;
}

sRequiredRadianceCurve& GlobalData::getRequiredRadianceCurve()
{
  return _requiredRadianceCurve;
}

int GlobalData::loadMaterialReflectanceSpectrum( char* fname)
{
  sMaterialReflectanceSpectrum tmp;
  int res = loadMaterial(fname,&tmp);
  if (res == ERROR_NONE)
    {
      Parameters::updateFileParameters(fname,
				       Parameters::_reflectanceMaterialSpectrumDirectory,
				       Parameters::_reflectanceMaterialSpectrumFilename,
				       Parameters::_reflectanceMaterialSpectrumFileExtension);
      strcpy(_materialReflectanceSpectrum._spectrumName,tmp._spectrumName);
      _materialReflectanceSpectrum._spectrumN = tmp._spectrumN;
      _materialReflectanceSpectrum._spectrumK = tmp._spectrumK;
      strcpy(_materialReflectanceSpectrum._spectrumType,tmp._spectrumType);
      _materialReflectanceSpectrum._spectrumDataSize = tmp._spectrumDataSize;
      _materialReflectanceSpectrum._spectrumPrice = tmp._spectrumPrice;
      for (int i=0;i<tmp._spectrumDataSize;i++)
	{
	  _materialReflectanceSpectrum._spectrumWavelength[i] = tmp._spectrumWavelength[i];
	  _materialReflectanceSpectrum._spectrumValue[i] = tmp._spectrumValue[i];
	}
      _materialReflectanceSpectrum._meanSpectrumValue = tmp._meanSpectrumValue;
    }
  return res;
}

sMaterialReflectanceSpectrum& GlobalData::getMaterialReflectanceSpectrum()
{
  return _materialReflectanceSpectrum;
}

int GlobalData::loadLightSource( char* fname)
{
  sLightSource tmp;
  enum eErrorCode err = loadLight(fname,&(tmp));
  if (err == ERROR_NONE)
    {
      Parameters::updateFileParameters(fname,
				       Parameters::_lightSourceDirectory,
				       Parameters::_lightSourceFilename,
				       Parameters::_lightSourceExtension);
      _lightSource = tmp;
    }
  return err;
}

sLightSource& GlobalData::getLightSource()
{
  return _lightSource;
}

int GlobalData::loadTunnelParameters(istream& inputfile)
{
  int tmp,nb;
  /* vidage de la memoire */
  _tunnelParameters.light._lightSelected.freeMemory();
  _tunnelParameters.material._floorSelected.freeMemory();
  _tunnelParameters.material._wallSelected.freeMemory();
  _tunnelParameters.material._roofSelected.freeMemory();
  /* lecture des listes de sources et de materiaux */
  loadLightList(inputfile);
  loadFloorList(inputfile);
  loadWallList(inputfile);
  loadRoofList(inputfile);
  /* tunnel general shape */
  inputfile >> tmp; _tunnelParameters.shape._roofShape = (enum TunnelShape)tmp;
  inputfile >> tmp; _tunnelParameters.shape._floorLanes = (enum TunnelLane)tmp;
  inputfile >> _tunnelParameters.shape._length;
  inputfile >> _tunnelParameters.shape._maxHeight;
  inputfile >> _tunnelParameters.shape._width;
  inputfile >> _tunnelParameters.shape._minHeight;
  inputfile >> _tunnelParameters.shape._angle;
  /* tunnel lights */
  inputfile >> nb;
  for (int i=1;i<=nb;i++) { inputfile >> tmp; _tunnelParameters.light._lightSelected.add(tmp); }
  inputfile >> tmp; _tunnelParameters.light._lightPosition = (enum TunnelLight)tmp;
  inputfile >> tmp; _tunnelParameters.light._lightSymmetrical = (enum OnOff)tmp;
  inputfile >> _tunnelParameters.light._lightSpacing;
  inputfile >> _tunnelParameters.light._lightSlope;
  /* tunnel materials */
  inputfile >> nb;
  for (int i=1;i<=nb;i++) { inputfile >> tmp; _tunnelParameters.material._floorSelected.add(tmp); }
  inputfile >> nb;
  for (int i=1;i<=nb;i++) { inputfile >> tmp; _tunnelParameters.material._wallSelected.add(tmp); }
  inputfile >> nb;
  for (int i=1;i<=nb;i++) { inputfile >> tmp; _tunnelParameters.material._roofSelected.add(tmp); }
  return ERROR_NONE;
}

int GlobalData::loadTunnelParameters( char* fname)
{
  int tmp,nb;
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadTunnelParameters(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_tunnelParametersDirectory,
				       Parameters::_tunnelFilename,
				       Parameters::_tunnelParametersFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::saveTunnelParameters(ostream& outputfile)
{
  /* sauvegarde des listes de sources et de materiaux */
  saveLightList(outputfile);
  saveFloorList(outputfile);
  saveWallList(outputfile);
  saveRoofList(outputfile);
  /* tunnel general shape */
  outputfile << _tunnelParameters.shape._roofShape << endl;
  outputfile << _tunnelParameters.shape._floorLanes << endl;
  outputfile << _tunnelParameters.shape._length << endl;
  outputfile << _tunnelParameters.shape._maxHeight << endl;
  outputfile << _tunnelParameters.shape._width << endl;
  outputfile << _tunnelParameters.shape._minHeight << endl;
  outputfile << _tunnelParameters.shape._angle << endl;
  /* tunnel lights */
  outputfile << _tunnelParameters.light._lightSelected.card();
  for (int i=1;i<=_tunnelParameters.light._lightSelected.card();i++) outputfile << "  " << _tunnelParameters.light._lightSelected[i];
  outputfile << endl;
  outputfile << _tunnelParameters.light._lightPosition << endl;
  outputfile << _tunnelParameters.light._lightSymmetrical << endl;
  outputfile << _tunnelParameters.light._lightSpacing << endl;
  outputfile << _tunnelParameters.light._lightSlope << endl;
  /* tunnel materials */
  outputfile << _tunnelParameters.material._floorSelected.card();
  for (int i=1;i<=_tunnelParameters.material._floorSelected.card();i++) outputfile << "  " << _tunnelParameters.material._floorSelected[i];
  outputfile << endl;
  outputfile << _tunnelParameters.material._wallSelected.card();
  for (int i=1;i<=_tunnelParameters.material._wallSelected.card();i++) outputfile << "  " << _tunnelParameters.material._wallSelected[i];
  outputfile << endl;
  outputfile << _tunnelParameters.material._roofSelected.card();
  for (int i=1;i<=_tunnelParameters.material._roofSelected.card();i++) outputfile << "  " << _tunnelParameters.material._roofSelected[i];
  outputfile << endl;
  return ERROR_NONE;
}

int GlobalData::saveTunnelParameters( char* fname)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveTunnelParameters(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_tunnelParametersDirectory,
				       Parameters::_tunnelFilename,
				       Parameters::_tunnelParametersFileExtension);
    }
  return ERROR_NONE;
}

void GlobalData::dumpTunnelParameters(const sTunnelParameters& tmp)
{
  /* tunnel general shape */
  cout << tmp.shape._roofShape << endl;
  cout << tmp.shape._floorLanes << endl;
  cout << tmp.shape._length << endl;
  cout << tmp.shape._maxHeight << endl;
  cout << tmp.shape._width << endl;
  cout << tmp.shape._minHeight << endl;
  cout << tmp.shape._angle << endl;
  /* tunnel lights */
  cout << tmp.light._lightSelected.card();
  for (int i=1;i<=tmp.light._lightSelected.card();i++) cout << "  " << tmp.light._lightSelected[i];
  cout << endl;
  cout << tmp.light._lightPosition << endl;
  cout << tmp.light._lightSymmetrical << endl;
  cout << tmp.light._lightSpacing << endl;
  cout << tmp.light._lightSlope << endl;
  /* tunnel materials */
  cout << tmp.material._floorSelected.card();
  for (int i=1;i<=tmp.material._floorSelected.card();i++) cout << "  " << tmp.material._floorSelected[i];
  cout << endl;
  cout << tmp.material._wallSelected.card();
  for (int i=1;i<=tmp.material._wallSelected.card();i++) cout << "  " << tmp.material._wallSelected[i];
  cout << endl;
  cout << tmp.material._roofSelected.card();
  for (int i=1;i<=tmp.material._roofSelected.card();i++) cout << "  " << tmp.material._roofSelected[i];
  cout << endl;
}

sTunnelParameters& GlobalData::getTunnelParameters()
{
  return _tunnelParameters;
}

void GlobalData::setTunnelParameters(sTunnelParameters& v)
{
  _tunnelParameters = v;
}

sTunnelParameters GlobalData::getTunnelDefaultParameters()
{
  sTunnelParameters v;
  v.shape._roofShape = TUNNELSHAPE_PLANAR;
  v.shape._floorLanes = TUNNELSHAPE_ONELANE;
  v.shape._length = 70.;
  v.shape._maxHeight = 7.;
  v.shape._width = 5.;
  v.shape._minHeight = 4.;
  v.shape._angle = 30.;
  v.light._lightSelected.freeMemory();
  v.light._lightPosition = TUNNELLIGHT_CENTER;
  v.light._lightSymmetrical = OFF;
  v.light._lightSpacing = 2.;
  v.light._lightSlope = 10.;
  cList < int > tmp;
  v.material._floorSelected = tmp;
  v.material._wallSelected = tmp;
  v.material._roofSelected = tmp;
  //  v.material._floorSelected.freeMemory();
  //  v.material._wallSelected.freeMemory();
  //  v.material._roofSelected.freeMemory();
  return v;
}


int GlobalData::loadSimulationParameters(istream& inputfile,sSimulationParameters& res)
{
  char tmp[255];
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._preprocessParameters._patchAreaThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._preprocessParameters._lightAreaThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._preprocessParameters._globalThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._preprocessParameters._errorThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%d",&(res._preprocessParameters._simulationMode));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%d",&(res._preprocessParameters._waveletBasis));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%d",&(res._configurationNumber));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._finalstepParameters._patchAreaThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._finalstepParameters._lightAreaThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._finalstepParameters._globalThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%lf",&(res._finalstepParameters._errorThreshold));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%d",&(res._finalstepParameters._simulationMode));
  inputfile.getline(tmp,255,'\n');  sscanf(tmp,"%d",&(res._finalstepParameters._waveletBasis));
  return ERROR_NONE;
}

int GlobalData::loadSimulationParameters(istream& inputfile)
{
  return loadSimulationParameters(inputfile,_simulParameters);
}

int GlobalData::loadSimulationParameters( char* fname,sSimulationParameters& res)
{
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadSimulationParameters(inputfile,res);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_simulationParametersDirectory,
				       Parameters::_simulationFilename,
				       Parameters::_simulationParametersFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::loadSimulationParameters( char* fname)
{
  return loadSimulationParameters(fname,_simulParameters);
}

int GlobalData::saveSimulationParameters(ostream& outputfile,sSimulationParameters& res)
{
  outputfile << res._preprocessParameters._patchAreaThreshold << endl;
  outputfile << res._preprocessParameters._lightAreaThreshold << endl;
  outputfile << res._preprocessParameters._globalThreshold << endl;
  outputfile << res._preprocessParameters._errorThreshold << endl;
  outputfile << res._preprocessParameters._simulationMode << endl;
  outputfile << res._preprocessParameters._waveletBasis << endl;
  outputfile << res._configurationNumber << endl;
  outputfile << res._finalstepParameters._patchAreaThreshold << endl;
  outputfile << res._finalstepParameters._lightAreaThreshold << endl;
  outputfile << res._finalstepParameters._globalThreshold << endl;
  outputfile << res._finalstepParameters._errorThreshold << endl;
  outputfile << res._finalstepParameters._simulationMode << endl;
  outputfile << res._finalstepParameters._waveletBasis << endl;
  return ERROR_NONE;
}

int GlobalData::saveSimulationParameters( char* fname,sSimulationParameters& res)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveSimulationParameters(outputfile,res);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_simulationParametersDirectory,
				       Parameters::_simulationFilename,
				       Parameters::_simulationParametersFileExtension);
    }
  return ERROR_NONE;
}

sSimulationParameters& GlobalData::getSimulationParameters()
{
  return _simulParameters;
}

void GlobalData::setSimulationParameters(sSimulationParameters& v)
{
  _simulParameters = v;
}

sSimulationParameters GlobalData::getSimulationDefaultParameters()
{
  sSimulationParameters v;
  v._preprocessParameters._patchAreaThreshold = 0.01;
  v._preprocessParameters._lightAreaThreshold = 0.05;
  v._preprocessParameters._globalThreshold = 0.1;
  v._preprocessParameters._errorThreshold = 0.1;
  v._preprocessParameters._simulationMode = SIMULATIONMODE_SHOOTING;
  v._preprocessParameters._waveletBasis = WAVELETBASIS_LINEAR;
  v._configurationNumber = 1;
  v._finalstepParameters._patchAreaThreshold = 0.01;
  v._finalstepParameters._lightAreaThreshold = 0.1;
  v._finalstepParameters._globalThreshold = 0.0001;
  v._finalstepParameters._errorThreshold = 0.01;
  v._finalstepParameters._simulationMode = SIMULATIONMODE_GATHERING;
  v._finalstepParameters._waveletBasis = WAVELETBASIS_QUADRATIC;
  return v;
}


int GlobalData::loadLightList(istream& inputfile)
{
  int nb;
  char tmp[255];
  _lightList.freeMemory();
  inputfile.getline(tmp,255,'\n');
  sscanf(tmp,"%d",&nb);
  for (int i=0;i<nb;i++)
    {
      inputfile.getline(tmp,255,'\n');
      sLightSource* light = new sLightSource;
      loadLight(tmp,light);
      _lightList.add(light);
    }
  return ERROR_NONE;
}

int GlobalData::loadLightList( char* fname)
{
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadLightList(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_lightSourceListDirectory,
				       Parameters::_lightSourceListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::saveLightList(ostream& outputfile)
{
  outputfile << _lightList.card() << endl;
  for (cListIterator< sLightSource* > iter = _lightList;!(iter.finished());++iter)
    {
      sLightSource* crt = iter.getCurrent();
      outputfile << crt->_filename << endl;
    }
  return ERROR_NONE;
}

int GlobalData::saveLightList( char* fname)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveLightList(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_lightSourceListDirectory,
				       Parameters::_lightSourceListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::getLightIndexFromName( char* lname)
{
  for (int i=1;i<=_lightList.card();i++)
    if (!strcmp(_lightList[i]->_name,lname)) return i;
  return -1;
}

cList< sLightSource* >& GlobalData::getLightList()
{
  return _lightList;
}

void GlobalData::setLightList(const cList< sLightSource* >& lst)
{
  _lightList = lst;
}

int GlobalData::loadFloorList(istream& inputfile)
{
  int nb;
  char tmp[255];
  _floorList.freeMemory();
  inputfile.getline(tmp,255,'\n');
  sscanf(tmp,"%d",&nb);
  for (int i=0;i<nb;i++)
    {
      inputfile.getline(tmp,255,'\n');
      sMaterialReflectanceSpectrum* material = new sMaterialReflectanceSpectrum;
      loadMaterial(tmp,material);
      _floorList.add(material);
    }
  return ERROR_NONE;
}

int GlobalData::loadFloorList( char* fname)
{
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadFloorList(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_materialListDirectory,
				       Parameters::_floorMaterialListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::saveFloorList(ostream& outputfile)
{
  outputfile << _floorList.card() << endl;
  for (cListIterator< sMaterialReflectanceSpectrum* > iter = _floorList;!(iter.finished());++iter)
    {
      sMaterialReflectanceSpectrum* crt = iter.getCurrent();
      outputfile << crt->_filename << endl;
    }
  return ERROR_NONE;
}

int GlobalData::saveFloorList( char* fname)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveFloorList(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_materialListDirectory,
				       Parameters::_floorMaterialListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::getFloorIndexFromName( char* lname)
{
  for (int i=1;i<=_floorList.card();i++)
    if (!strcmp(_floorList[i]->_spectrumName,lname)) return i;
  return -1;
}

cList< sMaterialReflectanceSpectrum* >& GlobalData::getFloorList()
{
  return _floorList;
}

void GlobalData::setFloorList(const cList< sMaterialReflectanceSpectrum* >& lst)
{
  _floorList = lst;
}

int GlobalData::loadWallList(istream& inputfile)
{
  int nb;
  char tmp[255];
  _wallList.freeMemory();
  inputfile.getline(tmp,255,'\n');
  sscanf(tmp,"%d",&nb);
  for (int i=0;i<nb;i++)
    {
      inputfile.getline(tmp,255,'\n');
      sMaterialReflectanceSpectrum* material = new sMaterialReflectanceSpectrum;
      loadMaterial(tmp,material);
      _wallList.add(material);
    }
  return ERROR_NONE;
}

int GlobalData::loadWallList( char* fname)
{
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadWallList(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_materialListDirectory,
				       Parameters::_wallMaterialListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::saveWallList(ostream& outputfile)
{
  outputfile << _wallList.card() << endl;
  for (cListIterator< sMaterialReflectanceSpectrum* > iter = _wallList;!(iter.finished());++iter)
    {
      sMaterialReflectanceSpectrum* crt = iter.getCurrent();
      outputfile << crt->_filename << endl;
    }
  return ERROR_NONE;
}

int GlobalData::saveWallList( char* fname)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveWallList(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_materialListDirectory,
				       Parameters::_wallMaterialListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::getWallIndexFromName( char* lname)
{
  for (int i=1;i<=_wallList.card();i++)
    if (!strcmp(_wallList[i]->_spectrumName,lname)) return i;
  return -1;
}

cList< sMaterialReflectanceSpectrum* >& GlobalData::getWallList()
{
  return _wallList;
}

void GlobalData::setWallList(const cList< sMaterialReflectanceSpectrum* >& lst)
{
  _wallList = lst;
}

int GlobalData::loadRoofList(istream& inputfile)
{
  int nb;
  char tmp[255];
  _roofList.freeMemory();
  inputfile.getline(tmp,255,'\n');
  sscanf(tmp,"%d",&nb);
  for (int i=0;i<nb;i++)
    {
      inputfile.getline(tmp,255,'\n');
      sMaterialReflectanceSpectrum* material = new sMaterialReflectanceSpectrum;
      loadMaterial(tmp,material);
      _roofList.add(material);
    }
  return ERROR_NONE;
}

int GlobalData::loadRoofList( char* fname)
{
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      loadRoofList(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_materialListDirectory,
				       Parameters::_roofMaterialListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::saveRoofList(ostream& outputfile)
{
  outputfile << _roofList.card() << endl;
  for (cListIterator< sMaterialReflectanceSpectrum* > iter = _roofList;!(iter.finished());++iter)
    {
      sMaterialReflectanceSpectrum* crt = iter.getCurrent();
      outputfile << crt->_filename << endl;
    }
  return ERROR_NONE;
}

int GlobalData::saveRoofList( char* fname)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveRoofList(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_materialListDirectory,
				       Parameters::_roofMaterialListFilename,
				       Parameters::_listFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::getRoofIndexFromName( char* lname)
{
  for (int i=1;i<=_roofList.card();i++)
    if (!strcmp(_roofList[i]->_spectrumName,lname)) return i;
  return -1;
}

cList< sMaterialReflectanceSpectrum* >& GlobalData::getRoofList()
{
  return _roofList;
}

void GlobalData::setRoofList(const cList< sMaterialReflectanceSpectrum* >& lst)
{
  _roofList = lst;
}

enum eTunnelModel GlobalData::getVisualizedModel()
{
  return _viewedScene;
}

void GlobalData::setVisualizedModel(enum eTunnelModel v)
{
  if (v == TUNNELMODEL_NFF)
    StaticMemory::free(maxNumberOfPointsInSpectrum,Parameters::_MaxM,GlobalData::getSimulationParameters()._finalstepParameters._waveletBasis);
  else
    StaticMemory::allocate(maxNumberOfPointsInSpectrum,Parameters::_MaxM,GlobalData::getSimulationParameters()._finalstepParameters._waveletBasis);
  _viewedScene = v;
}

/*
 * Private methods
 */

enum eErrorCode GlobalData::loadLightSpectrum( char* filename,sLightSourceSpectrum* spectrum)
{
  char fname[255]; sprintf(fname,"%s%s",Parameters::_lightSourceDirectory,filename);
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      spectrum->_nbvalues = 0;
      while (inputfile)
	{
	  char tmp[255]; inputfile.getline(tmp,255,'\n');
	  sscanf(tmp,"%lf%lf",&(spectrum->_values[0][spectrum->_nbvalues]),&(spectrum->_values[1][spectrum->_nbvalues]));
	  (spectrum->_nbvalues)++;
	}
      (spectrum->_nbvalues)--;
    }
  return ERROR_NONE;
}

enum eErrorCode GlobalData::loadLight( char* fname,sLightSource* light)
{
  double tmp;
  char tmpline[255];
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    return ERROR_FILEOPEN;
  else
    {
      strcpy(light->_filename,fname);
      inputfile.getline(light->_name,255,'\n');
      inputfile.getline(light->_data,255,'\n');
      sscanf(&(light->_data[1]),"%s%lf%lf%lf",light->_spectrum._name,&(light->_spectrum._c0),&(light->_spectrum._c1),&(light->_spectrum._c2));
      enum eErrorCode err;
      if ((err = loadLightSpectrum(light->_spectrum._name,&(light->_spectrum))) != ERROR_NONE) return err;
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%lf",&(light->L));
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%lf",&(light->l));
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%lf",&(light->_stepDir));
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%d",&(light->_nbDir));
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%lf",&(light->_stepPlans));
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%d",&(light->_nbPlans));
      inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%lf",&(light->_firstPlan));
       for (int i=0;i<light->_nbPlans;i++)
	{
	  for (int j=0;j<light->_nbDir;j++)
	    {
	      inputfile >> light->_values[i][j];
	    }
	}
    }
  return ERROR_NONE;
}

enum eErrorCode GlobalData::loadMaterial( char* fname,sMaterialReflectanceSpectrum* material)
{
  int res;
  char tmpline[255];
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    res = ERROR_FILEOPEN;
  else
    {
      strcpy(material->_filename,fname);
      material->_spectrumDataSize = 0;
      material->_meanSpectrumValue = 0.;
      inputfile.getline(tmpline,255,'\n');
      while (!inputfile.eof())
	{
	  switch (tmpline[0])
	    {
	    case '#':
	      break;
	    case 'm':
	      strcpy(material->_spectrumName,&(tmpline[2]));
	      break;
	    case 'n':
	      sscanf(&(tmpline[2]),"%lf",&(material->_spectrumN));
	      break;
	    case 'k':
	      sscanf(&(tmpline[2]),"%lf",&(material->_spectrumK));
	      break;
	    case 't':
	      strcpy(material->_spectrumType,&(tmpline[2]));
	      break;
	    case 'p':
	      sscanf(&(tmpline[5]),"%lf",&(material->_spectrumPrice));
	      break;
	    default:
	      if (isdigit(tmpline[0]))
		{
		  double xx,yy;
		  sscanf(tmpline,"%lf%lf",&xx,&yy);
		  material->_spectrumWavelength[material->_spectrumDataSize] = xx;
		  material->_spectrumValue[material->_spectrumDataSize] = yy;
		  material->_spectrumDataSize++;
		  material->_meanSpectrumValue += yy;
		}
	      break;
	    }
	  inputfile.getline(tmpline,255,'\n');
	}
      inputfile.close();
      material->_meanSpectrumValue /= material->_spectrumDataSize;
    }
  return ERROR_NONE;
}

enum eErrorCode GlobalData::loadResolutionSearchResult(istream& inputfile)
{
  ENTERINGFUNCTION;
  int tmpnb;
  char tmpline[255];
  inputfile.getline(tmpline,255,'\n');
  if ((tmpline[0] == '/') && (tmpline[1] == '*'))
    {
      while (!((tmpline[0] == '*') && (tmpline[1] == '/')))
	inputfile.getline(tmpline,255,'\n');
    }
  // chargement des solutions trouvees
  inputfile.getline(tmpline,255,'\n'); sscanf(tmpline,"%d",&tmpnb); nbSolutions = tmpnb;
  for (int i=0;i<nbSolutions;i++)
    {
      int r0,r1,r2,s0,s1,s2;
      char sname[255], fname[255], wname[255], rname[255];
      float p;
      inputfile.getline(tmpline,255,'\n');
      sscanf(tmpline,"%d%d%d%d%d%d %s %s %s %s%f",&r0,&r1,&r2,&s0,&s1,&s2,sname,fname,wname,rname,&p);
      lesSolutions[i].ramp[0] = r0; lesSolutions[i].ramp[1] = r1; lesSolutions[i].ramp[2] = r2;
      lesSolutions[i].nsou[0] = s0; lesSolutions[i].nsou[1] = s1; lesSolutions[i].nsou[2] = s2;
      lesSolutions[i].sou = _lightList[getLightIndexFromName(sname)];
      lesSolutions[i].fr = _floorList[getFloorIndexFromName(fname)];
      lesSolutions[i].wl = _wallList[getWallIndexFromName(wname)];
      lesSolutions[i].rf = _roofList[getRoofIndexFromName(rname)];
      lesSolutions[i].prix = p;
    }
  EXITINGFUNCTION;
  return ERROR_NONE;
}

enum eErrorCode GlobalData::loadResolutionSearchResult( char* fname)
{
  ENTERINGFUNCTION; TRACEVARIABLE(fname);
  ifstream inputfile(fname,ios::in);
  if (!inputfile)
    {
      EXITINGFUNCTION;
      return ERROR_FILEOPEN;
    }
  else
    {
      loadResolutionSearchResult(inputfile);
      inputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_resolutionSearchDirectory,
				       Parameters::_resolutionSearchFilename,
				       Parameters::_resolutionSearchFileExtension);
    }
  EXITINGFUNCTION;
  return ERROR_NONE;
}

enum eErrorCode GlobalData::saveResolutionSearchResult(ostream& outputfile)
{
  outputfile << "/*" << endl;
  // sauvegarde de la courbe de reflectance requise
  outputfile << Parameters::_requiredRadianceCurveFilename << endl;
  // sauvegarde des parametres de simulation
  saveSimulationParameters(outputfile,_simulParameters);
  // sauvegarde des parametres de creation du tunnel
  saveTunnelParameters(outputfile);
  outputfile << "*/" << endl;
  /* sauvegarde des solutions trouvees */
  outputfile << nbSolutions << endl;
  for (int i=0;i<nbSolutions;i++)
    {
      outputfile << lesSolutions[i].ramp[0] << " " << lesSolutions[i].ramp[1] << " " << lesSolutions[i].ramp[2] << " ";
      outputfile << lesSolutions[i].nsou[0] << " " << lesSolutions[i].nsou[1] << " " << lesSolutions[i].nsou[2] << " ";
      outputfile << lesSolutions[i].sou->_name << " " << lesSolutions[i].fr->_spectrumName << " ";
      outputfile << lesSolutions[i].wl->_spectrumName << " " << lesSolutions[i].rf->_spectrumName << " ";
      outputfile << lesSolutions[i].prix << endl;
    }
}

enum eErrorCode GlobalData::saveResolutionSearchResult( char* fname)
{
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveResolutionSearchResult(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_resolutionSearchDirectory,
				       Parameters::_resolutionSearchFilename,
				       Parameters::_resolutionSearchFileExtension);
    }
  return ERROR_NONE;
}

void GlobalData::initializeSceneNFF()
{
  if (scenenff != NULL) { delete scenenff; scenenff = NULL; }
}

void GlobalData::initializeSceneMaillage()
{
  if (maillage != NULL) { Surface* S = maillage->slist; while (S != NULL) { Surface* St = S->nexts; delete S; S = St; } delete maillage; maillage = NULL; }
  StaticMemory::stat();
  _solutionInitialized = 0;
}

SceneNFF* GlobalData::getInitialTunnelNFF()
{
  if (scenenff != NULL) { delete scenenff; scenenff = NULL; }
  return (scenenff = getTunnelNFF(_tunnelParameters));
}

Env* GlobalData::getMaillage3d()
{
  researchMode = 1;
  if (maillage != NULL) { Surface* S = maillage->slist; while (S != NULL) { Surface* St = S->nexts; delete S; S = St; } delete maillage; maillage = NULL; }
  int idxlight = (_tunnelParameters.light._lightSelected)[1] - 1;
  int idxfloor = (_tunnelParameters.material._floorSelected)[1] - 1;
  int idxwall = (_tunnelParameters.material._wallSelected)[1] - 1;
  int idxroof = (_tunnelParameters.material._roofSelected)[1] - 1;
  sLightSource* light = _lightList[idxlight];
  sMaterialReflectanceSpectrum* floor = _floorList[idxfloor];
  sMaterialReflectanceSpectrum* wall = _wallList[idxwall];
  sMaterialReflectanceSpectrum* roof = _roofList[idxroof];
//   double tunnellength = _tunnelParameters.shape._length;
//   _tunnelParameters.shape._length = Parameters::_surfaceMaximumLength - smallEpsilon;
  initializeSpectralProperties(_simulParameters._preprocessParameters,light);
  maillage = getMaillage(_tunnelParameters,_simulParameters._preprocessParameters);
  addLight(_tunnelParameters,_simulParameters._preprocessParameters,light,maillage);
  setPhotometricProperties(maillage,light);
  setPhotometricProperties(maillage,floor,wall,roof);
//   _tunnelParameters.shape._length = tunnellength;
  StaticMemory::stat();
  return maillage;
}

void GlobalData::saveMaillage3d(ostream& outputfile)
{
  int M = (int)(_simulParameters._finalstepParameters._waveletBasis);
  saveNFF(outputfile,M,maillage,ON,OFF);
}

int GlobalData::saveMaillage3d( char* fname)
{
  if (maillage == NULL) return ERROR_NONE;
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveMaillage3d(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_geometricDataDirectory,
				       Parameters::_geometricDataFilename,
				       Parameters::_geometricDataFileExtension);
    }
  return ERROR_NONE;
}

Env* GlobalData::getMaillageSolution()
{
  researchMode = 0;
  if (maillage != NULL) { Surface* S = maillage->slist; while (S != NULL) { Surface* St = S->nexts; delete S; S = St; } delete maillage; maillage = NULL; }
   sRepartition& solution(lesSolutions[Parameters::_solutionNumber]);
  sLightSource* light = solution.sou;
  sMaterialReflectanceSpectrum* floor = solution.fr;
  sMaterialReflectanceSpectrum* wall = solution.wl;
  sMaterialReflectanceSpectrum* roof = solution.rf;
  initializeSpectralProperties(_simulParameters._finalstepParameters,light);

  int M = (int)(_simulParameters._finalstepParameters._waveletBasis);
  double Area_Threshold = _simulParameters._finalstepParameters._patchAreaThreshold;
  double Threshold = _simulParameters._finalstepParameters._errorThreshold;

  if (_simulParameters._finalstepParameters._simulationMode == SIMULATIONMODE_GATHERING)
    {
      maillage = getMaillage(_tunnelParameters,_simulParameters._finalstepParameters);
      addLights(solution,_tunnelParameters,_simulParameters._finalstepParameters,light,maillage);
      setPhotometricProperties(maillage,light);
      setPhotometricProperties(maillage,floor,wall,roof);
      solve2(M,maillage, Threshold,Area_Threshold);
    }
  else
    {
      maillage = getMaillageNoLink(_tunnelParameters);
      addLuminaireMaillage(solution,_tunnelParameters,light,maillage);
      setPhotometricProperties(maillage,light);
      setPhotometricProperties(maillage,floor,wall,roof);
      if (Parameters::_interfaceMode == OFF) Shoot(maillage, Area_Threshold, Threshold, Threshold, M);
      else { _solutionInitialized = 1; initializeInteractiveShoot(); }
    }
  removeAllLinks(maillage);
  StaticMemory::stat();
  return maillage;
}

void GlobalData::saveMaillageSolution(ostream& outputfile)
{
  int M = (int)(_simulParameters._finalstepParameters._waveletBasis);
  saveNFF(outputfile,M,maillage,ON,ON);
}

int GlobalData::saveMaillageSolution( char* fname)
{
  if (maillage == NULL) return ERROR_NONE;
  ofstream outputfile(fname,ios::out);
  if (!outputfile)
    return ERROR_FILEOPEN;
  else
    {
      saveMaillageSolution(outputfile);
      outputfile.close();
      Parameters::updateFileParameters(fname,
				       Parameters::_geometricDataDirectory,
				       Parameters::_geometricDataFilename,
				       Parameters::_geometricDataFileExtension);
    }
  return ERROR_NONE;
}

int GlobalData::solutionInitialized()
{
  return _solutionInitialized;
}

 Surface* GlobalData::is_crtS = NULL;
 Surface* GlobalData::is_Ssou = NULL;
 int GlobalData::is_i = 0;
 int GlobalData::is_step = 1;
 Surface** GlobalData::is_seen = NULL;
 int GlobalData::is_nbseen = 0;
 int GlobalData::is_crtseen = 0;
 GlobalData::eInteractiveShootStep GlobalData::is_etape = GlobalData::is_findsource;

void GlobalData::initializeInteractiveShoot()
{
  is_crtS = NULL;
  is_Ssou = NULL;
  is_i = 0;
  is_step = 1;
  if (is_seen != NULL) delete [] is_seen;
  is_seen = (Surface**)(new (Surface*)[maillage->N_surfaces]);
  is_nbseen = 0;
  is_crtseen = 0;
  init_delta_B(maillage);
  is_etape = is_findsource;
}

int GlobalData::oneStepInteractiveShoot()
{
  switch (is_etape)
    {
    case is_findsource:
      TRACEFUNCTION; TRACEVARIABLE(is_step); TRACEMESSAGE("  find source ");
      is_Ssou = find_source(maillage, is_step); TRACEVARIABLE(is_Ssou);
      if (is_Ssou == NULL) is_step++;
      else {
	is_etape = is_linkshoot; is_crtS = maillage->slist;
	/* if (is_Ssou->already_done == 0) { is_etape = is_linkshoot; is_crtS = maillage->slist; } else { is_etape = is_getseen; } */
	is_nbseen = 0;
      }
      break;
    case is_linkshoot:
      if (is_crtS == NULL) { is_crtseen = 0; is_etape = is_shoot; TRACEVARIABLE(is_nbseen); }
      else {
	link_shoot_onestep(_simulParameters._finalstepParameters._waveletBasis, maillage, is_crtS, _simulParameters._finalstepParameters._patchAreaThreshold,
		   _simulParameters._finalstepParameters._errorThreshold, is_Ssou, is_seen, &is_nbseen);
	is_crtS = is_crtS->nexts;
      }
      break;
    case is_getseen:
      get_seen(&(is_Ssou->p), is_seen, &is_nbseen);
      is_crtseen = 0;
      is_etape = is_shoot; TRACEVARIABLE(is_nbseen);
      break;
    case is_shoot:
      if (is_crtseen == is_nbseen) { is_etape = is_initdB; }
      else {
	//TRACEFUNCTION; TRACEVARIABLE(is_Ssou); TRACEVARIABLE(is_seen[is_crtseen]);
	shoot_from_source_onestep(_simulParameters._finalstepParameters._waveletBasis,maillage,is_crtseen,_simulParameters._finalstepParameters._patchAreaThreshold,
				  _simulParameters._finalstepParameters._errorThreshold,is_Ssou, is_seen, is_nbseen);
	is_crtseen++;
      }
      break;
    case is_initdB:
      is_Ssou->already_done ++;
      for (int i=0;i<is_nbseen;i++) removeAllLinks(&(is_seen[i]->p));
      init_null_delta_B(&(is_Ssou->p));
      is_etape = is_findsource;
      break;
    }
  if (is_step >= 5) return 1;
  return 0;
}

void GlobalData::lancerRecherche()
{
  researchMode = 1;
  double tunnellength = _tunnelParameters.shape._length;
  _tunnelParameters.shape._length = Parameters::_surfaceMaximumLength - smallEpsilon;
  StaticMemory::allocate(maxNumberOfPointsInSpectrum,Parameters::_MaxM,GlobalData::getSimulationParameters()._finalstepParameters._waveletBasis);
  rechercheSolutions(_tunnelParameters,_simulParameters._preprocessParameters,_simulParameters._configurationNumber,_requiredRadianceCurve,_lightList,_floorList,_wallList,_roofList);
  StaticMemory::free(maxNumberOfPointsInSpectrum,Parameters::_MaxM,GlobalData::getSimulationParameters()._finalstepParameters._waveletBasis);
  _tunnelParameters.shape._length = tunnellength;
}

void GlobalData::dumpGlobalData()
{
  cout << "GlobalData::(" << endl;
  cout << "\t" << _tunnelParameters.shape._roofShape << endl;
  cout << "\t" << _tunnelParameters.shape._floorLanes << endl;
  cout << "\t" << _tunnelParameters.shape._length << endl;
  cout << "\t" << _tunnelParameters.shape._maxHeight << endl;
  cout << "\t" << _tunnelParameters.shape._width << endl;
  cout << "\t" << _tunnelParameters.shape._minHeight << endl;
  cout << "\t" << _tunnelParameters.shape._angle << endl;
  cout << "\t" << _tunnelParameters.light._lightPosition << endl;
  cout << "\t" << _tunnelParameters.light._lightSymmetrical << endl;
  cout << "\t" << _tunnelParameters.light._lightSpacing << endl;
  cout << "\t" << _tunnelParameters.light._lightSlope << endl;
  cout << "\t" << _simulParameters._finalstepParameters._patchAreaThreshold << endl;
  cout << "\t" << _simulParameters._finalstepParameters._lightAreaThreshold << endl;
  cout << "\t" << _simulParameters._finalstepParameters._globalThreshold << endl;
  cout << "\t" << _simulParameters._finalstepParameters._errorThreshold << endl;
  cout << "\t" << _simulParameters._finalstepParameters._simulationMode << endl;
  cout << "\t" << _simulParameters._finalstepParameters._waveletBasis << endl;
  cout << ")::GlobalData" << endl;
}
