#include "others/Parameters.h"

#include "others/GlobalData.h"
#include "others/SystemFuncs.h"

char Parameters::_programName[255];
char Parameters::_programDirectory[255];
char Parameters::_language[255];
char Parameters::_requiredRadianceCurveDirectory[255];
char Parameters::_reflectanceMaterialSpectrumDirectory[255];
char Parameters::_lightSourceDirectory[255];
char Parameters::_lightSourceListDirectory[255];
char Parameters::_materialListDirectory[255];
char Parameters::_tunnelParametersDirectory[255];
char Parameters::_simulationParametersDirectory[255];
char Parameters::_resolutionSearchDirectory[255];
char Parameters::_geometricDataDirectory[255];

char Parameters::_requiredRadianceCurveFileExtension[255];
char Parameters::_reflectanceMaterialSpectrumFileExtension[255];
char Parameters::_lightSourceExtension[255];
char Parameters::_listFileExtension[255];
char Parameters::_tunnelParametersFileExtension[255];
char Parameters::_simulationParametersFileExtension[255];
char Parameters::_resolutionSearchFileExtension[255];
char Parameters::_geometricDataFileExtension[255];

char Parameters::_requiredRadianceCurveFilename[255];
char Parameters::_reflectanceMaterialSpectrumFilename[255];
char Parameters::_lightSourceFilename[255];
char Parameters::_lightSourceListFilename[255];
char Parameters::_floorMaterialListFilename[255];
char Parameters::_wallMaterialListFilename[255];
char Parameters::_roofMaterialListFilename[255];
char Parameters::_tunnelFilename[255];
char Parameters::_simulationFilename[255];
char Parameters::_resolutionSearchFilename[255];
char Parameters::_geometricDataFilename[255];

float Parameters::_limiteEntreeDuTunnel = 10.;
float Parameters::_surfaceMaximumLength = 32.;
float Parameters::_surfaceMaximumAspectRatio = 2.;

int Parameters::_MaxM = 4;

int Parameters::_solutionNumber = -1;

enum OnOff Parameters::_interfaceMode = ON;
enum OnOff Parameters::_configurationSearch = OFF;
enum OnOff Parameters::_finalSimulation = OFF;
enum OnOff Parameters::_outputSearchResult = OFF;

void Parameters::updateParameters()
{
  sprintf(_language,"%s","english");

  sprintf(_requiredRadianceCurveDirectory,"%sRRC/",_programDirectory);
  if (!existDirectory(_requiredRadianceCurveDirectory)) strcpy(_requiredRadianceCurveDirectory,_programDirectory);
  sprintf(_reflectanceMaterialSpectrumDirectory,"%sMRS/",_programDirectory);
  if (!existDirectory(_reflectanceMaterialSpectrumDirectory)) strcpy(_reflectanceMaterialSpectrumDirectory,_programDirectory);
  sprintf(_lightSourceDirectory,"%sLights/",_programDirectory);
  if (!existDirectory(_lightSourceDirectory)) strcpy(_lightSourceDirectory,_programDirectory);
  sprintf(_lightSourceListDirectory,"%sLights/",_programDirectory);
  if (!existDirectory(_lightSourceListDirectory)) strcpy(_lightSourceListDirectory,_programDirectory);
  sprintf(_materialListDirectory,"%sMaterials/",_programDirectory);
  if (!existDirectory(_materialListDirectory)) strcpy(_materialListDirectory,_programDirectory);
  sprintf(_tunnelParametersDirectory,"%sTunnels/",_programDirectory);
  if (!existDirectory(_tunnelParametersDirectory)) strcpy(_tunnelParametersDirectory,_programDirectory);
  sprintf(_simulationParametersDirectory,"%sSimulation/",_programDirectory);
  if (!existDirectory(_simulationParametersDirectory)) strcpy(_simulationParametersDirectory,_programDirectory);
  sprintf(_resolutionSearchDirectory,"%sResolution/",_programDirectory);
  if (!existDirectory(_resolutionSearchDirectory)) strcpy(_resolutionSearchDirectory,_programDirectory);
  sprintf(_geometricDataDirectory,"%sGeometrics/",_programDirectory);
  if (!existDirectory(_geometricDataDirectory)) strcpy(_geometricDataDirectory,_programDirectory);

  strcpy(_requiredRadianceCurveFileExtension,".rrc");
  strcpy(_reflectanceMaterialSpectrumFileExtension,".mrs");
  strcpy(_lightSourceExtension,".light");
  strcpy(_listFileExtension,".list");
  strcpy(_tunnelParametersFileExtension,".tunnel");
  strcpy(_simulationParametersFileExtension,".simulation");
  strcpy(_resolutionSearchFileExtension,".resolution");
  strcpy(_geometricDataFileExtension,".nff");

  sprintf(_requiredRadianceCurveFilename,"%scourbe2%s",_requiredRadianceCurveDirectory,_requiredRadianceCurveFileExtension);
  sprintf(_reflectanceMaterialSpectrumFilename,"%sAcier%s",_reflectanceMaterialSpectrumDirectory,_reflectanceMaterialSpectrumFileExtension);
  sprintf(_lightSourceFilename,"%sLampe3%s",_lightSourceDirectory,_lightSourceExtension);
  sprintf(_lightSourceListFilename,"%sLights%s",_lightSourceListDirectory,_listFileExtension);
  sprintf(_floorMaterialListFilename,"%sFloor%s",_materialListDirectory,_listFileExtension);
  sprintf(_wallMaterialListFilename,"%sWall%s",_materialListDirectory,_listFileExtension);
  sprintf(_roofMaterialListFilename,"%sRoof%s",_materialListDirectory,_listFileExtension);
  sprintf(_tunnelFilename,"%sdefault%s",_tunnelParametersDirectory,_tunnelParametersFileExtension);
  sprintf(_simulationFilename,"%sdefault%s",_simulationParametersDirectory,_simulationParametersFileExtension);
  sprintf(_resolutionSearchFilename,"%sdefault%s",_resolutionSearchDirectory,_resolutionSearchFileExtension);
  sprintf(_geometricDataFilename,"%sdefault%s",_geometricDataDirectory,_geometricDataFileExtension);

  _limiteEntreeDuTunnel = 10.;
  _surfaceMaximumLength = 32.;
  _surfaceMaximumAspectRatio = 2.;

  _MaxM = 4;

  _solutionNumber = -1;

  _interfaceMode = ON;
  _configurationSearch = OFF;
  _finalSimulation = OFF;
  _outputSearchResult = OFF;
}

void Parameters::initializeDefaultParameters()
{
  strcpy(_programName,"tunnel");
  strcpy(_programDirectory,"./");
  /*
    on choisit le repertoire de travail :
      - si la variable 'TUNNELDATADIR' est definie, on la prends
      - sinon on utilise 'cwd'
  */
  char* datadir = getenv("TUNNELDATADIR"); cout << "TUNNELDATADIR == " << datadir << endl;
  char* workingdir = getenv("PWD"); cout << "PWD == " << workingdir << endl;
  if ((datadir != NULL) && (existDirectory(datadir))) sprintf(_programDirectory,"%s/",datadir);
  else sprintf(_programDirectory,"%s/",workingdir);
  /* initialisation des parametres */
  updateParameters();
//   if (datadir != NULL) delete [] datadir;
//   if (workingdir != NULL) delete [] workingdir;
}

void Parameters::initializeProgramParameters(char* argv)
{
  int loaded = 0;
  /* recuperation du nom du programme */
  int idslash,iddot;
  char** adir = getAbsoluteDirectory(argv);
  char* aname = concatSplittedDirectories(adir);
  getFilenameIndexes(aname,idslash,iddot);
  sprintf(_programName,"%s",&(aname[idslash+1]));
  /*
    Recherche du fichier de configuration par defaut. Dans l'ordre :
      - dans le '_programDirectory'
      - dans le 'cwd'
      - dans le repertoire de l'executable
  */
  char configFilename[255];
  sprintf(configFilename,"%s%s.config",_programDirectory,_programName);
  if (existFile(configFilename)) { loadConfiguration(configFilename); loaded = 1; }
  else {
    char* workingdir = getenv("PWD");
    sprintf(configFilename,"%s%s.config",workingdir,_programName);
    if (existFile(configFilename)) { loadConfiguration(configFilename); loaded = 1; }
    else {
      char c = aname[idslash+1]; aname[idslash+1] = '\0';
      sprintf(configFilename,"%s%s.config",aname,_programName);
      aname[idslash+1] = c;
      if (existFile(configFilename)) { loadConfiguration(configFilename); loaded = 1; }
      else {
	configFilename[0] = '\0';
      }
    }
//     delete workingdir;
  }
  cout << "fichier de configuration :: " << configFilename << endl;
  if (!loaded) {
    /* pas de fichier de configuration, on en cree un dans le '_programDirectory' */
      updateParameters();
      sprintf(configFilename,"%s%s.config",_programDirectory,_programName);
      saveConfiguration(configFilename);
  }
  freeSplittedDirectories(adir);
  delete [] aname;
}

int cantrecover;
int errorparse;
int needhelp;
int offlinemode;
int searchmode;
int simulatemode;
int outputrsf;
char cfname[255];
char rrcname[255];
char tpname[255];
char spname[255];
char rfiname[255];
char rfoname[255];
char nffname[255];
int numsol;

void initializeTemporaryParameters()
{
  cantrecover = 0;
  errorparse = 0;
  needhelp = 0;
  offlinemode = 0;
  searchmode = 0;
  simulatemode = 0;
  outputrsf = 0;
  cfname[0] = '\0';
  rrcname[0] = '\0';
  tpname[0] = '\0';
  spname[0] = '\0';
  rfiname[0] = '\0';
  rfoname[0] = '\0';
  nffname[0] = '\0';
  numsol = -1;
}

void Parameters::showCommandLineUsage(char* argv)
{
  cout << "Usage of " << argv << endl;
  cout << "\t" << "command-line arguments : " << endl;
  cout << "\t\t" << "[-h , --help]" << endl;
  cout << "\t\t" << "[-nogui , --offline-computation] : offline computation mode" << endl;
  cout << "\t\t" << "[-sch , --configuration-search] : configuration search in offline mode" << endl;
  cout << "\t\t" << "[-sim , --final-light-simulation] : final simulation in offline mode" << endl;
  cout << "\t\t" << "[-cf , --configuration-file] , requires a file name : configuration file specification" << endl;
  cout << "\t\t" << "[-rrc , --required-radiance] , requires a file name : required radiance curve file specification" << endl;
  cout << "\t\t" << "[-tp , --tunnel-parameters] , requires a file name : tunnel parameters file specification" << endl;
  cout << "\t\t" << "[-sp , --simulation-parameters] , requires a file name : simulation parameters file specification" << endl;
  cout << "\t\t" << "[-rfi , --resolution-input-filename] , requires a file name : resolution file (to read) specification" << endl;
  cout << "\t\t" << "[-rfo , --resolution-output-filename] , requires a file name : resolution file (to write) specification" << endl;
  cout << "\t\t" << "[-nff , --nff-output-filename] , requires a file name : nff file specification" << endl;
  cout << "\t\t" << "[-sol , --solution-number] , requires a number : solution number for final simulation step" << endl;
  cout << "\t" << "modes : " << endl;
  cout << "\t\t" << "interactive mode (with gui) : " << endl;
  cout << "\t\t\t" << "required   :" << "\t" << "[]" << endl;
  cout << "\t\t\t" << "optionnal  :" << "\t" << "[-cf , -rrc , -tp , -sp , -rfi]" << endl;
  cout << "\t\t\t" << "prohibited :" << "\t" << "[-nogui , -rfo , -nff , -sch , -sim , -sol]" << endl;
  cout << "\t\t" << "offline mode, configuration search : " << endl;
  cout << "\t\t\t" << "required   :" << "\t" << "[-nogui , -sch , -rfo]" << endl;
  cout << "\t\t\t" << "optionnal  :" << "\t" << "[-cf , -rrc , -tp , -sp]" << endl;
  cout << "\t\t\t" << "prohibited :" << "\t" << "[-rfi , -nff , -sim , -sol]" << endl;
  cout << "\t\t" << "offline mode, final simulation : " << endl;
  cout << "\t\t\t" << "required   :" << "\t" << "[-nogui , -sim , -rfi , -nff , -sol]" << endl;
  cout << "\t\t\t" << "optionnal  :" << "\t" << "[-cf , -rrc , -tp , -sp]" << endl;
  cout << "\t\t\t" << "prohibited :" << "\t" << "[-rfo , -sch]" << endl;
  cout << "\t\t" << "offline mode, configuration search and final simulation : " << endl;
  cout << "\t\t\t" << "required   :" << "\t" << "[-nogui , -rsh , -sim , -nff , -sol]" << endl;
  cout << "\t\t\t" << "optionnal  :" << "\t" << "[-cf , -rrc , -tp , -sp , -rfo]" << endl;
  cout << "\t\t\t" << "prohibited :" << "\t" << "[-rfi]" << endl;
}

void Parameters::verifyCommandLineParameters()
{
  if (!offlinemode) {
    if (searchmode || simulatemode) {
      cerr << " can't specify research nor simulation modes while in interactive mode";
      cerr << " : setting these options off (with associated parameters)" << endl;
      searchmode = 0; simulatemode = 0;
      rfiname[0] = '\0'; rfoname[0] = '\0'; nffname[0] = '\0';
      numsol = -1;
      errorparse = 1;
    }
    //if (rfiname[0] != '\0') { cerr << "can't specify input resolution filename in interactive mode : setting off" << endl; rfiname[0] = '\0'; errorparse = 1; }
    if (rfoname[0] != '\0') { cerr << "can't specify output resolution filename in interactive mode : setting off" << endl; rfoname[0] = '\0'; errorparse = 1; }
    if (nffname[0] != '\0') { cerr << "can't specify output nff filename in interactive mode : setting off" << endl; nffname[0] = '\0'; errorparse = 1; }
    if (numsol != -1) { cerr << "can't specify solution number in interactive mode : setting off" << endl; numsol = -1; errorparse = 1; }
  } else {
    if (!searchmode && !simulatemode) {
      cerr << "can't specify offline mode without one of research or simulation mode : unrecoverable error" << endl;
      errorparse = 1;
      cantrecover = 1;
    } else if (searchmode && !simulatemode) {
      if (rfoname[0] == '\0') { cerr << "you must specify output resolution filename in offline research mode : unrecoverable error" << endl; cantrecover = 1; errorparse = 1; }
      if (rfiname[0] != '\0') { cerr << "can't specify input resolution filename in offline research mode : setting off" << endl; rfiname[0] = '\0'; errorparse = 1; }
      if (nffname[0] != '\0') { cerr << "can't specify output nff filename in offline research mode : setting off" << endl; nffname[0] = '\0'; errorparse = 1; }
      if (numsol != -1) { cerr << "can't specify solution number in offline research mode : setting off" << endl; numsol = -1; errorparse = 1; }
    } else if (!searchmode && simulatemode) {
      if (rfiname[0] == '\0') { cerr << "you must specify input resolution filename in offline simulation mode : unrecoverable error" << endl; cantrecover = 1; errorparse = 1; }
      if (nffname[0] == '\0') { cerr << "you must specify output nff filename in offline simulation mode : unrecoverable error" << endl; cantrecover = 1; errorparse = 1; }
      if (numsol == -1) { cerr << "you must specify solution number in offline simulation mode : unrecoverable error" << endl; cantrecover = 1; errorparse = 1; }
      if (rfoname[0] != '\0') { cerr << "can't specify output resolution filename in offline simulation mode : setting off" << endl; rfoname[0] = '\0'; errorparse = 1; }
    } else if (searchmode && simulatemode) {
      if (nffname[0] == '\0') { cerr << "you must specify output nff filename in offline research and simulation mode : unrecoverable error" << endl; cantrecover = 1; errorparse = 1; }
      if (numsol == -1) { cerr << "you must specify solution number in offline research and simulation mode : exiting program" << endl; cantrecover = 1; errorparse = 1; }
      if (rfiname[0] != '\0') { cerr << "can't specify input resolution filename in offline research and simulation mode : setting off" << endl; rfoname[0] = '\0'; errorparse = 1; }
    }
  }
  if (rfoname[0] != '\0') outputrsf = 1;
}

void Parameters::initializeCommandLineModeParameters(int argc,char** argv)
{
  int i=1;
  while (i < argc)
    {
      if (argv[i][0] != '-') {
	cerr << "unrecognized option : " << argv[i] << " : forgetting it" << endl; i += 1;
      } else if ((!strcmp(argv[i],"-h")) || (!strcmp(argv[i],"--help"))) {
	needhelp = 1; i += 1;
      } else if ((!strcmp(argv[i],"-nogui")) || (!strcmp(argv[i],"--offline-computation"))) {
	offlinemode = 1; i += 1;
      } else if ((!strcmp(argv[i],"-sch")) || (!strcmp(argv[i],"--configuration-search"))) {
	searchmode = 1; i += 1;
      } else if ((!strcmp(argv[i],"-sim")) || (!strcmp(argv[i],"--final-light-simulation"))) {
	simulatemode = 1; i += 1;
      } else if ((!strcmp(argv[i],"-cf")) || (!strcmp(argv[i],"--configuration-file"))) {
	if ((i+1) < argc) { sprintf(cfname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-cf,--configuration-file] option requires a configuration file name : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-rrc")) || (!strcmp(argv[i],"--required-radiance"))) {
	if ((i+1) < argc) { sprintf(rrcname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-rrc,--required-radiance] option requires a radiance curve file name : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-tp")) || (!strcmp(argv[i],"--tunnel-parameters"))) {
	if ((i+1) < argc) { sprintf(tpname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-tp,--tunnel-parameters] option requires a tunnel parameter file name : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-sp")) || (!strcmp(argv[i],"--simulation-parameters"))) {
	if ((i+1) < argc) { sprintf(spname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-sp,--simulation-parameters] option requires a simulation parameter file name : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-rfi")) || (!strcmp(argv[i],"--resolution-input-filename"))) {
	if ((i+1) < argc) { sprintf(rfiname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-rfi,--resolution-input-filename] option requires a resolution file name fro input : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-rfo")) || (!strcmp(argv[i],"--resolution-output-filename"))) {
	if ((i+1) < argc) { sprintf(rfoname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-rfo,--resolution-output-filename] option requires a resolution file name for output : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-nff")) || (!strcmp(argv[i],"--nff-output-filename"))) {
	if ((i+1) < argc) { sprintf(nffname,"%s",argv[i+1]); i += 2; }
	else { cerr << "[-nff,--nff-output-filename] option requires a nff file name : forgetting it" << endl; i = argc; }
      } else if ((!strcmp(argv[i],"-sol")) || (!strcmp(argv[i],"--solution-number"))) {
	if ((i+1) < argc) { sscanf(argv[i+1],"%d",&numsol); i += 2; }
	else { cerr << "[-sol,--solution-number] option requires a number : forgetting it" << endl; i = argc; }
      } else {
	cerr << "unrecognized option : " << argv[i] << " : forgetting it" << endl; i += 1;
      }
    }
}

void Parameters::initializeCommandLineParameters(int argc,char** argv)
{
  /* etude de la ligne de commande */
  initializeTemporaryParameters();
  initializeCommandLineModeParameters(argc,argv);
  verifyCommandLineParameters();
  if (needhelp || errorparse) showCommandLineUsage(argv[0]);
  if (cantrecover) exit(ERROR_WRONGCOMMANDLINEARGUMENT);
  /* mise a jour des donnees */
  if (cfname[0] != '\0') loadConfiguration(cfname);
  if (rrcname[0] != '\0') GlobalData::loadRequiredRadianceCurve(rrcname);
  if (spname[0] != '\0') GlobalData::loadSimulationParameters(spname);
  if (tpname[0] != '\0') GlobalData::loadTunnelParameters(tpname);
  if (!offlinemode)
    {
      if (rfiname[0] != '\0') GlobalData::loadResolutionSearchResult(rfiname);
    }
  else
    {
      if (searchmode && !simulatemode)
	{
	  updateFileParameters(rfoname,_resolutionSearchDirectory,_resolutionSearchFilename,_resolutionSearchFileExtension);
	  _outputSearchResult = ON;
	}
      else if (!searchmode && simulatemode)
	{
	  GlobalData::loadResolutionSearchResult(rfiname);
	  updateFileParameters(nffname,_geometricDataDirectory,_geometricDataFilename,_geometricDataFileExtension);
	  _solutionNumber = numsol;
	}
      else if (searchmode && simulatemode)
	{
	  if (rfoname[0] != '\0') {
	    updateFileParameters(rfoname,_resolutionSearchDirectory,_resolutionSearchFilename,_resolutionSearchFileExtension);
	    _outputSearchResult = ON;
	  }
	  updateFileParameters(nffname,_geometricDataDirectory,_geometricDataFilename,_geometricDataFileExtension);
	  _solutionNumber = numsol;
	}
    }
  _interfaceMode = (offlinemode ? OFF : ON);
  _configurationSearch = (searchmode ? ON : OFF);
  _finalSimulation = (simulatemode ? ON : OFF);
}

eErrorCode Parameters::loadConfiguration(char* fname)
{
  char str[255];
  ifstream inputConfigFile(fname,ios::in);
  if (!inputConfigFile)
    {
      return ERROR_FILEOPEN;
    }
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_language,255,'\n');

  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_requiredRadianceCurveDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_reflectanceMaterialSpectrumDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_lightSourceDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_lightSourceListDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_materialListDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_tunnelParametersDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_simulationParametersDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_resolutionSearchDirectory,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_geometricDataDirectory,255,'\n');

  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_requiredRadianceCurveFileExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_reflectanceMaterialSpectrumFileExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_lightSourceExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_listFileExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_tunnelParametersFileExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_simulationParametersFileExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_resolutionSearchFileExtension,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_geometricDataFileExtension,255,'\n');

  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_requiredRadianceCurveFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_reflectanceMaterialSpectrumFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_lightSourceFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_lightSourceListFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_floorMaterialListFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_wallMaterialListFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_roofMaterialListFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_tunnelFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_simulationFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_resolutionSearchFilename,255,'\n');
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(_geometricDataFilename,255,'\n');

  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(str,255,'\n'); sscanf(str,"%f",&_limiteEntreeDuTunnel);
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(str,255,'\n'); sscanf(str,"%f",&_surfaceMaximumLength);
  inputConfigFile.getline(str,255,':'); goThroughBlankChar(inputConfigFile); inputConfigFile.getline(str,255,'\n'); sscanf(str,"%f",&_surfaceMaximumAspectRatio);

  inputConfigFile.close();

  GlobalData::initializeDefaultData();

  return ERROR_NONE;
}

eErrorCode Parameters::saveConfiguration(char* fname)
{
  ofstream outputConfigFile(fname,ios::out);
  if (!outputConfigFile)
    {
      return ERROR_FILEOPEN;
    }
  outputConfigFile << "language : " << _language << endl << endl;

  outputConfigFile << "radiance curve directory : " << _requiredRadianceCurveDirectory << endl;
  outputConfigFile << "material spectrum directory : " << _reflectanceMaterialSpectrumDirectory << endl;
  outputConfigFile << "light description directory : " << _lightSourceDirectory << endl;
  outputConfigFile << "light list directory : " << _lightSourceListDirectory << endl;
  outputConfigFile << "material list directory : " << _materialListDirectory << endl;
  outputConfigFile << "tunnel parameters directory : " << _tunnelParametersDirectory << endl;
  outputConfigFile << "simulation parameters directory : " << _simulationParametersDirectory << endl;
  outputConfigFile << "resolution directory : " << _resolutionSearchDirectory << endl << endl;
  outputConfigFile << "geometry directory : " << _geometricDataDirectory << endl << endl;

  outputConfigFile << "radiance curve extension : " << _requiredRadianceCurveFileExtension << endl;
  outputConfigFile << "material spectrum extension : " << _reflectanceMaterialSpectrumFileExtension << endl;
  outputConfigFile << "light description extension : " << _lightSourceExtension << endl;
  outputConfigFile << "list extension : " << _listFileExtension << endl;
  outputConfigFile << "tunnel parameters extension : " << _tunnelParametersFileExtension << endl;
  outputConfigFile << "simulation parameters extension : " << _simulationParametersFileExtension << endl << endl;
  outputConfigFile << "resolution search extension : " << _resolutionSearchFileExtension << endl << endl;
  outputConfigFile << "geometry extension : " << _geometricDataFileExtension << endl << endl;

  outputConfigFile << "initial radiance curve : " << _requiredRadianceCurveFilename << endl;
  outputConfigFile << "initial material spectrum  : " << _reflectanceMaterialSpectrumFilename << endl;
  outputConfigFile << "initial light description : " << _lightSourceFilename << endl;
  outputConfigFile << "initial light list : " << _lightSourceListFilename << endl;
  outputConfigFile << "initial floor list : " << _floorMaterialListFilename << endl;
  outputConfigFile << "initial wall list : " << _wallMaterialListFilename << endl;
  outputConfigFile << "initial roof list : " << _roofMaterialListFilename << endl;
  outputConfigFile << "initial tunnel parameters : " << _tunnelFilename << endl;
  outputConfigFile << "initial simulation parameters : " << _simulationFilename << endl << endl;
  outputConfigFile << "resolution search result : " << _resolutionSearchFilename << endl << endl;
  outputConfigFile << "geometry output : " << _geometricDataFilename << endl << endl;

  outputConfigFile << "tunnel entrance distance : " << _limiteEntreeDuTunnel << endl;
  outputConfigFile << "surface maximum length : " << _surfaceMaximumLength << endl;
  outputConfigFile << "surface maximum apect ratio : " << _surfaceMaximumAspectRatio << endl << endl;

  outputConfigFile.close();
  return ERROR_NONE;
}

void Parameters::updateFileParameters(char* filename,char* dir,char* file,char* ext)
{
  int idslash,iddot;
  getFilenameIndexes(filename,idslash,iddot);
  if (idslash >= 0)
    {
      char c = filename[idslash+1];
      filename[idslash+1] = '\0';
      sprintf(dir,"%s",filename);
      filename[idslash+1] = c;
    }
  else
    {
      dir[0] = '\0';
    }
  if (iddot < strlen(filename)) sprintf(ext,"%s",&(filename[iddot]));
  else ext[0] = '\0';
  sprintf(file,"%s",filename);
  char** sdir = getAbsoluteDirectory(dir);
  char* cdir = concatSplittedDirectories(sdir);
  sprintf(dir,"%s",cdir);
  freeSplittedDirectories(sdir);
  delete [] cdir;
}

void Parameters::dumpParameters()
{
  cout << "Parameters::(" << endl;
  cout << "\t" << _limiteEntreeDuTunnel << endl;
  cout << "\t" << _surfaceMaximumLength << endl;
  cout << "\t" << _surfaceMaximumAspectRatio << endl;
  cout << "\t" << _MaxM << endl;
  cout << ")::Parameters" << endl;
}
