#include "radiosite/Gauss.h"

#include "radiosite/Wavelet.h"

double gauss_quadrature_point[4][4]={
  {0.5,0,0,0},
  {0.21132487,0.78867513,0,0},
  {0.11270167,0.5,0.88729833,0},
  {0.069431845,0.33000948,0.66999052,0.93056816}
};

double gauss_quadrature_weight[4][4]={
  {1,0,0,0},
  {0.5, 0.5,0,0},
  {0.27777778,0.44444444,0.27777778,0},
  {0.17392742,0.32607258,0.32607258,0.17392742}
};

double projection_coeff_1v(int M,int m, int J,double *f)
{
  int two_power_J=get_two_power_J(J),u;
  double product=0;
  for(u=0;u<M;u++)
    product+=gauss_quadrature_weight[M-1][u]*f[u]*
      Phi(m,gauss_quadrature_point[M-1][u]);
  product /= sqrt((double)two_power_J);
  return(product);
}

double projection_coeff_2v(int M,int m,int n, int J,double *f)
{
  int two_power_J=get_two_power_J(J),u,v,index;
  double product_u=0,product_v;
  for(u=index=0;u<M;u++){
    product_v=0;
    for(v=0;v<M;v++,index++){
      product_v+=gauss_quadrature_weight[M-1][v]*f[index]*
	Phi(n,gauss_quadrature_point[M-1][v]);
    }
    product_u += product_v*gauss_quadrature_weight[M-1][u]*
      Phi(m,gauss_quadrature_point[M-1][u]);
  }
  product_u /= two_power_J;				
  return(product_u);
}

double projection_coeff_4v(int M,int m,int n,int o, int p, int J_a,int J_b,double *f)
{
  int two_power_J_a=get_two_power_J(J_a),
    two_power_J_b=get_two_power_J(J_b),u,v,s,t,index;
  double product_u=0,product_t,product_s,product_v;
  for(u=index=0;u<M;u++){
    product_v=0;
    for(v=0;v<M;v++){
      product_s=0;
      for(s=0;s<M;s++){
	product_t=0;
	for(t=0;t<M;t++,index++){
	  product_t+=gauss_quadrature_weight[M-1][t]*f[index]*
	    Phi(p,gauss_quadrature_point[M-1][t]);
	}
	product_s += product_t*gauss_quadrature_weight[M-1][s]*
	  Phi(o,gauss_quadrature_point[M-1][s]);
      }
      product_v += product_s*gauss_quadrature_weight[M-1][v]*
	Phi(n,gauss_quadrature_point[M-1][v]);
    }
    product_u += product_v*gauss_quadrature_weight[M-1][u]*
      Phi(m,gauss_quadrature_point[M-1][u]);
  }
  product_u /= two_power_J_a;
  product_u/= two_power_J_b;
  return(product_u);
}
