#include "radiosite/Kernel.h"

#include "radiosite/VecLib.h"
#include "radiosite/Visibility.h"
#include "radiosite/Lumin.h"

void area_related(Point3 *A,Point3 *B,Point3 *D,Point3 *C,double *a)
{
  Vector3 B_A,C_A,D_B,D_C,A_1,A_2,A_3,A_4;
  V3Sub(B,A,&B_A);V3Sub(C,A,&C_A);V3Sub(D,C,&D_C);V3Sub(D,B,&D_B);
  V3Cross(&B_A,&C_A,&A_1); a[0] = V3Length(&A_1);
  V3Cross(&B_A,&D_B,&A_2); a[1] = V3Length(&A_2);
  V3Cross(&D_C,&C_A,&A_3); a[2] = V3Length(&A_3);
  V3Cross(&D_C,&D_B,&A_4); a[3] = V3Length(&A_4);
}

double differential_area(Point3 *A,Point3 *B,Point3 *C,Point3 *D,double u_b,double v_b)
{
  double a_1,a_2,a_3,a_4;
  Vector3 B_A,C_A,D_B,D_C,A_1,A_2,A_3,A_4;
  V3Sub(B,A,&B_A);V3Sub(C,A,&C_A);V3Sub(D,C,&D_C);V3Sub(D,B,&D_B);
  V3Cross(&B_A,&C_A,&A_1); a_1 = V3Length(&A_1);
  V3Cross(&B_A,&D_B,&A_2); a_2 = V3Length(&A_2);
  V3Cross(&D_C,&C_A,&A_3); a_3 = V3Length(&A_3);
  V3Cross(&D_C,&D_B,&A_4); a_4 = V3Length(&A_4);
  return((1-v_b)*(1-u_b)*a_1 + (1-v_b)*u_b*a_2 +
	 v_b*(1-u_b)*a_3 + v_b*u_b*a_4);
}

double delta_A(Point3 *A,Point3 *B,Point3 *D,Point3 *C,double u_b,double v_b)
{
  return(differential_area(A,B,C,D,u_b,v_b));
}

void quadrilateral_point(Point3 *A, Point3 *B, Point3 *C, Point3 *D,double u,double v, Point3 *p)
{
  p->x = (1-v)*((1-u)*A->x + u*B->x)+ v*((1-u)*C->x + u*D->x);
  p->y = (1-v)*((1-u)*A->y + u*B->y)+ v*((1-u)*C->y + u*D->y);
  p->z = (1-v)*((1-u)*A->z + u*B->z)+ v*((1-u)*C->z + u*D->z);
}

double eval_kernel(int Vis, Surface *slist,Patch *a,double u_a,double v_a,Patch *b,double u_b,double v_b,int *shadow_flag)
{
  double value,cos_theta1,cos_theta2,length,area;
  Point3 p_a,p_b,*A,*B,*C,*D;
  Vector3 vec;
  A=&(a->P1);B=&(a->P2);C=&(a->P4);D=&(a->P3);
  quadrilateral_point(A,B,C,D,u_a,v_a,&p_a);
  A=&(b->P1);B=&(b->P2);C=&(b->P4);D=&(b->P3);
  quadrilateral_point(A,B,C,D,u_b,v_b,&p_b);
  if (Vis == PARTIALLY_VISIBLE)
    *shadow_flag = under_shadow(slist,a,&p_a,b,&p_b);
  else *shadow_flag = 0;
  if (*shadow_flag) return(0);
  V3Sub(&p_b,&p_a,&vec);
  length=V3Length(&vec);
  if (length != 0.){
    vec.x /= length; vec.y /= length; vec.z /= length;
  }
  else length = 1;
  {
    double u = b->min_u + (b->max_u - b->min_u)*u_b;
    double v = b->min_v + (b->max_v - b->min_v)*v_b;
    area=((1-v)*(1-u)*b->s->a[0] + (1-v)*u*b->s->a[1] +
	  v*(1-u)*b->s->a[2] + v*u*b->s->a[3]);
  }
  cos_theta1=V3Dot(&(a->s->normal),&vec);
  if(cos_theta1<0.)cos_theta1=0.;
  cos_theta2=-V3Dot(&(b->s->normal),&vec);
  if(cos_theta2<0.)cos_theta2=0.;
  value=((cos_theta1*cos_theta2*Jacobien(b,u_b,v_b))/(M_PI*length*length));
  return(value);
}
