#include "radiosite/Link.h"

#include "radiosite/Surface.h"
#include "radiosite/Patch.h"
#include "radiosite/Luminaire.h"
#include "radiosite/OpticalProperty.h"
#include "radiosite/Visibility.h"
#include "radiosite/Gauss.h"
#include "radiosite/Oracle.h"
#include "radiosite/StaticMemory.h"

#include "others/Debugger.h"

int Link::_numberOfInstances = 0;

void Link::stat()
{
  cout << "Link::stat(" << sizeof(Link) << ") :: " << _numberOfInstances << endl;
}

Link::Link() :
  source(NULL) ,
  no_surface_source(0) ,
  no_patch_source(0) ,
  no_cell_source(0) ,
  depth(0) ,
  next_link(NULL)
{
  ALLOCATEARRAY(F,double***,__simulationParameters._waveletBasis);
  //F = (double****)(new (double***)[Parameters::_MaxM]);
  for (int i=0;i<__simulationParameters._waveletBasis;i++)
    {
      ALLOCATEARRAY(F[i],double**,__simulationParameters._waveletBasis);
      //F[i] = (double***)(new (double**)[Parameters::_MaxM]);
      for (int j=0;j<__simulationParameters._waveletBasis;j++)
	{
	  ALLOCATEARRAY(F[i][j],double*,__simulationParameters._waveletBasis);
	  //F[i][j] = (double**)(new (double*)[Parameters::_MaxM]);
	  for (int k=0;k<__simulationParameters._waveletBasis;k++)
	    {
	      ALLOCATEARRAY(F[i][j][k],double,__simulationParameters._waveletBasis);
	      //F[i][j][k] = (double*)(new (double)[Parameters::_MaxM]);
	    }
	}
    }
  _numberOfInstances++;
}

Link::~Link()
{
  for (int i=0;i<__simulationParameters._waveletBasis;i++)
    {
      for (int j=0;j<__simulationParameters._waveletBasis;j++)
	{
	  for (int k=0;k<__simulationParameters._waveletBasis;k++)
	    {
	      FREEARRAY(F[i][j][k],double,__simulationParameters._waveletBasis);
	      //delete [] F[i][j][k];
	    }
	  FREEARRAY(F[i][j],double*,__simulationParameters._waveletBasis);
	  //delete [] F[i][j];
	}
      FREEARRAY(F[i],double**,__simulationParameters._waveletBasis);
      //delete [] F[i];
    }
  FREEARRAY(F,double***,__simulationParameters._waveletBasis);
  //delete [] F;
  _numberOfInstances--;
}

void create_link(int VM,int J_a,Patch *a,int J_b,Patch *b,double *kernel)
{
  int i,j,k,l;
  Link *L;
  /* Allocation dynamique du lien */
  L = new Link;
  L->no_cell_source = b->no_cellule;
  L->no_surface_source = b->no_surface;
  L->no_patch_source = b->no_patch;  
  L->source = b;
  for(i=0;i<VM;i++)
    for(j=0;j<VM;j++)
      for(k=0;k<VM;k++)
	for(l=0;l<VM;l++)
	  L->F[i][j][k][l] = projection_coeff_4v(VM,i+1,j+1,k+1,l+1,J_a,J_b,kernel);
  L->depth=J_b;
  L->next_link = a->link;
  a->link = L;
}

int already_exists_link(Patch *a, Patch *b)
{
  int found = 0;
  Link *l;
  for(l = a->link; found == 0 && l != NULL; l = l->next_link) {
    if( (l->no_surface_source == b->no_surface) &&
	(l->no_patch_source == b->no_patch)) found = 1;
  }
  return found;
}

void link(int vis,Surface *slist,int M,int J_a,Patch *a,int J_b,Patch *b,double Area_Threshold, double Threshold)
{
  // ENTERINGFUNCTION; TRACEVARIABLE(a); TRACEVARIABLE(b);
  double kernel[16];
  double ovalue;
  int nvis;

  ovalue = oracle(vis,slist,M,a,b,kernel,Area_Threshold,&nvis);

  if (ovalue < Threshold) {
    // TRACEFUNCTION;
    create_link(M,J_a,a,J_b,b,kernel);
  }
  else { /* Must be subdivided and tested for linking.  */
    if (which_one(a,b) == a){
      // TRACEFUNCTION;
      create_children(a);
      link(vis,slist,M,J_a+1,&(a->nextp[0]),J_b,b,Area_Threshold,Threshold);
      link(vis,slist,M,J_a+1,&(a->nextp[1]),J_b,b,Area_Threshold,Threshold);
      link(vis,slist,M,J_a+1,&(a->nextp[2]),J_b,b,Area_Threshold,Threshold);
      link(vis,slist,M,J_a+1,&(a->nextp[3]),J_b,b,Area_Threshold,Threshold);
    }else{
      // TRACEFUNCTION;
      create_children(b);	
      link(vis,slist,M,J_a,a,J_b+1,&(b->nextp[0]),Area_Threshold,Threshold);
      link(vis,slist,M,J_a,a,J_b+1,&(b->nextp[1]),Area_Threshold,Threshold);
      link(vis,slist,M,J_a,a,J_b+1,&(b->nextp[2]),Area_Threshold,Threshold);
      link(vis,slist,M,J_a,a,J_b+1,&(b->nextp[3]),Area_Threshold,Threshold);
    }
  }
  // EXITINGFUNCTION;
}

void link2(int vis,Surface *slist,int M,int J_a,Patch *a,int J_b,Patch *b,double Area_Threshold,double Area_Threshold_sources, double Threshold)
{
  double kernel[16];
  double ovalue = oracle_lum(vis,slist,M,a,b,kernel,Area_Threshold,Area_Threshold_sources);
  if (ovalue < Threshold)
    create_link(M,J_a,a,J_b,b,kernel);
  else { /* Must be subdivided and tested for linking.  */
    if (a->area > Area_Threshold){
      if (a->nextp == NULL) create_children(a);
      link2(vis,slist,M,J_a+1,&(a->nextp[0]),J_b,b,Area_Threshold,Area_Threshold_sources,Threshold);
      link2(vis,slist,M,J_a+1,&(a->nextp[1]),J_b,b,Area_Threshold,Area_Threshold_sources,Threshold);
      link2(vis,slist,M,J_a+1,&(a->nextp[2]),J_b,b,Area_Threshold,Area_Threshold_sources,Threshold);
      link2(vis,slist,M,J_a+1,&(a->nextp[3]),J_b,b,Area_Threshold,Area_Threshold_sources,Threshold);
    }else{
      if (b->nextp == NULL) create_children(b);
      link2(vis,slist,M,J_a,a,J_b+1,&(b->nextp[0]),Area_Threshold,Area_Threshold_sources,Threshold);
      link2(vis,slist,M,J_a,a,J_b+1,&(b->nextp[1]),Area_Threshold,Area_Threshold_sources,Threshold);
      link2(vis,slist,M,J_a,a,J_b+1,&(b->nextp[2]),Area_Threshold,Area_Threshold_sources,Threshold);
      link2(vis,slist,M,J_a,a,J_b+1,&(b->nextp[3]),Area_Threshold,Area_Threshold_sources,Threshold);
    }
  }
}

void link_lum3(int vis,int M,Surface *slist,double Area_Threshold,double Area_Threshold_sources,double Threshold,Surface* Si,Surface* Sj)
{
  if(visibility(slist,&(Si->p),&(Sj->p)) != INVISIBLE)
    {
      link2(vis,slist,M,0,&(Si->p), 0,&(Sj->p),Area_Threshold,Area_Threshold_sources,Threshold);
    }
}

void link_lum2(int vis,int M,Surface *slist,double Area_Threshold,double Area_Threshold_sources,double Threshold,Surface* Si,Surface* Sj)
{
  if (Sj->_splitting) {
    for (int i=0;i<Si->_splitting;i++) link_lum2(vis,M,slist,Area_Threshold,Area_Threshold_sources,Threshold,Si,Sj->_splitted[i]);
  } else link_lum3(vis,M,slist,Area_Threshold,Area_Threshold_sources,Threshold,Si,Sj);
}

void link_lum1(int vis,int M,Surface *slist,double Area_Threshold,double Area_Threshold_sources,double Threshold,Surface* Si,Surface* Sj)
{
  if (Si->_splitting) {
    for (int i=0;i<Si->_splitting;i++) link_lum1(vis,M,slist,Area_Threshold,Area_Threshold_sources,Threshold,Si->_splitted[i],Sj);
  } else link_lum2(vis,M,slist,Area_Threshold,Area_Threshold_sources,Threshold,Si,Sj);
}

void link_lum(int vis,int M,Surface *slist,double Area_Threshold,double Area_Threshold_sources,double Threshold)
{
  cout << "link_lum seuils " << M << " " << Area_Threshold << " " << Area_Threshold_sources << " " << Threshold << endl;
  int nb,i;
  nb = 0;
  Surface* Ssource = slist;
  while (Ssource != NULL)
    {
      while ((Ssource->nexts != NULL) && (Ssource->type_surface != TUNNELSURFACE_LIGHT)) Ssource = Ssource->nexts;
      if (Ssource->type_surface == TUNNELSURFACE_LIGHT)
	{
	  cout << "Light :: " << Ssource << endl;
	  nb++;
	  Surface* Si = slist;
	  while (Si != NULL)
	    {
	      if (Si->type_surface != TUNNELSURFACE_LIGHT) link_lum1(vis,M,slist,Area_Threshold,Area_Threshold_sources,Threshold,Si,Ssource);
	      Si = Si->nexts;
	    }
	  Ssource = Ssource->nexts;
	  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat();
	}
      else Ssource = NULL;
    }
  cout << "liens sur " << nb << " sources" << endl;
}

void link_surfaces3(int M,Surface *slist,double Area_Threshold,double Threshold,Surface* Si,Surface* Sj)
{
  int vflag;
  /* debug */ //fprintf(stderr,"[%d <==> %d]\n",Si,Sj);
  if (Si != Sj) {
    if((vflag=visibility(slist,&(Si->p),&(Sj->p))) != INVISIBLE)
      {
	link(vflag,slist,M,0,&(Si->p),0,&(Sj->p),Area_Threshold,Threshold);
      }
    /* debug */ //fprintf(stderr,"link_surfaces :: %d --> %d == %d\n",Si,Sj,vflag);
  }
}

void link_surfaces2(int M,Surface *slist,double Area_Threshold,double Threshold,Surface* Si,Surface* Sj)
{
  if (Sj->_splitting) {
    for (int i=0;i<Sj->_splitting;i++) link_surfaces2(M,slist,Area_Threshold,Threshold,Si,Sj->_splitted[i]);
  } else link_surfaces3(M,slist,Area_Threshold,Threshold,Si,Sj);
}

void link_surfaces1(int M,Surface *slist,double Area_Threshold,double Threshold,Surface* Si,Surface* Sj)
{
  if (Si->_splitting) {
    for (int i=0;i<Si->_splitting;i++) link_surfaces1(M,slist,Area_Threshold,Threshold,Si->_splitted[i],Sj);
  } else link_surfaces2(M,slist,Area_Threshold,Threshold,Si,Sj);
}

void link_surfaces(int M,Surface *slist,double Area_Threshold,double Threshold)
{
  cout << "link_surfaces seuils " << M << " " << Area_Threshold << " " << Threshold << endl;
  Surface* Si = slist;
  while (Si != NULL)
    {
      Vector3 ni = Si->normal;
      Surface* Sj = slist;
      while (Sj != NULL)
	{
	  link_surfaces1(M,slist,Area_Threshold,Threshold,Si,Sj);
	  Sj = Sj->nexts;
	}
      Si = Si->nexts;
//       cout << "lien sur surface" << endl;
//       Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat();
    }
}



void link_surfaces2(int M,Surface *slist,double Area_Threshold,double Area_Threshold_sources,double Threshold)
{
  Surface* Si = slist;
  while (Si != NULL)
    {
      Surface* Sj = slist;
      while (Sj != NULL)
	{
	  int vflag;
	  if (Si != Sj)
	    {
	      if((vflag=visibility(slist,&(Si->p),&(Sj->p))) != INVISIBLE)
		{
		  /* debug */ // fprintf(stderr,"[%d <==> %d]\n",i,j);
		  link2(vflag,slist,M,0,&(Si->p),0,&(Sj->p),Area_Threshold,Area_Threshold_sources,Threshold);
		}
	    }
	  Sj = Sj->nexts;
	}
      Si = Si->nexts;
    }
}
