#include "radiosite/Lumin.h"

#include "radiosite/Surface.h"
#include "radiosite/OpticalProperty.h"
#include "radiosite/Gauss.h"
#include "radiosite/VecLib.h"
#include "radiosite/Wavelet.h"

double get_Ld(Luminaire *luminaire, double thetaa, double psii, double Area)
{
  double theta, psi, psi0, psi1, theta0,theta1,psimin, psimax, thetamax, I1, I2, I3, I4;
  int itheta0, itheta1, ipsi0, ipsi1;
  theta=thetaa;						
  psi=psii;
  psimin=0.0;
  psimax=luminaire->step_plans*(luminaire->Nb_Plans-1);
  thetamax=luminaire->step_dir*(luminaire->Nb_dir-1);
  psi*=raddeg;
  theta*=raddeg;
  if  ((psi>psimax)&&(psi>180)) psi=360-psi;
  if  ((psi>psimax)&&(psi>90)) psi=180-psi;
  if (psi>psimax) cerr << "Bad light file (too few datas)" << endl;
  itheta0=(int)(theta/luminaire->step_dir);
  itheta1=itheta0+1;
  ipsi0=(int)(psi/luminaire->step_plans);
  ipsi1=ipsi0+1;
  theta0=itheta0*luminaire->step_dir/raddeg;                                   
  theta1=itheta1*luminaire->step_dir/raddeg;
  theta=theta/raddeg;
  psi0=ipsi0*luminaire->step_plans;
  psi1=ipsi1*luminaire->step_plans;
  I1=luminaire->val[ipsi0][itheta0];	
  I2=luminaire->val[ipsi0][itheta1];
  I3=luminaire->val[ipsi1][itheta0];	
  I4=luminaire->val[ipsi1][itheta1];
  I1=LERP( (theta-theta0)/(theta1-theta0), I1, I2);
  I2=LERP( (theta-theta0)/(theta1-theta0), I3, I4);
  I1=LERP( (psi-psi0)/(psi1-psi0), I1, I2);
  I1=I1/cos(theta)/Area;
  return(I1);
}

double get_rad(int k, Patch *b, Point3 *x, double Area)
{
  Vector3 u,v,w,z,n,nu,centerb;
  double phit, psi, theta, Ld;
  V3Lerp(&(b->P2),&(b->P3),0.5,&v);
  V3Lerp(&(b->P1),&(b->P4),0.5,&w);
  V3Lerp(&v,&w,0.5,&centerb);
  V3Sub(&v,&centerb,&v);
  n=b->s->normal;
  V3Cross(&n,&v,&w);
  V3Sub(x,&centerb,&nu);
  V3Cross(&nu,&n,&z);
  V3Cross(&z,&n,&u);
  psi=angle(&v,&u);				
  if (V3Dot(&u,&w)<0) psi=2*M_PI-psi;
  theta=angle(&n,&nu);					 
  Ld=get_Ld(b->s->luminaire,theta,psi, Area);
  phit=b->s->o->emmit;
  Ld*=b->s->o->spectrum[k]*M_PI*phit/1000 ;
  return(Ld);
}		

double Jacobien (Patch *a, double s, double t)
{
  Vector3 u, v, PP2, PP3, PP4;
  Vector2  P1, P2, P3, P4;
  double Jac;
  V3Sub(&(a->P2),&(a->P1), &PP2);
  V3Sub(&(a->P3),&(a->P1), &PP3);
  V3Sub(&(a->P4),&(a->P1), &PP4);
  u=PP2;
  V3Normalize(&u);
  V3Cross(&(a->s->normal),&u,&v);
  P1.x=0;
  P1.y=0;
  P2.x=V3Dot(&PP2, &u);
  P2.y=V3Dot(&PP2, &v);
  P3.x=V3Dot(&PP3, &u);
  P3.y=V3Dot(&PP3, &v);
  P4.x=V3Dot(&PP4, &u);
  P4.y=V3Dot(&PP4, &v);
  Jac=fabs(s*(-P3.x*P2.y-P2.x*P4.y+P2.x*P3.y+P4.x*P2.y)+t*(-P2.x*P4.y+P3.x*P4.y+P4.x*P2.y-P4.x*P3.y)+P2.x*P4.y-P4.x*P2.y);
  return(Jac);       
}
	
double eval_rad(int k, Patch *a,  Patch *b, double s, double t,  double Area)
{	
  double value;
  Point3 x;
  x.x=(1-t)*((1-s)*((a->P1).x)+s*((a->P2).x))+t*((1-s)*((a->P4).x)+s*((a->P3).x));
  x.y=(1-t)*((1-s)*((a->P1).y)+s*((a->P2).y))+t*((1-s)*((a->P4).y)+s*((a->P3).y));	
  x.z=(1-t)*((1-s)*((a->P1).z)+s*((a->P2).z))+t*((1-s)*((a->P4).z)+s*((a->P3).z));
  value=get_rad(k, b, &x, Area);
  return(value);
}

double rebuild(int M, double *B,int i, int j)
{
  double s=0;
  int index, k, l;
  for (k=index=0 ; k<M ; k++) 
    for (l=0 ; l<M ; l++, index++)	
      s+=MultiWavelet_basis_2v(0,k+1,0, gauss_quadrature_point[M-1][i] , l+1,0,gauss_quadrature_point[M-1][j])*B[index];
  return(s);
}
