#include "radiosite/Oracle.h"

#include "radiosite/Gauss.h"
#include "radiosite/Kernel.h"
#include "radiosite/Lumin.h"
#include "radiosite/Lambda.h"

double Polint(int M,double *x,double *y,double u)
{
  int i,m,index;
  double diff, diff1,c[4],d[4],v;
  for(i=0;i<M;i++){
    c[i] = d[i] = y[i];
  }
  index = 1;
  diff = fabs(u-x[0]);
  for(i=1;i<M;i++){
    diff1 = fabs(u-x[i]);
    if (diff1 < diff){
      index = i+1; diff = diff1;
    }
  }
  v = y[index-1];
  index--;
  for (m=1;m<=(M-1);m++){
    double dy;
    for(i=1;i<=(M-m);i++){
      double ho,hp,w,den;
      ho = x[i-1]-u; hp = x[i+m-1]-u;
      w=c[i]-d[i-1];
      den = ho-hp;
      if (den == 0.) cerr << "There are two coincident points in array x" << endl;
      den = w/den;
      d[i-1] = hp*den;
      c[i-1] = ho*den;
    }
    if ((2*index)< (M-m)) dy = c[index];
    else{
      dy = d[index-1];
      index--;
    }
    v += dy;
  }
  return(v);
}

double approx_kernel(int M,double *x, double *K, double u, double v,double s, double t)
{	
  int i,j,k,index;
  double y2tmp[4];
  if ((M < 1)||(M > Parameters::_MaxM)) cerr << "Vanishing moment must be 1-" << Parameters::_MaxM << endl;
  for (i=index=0;i<M;i++){
    double y3tmp[4];
    for (j=0;j<M;j++){
      double y4tmp[4];
      for (k=0;k<M;k++,index+=M)
	y4tmp[k] = Polint(M,x,K+index,t);
      y3tmp[j] = Polint(M,x,y4tmp,s);
    }
    y2tmp[i] = Polint(M,x,y3tmp,v);
  }
  return(Polint(M,x,y2tmp,u));
}

void compute_kernels(int vis,Surface *slist,int M,Patch *a, Patch *b,double *kernel)
{
  int u,v,s,t,index;
  for(u=index=0;u<M;u++)
    for(v=0;v<M;v++)
      for(s=0;s<M;s++)
	for(t=0;t<M;t++,index++)
	  {
	    int flg = 0;
	    kernel[index]=
	      eval_kernel(vis,slist,
			  a,gauss_quadrature_point[M-1][u],
			  gauss_quadrature_point[M-1][v],
			  b,gauss_quadrature_point[M-1][s],
			  gauss_quadrature_point[M-1][t],
			  &flg);
	  }
}

void compute_kernels2(int M,Patch *a, Patch *b,double *kernel,double Area_Threshold,double *kernel2)
{
  int u,v,s,t,index;
  double coef;
  for(u=index=0;u<M;u++)
    for(v=0;v<M;v++){
      coef=eval_rad(1,a,b,gauss_quadrature_point[M-1][u],
		gauss_quadrature_point[M-1][v],Area_Threshold);
      for(s=0;s<M;s++)
	for(t=0;t<M;t++,index++)
	  kernel2[index]= kernel[index]*coef;
    }
}

int emitter(Patch *a)
{
  int k;
  for(k=0;k<sampledSpectrumSize;k++)
    if (a->s->E[k] > 0.) return(1);
  return(0);
}

int recept(Patch *a)
{
  int k;
  for(k=0;k<sampledSpectrumSize;k++)
    if (a->s->rho[k] > 0.) return(1);
  return(0);
}

double oracle(int vis,Surface *slist,int M,Patch *a,Patch *b,double *kernel,double Area_Threshold,int *new_vis)
{
  int u,v,s,t,MM;
  double test_kernel,predict_kernel;
  double error_u,error_v,error_s,error_t;
  double gauss_u,gauss_v,gauss_s,gauss_t;
  int flg;
  double frac = 1.,coef;
  if ((M<1)||(M > Parameters::_MaxM)) cerr << "Vanishing moment must be 1-" << Parameters::_MaxM << endl;
  vis = *new_vis = FULLY_VISIBLE;
  compute_kernels(FULLY_VISIBLE,slist,M,a,b,kernel); 
  if (a->area < Area_Threshold && b->area < Area_Threshold)
    return(0.);
  MM = (M<=2)?M+1:(M-1);
  error_u=0;
  for(u=0;u<MM;u++){
    gauss_u=gauss_quadrature_point[MM-1][u];
    error_v=0;
    for(v=0;v<MM;v++){
      gauss_v=gauss_quadrature_point[MM-1][v];
      coef=1;
      error_s=0;
      for(s=0;s<MM;s++){
	gauss_s=gauss_quadrature_point[MM-1][s];
	error_t=0;
	for(t=0;t<MM;t++){
	  gauss_t=gauss_quadrature_point[MM-1][t];
	  test_kernel=frac * eval_kernel(FULLY_VISIBLE,slist,a,
					 gauss_u,gauss_v,b,gauss_s,gauss_t,&flg)*coef;
	  predict_kernel=approx_kernel(M,gauss_quadrature_point[M-1],
				       kernel,gauss_u,gauss_v,gauss_s,gauss_t);
	  error_t += gauss_quadrature_weight[MM-1][t]*
	    fabs(test_kernel-predict_kernel);
	}
	error_s += gauss_quadrature_weight[MM-1][s]*error_t;
      }
      error_v += gauss_quadrature_weight[MM-1][v]*error_s;
    }
    error_u += gauss_quadrature_weight[MM-1][u]*error_v;
  }
  return( (a->max_u-a->min_u)*(a->max_v-a->min_v)*
	  (b->max_u-b->min_u)*(b->max_v-b->min_v)*  
	  error_u);
}

double oracle_lum(int vis,Surface *slist,int M,Patch *a,Patch *b,double *kernel,double Area_Threshold,double Area_Threshold_sources)
{
  if ((M<1)||(M > Parameters::_MaxM)) cerr << "Vanishing moment must be 1-" << Parameters::_MaxM << endl;
  if (a->area < Area_Threshold && 
      b->area < Area_Threshold_sources){
    compute_kernels(vis,slist,M,a,b,kernel);
    return(0.);
  };
  return(hugeValue);
}
