#include "radiosite/Patch.h"

#include "radiosite/Surface.h"
#include "radiosite/Link.h"
#include "radiosite/VecLib.h"
#include "radiosite/Wavelet.h"
#include "radiosite/Lambda.h"
#include "radiosite/Spectre.h"
#include "radiosite/StaticMemory.h"

int Patch::_numberOfInstances = 0;

void Patch::stat()
{
  cout << "Patch::stat(" << sizeof(Patch) << ") :: " << _numberOfInstances << endl;
}

Patch::Patch()
{
  nextp = NULL;
  link = NULL;
  s = NULL;
  _numberOfInstances++;

  ALLOCATEARRAY(coeff,double*,sampledSpectrumSize);
  ALLOCATEARRAY(d_coeff,double*,sampledSpectrumSize);
  ALLOCATEARRAY(de_coeff,double*,sampledSpectrumSize);
//   coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   d_coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   de_coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
  for (int i=0;i<sampledSpectrumSize;i++)
    {
      ALLOCATEARRAY(coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
      ALLOCATEARRAY(d_coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
      ALLOCATEARRAY(de_coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
//       coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       d_coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       de_coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
      for (int j=0;j<__simulationParameters._waveletBasis*__simulationParameters._waveletBasis;j++)
	{
	  coeff[i][j] = d_coeff[i][j] = de_coeff[i][j] = 0.;
	}
    }
}

Patch::Patch(const Patch& p)
{
  no_cellule = p.no_cellule;
  no_surface = p.no_surface;
  no_patch = p.no_patch;
  P1 = p.P1;
  P2 = p.P2;
  P3 = p.P3;
  P4 = p.P4;
  area = p.area;
  min_u = p.min_u;
  min_v = p.min_v;
  max_u = p.max_u;
  max_v = p.max_v;

  ALLOCATEARRAY(coeff,double*,sampledSpectrumSize);
  ALLOCATEARRAY(d_coeff,double*,sampledSpectrumSize);
  ALLOCATEARRAY(de_coeff,double*,sampledSpectrumSize);
//   coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   d_coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   de_coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
  for (int i=0;i<sampledSpectrumSize;i++)
    {
      ALLOCATEARRAY(coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
      ALLOCATEARRAY(d_coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
      ALLOCATEARRAY(de_coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
//       coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       d_coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       de_coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
      for (int j=0;j<__simulationParameters._waveletBasis*__simulationParameters._waveletBasis;j++)
	{
	  coeff[i][j] = p.coeff[i][j]; d_coeff[i][j] = p.d_coeff[i][j]; de_coeff[i][j] = p.de_coeff[i][j];
	}
    }
//   coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   d_coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   de_coeff = (double**)(new (double*)[maxNumberOfPointsInSpectrum]);
//   for (int i=0;i<maxNumberOfPointsInSpectrum;i++)
//     {
//       coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       d_coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       de_coeff[i] = new double[Parameters::_MaxM*Parameters::_MaxM];
//       for (int j=0;j<Parameters::_MaxM*Parameters::_MaxM;j++)
// 	{
// 	coeff[i][j] = p.coeff[i][j];
// 	d_coeff[i][j] = p.d_coeff[i][j];
// 	de_coeff[i][j] = p.de_coeff[i][j];
// 	}
//     }
  if (p.nextp != NULL) { nextp = new Patch[4]; for (int i=0;i<4;i++) nextp[i] = p.nextp[i]; }
  link = NULL; /* a modifier */
  s = NULL; /* a modifier */
  _numberOfInstances++;
}

Patch::~Patch()
{
  Link* l = link; while (l != NULL) { Link* lt = l->next_link; delete l; l = lt; }
  if (nextp != NULL) delete [] nextp;
  for (int i=0;i<sampledSpectrumSize;i++)
    {
      FREEARRAY(coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
      FREEARRAY(d_coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
      FREEARRAY(de_coeff[i],double,__simulationParameters._waveletBasis*__simulationParameters._waveletBasis);
//       delete [] de_coeff[i];
//       delete [] d_coeff[i];
//       delete [] coeff[i];
    }
  FREEARRAY(coeff,double*,sampledSpectrumSize);
  FREEARRAY(d_coeff,double*,sampledSpectrumSize);
  FREEARRAY(de_coeff,double*,sampledSpectrumSize);
//   delete [] de_coeff;
//   delete [] d_coeff;
//   delete [] coeff;

  _numberOfInstances--;
}

Patch& Patch::operator=(const Patch& p)
{
  if (this != &p)
    {
      if (nextp != NULL) delete [] nextp;
      if (link != NULL) { /* a completer */ }
      no_cellule = p.no_cellule;
      no_surface = p.no_surface;
      no_patch = p.no_patch;
      P1 = p.P1;
      P2 = p.P2;
      P3 = p.P3;
      P4 = p.P4;
      area = p.area;
      min_u = p.min_u;
      min_v = p.min_v;
      max_u = p.max_u;
      max_v = p.max_v;
      for (int i=0;i<sampledSpectrumSize;i++)
	for (int j=0;j<__simulationParameters._waveletBasis*__simulationParameters._waveletBasis;j++)
	  {
	    coeff[i][j] = p.coeff[i][j];
	    d_coeff[i][j] = p.d_coeff[i][j];
	    de_coeff[i][j] = p.de_coeff[i][j];
	  }
      if (p.nextp != NULL) { nextp = new Patch[4]; for (int i=0;i<4;i++) nextp[i] = p.nextp[i]; }
      link = NULL; /* a modifier */
      s = NULL; /* a modifier */
    }
  return *this;
}

void Patch::getColor(int M,int puis,double u,double v,double* rr,double* gg,double* bb)
{
  double r,g,b;
  int i,j,index;
  double s[maxNumberOfPointsInSpectrum];
  for(int n=0;n<sampledSpectrumSize;n++)
    {
      s[n] = 0;
      for(i=index=0;i<M;i++)
	{
	  for(j=0;j<M;j++,index++)
	    {
	      s[n] += puis*2*MultiWavelet_basis_2v(0,i+1,0,u,j+1,0,v)*coeff[n][index];
	    }
	}
    }
  SPECTRUMtoRGB(s,&r,&g,&b);
  if (r > 1.) r = 1.; else if (r <0.) r = 0.;
  if (g > 1.) g = 1.; else if (g <0.) g = 0.;
  if (b > 1.) b = 1.; else if (b <0.) b = 0.;
  correctionGamma(&r,&g,&b);
  *rr = 255.*r;
  *gg = 255.*g;
  *bb = 255.*b;
}

void initSurface(Patch& p,Surface* s)
{
  p.s = s;
  if (p.nextp != NULL)
    for (int i=0;i<4;i++) initSurface(p.nextp[i],s);
}

double patch_area(Point3 *P1,Point3 *P2,Point3 *P3,Point3 *P4)
{
  double a_1,a_2,a_3,a_4;
  Vector3 *A,*B,*C,*D,B_A,C_A,D_B,D_C,A_1,A_2,A_3,A_4;
  /* 
     \int || f_u X F_v ||du dv computation.
  */
  A = P1; B = P2, C= P4, D = P3;
  V3Sub(B,A,&B_A);V3Sub(C,A,&C_A);V3Sub(D,C,&D_C);V3Sub(D,B,&D_B);
  V3Cross(&B_A,&C_A,&A_1); a_1 = V3Length(&A_1);
  V3Cross(&B_A,&D_B,&A_2); a_2 = V3Length(&A_2);
  V3Cross(&D_C,&C_A,&A_3); a_3 = V3Length(&A_3);
  V3Cross(&D_C,&D_B,&A_4); a_4 = V3Length(&A_4);
  return(0.25*(a_1+a_2+a_3+a_4));
}

void create_children(Patch *a)
{
  double mid_u,mid_v;
  Point3 P5,P6,P7,P8,P9;
  
  if (a->nextp != NULL) return;
  a->nextp = new Patch[4];

  a->nextp->no_surface = a->no_surface;
  (a->nextp+1)->no_surface = a->no_surface;
  (a->nextp+2)->no_surface = a->no_surface;
  (a->nextp+3)->no_surface = a->no_surface;
  
  a->nextp->no_patch = a->no_patch * 10 + 1;
  (a->nextp+1)->no_patch = a->no_patch * 10 + 2;
  (a->nextp+2)->no_patch = a->no_patch * 10 + 3;
  (a->nextp+3)->no_patch = a->no_patch * 10 + 4;

  if (__simulationParameters._simulationMode == SIMULATIONMODE_SHOOTING) {
    int i;
    for(i = 0; i < sampledSpectrumSize; i ++) {
      push_2v(__simulationParameters._waveletBasis, a->coeff[i], a->nextp->coeff[i], (a->nextp+1)->coeff[i],(a->nextp+2)->coeff[i], (a->nextp+3)->coeff[i]);
    }
  }

  a->nextp->s = (a->nextp+1)->s =	(a->nextp+2)->s = (a->nextp+3)->s = a->s;
  a->nextp->nextp = (a->nextp+1)->nextp =	(a->nextp+2)->nextp = (a->nextp+3)->nextp = NULL;
  
  V3Lerp(&(a->P1),&(a->P2),0.5,&P5);
  V3Lerp(&(a->P2),&(a->P3),0.5,&P6);
  V3Lerp(&(a->P4),&(a->P3),0.5,&P7);
  V3Lerp(&(a->P1),&(a->P4),0.5,&P8);
  V3Lerp(&P5,&P7,0.5,&P9);
  mid_u = (a->min_u+a->max_u)*0.5;
  mid_v = (a->min_v+a->max_v)*0.5;
  
  /* SW child */
  a->nextp->P1 = a->P1; a->nextp->P2 = P5;
  a->nextp->P3 = P9; a->nextp->P4 = P8;
  a->nextp->area = patch_area(&(a->nextp->P1),&(a->nextp->P2),&(a->nextp->P3),&(a->nextp->P4));
  a->nextp->min_u = a->min_u;
  a->nextp->min_v = a->min_v;
  a->nextp->max_u = mid_u;
  a->nextp->max_v = mid_v;
  /* NW child */
  (a->nextp+1)->P1=P8; (a->nextp+1)->P2=P9;
  (a->nextp+1)->P3=P7; (a->nextp+1)->P4=a->P4;
  (a->nextp+1)->area=patch_area(&((a->nextp+1)->P1),&((a->nextp+1)->P2),&((a->nextp+1)->P3),&((a->nextp+1)->P4));
  (a->nextp+1)->min_u = a->min_u;
  (a->nextp+1)->min_v = mid_v;
  (a->nextp+1)->max_u = mid_u;
  (a->nextp+1)->max_v = a->max_v;
  /* SE child */
  (a->nextp+2)->P1=P5; (a->nextp+2)->P2=a->P2;
  (a->nextp+2)->P3=P6; (a->nextp+2)->P4=P9;
  (a->nextp+2)->area=patch_area(&((a->nextp+2)->P1),&((a->nextp+2)->P2),&((a->nextp+2)->P3),&((a->nextp+2)->P4));
  (a->nextp+2)->min_u = mid_u;
  (a->nextp+2)->min_v = a->min_v;
  (a->nextp+2)->max_u = a->max_u;
  (a->nextp+2)->max_v = mid_v;
  /* NE child */
  (a->nextp+3)->P1=P9; (a->nextp+3)->P2=P6;
  (a->nextp+3)->P3=a->P3; (a->nextp+3)->P4=P7;
  (a->nextp+3)->area=patch_area(&((a->nextp+3)->P1),&((a->nextp+3)->P2),&((a->nextp+3)->P3),&((a->nextp+3)->P4));
  (a->nextp+3)->min_u = mid_u;
  (a->nextp+3)->min_v = mid_v;    
  (a->nextp+3)->max_u = a->max_u;
  (a->nextp+3)->max_v = a->max_v;
}

Patch *which_one(Patch *a, Patch *b)
{
  if (a->area >= b->area) return(a); else return(b);
}

/************************************************************
 * Fonction permettant de recuperer le spectre associe
 * A un point u,v d'un patch P
 *
 * ENTREE : Le vanishing moment,
 *          Le patch P,
 *          Le niveau dans l'arbre 2^niveau,
 *          u,v,
 *          le spectre a recuperer
 * SORTIE : VOID.
 */
void computeUVspectrum(int M,Patch *p,int factor, double u,double v,double* s,double area)
{
  /* Find the sub patch in which the (u,v) parameters lie */
  int k,i,j,index;
  for(k=0;k<sampledSpectrumSize;k++) {
    if (p->s->luminaire!=NULL) s[k]=1;
    else {
      s[k] = 0;
      for(i=index=0;i<M;i++){
	for(j=0;j<M;j++,index++){
	  s[k] +=factor*MultiWavelet_basis_2v(0,i+1,0,u,j+1,0,v)*p->coeff[k][index];
	}
      }
      s[k]*=area;
    }
  }
}/* fin computeUVspectrum */
