#include "radiosite/Solver.h"

#include "radiosite/StaticMemory.h"
#include "radiosite/Link.h"
#include "radiosite/Surface.h"
#include "radiosite/Lambda.h"
#include "radiosite/Lumin.h"
#include "radiosite/Gauss.h"
#include "radiosite/Wavelet.h"

double gather_with_push_pull(int M,double factor,Patch *p,double **coeffs, double Area)
{
  int i,j,k,m1,m2,m,n1,n2,n,index;
  double error;
  Link *l = p->link;
  while(l != NULL)
    { 
      for(k=0;k<sampledSpectrumSize;k++)
	{
	  for(m1=m=0;m1<M;m1++)
	    for(m2=0;m2<M;m2++,m++) 
	      for(n1=n=0;n1<M;n1++)
		for(n2=0;n2<M;n2++,n++) {
		  double a = p->s->rho[k];
		  double b = l->F[m1][m2][n1][n2];
		  double c = l->source->coeff[k][n];
		  coeffs[k][m] += (a*b*c);
		  //coeffs[k][m] +=p->s->rho[k]*l->F[m1][m2][n1][n2]* l->source->coeff[k][n];
		}
	  if (l->source->s->luminaire!=NULL){
	    double s;
	    for (i=index=0 ; i<M ; i++) 
	      for (j=0 ; j<M ; j++, index++)
		{
		  s=eval_rad(k,p,l->source,gauss_quadrature_point[M-1][i],gauss_quadrature_point[M-1][j],Area);
		  for(m1=m=0;m1<M;m1++)
		    for(m2=0;m2<M;m2++,m++) 
		      StaticMemory::Solver_gatherwithpushpull_B[m] =p->s->rho[k]*l->F[m1][m2][0][0]*s/get_two_power_J(l->depth);
		  StaticMemory::Solver_gatherwithpushpull_f[index]=rebuild(M,StaticMemory::Solver_gatherwithpushpull_B, i, j)/factor;
		}
	    for (i=index=0 ; i<M ; i++) 
	      for (j=0 ; j<M ; j++, index++)
		coeffs[k][index]+=projection_coeff_2v(M,i+1,j+1,0,StaticMemory::Solver_gatherwithpushpull_f)*factor;
	  }
	}
      l = l->next_link;
    }
    if (p->nextp == NULL)
      {
	error = 0;
	for(k=0;k<sampledSpectrumSize;k++)
	  {
	    if (fabs(p->coeff[k][0]-coeffs[k][0]) > error)
	      error=fabs(p->coeff[k][0]-coeffs[k][0]);
	    for(m1=m=0;m1<M;m1++)
	      for(m2=0;m2<M;m2++,m++)
		p->coeff[k][m] = coeffs[k][m];
	  }
      }
    else
      {
	double temp_error;
	int i, j;
	error = 0;
	for(k=0;k<sampledSpectrumSize;k++) push_2v(M,coeffs[k],StaticMemory::Solver_gatherwithpushpull_c[0][k],StaticMemory::Solver_gatherwithpushpull_c[1][k],
						   StaticMemory::Solver_gatherwithpushpull_c[2][k],StaticMemory::Solver_gatherwithpushpull_c[3][k]);
	for(i=0;i<4;i++)
	  {
	    temp_error = gather_with_push_pull(M,factor*0.5,&(p->nextp[i]),StaticMemory::Solver_gatherwithpushpull_c[i], Area);
	    if (temp_error > error) error = temp_error;
	  }
	for(k=0;k<sampledSpectrumSize;k++)
	  pull_2v(M,p->coeff[k],(p->nextp[0]).coeff[k],(p->nextp[1]).coeff[k],(p->nextp[2]).coeff[k],(p->nextp[3]).coeff[k]);
      }
    return(error);
}

void solve(int M,Env *env, double Error_Threshold, double Area)
{
  double convergence_error;
  int i, arret = 0;
  /* A coefficient storage structure used to gather
     coefficients and push them down.b
     */
  do{
    int i,j,k;
    convergence_error = 0;
    Surface* S = env->slist;
    while (S->nexts != NULL) /* a modifier pour aller dans l'autre sens */
      {
	double temp_error;
	for(k=0;k<sampledSpectrumSize;k++) for(j=0;j<Parameters::_MaxM*Parameters::_MaxM;j++) StaticMemory::Solver_solve_C[k][j]=0.;
	temp_error= gather_with_push_pull(M,1,&(S->p),StaticMemory::Solver_solve_C, Area);
	if (temp_error > convergence_error) 
	  convergence_error=temp_error;
	S = S->nexts; /* a modifier pour aller dans l'autre sens */
      }
    // fprintf(stderr,"   %f\n",convergence_error);
    arret++;
  } while((convergence_error>Error_Threshold)&&(arret<maximumIteration));
}

double solve2(int M,Env *env, double Error_Threshold, double Area,Surface* S)
{
  double convergence_error = 0.;
  double temp_error;
  int i,j,k;
  if (S->_splitting) {
    for (int i=0;i<S->_splitting;i++) {
      temp_error = solve2(M,env,Error_Threshold,Area,S->_splitted[i]);
      if (temp_error > convergence_error) 
	convergence_error=temp_error;
    }
  } else {
    for(k=0;k<sampledSpectrumSize;k++) for(j=0;j<Parameters::_MaxM*Parameters::_MaxM;j++) StaticMemory::Solver_solve2_C[k][j]=0.;
    convergence_error = gather_with_push_pull(M,1,&(S->p),StaticMemory::Solver_solve2_C, Area);
  }
  return convergence_error;
}

void solve2(int M,Env *env, double Error_Threshold, double Area)
{
  ENTERINGFUNCTION; TRACEVARIABLE(M); TRACEVARIABLE(Error_Threshold); TRACEVARIABLE(Area);
  double convergence_error;
  int i, arret = 0;
  /* 
     A coefficient storage structure used to gather coefficients and push them down.b
  */
  do{
    TRACEFUNCTION; TRACEMESSAGE("Iteration gathering"); TRACEVARIABLE(arret);
    int i,j,k;
    convergence_error = 0;
    Surface* S = env->slist;
    i = 0;
    while (S != NULL)
      {
	if (S->type_surface != TUNNELSURFACE_LIGHT)
	  {
	    double temp_error;
	    temp_error= solve2(M,env,Error_Threshold,Area,S);
	    if (temp_error > convergence_error) 
	      convergence_error=temp_error;
	  }
	S = S->nexts;
	i++;
      }
    arret++;
    TRACEVARIABLE(convergence_error);
  } while((convergence_error>Error_Threshold)&&(arret<maximumIteration));  
  EXITINGFUNCTION;
}
