#include "radiosite/VecLib.h"

/* returns squared length of input vector */	
double V3SquaredLength(Vector3 *a) 
{
  return((a->x * a->x)+(a->y * a->y)+(a->z * a->z));
}

/* returns length of input vector */
double V3Length(Vector3 *a) 
{
  return(sqrt(V3SquaredLength(a)));
}

/* normalizes the input vector and returns it */
Vector3 *V3Normalize(Vector3 *v) 
{
  double len = V3Length(v);
  if (len != 0.0) { v->x /= len;  v->y /= len; v->z /= len; }
  //else error("V3Normalize : Zero Length Vector.");
  return(v);
}

/* return vector sum c = a+b */
Vector3 *V3Add(Vector3 *a, Vector3 *b, Vector3 *c)
{
  c->x = a->x+b->x;  c->y = a->y+b->y;  c->z = a->z+b->z;
  return(c);
}
	
/* return vector difference c = a-b */
Vector3 *V3Sub(Vector3 *a, Vector3 *b, Vector3 *c)
{
  c->x = a->x-b->x;  c->y = a->y-b->y;  c->z = a->z-b->z;
  return(c);
}

/* return the dot product of vectors a and b */
double V3Dot(Vector3 *a, Vector3 *b) 
{
  return((a->x*b->x)+(a->y*b->y)+(a->z*b->z));
}

/* linearly interpolate between vectors by an amount alpha */
/* and return the resulting vector. */
/* When alpha=0, result=lo.  When alpha=1, result=hi. */
Vector3 *V3Lerp(Vector3 *lo, Vector3 *hi, double alpha, Vector3 *result) 
{
  result->x = LERP(alpha, lo->x, hi->x);
  result->y = LERP(alpha, lo->y, hi->y);
  result->z = LERP(alpha, lo->z, hi->z);
  return(result);
}

/* return the cross product c = a cross b */
Vector3 *V3Cross(Vector3 *a, Vector3 *b, Vector3 *c)
{
  c->x = (a->y*b->z) - (a->z*b->y);
  c->y = (a->z*b->x) - (a->x*b->z);
  c->z = (a->x*b->y) - (a->y*b->x);
  return(c);
}

/* Returns un unoriented angle between zero and pi */
double angle(Vector3 *a,Vector3 *b)
{
  double c= V3Dot(a,b)/(V3Length(a)*V3Length(b));
  if (V3Length(a)*V3Length(b)==0.0) return(0); 
  if (c>1) return(0);
  if (c<-1) return(M_PI);
  else return(acos(c));     
}
