#include "radiosite/Visibility.h"

#include "radiosite/VecLib.h"
#include "radiosite/Ray.h"
#include "radiosite/Kernel.h"

int whichside_point_plane(Point3 *p,Patch *b)
{
  if((b->s->normal.x*p->x+b->s->normal.y*p->y+b->s->normal.z*p->z+
      b->s->plane_constant) >= 0.) return(1);
  else return(-1);
}

int whichside_polygon_plane(Patch *a,Patch *b)
{
  int count;
  count  = whichside_point_plane(&(a->P1),b);
  count += whichside_point_plane(&(a->P2),b);
  count += whichside_point_plane(&(a->P3),b);
  count += whichside_point_plane(&(a->P4),b);
  if (abs(count) != 4) return(0);
  else if (count < 0) return(-1);
  else return(1);
}

int under_shadow(Surface *slist,Patch *a,Point3 *start,Patch *b,Point3 *end)
{
  Ray ray;
  double t;
  ray.number=++ray_count;
  ray.start = *start;
  V3Sub(end,start,&(ray.direction));
  t = V3Length(&(ray.direction)); if (t==0.) return(1);
  if ((V3Dot(&(ray.direction),&(a->s->normal)) <= 0.)
      || (V3Dot(&(ray.direction),&(b->s->normal)) >= 0.))
    return(1);
  ray.direction.x /=t;ray.direction.y /=t;ray.direction.z /=t;
  return(shadow_ray_test(&ray,slist,t));
}

int under_shadow2(Surface *slist,Patch *a,Point3 *start,Patch *b,Point3 *end)
{
  Ray ray;
  double t;
  ray.number=++ray_count;
  ray.start = *start;
  V3Sub(end,start,&(ray.direction));
  t = V3Length(&(ray.direction)); if (t==0.) return(1);
  if ((V3Dot(&(ray.direction),&(b->s->normal)) <= 0.)
      || (V3Dot(&(ray.direction),&(a->s->normal)) >= 0.))
    return(1);
  ray.direction.x /=t;ray.direction.y /=t;ray.direction.z /=t;
  return(shadow_ray_test(&ray,slist,t));
}

int complex_visibility_test(Surface *slist,Patch *a,Patch *b)
{
  double seuilvis = 0.008;
  double carre_x,carre_y;
  int j,coef=1,nbj=1;
  struct {
    double min_u,min_v;
  } magic_square_a[Magic_square_size]={
    {0.5,0.25},
    {0,0},
    {0.75,0.75},
    {0.25,0.5},
    {0,0.5},
    {0.5,0.75},
    {0.25,0},
    {0.75,0.25},
    {0.25,0.75},
    {0.75,0.5},
    {0,0.25},
    {0.5,0},
    {0.75,0},
    {0.25,0.25},
    {0.5,0.5},
    {0,0.75}
  }, magic_square_b[Magic_square_size]={
    {0.25,0.5},
    {0.75,0.75},
    {0.5,0},
    {0,0.25},
    {0,0},
    {0.5,0.25},
    {0.75,0.5},
    {0.25,0.75},
    {0.75,0.25},
    {0.25,0},
    {0,0.75},
    {0.5,0.5},
    {0.5,0.75},
    {0,0.5},
    {0.25,0.25},
    {0.75,0}
  };
  int i,count=0,count2=0;
  if (a->area > seuilvis || b->area > seuilvis){
    coef=2;nbj=4;
  };
  for (i=0;i< Magic_square_size;i++){
    double drand48();
    double u_a,v_a,u_b,v_b;
    for (j=1;j<=nbj;j++){
      if (j==1) {carre_x=0;carre_y=0;};
      if (j==2) {carre_x=0.5;carre_y=0;};
      if (j==3) {carre_x=0;carre_y=0.5;};
      if (j==4) {carre_x=0.5;carre_y=0.5;};
      u_a = (magic_square_a[i].min_u + 0.125)/coef + carre_x;
      v_a = (magic_square_a[i].min_v + 0.125)/coef + carre_y;
      u_b = (magic_square_b[i].min_u + 0.125)/coef + carre_x;
      v_b = (magic_square_b[i].min_v + 0.125)/coef + carre_y;
      {
	Point3 start;
	Point3 end;
	quadrilateral_point(&(a->P1),&(a->P2),&(a->P4),&(a->P3),u_a,v_a,&(start));
	quadrilateral_point(&(b->P1),&(b->P2),&(b->P4),&(b->P3),u_b,v_b,&(end));
	if (under_shadow(slist,a,&start,b,&end)) count++;
	quadrilateral_point(&(a->P1),&(a->P2),&(a->P4),&(a->P3),u_b,v_b,&(start));
	quadrilateral_point(&(b->P1),&(b->P2),&(b->P4),&(b->P3),u_a,v_a,&(end));
	if (under_shadow2(slist,b,&start,a,&end)) count2++;
      }
    }
  }
  return((count+count2)/(2*coef*coef));
}

int complex_visibility_test2(Surface *slist,Patch *a,Patch *b)
{
  struct {
    double min_u,min_v;
  } magic_square_a[Magic_square_size]={
    {0.5,0.25},
    {0,0},
    {0.75,0.75},
    {0.25,0.5},
    {0,0.5},
    {0.5,0.75},
    {0.25,0},
    {0.75,0.25},
    {0.25,0.75},
    {0.75,0.5},
    {0,0.25},
    {0.5,0},
    {0.75,0},
    {0.25,0.25},
    {0.5,0.5},
    {0,0.75}
  }, magic_square_b[Magic_square_size]={
    {0.25,0.5},
    {0.75,0.75},
    {0.5,0},
    {0,0.25},
    {0,0},
    {0.5,0.25},
    {0.75,0.5},
    {0.25,0.75},
    {0.75,0.25},
    {0.25,0},
    {0,0.75},
    {0.5,0.5},
    {0.5,0.75},
    {0,0.5},
    {0.25,0.25},
    {0.75,0}
  };
  int i,count=0;
  for (i=0;i< Magic_square_size;i++){
    double drand48();
    double u_a,v_a,u_b,v_b;
    u_a = magic_square_a[i].min_u + 0.125;
    v_a = magic_square_a[i].min_v + 0.125;
    u_b = magic_square_b[i].min_u + 0.125;
    v_b = magic_square_b[i].min_v + 0.125;
    {
      Point3 start;
      Point3 end;
      quadrilateral_point(&(a->P1),&(a->P2),&(a->P4),&(a->P3),u_a,v_a,&(start));
      quadrilateral_point(&(b->P1),&(b->P2),&(b->P4),&(b->P3),u_b,v_b,&(end));
      if (under_shadow(slist,a,&start,b,&end)) count++;
    }
  }
  return(count);
}

int visibility(Surface *slist,Patch *a,Patch *b)
{
  int side1,side2;
  side1 = whichside_polygon_plane(b,a);
  side2 = whichside_polygon_plane(a,b);
  if (negative(side1) && negative(side2)) return(INVISIBLE);
  if ((positive(side1) && negative(side2))||
      (positive(side2)&& negative(side1))) return(INVISIBLE);
  //if (fabs(V3Dot(&(a->s->normal),&(b->s->normal))-1.) < 1.e-3) return (INVISIBLE);
  return(PARTIALLY_VISIBLE);
}
