#include "radiosite/Voxel.h"

#include "radiosite/StaticMemory.h"
#include "radiosite/Surface.h"
#include "radiosite/VecLib.h"

Volume_grid volume_grid = { { { 0 , 0 , 0 } , { 0 , 0 , 0 } } , NULL , 0 , 0 , 0 , { 0 , 0 , 0 } , { 0 , 0 , 0 } };

void initializeVolumeGrid()
{
  int firsttime = 0;
  if (volume_grid.voxels == NULL)
    {
      firsttime = 1;
      volume_grid.x_subdivision=volume_grid.y_subdivision=volume_grid.z_subdivision=2.;
      volume_grid.voxels = new Voxel[volume_grid.x_subdivision*volume_grid.y_subdivision*volume_grid.z_subdivision];
    }
  for (int i=0;i<volume_grid.x_subdivision*volume_grid.y_subdivision*volume_grid.z_subdivision;i++)
    {
      volume_grid.voxels[i].nsurfaces = 0;
      if (!firsttime) delete [] volume_grid.voxels[i].intersecting_surface_list;
      volume_grid.voxels[i].intersecting_surface_list = NULL;
    }
}

void spacially_sub_divide2(Surface *S,int nbS,int& idS)
{
  if (S->_splitting == SPLITTING_NONE) {
    int x,y,z,xlo,ylo,zlo,xhi,yhi,zhi;
    adjust_limits(*S);
    for (x = xlo; x <= xhi; x++){
      for (y = ylo; y <= yhi; y++){
	for (z = zlo; z <= zhi; z++){
	  Voxel *v=voxel_addr(volume_grid,x,y,z);
	  if (v->nsurfaces==0) v->intersecting_surface_list = new int[nbS];
	  v->intersecting_surface_list[v->nsurfaces]=idS;
	  (v->nsurfaces)++;
	}
      }
    }
    idS++;
  } else {
    for (int i=0;i<S->_splitting;i++) {
      spacially_sub_divide2(S->_splitted[i],nbS,idS);
    }
  }
}

void spacially_sub_divide(Surface *slist,int nbS)
{
  int i = 0; Surface* S = slist;
  while (S != NULL) {
    spacially_sub_divide2(S,nbS,i);
    S = S->nexts;
  }
}

void preProcess(Surface *slist,int nbS)
{
  int i,j,k;
  Point3 min0;
  Voxel *vox;
  Box3 *b = &(volume_grid.extent);
  b->min.x= b->min.y= b->min.z= largeValue;
  b->max.x= b->max.y= b->max.z= -largeValue;
  Surface* S = slist;
  while (S != NULL) {
    Box3 *bb = &(S->bbox);
    bb->max.x=MAX(MAX(S->P1.x,S->P2.x),MAX(S->P4.x,S->P3.x));
    bb->max.y=MAX(MAX(S->P1.y,S->P2.y),MAX(S->P4.y,S->P3.y));
    bb->max.z=MAX(MAX(S->P1.z,S->P2.z),MAX(S->P4.z,S->P3.z));
    bb->min.x=MIN(MIN(S->P1.x,S->P2.x),MIN(S->P4.x,S->P3.x));
    bb->min.y=MIN(MIN(S->P1.y,S->P2.y),MIN(S->P4.y,S->P3.y));
    bb->min.z=MIN(MIN(S->P1.z,S->P2.z),MIN(S->P4.z,S->P3.z));
    update_scene_bound(bb,b);
    S->mail_box.ray_num= UNDEFINED;
    S->mail_box.t =  S->mail_box.u = S->mail_box.v = -1.0;
    S = S->nexts;
  }

  b->min.x-=epsilonValue;b->max.x+=epsilonValue;
  b->min.y-=epsilonValue;b->max.y+=epsilonValue;
  b->min.z-=epsilonValue;b->max.z+=epsilonValue;
  volume_grid.voxel_size.x=(volume_grid.extent.max.x-volume_grid.extent.min.x)/volume_grid.x_subdivision;
  volume_grid.voxel_size.y=(volume_grid.extent.max.y-volume_grid.extent.min.y)/volume_grid.y_subdivision;
  volume_grid.voxel_size.z=(volume_grid.extent.max.z-volume_grid.extent.min.z)/volume_grid.z_subdivision;
  volume_grid.voxel_density.x=1.0/volume_grid.voxel_size.x;
  volume_grid.voxel_density.y=1.0/volume_grid.voxel_size.y;
  volume_grid.voxel_density.z=1.0/volume_grid.voxel_size.z;
  min0=volume_grid.extent.min;
  for(i=0,vox=volume_grid.voxels;i<volume_grid.x_subdivision;i++,min0.x+=volume_grid.voxel_size.x){
    Point3 min1;
    min1=min0;
    for(j=0;j<volume_grid.y_subdivision;j++,min1.y+=volume_grid.voxel_size.y){
      Point3 min2;min2=min1;
      for(k=0;k<volume_grid.z_subdivision;k++,vox++,min2.z+=volume_grid.voxel_size.z){
	vox->extent.min=min2;
	V3Add(&(vox->extent.min),&(volume_grid.voxel_size),&(vox->extent.max));
      }
    }
  }
  spacially_sub_divide(slist,nbS);
}

void add_to_list(Vlist **vlist,int voxnum,double t1,double t2)
{
  Vlist *newvox;
  newvox = new Vlist;
  newvox->voxnum=voxnum;
  newvox->t_near=t1;
  newvox->t_far=t2;
  newvox->next=NULL;
  if (*vlist==NULL) *vlist = StaticMemory::Voxel_addtovlist_p = newvox;
  else { StaticMemory::Voxel_addtovlist_p->next = newvox; StaticMemory::Voxel_addtovlist_p = newvox; }
}

Vlist *create_vlist(Point3 *ray_start,Vector3 *ray_direction,double tmin, double tmax)
{
  Vlist *vlist=NULL;
  int x,stepX,outX;
  double tMaxX,tDeltaX;
  int y,stepY,outY;
  double tMaxY,tDeltaY;
  int z,stepZ,outZ;
  double tMaxZ,tDeltaZ;
  double t1;
  Point3 ray_entry,ray_exit;
  point_on_line(*ray_start,*ray_direction,tmin,ray_entry);
  point_on_line(*ray_start,*ray_direction,tmax,ray_exit);
  x=x2voxel(volume_grid,ray_entry.x); if (x == volume_grid.x_subdivision) x--;
  if (ray_direction->x < 0.){
    tMaxX = tmin + (voxel2x(volume_grid,x)-ray_entry.x)/ray_direction->x;
    tDeltaX =  volume_grid.voxel_size.x/(-ray_direction->x);
    stepX = outX = -1;
  }
  else if (ray_direction->x > 0.){
    tMaxX = tmin + (voxel2x(volume_grid,x+1)-ray_entry.x)/ray_direction->x;
    tDeltaX =  volume_grid.voxel_size.x/ray_direction->x;
    stepX = 1;
    outX = volume_grid.x_subdivision;
  }
  else{
    tMaxX=largeValue;
    tDeltaX = 0.;
  }
  y=y2voxel(volume_grid,ray_entry.y); if (y == volume_grid.y_subdivision) y--;
  if (ray_direction->y < 0.){
    tMaxY = tmin + (voxel2y(volume_grid,y)-ray_entry.y)/ray_direction->y;
    tDeltaY =  volume_grid.voxel_size.y/(-ray_direction->y);
    stepY = outY = -1;
  }
  else if (ray_direction->y > 0.){
    tMaxY = tmin + (voxel2y(volume_grid,y+1)-ray_entry.y)/ray_direction->y;
    tDeltaY =  volume_grid.voxel_size.y/ray_direction->y;
    stepY = 1;
    outY = volume_grid.y_subdivision;
  }
  else{
    tMaxY=largeValue;
    tDeltaY = 0.;
  }
  z=z2voxel(volume_grid,ray_entry.z); if (z == volume_grid.z_subdivision) z--;
  if (ray_direction->z < 0.){
    tMaxZ = tmin + (voxel2z(volume_grid,z)-ray_entry.z)/ray_direction->z;
    tDeltaZ =  volume_grid.voxel_size.z/(-ray_direction->z);
    stepZ = outZ = -1;
  }
  else if (ray_direction->z > 0.){
    tMaxZ = tmin + (voxel2z(volume_grid,z+1)-ray_entry.z)/ray_direction->z;
    tDeltaZ =  volume_grid.voxel_size.z/ray_direction->z;
    stepZ = 1;
    outZ = volume_grid.z_subdivision;
  }
  else{
    tMaxZ=largeValue;
    tDeltaZ = 0.;
  }
  t1 = tmin;
  while(1){
    int index=voxel_index(volume_grid,x,y,z);
    if ((tMaxX < tMaxY) && (tMaxX < tMaxZ)) {
      add_to_list(&vlist,index,t1,tMaxX); t1 = tMaxX;
      x += stepX;
      if ((tmax < tMaxX) || (x == outX)) break;
      tMaxX += tDeltaX;
    }else if (tMaxZ < tMaxY) {
      add_to_list(&vlist,index,t1,tMaxZ); t1 = tMaxZ;
      z += stepZ;
      if ((tmax < tMaxZ) || (z == outZ))
	break;
      tMaxZ += tDeltaZ;
    } else {
      add_to_list(&vlist,index,t1,tMaxY); t1 = tMaxY;
      y += stepY;
      if ((tmax < tMaxY) || (y == outY))
	break;
      tMaxY += tDeltaY;
    }
  }
  return(vlist);
}

void purge_vlist(Vlist *vlist)
{
  while(vlist != NULL){
    Vlist *tlist=vlist;
    vlist=vlist->next;
    delete tlist;
  }
}
