#include "radiosite/Wavelet.h"

#include "radiosite/StaticMemory.h"
#include "radiosite/Lambda.h"

/*
 	Multi-Wavelet Tools
	-----
	Author: S. N. Pattanaik
	Date  : 5/7/95
*/

double A_f[4][4] ={
  {0.70710678,0,0,0},
  {-0.61237244,0.35355339,0,0},
  {0,-0.6846532,0.1767767,0},
  {0.23385359,0.40504629,-0.52291252,0.088388348}
};

double C_f[4][4] ={
  {0.70710678,0,0,0},
  {0.61237244,0.35355339,0,0},
  {0,0.6846532,0.1767767,0},
  {-0.23385359,0.40504629,0.52291252,0.088388348}
};

int get_two_power_J(int J)
{
  int two_power_J=1;
  for(;J > 0; J--) two_power_J <<= 1;
  return(two_power_J);
}

double Phi(int M,double u)
{
  double v;
  if ((u < 0.)||(u > 1.)) v=0.;
  else if ((M < 1)||(M > 4)) {}
  else switch(M)
    {
    case 1 : v = 1.; break;
    case 2 : v = SQRT3*(2.*u-1); break;
    case 3 : v = SQRT5*(6.*u*u-6.*u+1.); break;
    case 4 : v = SQRT7*(20.*u*u*u-30.*u*u+12.*u-1.); break;
    }
  return(v);
}

double MultiWavelet_basis_1v(int J, int M, int k, double u)
{
  int two_power_J=get_two_power_J(J);
  return(sqrt((double)two_power_J)*Phi(M,(two_power_J*u-k)));
}

double MultiWavelet_basis_2v(int J,int M,int k,double u,int N,int l,double v)
{
  int two_power_J=get_two_power_J(J);
  return(two_power_J*Phi(M,(two_power_J*u-k))*Phi(N,(two_power_J*v-l)));
}

double MultiWavelet_basis_4v(int J,int M,int k,double u,int N,int l,double v,
				int O, int m, double s, int P, int n, double t)
{
  int two_power_J=get_two_power_J(J);
  return(two_power_J*two_power_J*
	 Phi(M,(two_power_J*u-k))*
	 Phi(N,(two_power_J*v-l))*
	 Phi(O,(two_power_J*s-m))*
	 Phi(P,(two_power_J*t-n))
	 );
}

void push_1v(int M,double *parent,double *L,double *R)
{
  int m,n;
  for(m=0;m<M;m++)
    {
      L[m] = R[m] = 0.;
      for(n=0;n<M;n++)
	{
	  L[m] += A_f[n][m]*parent[n];
	  R[m] += C_f[n][m]*parent[n];
	}
    }
}	

void pull_1v(int M,double *parent,double *L,double *R)
{
  int m,n;
  for(m=0;m<M;m++)
    {
      parent[m] = 0.;
      for(n=0;n<M;n++)
	{
	  parent[m] += A_f[m][n]*L[n]+
	    C_f[m][n]*R[n];
	}
    }
}

void push_2v(int M,double *parent,double *SW,double *NW,
		                  double *SE,double *NE)
{
  int m1,m2,n1,n2;
  for(m1=0;m1<M;m1++)
    for(m2=0;m2<M;m2++)
      {
	int m = m1*M+m2;
	SW[m] = NW[m] = NE[m] = SE[m] = 0.;
	for(n1=0;n1<M;n1++)
	  for(n2=0;n2<M;n2++)
	    {
	      int n = n1*M+n2;
	      SW[m] += A_f[n1][m1]*A_f[n2][m2]*parent[n];
	      NW[m] += A_f[n1][m1]*C_f[n2][m2]*parent[n];
	      SE[m] += C_f[n1][m1]*A_f[n2][m2]*parent[n];
	      NE[m] += C_f[n1][m1]*C_f[n2][m2]*parent[n];
	    }
      }
}

void pull_2v(int M,double *parent,double *SW,double *NW,
                                  double *SE,double *NE)
{
  int m,n,m1,m2,n1,n2;
  for(m1=m=0;m1<M;m1++)
    for(m2=0;m2<M;m2++,m++)
      {
	parent[m] = 0.;
	for(n1=n=0;n1<M;n1++)
	  for(n2=0;n2<M;n2++,n++)
	    {
	      parent[m] += A_f[m1][n1]*(A_f[m2][n2]*SW[n]+
					C_f[m2][n2]*NW[n])+
		C_f[m1][n1]*(A_f[m2][n2]*SE[n]+
			     C_f[m2][n2]*NE[n]);
	    }
      }
}

/*******************************************************************
 * Fonction permettant de remonter l'energie d'un patch P a sa
 * racine.
 *
 * ENTREE : La racine, le numero du patch, l'energie recue.
 * SORTIE : void
 */
void pull_up_energy_to_root(Patch *R, int N, int M, double **C)
{
  int k,m,n,m1,m2,n1,n2;
  int fils;
  for(k = 0; k < sampledSpectrumSize; k ++) /* a modifier pour tenir compte de l'echantillonnage du spectre */
    for(m = 0; m < M*M; m ++) {
      StaticMemory::Wavelet_pullupenergytoroot_PULL[k][m] = C[k][m];
      StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m] = 0.0;
    }
  while(N != 1) {
    fils = N % 10;
    N = N / 10;
    /* Pour chaque longueur d'onde */
    for(k = 0; k < sampledSpectrumSize; k++) { /* a modifier pour tenir compte de l'echantillonnage du spectre */
      /* On calcule les coeffs */
      for(m1=m=0;m1<M;m1++)
	for(m2=0;m2<M;m2++,m++){
	  for(n1=n=0;n1<M;n1++)
	    for(n2=0;n2<M;n2++,n++){
	      switch(fils) {
	      case 1 : /* SW */
		StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m] += A_f[m1][n1]*(A_f[m2][n2]*StaticMemory::Wavelet_pullupenergytoroot_PULL[k][n]);
		break;
	      case 2 : /* NW */
		StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m] += A_f[m1][n1]*(C_f[m2][n2]*StaticMemory::Wavelet_pullupenergytoroot_PULL[k][n]);
		break;
	      case 3 : /* SE */
		StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m] += C_f[m1][n1]*(A_f[m2][n2]*StaticMemory::Wavelet_pullupenergytoroot_PULL[k][n]);
		break;
	      case 4 : /* NE */
		StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m] += C_f[m1][n1]*(C_f[m2][n2]*StaticMemory::Wavelet_pullupenergytoroot_PULL[k][n]);
		break;
	      default :
		fprintf(stderr, "No de fils = %d\n", fils);
	      }
	    }
	}
    }/* k */
    for(k = 0; k < sampledSpectrumSize; k ++) /* a modifier pour tenir compte de l'echantillonnage du spectre */
      for(m = 0; m < M*M; m ++) {
	StaticMemory::Wavelet_pullupenergytoroot_PULL[k][m] = StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m];
	StaticMemory::Wavelet_pullupenergytoroot_TMP_PULL[k][m] = 0.0;
      }
  }/* while */
  for(k = 0; k < sampledSpectrumSize; k ++) /* a modifier pour tenir compte de l'echantillonnage du spectre */
    for(m = 0; m < M*M; m ++)
      R->de_coeff[k][m] += StaticMemory::Wavelet_pullupenergytoroot_PULL[k][m];
}
/* fin pull_up_energy_to_root */
