#include "recherche/Maillage.h"

#include "others/Parameters.h"

#include "radiosite/Surface.h"
#include "radiosite/Patch.h"
#include "radiosite/Link.h"
#include "radiosite/OpticalProperty.h"
#include "radiosite/Luminaire.h"
#include "radiosite/Voxel.h"
#include "radiosite/Lambda.h"
#include "radiosite/Spectre.h"

#include "radiosite/StaticMemory.h"

#define SEMIPOINTMAILLAGE(a,b,p) { p[0] = (a[0]+b[0])/2.; p[1] = (a[1]+b[1])/2.; p[2] = (a[2]+b[2])/2.; }
#define COPYPOINTMAILLAGEV(a,b) { a.x = b[0]; a.y = b[1]; a.z = b[2]; }
#define COPYPOINTMAILLAGE(a,b) { a.x = b.x; a.y = b.y; a.z = b.z; }
#define VECTORMAILLAGE(a,b,p) { p[0] = ((b)[0]-(a)[0])/2.; p[1] = ((b)[1]-(a)[1])/2.; p[2] = ((b)[2]-(a)[2])/2.; }
#define CROSSPRODMAILLAGE(a,b,p) { p[0] = ((a)[1]*(b)[2])-((a)[2]*(b)[1]); p[1] = ((a)[2]*(b)[0])-((a)[0]*(b)[2]); p[2] = ((a)[0]*(b)[1])-((a)[1]*(b)[0]); }
#define SUMMMAILLAGE(a,b,p) { p[0] = (a)[0]+(b)[0]; p[1] = (a)[1]+(b)[1]; p[2] = (a)[2]+(b)[2]; }
#define PRODMAILLAGE(a,v,p) { p[0] = (a)[0]*v; p[1] = (a)[1]*v; p[2] = (a)[2]*v; }

Surface* createPolygonMaillage3d(enum eTunnelSurface type,double p0[3],double p1[3],double p2[3],double p3[3])
{
  Surface* S = new Surface(type, p0, p1, p2, p3);
  return S;
}

void setPolygonMaillage3d(Surface** rootS,enum eTunnelSurface type,double p0[3],double p1[3],double p2[3],double p3[3])
{
  Surface* S = createPolygonMaillage3d(type, p0, p1, p2, p3);
  S->nexts = (*rootS);
  (*rootS) = S;
}

int researchMode = 1;

void splittingPolygonMaillage3d(Surface* S,enum eTunnelSurface type,double p0[3],double p1[3],double p2[3],double p3[3])
{
  double d0 = DISTANCE(p0,p1);
  double d1 = DISTANCE(p1,p2);
  double p0p1[3]; SEMIPOINTMAILLAGE(p0,p1,p0p1);
  double p1p2[3]; SEMIPOINTMAILLAGE(p1,p2,p1p2);
  double p2p3[3]; SEMIPOINTMAILLAGE(p2,p3,p2p3);
  double p3p0[3]; SEMIPOINTMAILLAGE(p3,p0,p3p0);
  double p0p2[3]; SEMIPOINTMAILLAGE(p0,p2,p0p2);
  if ((d0 > (Parameters::_surfaceMaximumAspectRatio * d1)) && (!researchMode))
    {
      Surface* S0 = createPolygonMaillage3d(type,p0,p0p1,p2p3,p3); splittingPolygonMaillage3d(S0,type,p0,p0p1,p2p3,p3);
      Surface* S1 = createPolygonMaillage3d(type,p0p1,p1,p2,p2p3); splittingPolygonMaillage3d(S1,type,p0p1,p1,p2,p2p3);
      S->_splitting = SPLITTING_BIN;
      ALLOCATEARRAY(S->_splitted,Surface*,2);
      S->_splitted[0] = S0; S0->_upperLevelParent = S;
      S->_splitted[1] = S1; S1->_upperLevelParent = S;
    }
  else if ((d1 > (Parameters::_surfaceMaximumAspectRatio * d0)) && (!researchMode))
    {
      Surface* S0 = createPolygonMaillage3d(type,p0,p1,p1p2,p3p0); splittingPolygonMaillage3d(S0,type,p0,p1,p1p2,p3p0);
      Surface* S1 = createPolygonMaillage3d(type,p1p2,p2,p3,p3p0); splittingPolygonMaillage3d(S1,type,p1p2,p2,p3,p3p0);
      S->_splitting = SPLITTING_BIN;
      ALLOCATEARRAY(S->_splitted,Surface*,2);
      S->_splitted[0] = S0; S0->_upperLevelParent = S;
      S->_splitted[1] = S1; S1->_upperLevelParent = S;
    }
  else if ((d0 > Parameters::_surfaceMaximumLength) || (d1 > Parameters::_surfaceMaximumLength))
    {
      Surface* S0 = createPolygonMaillage3d(type,p0,p0p1,p0p2,p3p0); splittingPolygonMaillage3d(S0,type,p0,p0p1,p0p2,p3p0);
      Surface* S1 = createPolygonMaillage3d(type,p1,p1p2,p0p2,p0p1); splittingPolygonMaillage3d(S1,type,p1,p1p2,p0p2,p0p1);
      Surface* S2 = createPolygonMaillage3d(type,p2,p2p3,p0p2,p1p2); splittingPolygonMaillage3d(S2,type,p2,p2p3,p0p2,p1p2);
      Surface* S3 = createPolygonMaillage3d(type,p3,p3p0,p0p2,p2p3); splittingPolygonMaillage3d(S3,type,p3,p3p0,p0p2,p2p3);
      S->_splitting = SPLITTING_QUAD;
      ALLOCATEARRAY(S->_splitted,Surface*,4);
      S->_splitted[0] = S0; S0->_upperLevelParent = S;
      S->_splitted[1] = S1; S1->_upperLevelParent = S;
      S->_splitted[2] = S2; S2->_upperLevelParent = S;
      S->_splitted[3] = S3; S3->_upperLevelParent = S;
    }
}

void initHighLevelParent(Surface* Sparent,Surface* S)
{
  S->_highLevelParent = Sparent;
  if (S->_splitting) for (int i=0;i<S->_splitting;i++) initHighLevelParent(Sparent,S->_splitted[i]);
}

void addPolygonMaillage3d(Surface** rootS,int* nbS,enum eTunnelSurface type,double p0[3],double p1[3],double p2[3],double p3[3])
{
  double d0 = DISTANCE(p0,p1);
  double d1 = DISTANCE(p1,p2);
  double p0p1[3]; SEMIPOINTMAILLAGE(p0,p1,p0p1);
  double p1p2[3]; SEMIPOINTMAILLAGE(p1,p2,p1p2);
  double p2p3[3]; SEMIPOINTMAILLAGE(p2,p3,p2p3);
  double p3p0[3]; SEMIPOINTMAILLAGE(p3,p0,p3p0);
  double p0p2[3]; SEMIPOINTMAILLAGE(p0,p2,p0p2);

  setPolygonMaillage3d(rootS,type,p0,p1,p2,p3);
  Surface* refS = *rootS; splittingPolygonMaillage3d(refS,type,p0,p1,p2,p3); initHighLevelParent(refS,refS);
}

void generateRoofPolygonMaillage(Surface** rootS,int* nbS,double p0[3],double p1[3],double p2[3],double p3[3],double d,int lvl)
{
  if (lvl == 0) addPolygonMaillage3d(rootS,nbS,TUNNELSURFACE_ROOF,p0,p1,p2,p3);
  else
    {
      double p0p1[3]; SEMIPOINTMAILLAGE(p0,p1,p0p1);
      double p2p3[3]; SEMIPOINTMAILLAGE(p2,p3,p2p3);
      double v0v1[3]; VECTORMAILLAGE(p0,p1,v0v1);
      double v0v3[3]; VECTORMAILLAGE(p0,p3,v0v3);
      double vn[3]; CROSSPRODMAILLAGE(v0v3,v0v1,vn);
      double tmp = sqrt(vn[0]*vn[0]+vn[1]*vn[1]+vn[2]*vn[2]);
      double nt[3]; PRODMAILLAGE(vn,d/tmp,nt);
      double p01[3]; SUMMMAILLAGE(p0p1,nt,p01);
      double p32[3]; SUMMMAILLAGE(p2p3,nt,p32);
      generateRoofPolygonMaillage(rootS,nbS,p0,p01,p32,p3,d/4.,lvl-1);
      generateRoofPolygonMaillage(rootS,nbS,p2,p32,p01,p1,d/4.,lvl-1);
    }
}

void initializeSurfaceIndexes(Surface* S,int& index)
{
  S->p.s = S;
  S->no_surface = index;
  S->p.no_surface = index;
  S->p.no_patch = 1;
  S->already_done = 0;
  index++;
  if (S->_splitting) for (int i=0;i<S->_splitting;i++) initializeSurfaceIndexes(S->_splitted[i],index);
}

void addShapeMaillage( sTunnelParameters& tunnelParameters,
		      Env* env)
{
  cout << "addShapeMaillage(" << tunnelParameters.shape._length << ")" << endl;
  int nbS = 0;
  Surface* rootS = NULL;
  {
    /* pour le sol */
    double p0[] = { tunnelParameters.shape._width/2. , 0. , 0. };
    double p1[] = { -tunnelParameters.shape._width/2. , 0. , 0. };
    double p2[] = { -tunnelParameters.shape._width/2. , tunnelParameters.shape._length , 0. };
    double p3[] = { tunnelParameters.shape._width/2. , tunnelParameters.shape._length , 0. };
    addPolygonMaillage3d(&rootS,&nbS,TUNNELSURFACE_FLOOR,p3,p2,p1,p0);
  }
  /* murs */
  double hmur = (tunnelParameters.shape._roofShape == TUNNELSHAPE_PLANAR) ? tunnelParameters.shape._maxHeight : tunnelParameters.shape._minHeight;
  {
    /* mur gauche */
    double p0[] = { tunnelParameters.shape._width/2.,0.,0.};
    double p1[] = { tunnelParameters.shape._width/2.,tunnelParameters.shape._length,0.};
    double p2[] = { tunnelParameters.shape._width/2.,tunnelParameters.shape._length,hmur};
    double p3[] = { tunnelParameters.shape._width/2.,0.,hmur};
    addPolygonMaillage3d(&rootS,&nbS,TUNNELSURFACE_WALL,p3,p2,p1,p0);
  }
  {
    /* mur droit */
    double p0[] = { -tunnelParameters.shape._width/2.,0.,0. };
    double p1[] = { -tunnelParameters.shape._width/2.,0.,hmur };
    double p2[] = { -tunnelParameters.shape._width/2.,tunnelParameters.shape._length,hmur };
    double p3[] = { -tunnelParameters.shape._width/2.,tunnelParameters.shape._length,0. };
    addPolygonMaillage3d(&rootS,&nbS,TUNNELSURFACE_WALL,p3,p2,p1,p0);
  }
  {
    /* plafond */
    double p0[] = { -tunnelParameters.shape._width/2.,0.,hmur };
    double p1[] = { tunnelParameters.shape._width/2.,0.,hmur };
    double p2[] = { tunnelParameters.shape._width/2.,tunnelParameters.shape._length,hmur };
    double p3[] = { -tunnelParameters.shape._width/2.,tunnelParameters.shape._length,hmur };
  switch (tunnelParameters.shape._roofShape)
    {
    case TUNNELSHAPE_PLANAR:
      generateRoofPolygonMaillage(&rootS,&nbS,p3,p2,p1,p0,0.,0);
      break;
    case TUNNELSHAPE_TRIANGULAR:
      generateRoofPolygonMaillage(&rootS,&nbS,p3,p2,p1,p0,tunnelParameters.shape._maxHeight - tunnelParameters.shape._minHeight,1);
      break;
   case TUNNELSHAPE_SPHERICAL:
      generateRoofPolygonMaillage(&rootS,&nbS,p3,p2,p1,p0,tunnelParameters.shape._maxHeight - tunnelParameters.shape._minHeight,4);
      break;
    }
  }

  /* on recopie dans la structure de type Env */
  env->slist = rootS;
  int id = 0;
  Surface* S = rootS;
  while (S != NULL)
    {
      initializeSurfaceIndexes(S,id);
      S = S->nexts;
    }
  env->N_surfaces=id;
}

void addlight(double x,double y,double dx,double alpha,
	      sLightSource* light, sTunnelParameters& tunnelParameters,Env* env)
{
  double longueur,largeur,hauteur;
  double hauteur_tun, largeur_tun, hauteurl, espacement, inclinaison;
  double emplacement, nbvoies, forme, symetrie;
  longueur = light->L;
  largeur = light->l;
  hauteur = tunnelParameters.shape._maxHeight - 0.01;
  hauteur_tun = tunnelParameters.shape._maxHeight;
  hauteurl = tunnelParameters.shape._minHeight;
  if (hauteurl == 0.0) hauteurl = hauteur_tun;
  largeur_tun = tunnelParameters.shape._width;

  espacement = tunnelParameters.light._lightSpacing;
  emplacement = tunnelParameters.light._lightPosition;
  inclinaison = tunnelParameters.light._lightSlope;
  nbvoies = tunnelParameters.shape._floorLanes;
  forme = tunnelParameters.shape._roofShape;
  symetrie = tunnelParameters.light._lightSymmetrical;

  double lon=longueur/2;
  double larg=largeur/2;
  double p0[] = {x+((largeur_tun/2)*(1-(1/nbvoies))) + dx + larg * cos(M_PI * (alpha/180)),y+lon,(hauteur_tun + hauteurl)/2 - 0.2 + larg * sin(M_PI * (alpha/180))};
  double p1[] = {x+((largeur_tun/2)*(1-(1/nbvoies))) + dx + larg * cos(M_PI * (alpha/180)),y-lon,(hauteur_tun + hauteurl)/2 - 0.2 + larg * sin(M_PI * (alpha/180))};
  double p2[] = {x+((largeur_tun/2)*(1-(1/nbvoies))) + dx - larg * cos(M_PI * (alpha/180)),y-lon,(hauteur_tun + hauteurl)/2 - 0.2 - larg * sin(M_PI * (alpha/180))};
  double p3[] = {x+((largeur_tun/2)*(1-(1/nbvoies))) + dx - larg * cos(M_PI * (alpha/180)),y+lon,(hauteur_tun + hauteurl)/2 - 0.2 - larg * sin(M_PI * (alpha/180))};
  Surface* S = new Surface(TUNNELSURFACE_LIGHT,p0,p1,p2,p3);

  /* ajout de la surface dans le maillage */
  int nS=env->N_surfaces;
  env->N_surfaces++;  /* +1 pour la source rajoutee */
  Surface* Sp; for (Sp=env->slist;Sp->nexts != NULL;Sp = Sp->nexts) {}
  Sp->nexts = S;
  S->p.s = S;
  S->p.no_surface = nS;
  S->p.no_patch = 1;
  S->already_done = 0;
}

void addLuminaireMaillage( sRepartition& solution,
			   sTunnelParameters& tunnelParameters,
			  sLightSource* light,
			  Env* env)
{
  cout << "ajout des sources au maillage " << light->L << "," << light->l << "," << light->_name << endl;
  double espacement = tunnelParameters.light._lightSpacing;
  double inclinaison = tunnelParameters.light._lightSlope;
  double zz = tunnelParameters.shape._maxHeight - 0.01;
  double maxy = tunnelParameters.shape._length;
  double y0 = solution.nsou[0];
  for (double yy = y0;yy < maxy;yy+=espacement) {
    int i=2;
    if ((yy < solution.nsou[2]) || (solution.ramp[2] == 0)) i = 1;
    if ((yy < solution.nsou[1]) || (solution.ramp[1] == 0)) i = 0;
    switch (solution.ramp[i]) {
    case 0:
      break;
    case 1:
      addlight(0.,yy,0.,0.,light,tunnelParameters,env);
      break;
    case 2:
      addlight(0.,yy,(.5*light->l),inclinaison,light,tunnelParameters,env);
      addlight(0.,yy,-(.5*light->l),-inclinaison,light,tunnelParameters,env);
      break;
    case 3:
      addlight(0.,yy,0.,0.,light,tunnelParameters,env);
      addlight(0.,yy,(1.5*light->l),inclinaison,light,tunnelParameters,env);
      addlight(0.,yy,-(1.5*light->l),-inclinaison,light,tunnelParameters,env);
      break;
    }
  }
  cout << "Apres creation de la source" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();
}

void addLuminaireMaillage( sTunnelParameters& tunnelParameters,
			  sLightSource* light,
			  Env* env)
{
  cout << "ajout de la source au maillage " << light->L << "," << light->l << "," << light->_name << endl;
  addlight(0.,(tunnelParameters.shape._length)/2,0.,0.,light,tunnelParameters,env);

  cout << "Apres creation de la source" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();
}

Env* getMaillageNoLink( sTunnelParameters& tunnelParameters)
{
  Env* res = new Env;
  res ->N_surfaces = 0;
  res->slist = NULL;

  initializeVolumeGrid();

  cout << "Avant tout" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();

  addShapeMaillage(tunnelParameters,res);

  cout << "Apres creation des surfaces initiales" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();

  return res;
}

Env* getMaillage( sTunnelParameters& tunnelParameters, sIlluminationParameters& simulationParameters)
{
  int M = (int)(simulationParameters._waveletBasis);
  double Area_Threshold = simulationParameters._patchAreaThreshold;
  double Threshold = simulationParameters._globalThreshold;
  __simulationParameters = simulationParameters;
  Env* res = getMaillageNoLink(tunnelParameters);

  link_surfaces(M,res->slist,Area_Threshold,Threshold);

  cout << "Apres lien sur les surfaces initiales" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();

  return res;
}

void addLights( sRepartition& solution,
	        sTunnelParameters& tunnelParameters, sIlluminationParameters& simulationParameters,
	       sLightSource* light,
	       Env* env)
{
  int M = (int)(simulationParameters._waveletBasis);
  double Area_Threshold = simulationParameters._patchAreaThreshold;
  double Area_Threshold_sources = simulationParameters._lightAreaThreshold;
  double Threshold = simulationParameters._globalThreshold;
  __simulationParameters = simulationParameters;

  addLuminaireMaillage(solution,tunnelParameters,light,env);

  link_lum(1,M,env->slist,Area_Threshold,Area_Threshold_sources,Threshold);

  cout << "Apres lien sur les sources" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();

  preProcess(env->slist,env->N_surfaces);
}

void addLight( sTunnelParameters& tunnelParameters, sIlluminationParameters& simulationParameters,
	      sLightSource* light,
	      Env* env)
{
  int M = (int)(simulationParameters._waveletBasis);
  double Area_Threshold = simulationParameters._patchAreaThreshold;
  double Area_Threshold_sources = simulationParameters._lightAreaThreshold;
  double Threshold = simulationParameters._globalThreshold;
  __simulationParameters = simulationParameters;

  addLuminaireMaillage(tunnelParameters,light,env);

  link_lum(1,M,env->slist,Area_Threshold,Area_Threshold_sources,Threshold);

  cout << "Apres lien sur la source" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();

  preProcess(env->slist,env->N_surfaces);
}

void initializeSpectralProperties( sIlluminationParameters& simulationParameters,sLightSource* light)
{
  __simulationParameters = simulationParameters;
  calculSpectreReduit2(light);
}

void initializePatch(Patch* p)
{
  for (int i=0;i<sampledSpectrumSize;i++)
    for (int j=0;j<__simulationParameters._waveletBasis*__simulationParameters._waveletBasis;j++)
      {
	p->coeff[i][j] = p->d_coeff[i][j] = p->de_coeff[i][j] = 0.;
      }
  if (p->nextp != NULL)
    for (int i=0;i<4;i++) initializePatch(&(p->nextp[i]));
}

void initializeMaterial(Surface* S,sMaterialReflectanceSpectrum* mat)
{
  strcpy(S->o->material,mat->_spectrumName);
  reduireSpectre(S->rho,mat->_spectrumWavelength,mat->_spectrumValue);
  for (int i=0;i<sampledSpectrumSize;i++) S->E[i] = 0.;
  initializePatch(&(S->p));
  if (S->_splitting)
    for (int i=0;i<S->_splitting;i++)
      initializeMaterial(S->_splitted[i],mat);
}

void initializeLight(Surface* S,sLightSource* light)
{
  int i,j,k,nS;
  char name[512],s[80];
  double m;
  char nomspectre[80];
  Optical_prop *o;
  o = S->o;
  sscanf(&(light->_data[1]),"%s%lf%lf%lf", nomspectre, &(o->refl), &m, &(o->emmit));
  { // lecture du spectre reduit
    int i=0;
    double absc,val;
    char name[512];
    sprintf(name,"%s.res",light->_filename);
    FILE* f = fopen(name,"r");
    while (!feof(f))
      {
	fscanf(f, "%lf %lf", &absc, &val);
	if (!feof(f)){
	  o->spectrum[i]=val;
	  i++;
	}
      }
    fclose(f);
  }
  Luminaire* luminaire = S->luminaire = new Luminaire;
  luminaire->step_dir = light->_stepDir;
  luminaire->Nb_dir = light->_nbDir;
  luminaire->step_plans = light->_stepPlans;
  luminaire->Nb_Plans = light->_nbPlans;
  luminaire->first_plan = light->_firstPlan;  
  for (i=0;i<luminaire->Nb_Plans;i++)
    {
      for (j=0;j<luminaire->Nb_dir;j++) luminaire->val[i][j] = light->_values[i][j];
      for (j=luminaire->Nb_dir; j<maxNumberOfLightSubdivisionDirection;j++) luminaire->val[i][j]=0;
    }
  for (i=luminaire->Nb_Plans;i<maxNumberOfLightSubdivisionPlane;i++)
    for (j=0;j<maxNumberOfLightSubdivisionDirection;j++) luminaire->val[i][j]=0;
  if (o->emmit > 0.)
    for(k=0;k<sampledSpectrumSize;k++) { S->E[k]=o->emmit*o->spectrum[k]; }
  if (S->_splitting)
    for (int i=0;i<S->_splitting;i++)
      initializeLight(S->_splitted[i],light);
}

void setPhotometricProperties(Env* env,sLightSource* light)
{
  Surface* S = env->slist;
  while (S != NULL)
    {
      if (S->type_surface == TUNNELSURFACE_LIGHT)
	initializeLight(S,light);
      S = S->nexts;
    }
}

void setPhotometricProperties(Env* env,sMaterialReflectanceSpectrum* floor,
			      sMaterialReflectanceSpectrum* wall,
			      sMaterialReflectanceSpectrum* roof)
{
  ENTERINGFUNCTION; TRACEVARIABLE(env); TRACEVARIABLE(floor); TRACEVARIABLE(wall); TRACEVARIABLE(roof);
  Surface* S = env->slist;
  while (S != NULL)
    {
      switch (S->type_surface)
	{
	  case TUNNELSURFACE_FLOOR:
	    initializeMaterial(S,floor);
	    break;
	  case TUNNELSURFACE_WALL:
	    initializeMaterial(S,wall);
	    break;
	  case TUNNELSURFACE_ROOF:
	    initializeMaterial(S,roof);
	    break;
	  case TUNNELSURFACE_LIGHT:
	    break;
	}
      S = S->nexts;
    }
  EXITINGFUNCTION;
}

void resetLightLinks(Patch* p)
{
  Link* llp = NULL;
  Link* lp = NULL;
  Link* l = p->link;
  while (l != NULL)
    {
      if (l->source->s->type_surface == TUNNELSURFACE_LIGHT)
	{
	  lp->next_link = l->next_link;
	  delete l;
	  l = lp->next_link;
	}
      else
	{
	  llp = lp;
	  lp = l;
	  l = l->next_link;
	}
    }
}

void resetLightLinks(Env* env)
{
  Surface* S = env->slist;
  while (S != NULL)
    {
      if (S->type_surface != TUNNELSURFACE_LIGHT)
	{
	  resetLightLinks(&(S->p));
	}
      S = S->nexts;
    }
}

void removeAllLinks(Patch* p)
{
  Link* l = p->link;
  while (l != NULL)
    {
      Link* lt = l->next_link;
      delete l;
      l = lt;
    }
  p->link = NULL;
  if (p->nextp != NULL)
    for (int i=0;i<4;i++) removeAllLinks(&(p->nextp[i]));
}

void removeAllLinks(Surface* s)
{
  Surface* S = s;
  while (S != NULL)
    {
      removeAllLinks(&(S->p));
      S = S->nexts;
    }
}

void removeAllLinks(Env* e)
{
  removeAllLinks(e->slist);
  cout << "Apres suppression des liens" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat(); Surface::stat();
}

void saveNFF(ostream& outputfile,int M,int level,Patch* p,enum OnOff mat,enum OnOff lsom)
{
  if (p->nextp != NULL)
    {
      for (int i=0;i<4;i++)
	saveNFF(outputfile,M,2*level,&(p->nextp[i]),mat,lsom);
    }
  else
    {
      double c[4][3];
      if (p->s->type_surface != TUNNELSURFACE_LIGHT)
	{
	  double S1[maxNumberOfPointsInSpectrum];
	  double S2[maxNumberOfPointsInSpectrum];
	  double S3[maxNumberOfPointsInSpectrum];
	  double S4[maxNumberOfPointsInSpectrum];
	  computeUVspectrum(M,p,level,0,0,S1,p->area);  /* Pour le point P1 */
	  computeUVspectrum(M,p,level,1,0,S2,p->area);  /* Pour le point P2 */
	  computeUVspectrum(M,p,level,1,1,S3,p->area);  /* Pour le point P3 */
	  computeUVspectrum(M,p,level,0,1,S4,p->area);  /* Pour le point P4 */
	  SPECTRUMtoCorrectedRGB(S1,&(c[0][0]),&(c[0][1]),&(c[0][2]));
	  SPECTRUMtoCorrectedRGB(S2,&(c[1][0]),&(c[1][1]),&(c[1][2]));
	  SPECTRUMtoCorrectedRGB(S3,&(c[2][0]),&(c[2][1]),&(c[2][2]));
	  SPECTRUMtoCorrectedRGB(S4,&(c[3][0]),&(c[3][1]),&(c[3][2]));
	}
      else
	{
	  for (int i=0;i<4;i++) for (int j=0;j<3;j++) c[i][j] = 255.;
	}
// 	  double r,g,b;
// 	  p->getColor(M,level,(p->min_u+p->max_u)*0.5,(p->min_v+p->max_v)*0.5,&r,&g,&b);
// 	  outputfile << "f " << r/255. << " " << g/255. << " " << b/255. << endl;
      outputfile << "p 4" << endl;
      outputfile << p->P1.x << " " << p->P1.y << " " << p->P1.z << " " << c[0][0]/255. << " " << c[0][1]/255. << " " << c[0][2]/255. << endl;
      outputfile << p->P2.x << " " << p->P2.y << " " << p->P2.z << " " << c[1][0]/255. << " " << c[1][1]/255. << " " << c[1][2]/255. << endl;
      outputfile << p->P3.x << " " << p->P3.y << " " << p->P3.z << " " << c[2][0]/255. << " " << c[2][1]/255. << " " << c[2][2]/255. << endl;
      outputfile << p->P4.x << " " << p->P4.y << " " << p->P4.z << " " << c[3][0]/255. << " " << c[3][1]/255. << " " << c[3][2]/255. << endl;
  /*
    outputfile << "f 1. 1. 1." << endl;
    outputfile << "p 4" << endl;
    outputfile << p->P1.x << " " << p->P1.y << " " << p->P1.z << endl;
    outputfile << p->P2.x << " " << p->P2.y << " " << p->P2.z << endl;
    outputfile << p->P3.x << " " << p->P3.y << " " << p->P3.z << endl;
    outputfile << p->P4.x << " " << p->P4.y << " " << p->P4.z << endl;
  */
    }
}

int surfacemaxlevel = 500;

void saveNFF(ostream& outputfile,int M,Surface* S,int lvl,enum OnOff mat,enum OnOff lsom)
{
  TRACEFUNCTION; TRACEVARIABLE(lvl); TRACEVARIABLE(S);
  if ((S->_splitting) && (lvl <= surfacemaxlevel)) {
    for (int i=0;i<S->_splitting;i++) saveNFF(outputfile,M,S->_splitted[i],lvl+1,mat,lsom);
  } else {
    if (S->type_surface != TUNNELSURFACE_LIGHT) outputfile << "# suface materiau : " << S->o->material << endl;
    else outputfile << "# surface d'une source lumineuse" << endl;
    saveNFF(outputfile,M,1,&(S->p),mat,lsom);
  }
}

void saveNFF(ostream& outputfile,int M,Env* env,enum OnOff mat,enum OnOff lsom)
{
  TRACEFUNCTION;
  Surface* S = env->slist;
  while (S != NULL)
    {
      saveNFF(outputfile,M,S,0,mat,lsom);
      S = S->nexts;
    }
}

void saveNFF(char* fname,int M,Env* env,enum OnOff mat,enum OnOff lsom)
{
  ofstream outputfile(fname,ios::out);
  saveNFF(outputfile,M,env,mat,lsom);
  outputfile.close();
}
