#include "recherche/Motif.h"

#include "radiosite/Surface.h"
#include "radiosite/Patch.h"
#include "radiosite/Spectre.h"
#include "radiosite/Lambda.h"
#include "radiosite/Wavelet.h"

#define maxSurfacesInScene 100

/*
  calcul du motif de base obtenu sur le sol apres simulation d'eclairage
  Dans le code d'eric cela correspond a l'appel (en partie) :
      extract(2,env,view,m1, Area_Threshold);
*/
int my_factor;
int my_puis;
int my_init_global_puis;
int my_global_puis;
int my_prof;
int my_nbp;
int my_id;
double min_distancePoint3 = 1.e-3;

double min2(double a,double b)
{
  return (a < b ? a : b);
}

double max2(double a,double b)
{
  return (a > b ? a : b);
}

double min4(double a, double b, double c,double d)
{
  return min2(min2(a,b),min2(c,d));
}

double max4(double a, double b, double c,double d)
{
  return max2(max2(a,b),max2(c,d));
}

void f_nbpoint(Surface& S)
{
  Patch *p,*q;
  my_factor = 1; my_puis = 1;
  p = &(S.p);
  q = &(p->nextp[0]);
  while (&(q->nextp[0]) != NULL)
    {
      q = &(q->nextp[0]);
      my_factor++;
      my_puis *= 2;
    }
  if (my_init_global_puis) my_global_puis += my_puis;
  my_prof = my_factor - 1;
  my_nbp += ((my_puis * 2) - 1);
}

void f_absrad(Surface& S)
{
  int k;
  my_factor = 1; my_puis = 1; my_nbp = 1; f_nbpoint(S);
  double maxy = max4(S.P1.y,S.P2.y,S.P3.y,S.P4.y);
  double miny = min4(S.P1.y,S.P2.y,S.P3.y,S.P4.y);
  double incr = (double)(maxy - miny) / my_nbp;
  for (k = 1;k <= my_nbp;k ++)
    {
      _mbase.absrad[my_id + k] = _mbase.absrad[my_id + k - 1] + incr;
    }
  my_id += my_nbp;
}

void f_radio(Surface& S)
{
  typedef int Tabl[6];
  Tabl tabl;
  double R,u1,v1;
  int l,m,n,u,t,index;
  int i,j,k;
  double s[maxNumberOfPointsInSpectrum];
  Patch *q, *p;
  my_factor = 1; my_puis = 1; my_nbp = 1; f_nbpoint(S);
  p=&(S.p);
  for (i=0;i<my_prof;i++) tabl[i]=2;
  /* remplissage de la premiere moitie du tableau radio */
  for (k=0;k<my_puis;k++) {
    q=&(p->nextp[0]);
    for (m=0; m<my_prof; m++) {
      l=tabl[m];
      q=&(q->nextp[l]);
    }
    u1=(q->min_u+q->max_u)*0.5;
    v1=(q->min_v+q->max_v)*0.5;
    for(n=0;n<sampledSpectrumSize;n++) {
      s[n] = 0;
      for(i=index=0;i<MM;i++) {
	for(j=0;j<MM;j++,index++) {
	  s[n] +=my_puis*2*MultiWavelet_basis_2v(0,i+1,0,u1,j+1,0,v1)*q->coeff[n][index];
	}
      }
    }
    SPECTRUMtoRADIANCE(s,&R);
    _mbase.radio[my_id+k]=(double)(R*AArea_Threshold);
    /* nouveau chemin de descente dans l'arbre de subdivision */
    if (l==2) tabl[m-1]=3;
    else {
      u=my_prof-2;
      while (tabl[u]==3 && u>0) u--; 
      tabl[u]=3;  
      for(t=u+1;t<my_prof;t++) tabl[t]=2;
    }
  }
  for (i=0;i<my_prof;i++) tabl[i]=2;
  /* remplissage de la deuxieme moitie du tableau radio */
  for (k=my_puis;k<(my_puis*2);k++) {
    q=&(p->nextp[1]);
    for (m=0; m<my_prof; m++) {
      l=tabl[m];
      q=&(q->nextp[l]);
    }
    u1=(q->min_u+q->max_u)*0.5;
    v1=(q->min_v+q->max_v)*0.5;
    for(n=0;n<sampledSpectrumSize;n++) {
      s[n] = 0;
      for(i=index=0;i<MM;i++) {
	for(j=0;j<MM;j++,index++) {
	  s[n] +=my_puis*2*MultiWavelet_basis_2v(0,i+1,0,u1,j+1,0,v1)*q->coeff[n][index];
	}
      }
    }
    SPECTRUMtoRADIANCE(s,&R);
    _mbase.radio[my_id+k]=(double)(R*AArea_Threshold);
    /* nouveau chemin de descente dans l'arbre de subdivision */
    if (l==2)
      tabl[m-1]=3;
    else {
      u=my_prof-2;
      while (tabl[u]==3 && u>0) u--; 
      tabl[u]=3;  
      for(t=u+1;t<my_prof;t++) tabl[t]=2;
    }
  }
  my_id += my_puis*2;
}

double distancePoint3(Point3& p1,Point3& p2)
{
  double dx,dy,dz;
  dx = p2.x - p1.x;
  dy = p2.y - p1.y;
  dz = p2.z - p1.z;
  return sqrt(dx * dx + dy * dy + dz * dz);
}

int equalPoint3(Point3& p1,Point3& p2)
{
  return (distancePoint3(p1,p2) < min_distancePoint3);
}

Surface* findNearestSurface(Surface** S,int nbS,Point3& Pref)
{
  Surface* Sres;
  double dist;
  int i;
  Sres = S[0];
  dist = distancePoint3(Sres->P2,Pref);
  for (i = 1;i < nbS;i ++)
    {
      double dtemp;
      if ((dtemp = distancePoint3(S[i]->P2,Pref)) < dist)
	{
	  dist = dtemp;
	  Sres = S[i];
	}
    }
  return Sres;
}

int traiter_ligne(Surface** S,int nbS,Point3& Pinit,Point3& Pfin,f_func f)
{
  // cout << "traiter_ligne(" << nbS << ")" << endl;
  int nbSiter;
  Point3 P0;
  Surface *S0;
  nbSiter = 0;
  P0.x = Pinit.x; P0.y = Pinit.y; P0.z = Pinit.z;
  S0 = findNearestSurface(S,nbS,P0);
  while ((S0 != NULL) && (!equalPoint3(P0,Pfin)))
    {
      f(*S0);
      nbSiter++;
      P0.x = Pinit.x; P0.y = max4(S0->P1.y,S0->P2.y,S0->P3.y,S0->P4.y); P0.z = 0.;
      S0 = findNearestSurface(S,nbS,P0);
    }
  return nbSiter;
}

void findFloorSurfaces(Surface** Sfloor,int* nbS)
{
  (*nbS) = 0;
  Surface* S = _maillage->slist;
  while (S != NULL)
    {
      if (S->type_surface == TUNNELSURFACE_FLOOR)
	{
	  Sfloor[(*nbS)] = S;
	  (*nbS)++;
	}
      S = S->nexts;
    }
  //return Sfloor;
}

void calculerMotifDeBase()
{
  // cout << "calculerMotifDeBase()" << endl;
  Surface* S = _maillage->slist; while (S->type_surface != TUNNELSURFACE_LIGHT) S = S->nexts;
  _mbase.ysource = (S->P1.y + S->P3.y) / 2.;
  Surface* Sfloor[maxSurfacesInScene];
  int nbSfloor,i,nbiter;
  Point3 Pinit,Pfin;
  double xmin = 1.e5,ymin = 1.e5,xmax = -1.e5,ymax = -1.e5;
  findFloorSurfaces(Sfloor,&nbSfloor);
  for (i = 0;i < nbSfloor;i ++)
    {
      double xmint,ymint,xmaxt,ymaxt;
      Surface *S = Sfloor[i];
      xmint = min4(S->P1.x,S->P2.x,S->P3.x,S->P4.x); xmin = min2(xmin,xmint);
      ymint = min4(S->P1.y,S->P2.y,S->P3.y,S->P4.y); ymin = min2(ymin,ymint);
      xmaxt = max4(S->P1.x,S->P2.x,S->P3.x,S->P4.x); xmax = max2(xmax,xmaxt);
      ymaxt = max4(S->P1.y,S->P2.y,S->P3.y,S->P4.y); ymax = max2(ymax,ymaxt);
    }
  Pinit.x = (xmin + xmax) / 2.; Pinit.y = ymin; Pinit.z = 0.;
  Pfin.x = (xmin + xmax) / 2.; Pfin.y = ymax; Pfin.z = 0.;
  my_factor = 1; my_puis = 1; my_nbp = 0; my_init_global_puis = 1; my_global_puis = 0;
  nbiter = traiter_ligne(Sfloor,nbSfloor,Pinit,Pfin,((f_func)f_nbpoint)); my_init_global_puis = 0;
  my_nbp += nbiter;
  _mbase.npoint = my_nbp;
  _mbase.absrad[0] = ymin; my_id = 0;
  traiter_ligne(Sfloor,nbSfloor,Pinit,Pfin,((f_func)f_absrad));
  my_id = 0;
  traiter_ligne(Sfloor,nbSfloor,Pinit,Pfin,((f_func)f_radio));
}

/*
  procedure rendant le motif de base connaissant l'espacement entre
  les sources et le motif initial du a une source
*/
void calculerMotifDeRecouvrement()
{
  // cout << "calculerMotifDeRecouvrement()" << endl;
  double d,xj,xs;
  int i,j,k,l,n,nontrouve;
  n=_mbase.npoint;
  d=_mbase.absrad[n]-_mbase.absrad[0]; /*support du motif*/
  k=floor(d/_tp.light._lightSpacing);	/* nbr de sources participant 2k+1 */
  xs=_mbase.ysource;
  /*initialisation du motif de base */
  for (j=0;j<=n;j++){
    _mrecouv.absrad[j]=_mbase.absrad[j];
    _mrecouv.radio[j]=_mbase.radio[j];   /*_mbase.radio[n] a une valeur sans 
				      signification*/
  };
  _mrecouv.ysource=xs;
  _mrecouv.npoint=n;
  /*Recherche et recouvrement*/
  for(i=1;i<=k;i++){
    for(j=0;j<n;j++){
      xj=_mbase.absrad[j]+i*_tp.light._lightSpacing;
      l=0;
      nontrouve=1;
      while (nontrouve && xj<_mbase.absrad[n])
	{if (xj>=_mbase.absrad[l] && xj<_mbase.absrad[l+1])
	  nontrouve=0;
	else l++;
	};
      if (xj<_mbase.absrad[n]) _mrecouv.radio[l]+=_mbase.radio[j];
    }
  }
  for(i=1;i<=k;i++){
    for(j=n-1;j>=0;j--){
      xj=_mbase.absrad[j]-i*_tp.light._lightSpacing;
      l=0;
      nontrouve=1;
      while (nontrouve && xj>=_mbase.absrad[0])
	{if (xj>=_mbase.absrad[l] && xj<_mbase.absrad[l+1])
	  nontrouve=0;
	else l++;
	};
      if (xj>=_mbase.absrad[0]) _mrecouv.radio[l]+=_mbase.radio[j];
    }
  }
}
