#include "recherche/Nff.h"

#include "others/Parameters.h"

void addPolygonNFF(SceneNFF* scn,enum eTunnelSurface type,sLightSource* light,sMaterialReflectanceSpectrum* mat,PointNFF& p0,PointNFF& p1,PointNFF& p2,PointNFF& p3)
{
  double d0 = DISTANCE(p0,p1);
  double d1= DISTANCE(p1,p2);
  if (d0 > (Parameters::_surfaceMaximumAspectRatio * d1))
    {
      PointNFF tmp0 = SEMIPOINTNFF(p0,p1);
      PointNFF tmp1 = SEMIPOINTNFF(p2,p3);
      addPolygonNFF(scn,type,light,mat,p0,tmp0,tmp1,p3);
      addPolygonNFF(scn,type,light,mat,tmp0,p1,p2,tmp1);
    }
  else if (d1 > (Parameters::_surfaceMaximumAspectRatio * d0))
    {
      PointNFF tmp0 = SEMIPOINTNFF(p1,p2);
      PointNFF tmp1 = SEMIPOINTNFF(p3,p0);
      addPolygonNFF(scn,type,light,mat,p0,p1,tmp0,tmp1);
      addPolygonNFF(scn,type,light,mat,tmp0,p2,p3,tmp1);
    }
  else if ((d0 > Parameters::_surfaceMaximumLength) || (d1 > Parameters::_surfaceMaximumLength))
    {
      PointNFF tmp0 = SEMIPOINTNFF(p0,p1);
      PointNFF tmp1 = SEMIPOINTNFF(p2,p3);
      PointNFF tmp2 = SEMIPOINTNFF(p1,p2);
      PointNFF tmp3 = SEMIPOINTNFF(p3,p0);
      PointNFF tmp4 = SEMIPOINTNFF(p0,p2);
      addPolygonNFF(scn,type,light,mat,p0,tmp0,tmp4,tmp3);
      addPolygonNFF(scn,type,light,mat,p1,tmp2,tmp4,tmp0);
      addPolygonNFF(scn,type,light,mat,p2,tmp1,tmp4,tmp2);
      addPolygonNFF(scn,type,light,mat,p3,tmp3,tmp4,tmp1);
    }
  else
    {
      PolygonNFF* poly = new PolygonNFF();
      poly->add(p0);
      poly->add(p1);
      poly->add(p2);
      poly->add(p3);
      poly->set(type);
      poly->set(light);
      poly->set(mat);
      scn->add(*poly);
      delete poly;
    }
}

void generateRoofPolygonNFF(SceneNFF* scn,sLightSource* light,sMaterialReflectanceSpectrum* mat,PointNFF& p0,PointNFF& p1,PointNFF& p2,PointNFF& p3,double d,int lvl)
{
  if (lvl == 0) addPolygonNFF(scn,TUNNELSURFACE_ROOF,light,mat,p0,p1,p2,p3);
  else
    {
      PointNFF v01 = VECTORNFF(p0,p1);
      PointNFF v03 = VECTORNFF(p0,p3);
      PointNFF vn = CROSSPRODNFF(v01,v03);
      double tmp = sqrt(vn[0]*vn[0]+vn[1]*vn[1]+vn[2]*vn[2]);
      PointNFF nt = PRODNFF(vn,d/tmp);
      PointNFF p01 = SUMMNFF(SEMIPOINTNFF(p0,p1),nt);
      PointNFF p32 = SUMMNFF(SEMIPOINTNFF(p3,p2),nt);
      generateRoofPolygonNFF(scn,light,mat,p0,p01,p32,p3,d/4.,lvl-1);
      generateRoofPolygonNFF(scn,light,mat,p2,p32,p01,p1,d/4.,lvl-1);
    }
}

SceneNFF* getTunnelNFF(const sTunnelParameters& tp)
{
  PointNFF pt;
  SceneNFF* res = new SceneNFF();
  /* sol */
  {
    PointNFF p0(tp.shape._width/2.,0.,0.);
    PointNFF p1(-tp.shape._width/2.,0.,0.);
    PointNFF p2(-tp.shape._width/2.,tp.shape._length,0.);
    PointNFF p3(tp.shape._width/2.,tp.shape._length,0.);
    addPolygonNFF(res,TUNNELSURFACE_FLOOR,NULL,NULL,p0,p1,p2,p3);
  }
  /* murs */
  double hmur = (tp.shape._roofShape == TUNNELSHAPE_PLANAR) ? tp.shape._maxHeight : tp.shape._minHeight;
  {
    PointNFF p0(tp.shape._width/2.,0.,0.);
    PointNFF p1(tp.shape._width/2.,tp.shape._length,0.);
    PointNFF p2(tp.shape._width/2.,tp.shape._length,hmur);
    PointNFF p3(tp.shape._width/2.,0.,hmur);
    addPolygonNFF(res,TUNNELSURFACE_WALL,NULL,NULL,p0,p1,p2,p3);
  }
  {
    PointNFF p0(-tp.shape._width/2.,0.,0.);
    PointNFF p1(-tp.shape._width/2.,0.,hmur);
    PointNFF p2(-tp.shape._width/2.,tp.shape._length,hmur);
    PointNFF p3(-tp.shape._width/2.,tp.shape._length,0.);
    addPolygonNFF(res,TUNNELSURFACE_WALL,NULL,NULL,p0,p1,p2,p3);
  }
  /* plafond */
  switch (tp.shape._roofShape)
    {
    case TUNNELSHAPE_PLANAR:
      {
	PointNFF p0(-tp.shape._width/2.,0.,hmur);
	PointNFF p1(tp.shape._width/2.,0.,hmur);
	PointNFF p2(tp.shape._width/2.,tp.shape._length,hmur);
	PointNFF p3(-tp.shape._width/2.,tp.shape._length,hmur);
	generateRoofPolygonNFF(res,NULL,NULL,p0,p1,p2,p3,0.,0);
      }
      break;
    case TUNNELSHAPE_TRIANGULAR:
      {
	PointNFF p0(-tp.shape._width/2.,0.,tp.shape._minHeight);
	PointNFF p1(tp.shape._width/2.,0.,tp.shape._minHeight);
	PointNFF p2(tp.shape._width/2.,tp.shape._length,tp.shape._minHeight);
	PointNFF p3(-tp.shape._width/2.,tp.shape._length,tp.shape._minHeight);
	generateRoofPolygonNFF(
			       res,NULL,NULL,p0,p1,p2,p3,
			       tp.shape._maxHeight - tp.shape._minHeight,
			       1
			       );
      }
      break;
   case TUNNELSHAPE_SPHERICAL:
     {
       PointNFF p0(-tp.shape._width/2.,0.,tp.shape._minHeight);
       PointNFF p1(tp.shape._width/2.,0.,tp.shape._minHeight);
       PointNFF p2(tp.shape._width/2.,tp.shape._length,tp.shape._minHeight);
       PointNFF p3(-tp.shape._width/2.,tp.shape._length,tp.shape._minHeight);
       generateRoofPolygonNFF(
			      res,NULL,NULL,p0,p1,p2,p3,
			      tp.shape._maxHeight - tp.shape._minHeight,
			      4
			      );
     }
      break;
    }
  cout << endl << "Scene Nff :: " << res->card() << endl;
  return res;
}
