#include "recherche/Recherche.h"

#include "others/Parameters.h"

#include "recherche/Maillage.h"
#include "recherche/Motif.h"

#include "radiosite/Surface.h"
#include "radiosite/Patch.h"
#include "radiosite/Solver.h"
#include "radiosite/Spectre.h"
#include "radiosite/Lambda.h"
#include "radiosite/Wavelet.h"

long nbinsertcall = 0;

template < class T > int isInList(cList< T >& l,T elt)
{
  for (int i=1;i<=l.card();i++)
    if (l[i] == elt) return 1;
  return 0;
}

template < class T > void copyList(cList< T* >& l,cList< T* >& lt)
{
  lt.freeMemory();
  for (int i=1;i<=l.card();i++)
    {
      lt.add(l[i]);
    }
}

template < class T > void initializeList(cList< T* >& l,cList< T* >& lt,cList< int >& li,int decal = 0)
{
  lt.freeMemory();
  for (int i=1;i<=li.card();i++)
    {
      if (((li[i]+decal) > 0) && ((li[i]+decal) <= l.card())) lt.add(l[li[i]+decal]);
    }
}

void sortMaterialList(cList< sMaterialReflectanceSpectrum* >& lm,cList< sMaterialReflectanceSpectrum* >& lmt,cList< int >& li)
{
  int sortedindexeslist[100];
  int nbsortindex = 0;
  int index = 0;
  lmt.freeMemory();
  if (isInList(li,1))
    { // use all materials
      for (int i=1;i<=lm.card();i++)
	{
	  sMaterialReflectanceSpectrum* crtm = lm[i];
	  index = 0; while ((index < nbsortindex) && (crtm->_meanSpectrumValue < lm[sortedindexeslist[index]]->_meanSpectrumValue)) { index++; }
	  for (int j=nbsortindex-1;j>=index;j--) sortedindexeslist[j+1] = sortedindexeslist[j];
	  sortedindexeslist[index] = i; nbsortindex++;
	}
    }
  else
    { // use specified materials
      for (int i=1;i<=li.card();i++)
	{
	  sMaterialReflectanceSpectrum* crtm = lm[li[i]-1];
	  index = 0; while ((index < nbsortindex) && (crtm->_meanSpectrumValue < lm[sortedindexeslist[index]]->_meanSpectrumValue)) { index++; }
	  for (int j=nbsortindex-1;j>=index;j--) sortedindexeslist[j+1] = sortedindexeslist[j];
	  sortedindexeslist[index] = li[i]-1; nbsortindex++;
	}
    }
  for (int i=0;i<nbsortindex;i++)
    {
      // cout << i << " :: " << sortedindexeslist[i] << " :: " << lm[sortedindexeslist[i]] << " == " << lm[sortedindexeslist[i]]->_spectrumPrice << endl;
      lmt.add(lm[sortedindexeslist[i]]);
    }
}

void initializeSeekingData()
{
  nbinsertcall = 0;
  lfi = -1;
  lfw = -1;
  lfr = -1;
  _mbase.npoint = 0;
  _mrecouv.npoint = 0;
  tres.ramp[0] = tres.ramp[1] = tres.ramp[2] = 0;
  tres.nsou[0] = tres.nsou[1] = tres.nsou[2] = 0;
  tres.sou = NULL;
  tres.fr = tres.wl = tres.rf = NULL;
  nbSolutions = 0;
  _maillage = NULL;
  coutmin = hugeValue;
  _numberOfRamps = 0;
  _positionOnMotif = 0;
}

int inserer( sRepartition& r)
{
  ENTERINGFUNCTION;
  nbinsertcall++;
  TRACEFUNCTION; TRACEVARIABLE(_maillage); TRACEVARIABLE(maxSolutions); TRACEVARIABLE(nbinsertcall);
  if ((nbSolutions == maxSolutions) && (r.prix > lesSolutions[maxSolutions-1].prix)) { TRACEFUNCTION; TRACEVARIABLE(_maillage); EXITINGFUNCTION; return -1; }
  for (int k=0;k<nbSolutions;k++) if (vraiegal(r,lesSolutions[k])) { TRACEFUNCTION; TRACEVARIABLE(_maillage); EXITINGFUNCTION; return k; }
  int i = nbSolutions-1;
  while (i >= 0) { if (lesSolutions[i].prix > r.prix) i--; else break; }
  if (nbSolutions == maxSolutions) { TRACEFUNCTION; TRACEVARIABLE(_maillage); EXITINGFUNCTION; return -1; }
  for (int j=nbSolutions-1;j>i;j--) ident(lesSolutions[j+1],lesSolutions[j]);
  ident(lesSolutions[i+1],r);
  nbSolutions++;
  TRACEFUNCTION; TRACEVARIABLE(_maillage);
  EXITINGFUNCTION;
  return i+1;
}

double evaluerCout( sRepartition& r)
{
  ENTERINGFUNCTION; TRACEVARIABLE(r);
  TRACEFUNCTION; TRACEVARIABLE((*(r.fr)));
  TRACEFUNCTION; TRACEVARIABLE((*(r.wl)));
  TRACEFUNCTION; TRACEVARIABLE((*(r.rf)));
  int i,nbsource,nbr;
  double prix,prixsource;
  nbsource=0;
  prixsource = 900.;   /*revoir : recuperer le prix*/
  nbr=0;
  for(i=0;i<=2;i++)
    {
      nbsource+=(r.nsou[i]-nbr)*r.ramp[i];
      nbr=r.nsou[i];
    }
  TRACEFUNCTION;
  r.prix= r.fr->_spectrumPrice + 2*r.wl->_spectrumPrice + r.rf->_spectrumPrice + nbsource*prixsource;
  EXITINGFUNCTION;
  return r.prix;
}

/*
  simulation d'eclairage dans la scene
  light <- source utilisee
  indexfloor <- indice du materiau pour le sol dans la liste _lf
  indexwall <- indice du materiau pour le sol dans la liste _lw
  indexroof <- indice du materiau pour le sol dans la liste _lr
  Dans le code d'eric cela correspond a l'appel (en partie) :
      emet_source(env,view,Error_Threshold,n1,0,0,m1,Area_Threshold);
*/
void evaluerSolution(sLightSource* light,
		     int indexfloor,
		     int indexwall,
		     int indexroof)
{
  ENTERINGFUNCTION; TRACEVARIABLE(light); TRACEVARIABLE(indexfloor); TRACEVARIABLE(indexwall); TRACEVARIABLE(indexroof);
  if (!((lfi == indexfloor) && (lfw == indexwall) && (lfr == indexroof)))
    {
      sMaterialReflectanceSpectrum* floor = _lf[indexfloor];
      sMaterialReflectanceSpectrum* wall = _lw[indexwall];
      sMaterialReflectanceSpectrum* roof = _lr[indexroof];
      TRACEFUNCTION; TRACEVARIABLE(_maillage); TRACEVARIABLE(floor); TRACEVARIABLE(wall); TRACEVARIABLE(roof);
      setPhotometricProperties(_maillage,floor,wall,roof);
      TRACEFUNCTION; TRACEVARIABLE(MM); TRACEVARIABLE(AArea_Threshold); TRACEVARIABLE(TThreshold);
      solve2(MM,_maillage, TThreshold,AArea_Threshold);
      lfi = indexfloor;
      lfw = indexwall;
      lfr = indexroof;
    }
  EXITINGFUNCTION;
}


/*
  regarde si il est possible de placer des source pour obtenir + de lum que necessaire
  on en profite pour calculer le nombre de rampes et la position de la premiere rampe
  par rapport au motif

  Test rapide permettant d'eviter des calculs lorsqu les radiosites requises sont trop importantes.
  On regarde ce qui se passe un peu plus loin qu'a l'entree car on peut estimer qu'a l'entree du
  tunnel, la lumiere exterieure participe aux niveaux requis sur la chaussee.

  Dans le code d'eric cela correspond a la fonction (en partie) :
       static int recouv_poss( Rad_req *courbe, Motif *mbase,int *nbramp,int *l1)
*/
int recouvrementPossible()
{
  // cout << "recouvrementPossible()" << endl;
  double xs,xk,x1,x2,y1,y2,yrequis,d;
  int nb=1,j,l,k,nontrouve,n;  
  n=_mrecouv.npoint;
  d=_mrecouv.absrad[n]-_mrecouv.absrad[0];  /*support*/	
  xs=_mrecouv.ysource;
  j=1;
  while (_mrecouv.absrad[j] < Parameters::_limiteEntreeDuTunnel) j++;
  xk=_mrecouv.absrad[j]-_mrecouv.absrad[0];
  l=0;
  nontrouve=1;
  while (nontrouve)
    {
      if (xk>=_rrc._curveAbcisse[l] && xk<_rrc._curveAbcisse[l+1])
	nontrouve=0;
      else l++;
    };
  /*interpolation lineaire :radiosite requise au point xk*/
  x1=_rrc._curveAbcisse[l];
  x2=_rrc._curveAbcisse[l+1];
  y1=_rrc._curveValue[l];
  y2=_rrc._curveValue[l+1];
  yrequis=((y2-y1)*xk+(x2*y1-x1*y2))/(x2-x1);
  if (_mrecouv.radio[j] < yrequis)
    nb=2;
  if (2*_mrecouv.radio[j] < yrequis)
    nb=3;
  if (3*_mrecouv.radio[j] < yrequis)
    {nb=4;return 0;};
  for(k=j+1;k<_mrecouv.npoint;k++)
    { xk=_mrecouv.absrad[k]-_mrecouv.absrad[0];
    if (xk>x2)
      {
	l++;
	x1=x2;
	x2=_rrc._curveAbcisse[l+1];
	y1=y2;
	y2=_rrc._curveValue[l+1];
	yrequis=((y2-y1)*xk+(x2*y1-x1*y2))/(x2-x1);
      };
    if (_mrecouv.radio[k] < yrequis && nb<2)
      nb=2;
    if (2*_mrecouv.radio[k] < yrequis && nb<3)
      nb=3;
    if (3*_mrecouv.radio[k] < yrequis)
      {nb=4;return 0;};
    };
  _positionOnMotif = l;
  _numberOfRamps = nb;
  // cout << "recouvrementPossible(" << _positionOnMotif << "," << _numberOfRamps << ")" << endl;
  return 1;
}

/*
  verifie si cette solution est realisable
*/
int nonRealisable(sLightSource* light,
		   int idfloor,int idwall,int idroof)
{
  ENTERINGFUNCTION; TRACEVARIABLE(light); TRACEVARIABLE(idfloor); TRACEVARIABLE(idwall); TRACEVARIABLE(idroof);
  TRACEFUNCTION; TRACEVARIABLE(_maillage);
  evaluerSolution(light,idfloor,idwall,idroof);
  calculerMotifDeBase();
  calculerMotifDeRecouvrement();
  int res = recouvrementPossible();
  EXITINGFUNCTION; TRACEVARIABLE(res); TRACEVARIABLE(_numberOfRamps); TRACEVARIABLE(_positionOnMotif);
  return !res;
}

/*
  simule l'eclairage (mais deja fait avec nonRealisable)
  calcule les repartition des sources
  stocke la solution
  en resultat renvoie la repartition des sources
  Dans le code d'eric cela correspond a la fonction (en partie) :
       repartition(courbe,mbase,tres4,h,&pos)
*/
sRepartition simulate(sLightSource* light,
		      int idfloor,int idwall,int idroof)
{
  ENTERINGFUNCTION; TRACEVARIABLE(light); TRACEVARIABLE(idfloor); TRACEVARIABLE(idwall); TRACEVARIABLE(idroof);
   sRepartition res;
  double xk,x1,x2,y1,y2,yrequis,d;
  int cpt,k,nummotif,dec=0,n,deb,pos;

  TRACECALL(nonRealisable); pos = !nonRealisable(light,idfloor,idwall,idroof);

  res.ramp[0]=_numberOfRamps; 
  if (_numberOfRamps==1) {
    TRACEFUNCTION;
    res.nsou[0]=floor(_rrc._curveStopDistance/_tp.light._lightSpacing);
    for (k=1;k<=2;k++){
      res.ramp[k]=0;
      res.nsou[k]=0;
    }
    TRACEFUNCTION;
    res.sou = light;
    res.fr = _lf[idfloor];
    res.wl = _lw[idwall];
    res.rf = _lr[idroof];
    EXITINGFUNCTION; TRACEVARIABLE(res);
    return res;
  }
  _positionOnMotif=_positionOnMotif+1;
  n=_mrecouv.npoint;
  d=_mrecouv.absrad[n]-_mrecouv.absrad[0];  /*support*/	    
  x1=_rrc._curveAbcisse[_positionOnMotif];
  if (x1<_rrc._curveStopDistance)
    {
      TRACEFUNCTION; TRACEVARIABLE(x1); TRACEVARIABLE(_rrc._curveStopDistance); TRACEVARIABLE(_numberOfRamps); TRACEVARIABLE(_positionOnMotif);
      x2=_rrc._curveAbcisse[_positionOnMotif+1];
      y1=_rrc._curveValue[_positionOnMotif];
      y2=_rrc._curveValue[_positionOnMotif+1];
      deb=floor((x2-x1)*n/d) +1;
      nummotif=1;
      while(x2<_rrc._curveStopDistance && _numberOfRamps>1)
	{
	  TRACEFUNCTION; TRACEVARIABLE(x2); TRACEVARIABLE(_rrc._curveStopDistance); TRACEVARIABLE(_numberOfRamps); TRACEVARIABLE(_positionOnMotif);
	  nummotif++;
	  cpt=0;
	  for(k=deb;k<_mrecouv.npoint;k++)
	    {
	      xk=_mrecouv.absrad[k]+(nummotif-1)*d-_mrecouv.absrad[0];
	      if (xk>x2 && xk<_rrc._curveStopDistance)
		{
		  _positionOnMotif++;
		  x1=x2;
		  x2=_rrc._curveAbcisse[_positionOnMotif+1];
		  y1=y2;
		  y2=_rrc._curveValue[_positionOnMotif+1];
		}
	      yrequis=((y2-y1)*xk+(x2*y1-x1*y2))/(x2-x1);
	      if ((_numberOfRamps-1)*_mrecouv.radio[k] >= yrequis)
		cpt++;
	    }
	  if (cpt==_mrecouv.npoint-deb)
	    {
	      /*sur tout le motif on etait au dessus */
	      /*avec une rampe de moins*/
	      res.nsou[dec]=floor((nummotif-1)*d/_tp.light._lightSpacing);   /*nbr d'espacement*/
	      _numberOfRamps--;
	      dec++;
	      res.ramp[dec]=_numberOfRamps;
	    }
	}
      TRACEFUNCTION;
    }
  res.nsou[dec]=floor(_rrc._curveStopDistance/_tp.light._lightSpacing);
  for (k=dec+1;k<=2;k++)
    {
      TRACEFUNCTION;
      res.ramp[k]=0;
      res.nsou[k]=0;
    }
  res.sou = light;
  res.fr = _lf[idfloor];
  res.wl = _lw[idwall];
  res.rf = _lr[idroof];
  res.prix = 0.;
  EXITINGFUNCTION; TRACEVARIABLE(res);
  return res;
}

void completerSolutions(int a,int b,int c,int place,int *i,int *j,int *k,
			sLightSource* light,
			int mnfloor,int mnwall,int mnroof,
			int mxfloor,int mxwall,int mxroof,
			 sRepartition& tres2)
{
  ENTERINGFUNCTION;
  TRACEVARIABLE(mnfloor); TRACEVARIABLE(mnwall); TRACEVARIABLE(mnroof);
  TRACEVARIABLE(mxfloor); TRACEVARIABLE(mxwall); TRACEVARIABLE(mxroof);
  int i1,j1,k1,pl,pos;
  double prix;
   sRepartition tres3;
  pl=place;
  if (c > mnroof)
    {
      //cout << "completerSolutions " << __LINE__ << " :: if (" << c << " > " << mnroof << ")" << endl;
      i1=a;
      j1=b;
      k1=c-1;
      //cout << "completerSolutions " << __LINE__ << " :: fin if (" << c << " > " << mnroof << ")" << endl;
    } /* if (c > mnroof) */
  else /* if (c > mnroof) */
    {
      //cout << "completerSolutions " << __LINE__ << " :: else if (" << c << " > " << mnroof << ")" << endl;
      k1=mxroof;
      if (b > mnwall)
	{
	  //cout << "completerSolutions " << __LINE__ << " :: if (" << b << " > " << mnwall << ")" << endl;
	  i1=a;
	  j1=b-1;
	  //cout << "completerSolutions " << __LINE__ << " :: fin if (" << b << " > " << mnwall << ")" << endl;
	} /* if (b > mnwall) */
      else
	{ /* if (b > mnwall) */
	  //cout << "completerSolutions " << __LINE__ << " :: else if (" << b << " > " << mnwall << ")" << endl;
	  j1=mxwall;
	  i1=a-1;
	  //cout << "completerSolutions " << __LINE__ << " :: fin else if (" << b << " > " << mnwall << ")" << endl;
	} /* else if (b > mnwall) */
      //cout << "completerSolutions " << __LINE__ << " :: fin else if (" << c << " > " << mnroof << ")" << endl;
    } /* else if (c > mnroof) */
  while (pl > 0)
    {
      //cout << "completerSolutions " << __LINE__ << " :: while (" << pl << " > 0) :: " << i1 << "," << j1 << "," << k1 << endl;
      TRACEFUNCTION; TRACEVARIABLE(_maillage);
      TRACECALL(nonRealisable); pos = !nonRealisable(light,i1,j1,k1);
      TRACEFUNCTION; TRACEVARIABLE(_maillage);
      if (k1 == mxroof)
	{
	  //cout << "completerSolutions " << __LINE__ << " :: if (" << k1 << " == " << mxroof << ") :: " << pos << endl;
	  TRACECALL(simulate); sRepartition tmprep = simulate(light,i1,j1,k1); ident(tres2,tmprep);
	  if (pos!=0)
	    {
	      //cout << "completerSolutions " << __LINE__ << " :: if (" << pos << " != 0) :: " << tres3 << endl;
	      prix = evaluerCout(tres2);
	      pl = inserer(tres2);
	      //cout << "completerSolutions " << __LINE__ << " :: fin if (" << pos << " != 0) :: " << prix << "," << pl << endl;
	    } /* if (pos != 0) */
	  else
	    {
	      pl=-1;
	    }
	  //cout << "completerSolutions " << __LINE__ << " :: fin if (" << k1 << " == " << mxroof << ") :: " << pos << endl;
	} /* if (k1 == mxroof) */
      else /* if (k1 == mxroof) */
	{
	  //cout << "completerSolutions " << __LINE__ << " :: else (" << k1 << " == " << mxroof << ") :: " << pos << endl;
	  TRACECALL(simulate); sRepartition tmprep = simulate(light,i1,j1,k1); ident(tres3,tmprep);
	  TRACEFUNCTION; TRACEVARIABLE(_maillage);
	  if (pos != 0)
	    {
	      //cout << "completerSolutions " << __LINE__ << " :: if (" << pos << " != 0) :: " << tres3 << endl;
	      TRACEFUNCTION; TRACEVARIABLE(_maillage);
	      prix = evaluerCout(tres3);
	      TRACEFUNCTION; TRACEVARIABLE(_maillage);
	      pl = inserer(tres3);
	      TRACEFUNCTION; TRACEVARIABLE(_maillage);
	      //cout << "completerSolutions " << __LINE__ << " :: fin if (" << pos << " != 0) :: " << prix << "," << pl << endl;
	    } /* if (pos != 0) */
	  else
	    {
	      //cout << "completerSolutions " << __LINE__ << " :: else if (" << pos << " != 0) :: " << tres3 << endl;
	      TRACEFUNCTION; TRACEVARIABLE(_maillage);
	      pl=-1;
	      //cout << "completerSolutions " << __LINE__ << " :: fin else if (" << pos << " != 0) :: " << tres3 << endl;
	    }
	  //cout << "completerSolutions " << __LINE__ << " :: fin else (" << k1 << " == " << mxroof << ") :: " << pos << endl;
	} /* else if (k1 == mxroof) */
      if (pl < 1)
	{
	  //cout << "completerSolutions " << __LINE__ << " :: if (" << pl << " < 1) :: " << i1 << "," << j1 << "," << k1 << endl;
	  *i=i1;*j=j1;*k=k1;
	  EXITINGFUNCTION; return;
	  //cout << "completerSolutions " << __LINE__ << " :: fin if (" << pl << " < 1) :: " << i1 << "," << j1 << "," << k1 << endl;
	} /* if (pl < 1) */
      if (k1 > mnroof)
	{
	  //cout << "completerSolutions " << __LINE__ << " :: if (" << k1 << " > " << mnroof << ") :: " << endl;
	  TRACEFUNCTION; TRACEVARIABLE(_maillage);
	  k1--;
	  //cout << "completerSolutions " << __LINE__ << " :: fin if (" << k1 << " > " << mnroof << ") :: " << endl;
	} /* if (k1 > mnroof) */
      else /* if (k1 > mnroof) */
	{
	  //cout << "completerSolutions " << __LINE__ << " :: else if (" << k1 << " > " << mnroof << ") :: " << endl;
	  TRACEFUNCTION; TRACEVARIABLE(_maillage);
	  k1=mxroof;
	  if (j1 > mnwall)
	    {
	      //cout << "completerSolutions " << __LINE__ << " :: if (" << j1 << " > " << mnwall << ") :: " << endl;
	      TRACEFUNCTION; TRACEVARIABLE(_maillage);
	      j1--;
	      //cout << "completerSolutions " << __LINE__ << " :: fin if (" << j1 << " > " << mnwall << ") :: " << endl;
	    } /* if (j1 > mnwall) */
	  else /* if (j1 > mnwall) */
	    {
	      //cout << "completerSolutions " << __LINE__ << " :: else if (" << j1 << " > " << mnwall << ") :: " << endl;
	      TRACEFUNCTION; TRACEVARIABLE(_maillage);
	      j1=mxwall;
	      if (i1 > mnfloor)
		{
		  //cout << "completerSolutions " << __LINE__ << " :: if (" << i1 << " > " << mnfloor << ") :: " << endl;
		  TRACEFUNCTION; TRACEVARIABLE(_maillage);
		  i1--;
		  //cout << "completerSolutions " << __LINE__ << " :: fin if (" << i1 << " > " << mnfloor << ") :: " << endl;
		}
	      else
		{
		  //cout << "completerSolutions " << __LINE__ << " :: else if (" << i1 << " > " << mnfloor << ") :: " << endl;
		  *i=i1;*j=j1;*k=k1;
		  EXITINGFUNCTION; return;
		  //cout << "completerSolutions " << __LINE__ << " :: fin else if (" << i1 << " > " << mnfloor << ") :: " << endl;
		}
	      //cout << "completerSolutions " << __LINE__ << " :: fin else if (" << j1 << " > " << mnwall << ") :: " << endl;
	    } /* else if (j1 > mnwall) */
	  //cout << "completerSolutions " << __LINE__ << " :: fin else if (" << k1 << " > " << mnroof << ") :: " << endl;
	} /* else if (k1 > mnroof) */
      TRACEFUNCTION; TRACEVARIABLE(_maillage);
    } /* while (pl > 0) */
  EXITINGFUNCTION;
}

void rechercheNiveau1(sLightSource* light,
		      int mnfloor,int mnwall,int mnroof,
		      int mxfloor,int mxwall,int mxroof)
{
  ENTERINGFUNCTION; TRACEVARIABLE(light);
  TRACEVARIABLE(mnfloor); TRACEVARIABLE(mnwall); TRACEVARIABLE(mnroof);
  TRACEVARIABLE(mxfloor); TRACEVARIABLE(mxwall); TRACEVARIABLE(mxroof);
   sRepartition tres4,tres2;
  int res,place,i,j,k,i1,j1,k1;
  double prix;
  int n1 = (int)(fabs((mxfloor+mnfloor+1)/2.));
  int rhoSi = n1;
  int rhoMi = mnwall;
  int rhoPi = mnroof;

  TRACECALL(nonRealisable); res = !nonRealisable(light,rhoSi,rhoMi,rhoPi);
  if (!res)
    {
      //cout << "nonRealisable(" << light << "," << rhoSi << "," << rhoMi << "," << rhoPi << ")" << endl;
      if ((n1 > 0) && (mxfloor >= mnfloor))
	{
	  TRACECALL(rechercheNiveau1); rechercheNiveau1(light,mnfloor,mnwall,mnroof,n1-1,mxwall,mxroof);
	}
    } /* if (!res) */
  else /* if (!res) */
    {
      TRACECALL(simulate); sRepartition tmprep = simulate(light,rhoSi,rhoMi,rhoPi); ident(tres4,tmprep);
      prix = evaluerCout(tres4);
      place = inserer(tres4);
      rhoMi = mxwall;
      rhoPi = mxroof;
      j = mxwall;
      TRACECALL(nonRealisable); res = !nonRealisable(light,rhoSi,rhoMi,rhoPi);
      if (res)
	{
	  TRACECALL(simulate); sRepartition tmprep = simulate(light,rhoSi,rhoMi,rhoPi); ident(tres2,tmprep);
	  prix = evaluerCout(tres2);
	  place = inserer(tres2);
	  if (n1 < mxfloor)
	    {
	      rechercheNiveau1(light,n1+1,mnwall,mnroof,mxfloor,mxwall,mxroof);
	      place = inserer(tres2);
	    } /* if (n1 < mxfloor) */
	  if (place > 0)
	    {
	      /* appel a complete_cout */
	      //cout << "rechercheNiveau1 :: " << __LINE__ << " completion des solutions" << endl;
	      TRACECALL(completerSolutions); completerSolutions(n1,mxwall,mxroof,place,&i,&j,&k,light,mnfloor,mnwall,mnroof,mxfloor,mxwall,mxroof,tres2);
	    } /* if (place > 0) */
	  else /* if (place > 0) */
	    {
	      i = n1;
	      j = mxwall;
	      k = mxroof;
	    } /* else de if (place > 0) */
	  if (egal(tres4,tres2))
	    {
	      i--;
	      if (i >= mnfloor)
		{
		  j = mxwall; k = mxroof;
		  TRACECALL(nonRealisable); res = !nonRealisable(light,i,j,k);
		  TRACECALL(simulate); sRepartition tmprep = simulate(light,i,j,k); ident(tres2,tmprep);
		  prix = evaluerCout(tres2);
		  place = inserer(tres2);
		  if (place > 0)
		    {
		      i1 = i; j1 = j; k1 = k;
		      /* appel a complete_cout */
		      //cout << "rechercheNiveau1 :: " << __LINE__ << " completion des solutions" << endl;
		      TRACECALL(completerSolutions); completerSolutions(i1,j1,k1,place,&i,&j,&k,light,mnfloor,mnwall,mnroof,mxfloor,mxwall,mxroof,tres2);
		    } /* if (place > 0) */
		} /* if (i >= mnfloor) */
	    } /* if (egal(tres4,tres2)) */
	  while (i >= mnfloor)
	    {
	      //cout << "rechercheNiveau1 " << __LINE__ << " while (" << i << " >= " << mnfloor << ")" << endl;
	      j++;
	      while (j > mnwall)
		{
		  //cout << "rechercheNiveau1 " << __LINE__ << " while (" << j << " > " << mnwall << ")" << endl;
		  ident(tres,tres2);
		  while (egal(tres,tres2) && (j > mnwall))
		    {
		      //cout << "rechercheNiveau1 " << __LINE__ << " while (egal(" << tres << "," << tres2 << ") && (" << j << " > " << mnwall << "))" << endl;
		      j--;
		      if (k != mnroof)
			{
			  TRACECALL(nonRealisable); res = !nonRealisable(light,i,j,mnroof);
			  TRACECALL(simulate); sRepartition tmprep = simulate(light,i,j,mnroof); ident(tres,tmprep);
			} /* if (k != mnroof) */
		      else k = mxroof;
		    } /* while (egal(tres,tres2) && (j > mnwall)) */
		  if (!egal(tres,tres2))
		    {
		      prix = evaluerCout(tres);
		      place = inserer(tres);
		      for (k1 = mnroof;k1<k;k1++)
			{
			  TRACECALL(nonRealisable); res = !nonRealisable(light,i,j,k1);
			  TRACECALL(simulate); sRepartition tmprep = simulate(light,i,j,k1); ident(tres,tmprep);
			  prix = evaluerCout(tres);
			  place = inserer(tres);
			} /* for (k1 = mnroof;k1<k;k++) */
		      if (j > mnwall)
			{
			  k = mxroof;
			  TRACECALL(nonRealisable); res = !nonRealisable(light,i,j-1,k);
			  TRACECALL(simulate); sRepartition tmprep = simulate(light,i,j-1,k); ident(tres2,tmprep);
			  prix = evaluerCout(tres2);
			  place = inserer(tres2);
			  if (place > 0)
			    {
			      i1 = i; j1 = j; k1 = k;
			      /* appel a complete_cout */
			      TRACECALL(completerSolutions); completerSolutions(i1,j1,k1,place,&i,&j,&k,light,mnfloor,mnwall,mnroof,mxfloor,mxwall,mxroof,tres2);
			    } /* if (place > 0) */
			} /* if (j > mnwall) */
		    } /* if (!egal(tres,tres2)) */
		} /* while (j > mnwall) */
	      j = mxwall;
	      i--;
	      if (i >= mnfloor)
		{
		  j = mxwall;
		  k = mxroof;
		  TRACECALL(nonRealisable); res = !nonRealisable(light,i,j,k);
		  TRACECALL(simulate); sRepartition tmprep = simulate(light,i,j,k); ident(tres2,tmprep);
		  prix = evaluerCout(tres2);
		  place = inserer(tres2);
		  if (place > 0)
		    {
		      i1 = i; j1 = j; k1 = k;
		      /* appel a complete_cout */
		      //cout << "rechercheNiveau1 :: " << __LINE__ << " completion des solutions" << endl;
		      TRACECALL(completerSolutions); completerSolutions(i1,j1,k1,place,&i,&j,&k,light,mnfloor,mnwall,mnroof,mxfloor,mxwall,mxroof,tres2);
		    } /* if (place > 0) */
		} /* if (i >= mnfloor) */
	    } /* while (i >= mnfloor) */
	} /* if (res) */
      else /* if (res) */
	{
	  while ((!res) && (j > mnwall))
	    {
	      //cout << "rechercheNiveau1 " << __LINE__ << " while ((!" << res << ") && (" << j << " > " << mnwall << "))" << endl;
	      TRACECALL(nonRealisable); res = !nonRealisable(light,n1,j,mnroof);
	      TRACECALL(simulate); sRepartition tmprep = simulate(light,n1,j,mnroof); ident(tres2,tmprep);
	      if (!res) j--;
	    } /* while ((!res) && (j > mnwall)) */
	  TRACECALL(rechercheNiveau1); rechercheNiveau1(light,n1,mnwall,mnroof,mxfloor,j,mxroof);
	  TRACECALL(rechercheNiveau1); rechercheNiveau1(light,mnfloor,mnwall,mnroof,n1-1,mxwall,mxroof);
	} /* else if (res) */
    } /* else if (!res) */
  EXITINGFUNCTION;
}

void rechercheSolutions( sTunnelParameters& tp,
			 sIlluminationParameters& sp,
			int nbconfigmax,
			 sRequiredRadianceCurve& rrc,
			cList< sLightSource* >& ls,
			cList< sMaterialReflectanceSpectrum* >& lf,
			cList< sMaterialReflectanceSpectrum* >& lw,
			cList< sMaterialReflectanceSpectrum* >& lr)
{
  ENTERINGFUNCTION;
  maxSolutions = nbconfigmax;
  initializeSeekingData();
  _tp = tp; _sp = sp; _rrc = rrc;
  MM = (int)(sp._waveletBasis);
  AArea_Threshold = sp._patchAreaThreshold;
  TThreshold = sp._errorThreshold;

  if (isInList(tp.light._lightSelected,1)) copyList(ls,_ls); else initializeList(ls,_ls,tp.light._lightSelected,-1);
  sortMaterialList(lf,_lf,tp.material._floorSelected);
  sortMaterialList(lw,_lw,tp.material._wallSelected);
  sortMaterialList(lr,_lr,tp.material._roofSelected);

  int indexLight = 1;
  sLightSource* light = _ls[indexLight];

  initializeSpectralProperties(sp,light);
  _maillage = getMaillage(tp,sp);
  addLight(tp,sp,light,_maillage);
  setPhotometricProperties(_maillage,light);
  rechercheNiveau1(light,1,1,1,_lf.card(),_lw.card(),_lr.card());

  Surface* S = _maillage->slist;
  while (S != NULL)
    {
      Surface* St = S->nexts;
      _maillage->slist = St;
      delete S;
      S = St;
    }
  delete _maillage;
  _maillage = NULL;
  EXITINGFUNCTION; TRACEVARIABLE(nbSolutions); TRACEVARIABLE(nbinsertcall);
}
